// $Id: Cursor.h,v 1.2 2009/02/02 23:34:42 avalassi Exp $
#ifndef CORALACCESS_CURSOR_H
#define CORALACCESS_CURSOR_H 1

// Include files
#include <memory>
#include <vector>
#include "RelationalAccess/ICursor.h"
#include "CoralServerBase/ICoralFacade.h"
#include "CoralServerBase/QueryDefinition.h"

namespace coral 
{

  namespace CoralAccess 
  {

    // Forward declarations
    class Cursor;
    class SessionProperties;

    /** @class Cursor
     *
     *  @author Andrea Valassi
     *  @date   2007-12-05
     */

    class Cursor : virtual public ICursor
    {

    public:

      /// Constructor
      Cursor( const SessionProperties& sessionProperties,
              const Token cursorID,
              const QueryDefinition& queryDef,
              AttributeList& rowBuffer,
              size_t rowBulkSize );

      /// Destructor
      virtual ~Cursor();

      /**
       * Positions the cursor to the next available row in the result set.
       * If there are no more rows in the result set false is returned.
       */
      bool next();
      
      /**
       * Returns a reference to output buffer holding the data of the last
       * row fetched.
       */
      const AttributeList& currentRow() const;
      
      /**
       * Explicitly closes the cursor, releasing the resources on the server.
       */
      void close();

      /// Returns the facade for the CORAL server connection.
      const ICoralFacade& facade() const;

    private:

      /// The properties of this remote database session.
      const SessionProperties& m_sessionProperties;

      /// The cursorID of the remote cursor.
      const Token m_cursorID;

      /// The query definition.
      const QueryDefinition& m_queryDef;
      
      /// The row buffer.
      AttributeList& m_rowBuffer;

      /// The maximum size of each bulk of rows.
      size_t m_maxRows;

      /// The current bulk of rows.
      IRowIteratorPtr m_rows;

      /// The number of rows read in the iteration over the current bulk.
      size_t m_nRows;

    };

  }

}
#endif // CORALACCESS_CURSOR_H
