// $Id: Query.cpp,v 1.5 2009/02/05 11:34:23 avalassi Exp $

// Include files
#include <iostream>
#include "CoralServerBase/NotImplemented.h"

// Local include files
#include "ConnectionProperties.h"
#include "Cursor.h"
#include "Query.h"
#include "SessionProperties.h"
#include "logger.h"

// Namespace
using namespace coral::CoralAccess;

//-----------------------------------------------------------------------------

Query::Query( const SessionProperties& sessionProperties,
              const std::string& schemaName )
  : QueryDefinition( schemaName )
  , m_sessionProperties( sessionProperties )
  , m_cursor( 0 )
  , m_rowCacheSize( 50 ) // default
  , m_pOutputBuffer( 0 )
{
  logger << "Create Query" << endlog;
}

//-----------------------------------------------------------------------------

Query::~Query()
{
  logger << "Delete Query" << endlog;
  if ( m_cursor ) delete m_cursor;
  m_cursor = 0;
}

//-----------------------------------------------------------------------------

void Query::setForUpdate()
{
  throw NotImplemented("Query::setForUpdate");
}

//-----------------------------------------------------------------------------

void Query::setRowCacheSize( int numberOfCachedRows )
{
  if ( numberOfCachedRows < 0 )
  {
    logger << "Query::setRowCacheSize called with #rows < 0 : will use 0" << endlog;
    m_rowCacheSize = 0;
  }
  m_rowCacheSize = (size_t)numberOfCachedRows;
}

//-----------------------------------------------------------------------------

void Query::setMemoryCacheSize( int /*sizeInMB*/ )
{
  throw NotImplemented("Query::setMemoryCacheSize");
}

//-----------------------------------------------------------------------------

void Query::defineOutputType( const std::string& /*outputIdentifier*/,
                              const std::string& /*cppTypeName*/ )
{
  throw NotImplemented("Query::defineOutputType");
}

//-----------------------------------------------------------------------------

void Query::defineOutput( coral::AttributeList& outputDataBuffer )
{
  m_pOutputBuffer = &outputDataBuffer;
}

//-----------------------------------------------------------------------------

coral::ICursor& Query::execute()
{
  if ( m_cursor ) 
    throw Exception( "Query is already executing", 
                     "Query::execute",
                     "coral::CoralAccess" );

  /*
  // New behaviour for CoralAccess (users must call defineOutput)??
  if ( !m_pOutputBuffer ) 
  {
    logger << Error << "Query::defineOutput has not been called" << endlog;
    throw Exception( "Query::defineOutput has not been called", 
                     "Query::execute", 
                     "coral::CoralAccess" );
  }
  */  

  coral::Token cursorID;
  if ( m_rowCacheSize > 0 ) 
  {
    logger << "Execute query (fetch " << m_rowCacheSize << " rows at a time)" << endlog;
    cursorID = facade().executeQuery( m_sessionProperties.sessionID(), *this, m_pOutputBuffer );
  }
  else 
  {
    logger << "Execute query (fetch ALL rows in one go)" << endlog;
    cursorID = 0;
  }
  logger << "Create cursor" << endlog;
  m_cursor = new Cursor( m_sessionProperties, cursorID, *this, *m_pOutputBuffer, m_rowCacheSize );
  return *m_cursor;
}

//-----------------------------------------------------------------------------

const coral::ICoralFacade& Query::facade() const
{
  return m_sessionProperties.connectionProperties().facade();
}

//-----------------------------------------------------------------------------
