// $Id: Table.cpp,v 1.5 2009/02/02 23:20:17 avalassi Exp $

// Include files
#include <iostream>
#include "CoralServerBase/InternalErrorException.h"
#include "CoralServerBase/NotImplemented.h"

// Local include files
#include "ConnectionProperties.h"
#include "SessionProperties.h"
#include "Table.h"
#include "TableDataEditor.h"
#include "logger.h"

// Namespace
using namespace coral::CoralAccess;

//-----------------------------------------------------------------------------

Table::Table( const SessionProperties& sessionProperties,
              const std::string& schemaName,
              const TableDescription& desc )
  : m_sessionProperties( sessionProperties )
  , m_schemaName( schemaName )
  , m_descriptionProxy( new TableDescription( desc ) )
  , m_dataEditor( new TableDataEditor( sessionProperties, *m_descriptionProxy ) )
{
  logger << "Create Table '" << desc.name() << "'" << endlog;
}

//-----------------------------------------------------------------------------

Table::~Table()
{
  if ( m_descriptionProxy )
    logger << "Delete Table '" << m_descriptionProxy->name() << "'" << endlog;
  else
    logger << "Delete Table (WARNING: invalid description)" << endlog;
  if ( m_dataEditor ) delete m_dataEditor;
  m_dataEditor = 0;
  if ( m_descriptionProxy ) delete m_descriptionProxy;
  m_descriptionProxy = 0;
}

//-----------------------------------------------------------------------------

const coral::ITableDescription& Table::description() const
{
  if ( ! m_descriptionProxy )
    throw InternalErrorException( "PANIC! Invalid table description proxy", 
                                  "Table::description", 
                                  "coral::CoralAccess" );
  return *m_descriptionProxy;
}

//-----------------------------------------------------------------------------
    
coral::ITableSchemaEditor& Table::schemaEditor()
{
  throw NotImplemented("Table::schemaEditor");
}

//-----------------------------------------------------------------------------

coral::ITableDataEditor& Table::dataEditor()
{
  if ( ! m_dataEditor )
    throw InternalErrorException( "PANIC! Invalid table data editor", 
                                  "Table::dataEditor", 
                                  "coral::CoralAccess" );
  return *m_dataEditor;
}

//-----------------------------------------------------------------------------

coral::ITablePrivilegeManager& Table::privilegeManager()
{
  throw NotImplemented("Table::privilegeManager");
}

//-----------------------------------------------------------------------------

coral::IQuery* Table::newQuery() const
{
  throw NotImplemented("Table::newQuery");
}

//-----------------------------------------------------------------------------


