// $Id: Table.h,v 1.3 2009/02/02 15:07:23 avalassi Exp $
#ifndef CORALACCESS_TABLE_H
#define CORALACCESS_TABLE_H 1

// Include files
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/TableDescription.h" // TEMPORARY
#include "CoralServerBase/ICoralFacade.h"

namespace coral 
{

  namespace CoralAccess 
  {

    // Forward declarations
    class Cursor;
    class SessionProperties;
    class TableDataEditor;

    /** @class Table
     *
     *  @author Andrea Valassi
     *  @date   2007-12-05
     */

    class Table : virtual public coral::ITable
    {

    public:

      /// Constructor
      Table( const SessionProperties& sessionProperties,
             const std::string& schemaName,
             const TableDescription& desc );

      /// Destructor
      virtual ~Table();

      /**
       * Returns the description of the table.
       */
      const ITableDescription& description() const;

      /**
       * Returns a reference to the schema editor for the table.
       */
      ITableSchemaEditor& schemaEditor();

      /**
       * Returns a reference to the ITableDataEditor object  for the table.
       */
      ITableDataEditor& dataEditor();

      /**
       * Returns a reference to the privilege manager of the table.
       */
      ITablePrivilegeManager& privilegeManager();

      /**
       * Returns a new query object for performing a query involving this table only.
       */
      IQuery* newQuery() const;

      /// Returns the facade for the CORAL server connection.
      const coral::ICoralFacade& facade() const;

    private:

      /// The properties of this remote database session.
      const SessionProperties& m_sessionProperties;

       /// The schema name ("" means nominal schema).
      const std::string m_schemaName;

      /// The table description proxy (owned by this Table).
      TableDescription* m_descriptionProxy;

      /// The table data editor (owned by this Table).
      TableDataEditor* m_dataEditor;

    };

  }

}
#endif // CORALACCESS_TABLE_H
