// $Id: QueryMgr.cpp,v 1.2 2009/02/05 11:34:23 avalassi Exp $

// Include files
#include "RelationalAccess/ISchema.h"

// Local include files
#include "QueryMgr.h"

// Namespaces
using namespace coral::CoralServer;

//---------------------------------------------------------------------------

coral::IQuery* QueryMgr::newQuery( coral::AttributeList* pDataBuffer,
                                   coral::ISessionProxy& session,
                                   const coral::QueryDefinition& qd )
{
  // Get a new query in the appropriate schema ---
  coral::IQuery* pQ;
  const std::string& schemaName = qd.getSchemaName();
  if ( schemaName == "" ) pQ = session.nominalSchema().newQuery();
  else pQ = session.schema( schemaName ).newQuery();

  // Prepare the query from its definition ---
  prepareQuery( *pQ, qd );

  // Define the output buffer ---
  if ( pDataBuffer ) pQ->defineOutput( *pDataBuffer );

  // Set the prefetch row cache size
  //const int newRowCacheSize = 100;
  //query->setRowCacheSize( newRowCacheSize );

  // Return the query
  return pQ;

}

//---------------------------------------------------------------------------

void QueryMgr::prepareQuery( coral::IQueryDefinition& query,
                             const coral::QueryDefinition& qd )
{
  
  // Distinct flag
  if ( qd.getDistinct() ) query.setDistinct();

  // Output list
  const std::vector< std::pair<std::string,std::string> >& cols = qd.getOutputList();
  for ( unsigned int i=0; i<cols.size(); i++ )
    query.addToOutputList( cols[i].first, cols[i].second );

  // Table list
  const std::vector< std::pair<std::string,std::string> >& tabs = qd.getTableList();
  for ( unsigned int i=0; i<tabs.size(); i++ )
    query.addToTableList( tabs[i].first, tabs[i].second );

  // Subqueries
  const std::map< std::string, QueryDefinition >& sqds = qd.getSubQueries();
  for ( std::map< std::string, QueryDefinition >::const_iterator 
          sqd = sqds.begin(); sqd != sqds.end(); sqd++ )
  {
    IQueryDefinition& sq = query.defineSubQuery( sqd->first );
    prepareQuery( sq, sqd->second );
  }

  // Condition
  query.setCondition( qd.getCondition(), qd.getConditionData() );

  // Grouping
  query.groupBy( qd.getGroupBy() );

  // Order
  const std::vector< std::string >& ords = qd.getOrderList();
  for ( unsigned int i=0; i<ords.size(); i++ )
    query.addToOrderList( ords[i] );

  // Limit returned rows
  const std::pair<int,int>* rowLimit = qd.getRowLimitAndOffset();
  if ( rowLimit )
    query.limitReturnedRows( rowLimit->first, rowLimit->second );

  // Set operation
  const std::pair<IQueryDefinition::SetOperation,QueryDefinition>* setOp = qd.getSetOperation();
  if ( setOp )
  {
    IQueryDefinition& sq = query.applySetOperation( setOp->first );
    prepareQuery( sq, setOp->second );
  }

}
      
//---------------------------------------------------------------------------
