// $Id: coralServer.cpp,v 1.2 2009/02/10 13:15:32 avalassi Exp $

// Include files
#include <iostream>
#include <stdlib.h> // For setenv and _putenv
#include "CoralKernel/Context.h"
#include "CoralServer/CoralServerFacade.h"
#include "CoralSockets/SocketServer.h"
#include "CoralStubs/ServerStub.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"

// Message output
#define LOG std::cout << "__coralServer "

// Namespace
using namespace coral;

//-----------------------------------------------------------------------------

/** @file coralServer.cpp
 *
 *  @author Andrea Valassi
 *  @date   2009-02-09
 */

int main( int argc, char** argv )
{

  LOG << "Entering main" << std::endl;
  try 
  {
    
    // Get the server parameters
    std::string host;
    int port;
    if ( argc == 1 ) 
    {
      host = "localhost";
      port = 40007;
    }
    else if ( argc == 3 ) 
    {
      host = argv[1];
      port = atoi( argv[2] );
    } 
    else 
    {
      LOG << "Usage:   " << argv[0] << " [host port]" << std::endl;
      LOG << "Example: " << argv[0] << " localhost 40007" << std::endl;
      return 1;
    }

    // Setup CORAL db lookup and authentication from the user's AFS private 
    /*
    std::string user( getenv("USER") );
    std::string afsPrivate = "/afs/cern.ch/user/"+user.substr(0,1)+"/"+user+"/private/CoralServer";
#ifdef WIN32
    _putenv( ( "CORAL_DBLOOKUP_PATH=" + afsPrivate ).c_str() );
    _putenv( ( "CORAL_AUTH_PATH" + afsPrivate ).c_str() );
#else
    setenv( "CORAL_DBLOOKUP_PATH", afsPrivate.c_str(), 1 );
    setenv( "CORAL_AUTH_PATH", afsPrivate.c_str(), 1 );
#endif
    */
    if ( !getenv( "CORAL_AUTH_PATH" ) )
    {
      LOG << "ERROR! Env CORAL_AUTH_PATH is not defined" << std::endl;
      return 1;
    }
    if ( !getenv( "CORAL_DBLOOKUP_PATH" ) )
    {
      LOG << "ERROR! Env CORAL_DBLOOKUP_PATH is not defined" << std::endl;
      return 1;
    }
    LOG << "Using CORAL_AUTH_PATH='" 
        << getenv( "CORAL_AUTH_PATH" ) <<  "'" << std::endl;
    LOG << "Using CORAL_DBLOOKUP_PATH='" 
        << getenv( "CORAL_DBLOOKUP_PATH" ) <<  "'" << std::endl;

    // Load ConnectionService2
    std::string connSvc2Name = "CORAL/Services/ConnectionService2";
    LOG << "Load " << connSvc2Name << std::endl;
    Context::instance().loadComponent( connSvc2Name );
    IHandle<IConnectionService> connSvc2 = 
      Context::instance().query<IConnectionService>( connSvc2Name );
    if ( !connSvc2.isValid() )
    {
      LOG << "ERROR! Could not load " << connSvc2Name << std::endl;
      return 1;
    }

    // Configure ConnectionService2
    LOG << "Configure " << connSvc2Name << std::endl;
    connSvc2->configuration().disablePoolAutomaticCleanUp(); 
    connSvc2->configuration().setConnectionTimeOut(0);

    // Start the coralServer
    {
      LOG << "Create a CoralServerFacade" << std::endl;
      CoralServer::CoralServerFacade facade;
      LOG << "Create a ServerStub" << std::endl;
      CoralStubs::ServerStub handler( facade );
      LOG << "Start the server on " << host << ":" << port << std::endl;
      CoralSockets::SocketServer server( handler, host, port );
      server.run();
      LOG << "The server has been stopped" << std::endl;
    }
  }
  
  catch( std::exception& e )
  {
    LOG << "ERROR! Standard C++ exception: '" << e.what() << "'" << std::endl;
    return 1;
  }

  catch( ... ) 
  {
    LOG << "ERROR! Unknown exception caught" << std::endl;
    return 1;
  }

  // Successful program termination
  LOG << "Exiting main" << std::endl;
  return 0;
  
}

//-----------------------------------------------------------------------------
