// $Id: IRequestHandler.h,v 1.7 2009/01/31 00:04:39 avalassi Exp $
#ifndef CORALSERVERBASE_IREQUESTHANDLER_H
#define CORALSERVERBASE_IREQUESTHANDLER_H 1

// Include files
#include <memory>
#include "CoralServerBase/ByteBuffer.h"
#include "CoralServerBase/IByteBufferIterator.h"

namespace coral 
{

  /// Buffer iterator pointer.
  typedef std::auto_ptr<IByteBufferIterator> IByteBufferIteratorPtr;

  /** @class IRequestHandler
   *
   *  Interface to a request handler for the CORAL server project.  
   *
   *  A request handler delivers one logical reply to each logical request.
   *  One logical reply can be segmented in several physical buffers.
   *  One logical request is presently described by only one physical buffer.
   *
   *  @author Andrea Valassi, Martin Wache and Alexander Kalkhof
   *  @date   2007-12-04
   */
  
  class IRequestHandler
  { 
    
  public:
    
    // Destructor.
    virtual ~IRequestHandler(){}
    
    // Handle a request message and return an iterator over reply messages.
    // This method cannot be const because we need to lock mutexes in it!
    // The iterator is positioned before the first reply in the loop:
    // the next() method must be called to retrieve the first reply.
    // The request handler must be kept alive while the iterator is used:
    // the iterator uses a byte buffer pool owned by the request handler. 
    virtual IByteBufferIteratorPtr replyToRequest( const ByteBuffer& requestBuffer ) = 0;
    
  };
  
}
#endif // CORALSERVERBASE_IREQUESTHANDLER_H
