// $Id: ByteBuffer.cpp,v 1.4 2009/02/04 10:09:37 avalassi Exp $

// Include files
#include "CoralBase/Exception.h"
#include "CoralServerBase/ByteBuffer.h"

// Namespace
using namespace coral;

//-----------------------------------------------------------------------------

ByteBuffer::ByteBuffer()
  : m_maxSize( 0 )
  , m_usedSize( 0 )
  , m_data( (unsigned char*)::malloc( m_maxSize ) )
{
}

//-----------------------------------------------------------------------------

ByteBuffer::ByteBuffer( size_t newSize )
  : m_maxSize( newSize )
  , m_usedSize( 0 )
  , m_data( (unsigned char*)::malloc( m_maxSize ) )
{
}

//-----------------------------------------------------------------------------

ByteBuffer::~ByteBuffer()
{
  ::free( m_data );
}

//-----------------------------------------------------------------------------

void ByteBuffer::setUsedSize( size_t newUsedSize )
{
  if ( newUsedSize > maxSize() )
    throw Exception( "Attempt to set new usedSize > maxSize", 
                     "ByteBuffer::setUsedSize",
                     "coral::CoralServerBase" );
  m_usedSize = newUsedSize;
}

//-----------------------------------------------------------------------------

void ByteBuffer::resize( size_t newMaxSize )
{
  if ( newMaxSize == maxSize() ) return; // do nothing
  m_maxSize = newMaxSize;
  if ( m_usedSize > newMaxSize ) m_usedSize = newMaxSize;
  m_data = (unsigned char*)::realloc( m_data, newMaxSize );
}

//-----------------------------------------------------------------------------

std::ostream& ByteBuffer::print( std::ostream& out ) const
{
  // Header
  out << "ByteBuffer[ maxSize=" << m_maxSize 
      << ", usedSize=" << m_usedSize << ", ";
  // ASCII dump
  out << "ascii='";
  for ( size_t i=0; i<m_usedSize; i++ )
  {
    unsigned char cTmp = *(m_data+i);
    if ( !isprint( cTmp ) ) out << ".";
    else out << cTmp;
  }
  out << "', ";
  // HEX dump
  out << "hex=0x";
  for ( size_t i=0; i<m_usedSize; i++ )
  {
    unsigned char cTmp = *(m_data+i); // NB Need __unsigned__ char conversion!
    char xTmp[2];
    sprintf( xTmp, "%02X", cTmp );      
    out << xTmp;
  }
  // Footer
  out << " ]";
  return out;
}      

//-----------------------------------------------------------------------------
