// $Id: QueryDefinition.cpp,v 1.5 2009/02/05 10:45:56 avalassi Exp $

// Include files
#include "CoralBase/Exception.h"
#include "CoralServerBase/QueryDefinition.h"

// Namespace
using namespace coral;

//-----------------------------------------------------------------------------

QueryDefinition::QueryDefinition( const std::string& schemaName )
  : m_schemaName( schemaName )
  , m_distinct( false )
  , m_outputList()
  , m_tableList()
  , m_subqueries()
  , m_condition("")
  , m_conditionData()
  , m_groupBy("")
  , m_orderList()
  , m_rowLimitAndOffset()
  , m_setOperation()
{
}

//-----------------------------------------------------------------------------

QueryDefinition::QueryDefinition( const QueryDefinition& rhs )
  : IQueryDefinition( rhs )
{
  m_schemaName = rhs.m_schemaName;
  m_distinct = rhs.m_distinct;
  m_outputList = rhs.m_outputList;
  m_tableList = rhs.m_tableList;
  m_subqueries = rhs.m_subqueries;
  m_condition = rhs.m_condition;
  m_conditionData = rhs.m_conditionData;
  m_groupBy = rhs.m_groupBy;
  m_orderList = rhs.m_orderList;
  if ( rhs.m_rowLimitAndOffset.get() )
    m_rowLimitAndOffset.reset( new std::pair<int,int>( *(rhs.m_rowLimitAndOffset) ) );
  if ( rhs.m_setOperation.get() )
    m_setOperation.reset( new std::pair<SetOperation,QueryDefinition>( *(rhs.m_setOperation) ) );
}

//-----------------------------------------------------------------------------

QueryDefinition& QueryDefinition::operator=( const QueryDefinition& rhs )
{
  m_schemaName = rhs.m_schemaName;
  m_distinct = rhs.m_distinct;
  m_outputList = rhs.m_outputList;
  m_tableList = rhs.m_tableList;
  m_subqueries = rhs.m_subqueries;
  m_condition = rhs.m_condition;
  m_conditionData = rhs.m_conditionData;
  m_groupBy = rhs.m_groupBy;
  m_orderList = rhs.m_orderList;
  if ( rhs.m_rowLimitAndOffset.get() )
    m_rowLimitAndOffset.reset( new std::pair<int,int>( *(rhs.m_rowLimitAndOffset) ) );
  if ( rhs.m_setOperation.get() )
    m_setOperation.reset( new std::pair<SetOperation,QueryDefinition>( *(rhs.m_setOperation) ) );
  return *this;
}

//-----------------------------------------------------------------------------

QueryDefinition::~QueryDefinition()
{
}

//-----------------------------------------------------------------------------

IQueryDefinition& QueryDefinition::defineSubQuery( const std::string& alias )
{
  if ( m_subqueries.find( alias ) != m_subqueries.end() )
    throw Exception( "Subquery "+alias+" already exists",
                     "QueryDefinition::defineSubQuery",
                     "coral::CoralServerBase" );
  m_subqueries[ alias ] = QueryDefinition();
  return m_subqueries[ alias ];
}

//-----------------------------------------------------------------------------
