// $Id: test_ByteBuffer.cpp,v 1.4 2009/01/27 09:46:49 avalassi Exp $
#include "../Common/CoralServerUnitTest.h"

// Include files
#include <sstream>
#include "CoralBase/Exception.h"
#include "CoralServerBase/ByteBuffer.h"
#ifdef _WIN32
#include "CoralServerBase/wintypes.h"
#endif

// Namespace
using namespace coral;

namespace coral 
{

  class ByteBufferTest : public CoralServerUnitTest 
  {

    CPPUNIT_TEST_SUITE( ByteBufferTest );    
    CPPUNIT_TEST( test_buffer );
    CPPUNIT_TEST_SUITE_END();
		
  public:

    void setUp() {}

    void tearDown() {}

    // ------------------------------------------------------

    void test_buffer() 
    {
      {
        ByteBuffer buffer;
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "maxSize0", (size_t)0, buffer.maxSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "usedSize0", (size_t)0, buffer.usedSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "freeSize0", (size_t)0, buffer.freeSize() );
        std::string expOut = 
          "ByteBuffer[ maxSize=0, usedSize=0, ascii='', hex=0x ]";
        std::stringstream out;
        out << buffer;
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "print0", expOut, out.str() );
      }
      {
        ByteBuffer buffer( 10 );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "maxSize10a", (size_t)10, buffer.maxSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "usedSize10a", (size_t)0, buffer.usedSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "freeSize10a", (size_t)10, buffer.freeSize() );
        {
          std::string expOut = 
            "ByteBuffer[ maxSize=10, usedSize=0, ascii='', hex=0x ]";
          std::stringstream out;
          out << buffer;
          CPPUNIT_ASSERT_EQUAL_MESSAGE
            ( "print10a", expOut, out.str() );
        }
        // Write data and check again
        uint32_t iHex = 0xF1F2F3F4; // Non-printable
        ::memcpy( buffer.data()+buffer.usedSize(), &iHex, 4 );
        buffer.setUsedSize( buffer.usedSize()+4 );
        uint32_t iCiao = 0x6F616943; // "oaiC"... will be streamed as "Ciao"!
        ::memcpy( buffer.data()+buffer.usedSize(), &iCiao, 4 );
        buffer.setUsedSize( buffer.usedSize()+4 );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "maxSize10b", (size_t)10, buffer.maxSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "usedSize10b", (size_t)8, buffer.usedSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "freeSize10b", (size_t)2, buffer.freeSize() );
        {
          std::string expOut = 
            "ByteBuffer[ maxSize=10, usedSize=8, ascii='....Ciao', "
            "hex=0xF4F3F2F14369616F ]";
          std::stringstream out;
          out << buffer;
          CPPUNIT_ASSERT_EQUAL_MESSAGE
            ( "print10b", expOut, out.str() );
        }
        // Resize and check again
        buffer.resize( 4 );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "maxSize10c", (size_t)4, buffer.maxSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "usedSize10c", (size_t)4, buffer.usedSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "freeSize10c", (size_t)0, buffer.freeSize() );
        {
          std::string expOut = 
            "ByteBuffer[ maxSize=4, usedSize=4, ascii='....', "
            "hex=0xF4F3F2F1 ]";
          std::stringstream out;
          out << buffer;
          CPPUNIT_ASSERT_EQUAL_MESSAGE
            ( "print10c", expOut, out.str() );
        }
        // Resize and check again
        buffer.resize( 10 );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "maxSize10d", (size_t)10, buffer.maxSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "usedSize10d", (size_t)4, buffer.usedSize() );
        CPPUNIT_ASSERT_EQUAL_MESSAGE
          ( "freeSize10d", (size_t)6, buffer.freeSize() );
        {
          std::string expOut = 
            "ByteBuffer[ maxSize=10, usedSize=4, ascii='....', "
            "hex=0xF4F3F2F1 ]";
          std::stringstream out;
          out << buffer;
          CPPUNIT_ASSERT_EQUAL_MESSAGE
            ( "print10d", expOut, out.str() );
        }
        // Move the used size beyond the allocated max size
        try
        {  
          buffer.setUsedSize( 11 );
          CPPUNIT_FAIL( "Expected an exception" );
        }
        catch( std::exception& e )
        {
          std::stringstream expMsg;
          expMsg << "Attempt to set new usedSize > maxSize";
          CPPUNIT_ASSERT_EQUAL_MESSAGE
            ( "Exception message", 
              expMsg.str(), 
              std::string( e.what(), 
                           expMsg.str().size() ) ); // cut trailing CORAL info
        }
      }
    }

    // ------------------------------------------------------
 
  };
  
  CPPUNIT_TEST_SUITE_REGISTRATION( ByteBufferTest );
  
}

CORALSERVERTEST_MAIN( ByteBufferTest )
