// $Id: SocketServer.cpp,v 1.4 2009/02/09 13:56:14 mwache Exp $

// Include files
#ifndef WIN32
#include <iostream>
#endif
#include <sstream>

#include "CoralSockets/SocketServer.h"

// Local include files
#include "ThreadManager.h"
#include "SocketThread.h"
#include "TcpSocket.h"
#include "PacketSocket.h"

// Namespace
using namespace coral::CoralSockets;

//-----------------------------------------------------------------------------

SocketServer::SocketServer( coral::IRequestHandler& handler, 
               const std::string& host, int port,
               int nHandlerThreadsPerSocket,bool debug )
  : m_isActive(true)
  , m_handler( handler )
  , m_host( host )
  , m_port( port )
  , m_nHandlerThreadsPerSocket( nHandlerThreadsPerSocket )
  , m_debug( debug )
  , m_thrManager( new ThreadManager() )
{
  std::cout << "Create SocketServer"
            << " on host " << host << " and port " << port << std::endl;
  if (!nHandlerThreadsPerSocket) 
    throw GenericSocketException( "Invalid # handler threads per socket" );
}

//-----------------------------------------------------------------------------

SocketServer::~SocketServer()
{
  std::cout << "Delete SocketServer" << std::endl;
}

//-----------------------------------------------------------------------------

void SocketServer::stopServer()
{
  m_isActive=false;
}

//-----------------------------------------------------------------------------

void SocketServer::run() {
  TcpSocketPtr listen=TcpSocket::listenOn( m_host, m_port );

  int tries=0;
  while ( m_isActive && tries<60 ) {
    m_thrManager->housekeeping();
    // poll if any new clients want to connect
    if ( !listen->poll( TcpSocket::P_READ, 500) ) {
      tries++;
      continue;
    };
    try {
      tries=0;
      PacketSocketPtr cSocket( new PacketSocket( listen->accept() ) );
      SocketThread *cThread = 
        new SocketThread( m_handler, cSocket,
            m_nHandlerThreadsPerSocket, m_debug );
      m_thrManager->addThread( cThread );
    }
    catch (GenericSocketException &e) {
      std::cerr << "Caught exception during accept: " << e.what() << std::endl;
    };
  };

  std::cout << "Close the server socket" << std::endl;  
  listen->close();

  std::cout << "Waiting for client connections to terminate:" << std::endl;

  m_thrManager->joinAll();

  std::cout << "All client connections are closed" << std::endl;  
  std::cout << "Shutdown the server" << std::endl;  
}
  
//-----------------------------------------------------------------------------


