// $Id: SocketThread.h,v 1.5 2009/02/09 12:25:45 mwache Exp $
#ifndef CORALSOCKETS_SOCKETTHREAD_H 
#define CORALSOCKETS_SOCKETTHREAD_H 1

#include <vector>
#include <queue>

#include "CoralServerBase/IRequestHandler.h"
#include "CoralSockets/BoostThread_headers.h"

#include "ThreadManager.h"
#include "PacketSocket.h"

namespace coral {

  namespace CoralSockets {

    class SocketThread  : public Runnable {

      private:
        // forward definition
        class HandlerThread;

      public: 

        // Do not print a message - this is copy-constructed several times
        virtual ~SocketThread() {}

        // Do not print a message - this is copy-constructed several times
        SocketThread( coral::IRequestHandler& handler,
            PacketSocketPtr cSocket,
            int nHandlerThreadsPerSocket,
            bool debug )
          : Runnable()
            , m_handler( handler ) 
            , m_cSocket( cSocket )
            , m_debug( debug )
            , m_nHThreads( nHandlerThreadsPerSocket )
            , m_isActive( true ) 
      {}

        // Do not print a message - this is copy-constructed several times
        SocketThread( const SocketThread& rhs )
          :// coral::CoralServerBase::IRequestHandlerFactory()
            Runnable()
            , m_handler( rhs.m_handler )
            , m_cSocket( rhs.m_cSocket ) 
            , m_debug( rhs.m_debug )
            , m_nHThreads( rhs.m_nHThreads )
            , m_isActive( true ) 
      {}

        coral::IRequestHandler& getRequestHandler() 
        {
          return m_handler;
        }

        /// main loop of thread
        void operator()();

        /// send packets over the socket
        void send( const PacketSLAC& packet );
 
        /// get new incomming request
        PacketPtr getNewRequest();


        void endThread()
        {
          m_isActive=false;
        }

        bool isActive()
        {
          return m_isActive;
        }

      private:

        /// Standard constructor is private
        SocketThread();

        //GenericSocketServerMT_SLAC* m_server;
        coral::IRequestHandler& m_handler;

        PacketSocketPtr m_cSocket;

        const std::string m_cHost;

        const bool m_debug;

        // Pool of handler threads
        const int m_nHThreads;                    // target size of hThread pool
        std::vector<boost::thread*> m_hThreads; 
        boost::mutex m_mutexForHThreads;

        // Forward declaration
        class HandlerThread;

        std::queue<PacketPtr> m_requests;
        boost::mutex m_mutexForRequests;
        boost_condition m_condForRequests;

        boost::mutex m_mutexForSend;

        bool m_isActive;

        ThreadManager m_thrManager;
    };
  }
}


#endif //CORALSOCKETS_SOCKETTHREAD_H 
