// $Id: ThreadManager.cpp,v 1.3 2009/02/09 12:27:55 mwache Exp $

#include "ThreadManager.h"

#include "CoralSockets/BoostThread_headers.h"

using namespace coral::CoralSockets;

#include <iostream>
#include <sys/time.h> // for gettimeof day debug

//#define DEBUG( out ) do { timeval val; gettimeofday( &val, 0); std::cerr << "["<< val.tv_usec << "](" << this << ")"<< out << std::endl; } while (0)

#ifndef DEBUG
#define DEBUG( out )
#endif

//----------------------------------------------------------------------------

ThreadManager::~ThreadManager()
{
  DEBUG( "ThreadManager::~ThreadManager called" );
  joinAll();
  DEBUG( "ThreadManager::~ThreadManager call finished" );
}

//----------------------------------------------------------------------------

void ThreadManager::addThread( Runnable *runClass )
{
  if ( existsThread( runClass ) )
    throw GenericSocketException("can't add a class twice to a thread"
        " manager","ThreadManager::createThread");

  DEBUG( "ThreadManager::addThread("<<runClass<<") called" );
  ThreadWrapper wrap( this, runClass );
  ThreadPtr tmp( new boost::thread( wrap ) );

  boost::mutex::scoped_lock lock( m_threadsMutex );
  m_group.add_thread( tmp );
  m_threads[ runClass ] = tmp;
  m_threadsDone[ runClass ] = false;
}

//----------------------------------------------------------------------------

bool ThreadManager::existsThread( Runnable *runClass )
{
  DEBUG( "ThreadManager::existsThread("<<runClass<<") called" );
  boost::mutex::scoped_lock lock( m_threadsMutex );
  bool ret=m_threads.find( runClass) != m_threads.end();
  return ret;
}

//----------------------------------------------------------------------------

void ThreadManager::markDoneThread( Runnable *runClass ) 
{
  DEBUG( "ThreadManager::markDoneThread("<< runClass<<") called" );
  if (!existsThread( runClass ) )
    throw GenericSocketException("runClass doesn't exist!",
        "ThreadManager::markDoneThread");

  boost::mutex::scoped_lock lock( m_threadsMutex );
  m_threadsDone[ runClass ] = true;
}

//----------------------------------------------------------------------------

void ThreadManager::housekeeping() {
  DEBUG("ThreadManager::housekeeping()");
  boost::mutex::scoped_lock lock( m_threadsMutex );

  std::map<Runnable*, bool >::iterator p;
  for(p = m_threadsDone.begin(); p!=m_threadsDone.end(); ) 
  {
    if ( p->second ) {
      // thread is done
      DEBUG( "thread "<< p->first <<" is done" );

      // make sure it is _really_ done
      m_threads[ p->first ]->join();
      m_group.remove_thread( m_threads[ p->first ] );
      delete m_threads[ p->first ];
      delete p->first;

      m_threads.erase( p->first );
      m_threadsDone.erase( p++ );
    } 
    else 
      ++p;
  };
}

//----------------------------------------------------------------------------

void ThreadManager::joinAll() {
  DEBUG( "ThreadManager::joinAll() called" );
  m_group.join_all();

  housekeeping();
  DEBUG( "ThreadManager::joinAll() finished" );
}

