// $Id: test_PacketSocket.cpp,v 1.8 2009/02/11 11:34:26 mwache Exp $

#include <iostream>

#include "CoralServerBase/hexstring.h"

#include "../../../CoralServerBase/tests/Common/CoralServerUnitTest.h"

#include "CoralServerBase/CTLMagicWordException.h"

// Local include files 
#include "../../CoralSockets/BoostThread_headers.h"
#include "../../CoralSockets/GenericSocketException.h"
#include "../../src/PacketSocket.h"
#include "../../src/RingBufferSocket.h"
#include "../../src/TcpSocket.h"


// Namespace
using namespace coral::CoralSockets;
using namespace coral;


// port used for testing
const int testPort = 50007;

std::auto_ptr<RingBufferPipes> pipes;
TcpSocketPtr listenSocket;
 
namespace coral 
{

  class PacketSocketTest : public CoralServerUnitTest 
  {

    CPPUNIT_TEST_SUITE( PacketSocketTest );
    CPPUNIT_TEST( test_PacketSLAC );
    CPPUNIT_TEST( test_PacketSLAC_errorPackets );
    CPPUNIT_TEST( test_ReceiveInvalidPackets );
    CPPUNIT_TEST( test_sendPackets );
    CPPUNIT_TEST_SUITE_END();
		
  public:

    void setUp() {}

    void tearDown() {
    }

    //-------------------------------------------------------------
    // helper methods

    void createSockets( ISocketPtr &src, ISocketPtr &dst)
    {
#if 1
      pipes = std::auto_ptr<RingBufferPipes>(
          new RingBufferPipes( 10000, "test pipes") );

      src=pipes->getSrc();
      dst=pipes->getDst();
#else
      listenSocket = TcpSocketPtr( (TcpSocket*)0);

      while ( listenSocket.get() == 0 )
      {
        try {
          listenSocket = TcpSocket::listenOn( "localhost", testPort );
        }
        catch (ErrorInHeaderException &e) {
          std::string expMsg="Error binding socket";
          if (expMsg == std::string( e.what(), expMsg.size() ) ) {
            std::cout << "Caught exeption: '" << e.what() 
              << "' sleeping."<< std::endl;
            sleep( 10 );
            continue;
          }
          throw;
        }
      };


      src = TcpSocket::connectTo( "localhost", testPort );
      dst = listenSocket->accept();
#endif

    }

    void checkByteBuffer( const ByteBuffer & buf1, const ByteBuffer & buf2 )
    {
      CPPUNIT_ASSERT_EQUAL_MESSAGE( "buffer used size",
          buf1.usedSize(), buf2.usedSize() );

      for (unsigned int i = 0; i< buf1.usedSize(); i++) {
        if ( *(buf1.data()+i) != *(buf2.data()+i) )
          CPPUNIT_FAIL("buffer content not equal");
      };
    };


    void checkHeader(const CTLPacketHeader &header, 
        const CTLPacketHeader& eHeader) 
    {
      CPPUNIT_ASSERT_EQUAL_MESSAGE( "version check",
          header.version(), eHeader.version() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "status check",
          header.status(), header.status() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "packetSize check",
          header.packetSize(), 
          eHeader.packetSize() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "requestID check",
          header.requestID(), eHeader.requestID() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "clientID check",
          header.clientID(), eHeader.clientID() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("segmentNumber check",
          header.segmentNumber(), eHeader.segmentNumber() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("more Segments check",
          header.moreSegments(), eHeader.moreSegments() );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("payloadChecksum",
          header.payloadChecksum(),
          eHeader.payloadChecksum() );

    };

    void checkPacket(const CTLPacketHeader &header, const ByteBuffer & payload,
        uint32_t ctlRequestID, uint32_t clientID, uint32_t ctlSegmentNumber,
        bool ctlMoreSegments, const ByteBuffer& ePayload) 
    {
      CPPUNIT_ASSERT_EQUAL_MESSAGE( "version check",
          header.version(), CTLPACKET_CURRENT_VERSION );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "status check",
          header.status(), CTLOK );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "packetSize check",
          header.packetSize(), 
          (uint32_t)( CTLPACKET_HEADER_SIZE + payload.usedSize() ) );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "requestID check",
          header.requestID(), ctlRequestID );

      CPPUNIT_ASSERT_EQUAL_MESSAGE( "clientID check",
          header.clientID(), clientID );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("segmentNumber check",
          header.segmentNumber(), ctlSegmentNumber );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("more Segments check",
          header.moreSegments(), ctlMoreSegments );

      CPPUNIT_ASSERT_EQUAL_MESSAGE("payloadChecksum",
          header.payloadChecksum(),
          CTLPacketHeader::computeChecksum( 
            ePayload.data(), ePayload.usedSize() ) );

      checkByteBuffer( payload, ePayload );

    }

    void testPacket( uint32_t ctlRequestID, uint32_t clientID,
            uint32_t ctlSegmentNumber, bool ctlMoreSegments, 
            const ByteBuffer& payload) 
    {
      PacketSLAC packet( ctlRequestID, clientID, ctlSegmentNumber,
          ctlMoreSegments, payload);
      checkPacket(packet.getHeader(), packet.getPayload(),
          ctlRequestID, clientID, ctlSegmentNumber, ctlMoreSegments,
          payload);
    };


    // ------------------------------------------------------
    void test_PacketSLAC() 
    {
      {
        const int payloadSize=100;
        ByteBuffer buffer( payloadSize );
        for (int i= 0; i< payloadSize; i++ )
          *(buffer.data()+i)= (unsigned char) i&0xff;

        testPacket( 0xFFFFFFFF, 1,  0x00000000, false, buffer);
        testPacket( 0xFFFFFFFF, 2, 0x7FFFFFFF, false, buffer);
        testPacket( 0xFFFFFFFF, 3, 0x7FFFFFFF,  true, buffer);
        testPacket( 0xFFFFFFFF, 4, 0x00000000,  true, buffer);

      }; 

    };

    // ------------------------------------------------------

    void test_PacketSLAC_errorPackets() 
    {
      {

        // error message constructor
        PacketSLAC packet( CTLWrongMagicWord );

        // construct the package by hand
        ByteBuffer buffer( CALPACKET_HEADER_SIZE );
        CALPacketHeader calHeader( 0 /*opcode*/, false /*fromProxy*/,
            false /*cacheable*/, 0 /*cacheablesize*/);
        ::memcpy( buffer.data(), calHeader.data(), CALPACKET_HEADER_SIZE );
        buffer.setUsedSize( CALPACKET_HEADER_SIZE );

        CTLPacketHeader header(CTLWrongMagicWord, 
            CTLPACKET_HEADER_SIZE+buffer.usedSize(), /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )
            );


        CPPUNIT_ASSERT_EQUAL_MESSAGE("header 1", 0, 
            memcmp( header.data(), packet.getHeader().data(), 
              CTLPACKET_HEADER_SIZE ) );
        CPPUNIT_ASSERT_EQUAL_MESSAGE("payload 1", 0,
            memcmp( buffer.data(), packet.getPayload().data(), 
              packet.getPayload().usedSize() ) );

      }; 

    };

    // ------------------------------------------------------
    void test_ReceiveInvalidPackets()
    {
      ISocketPtr src;
      ISocketPtr dst;


      {
        createSockets( src, dst );
        PacketSocket pSocket( dst );

        // setup home made correct packet
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            123, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        PacketPtr recPacket = pSocket.receivePacket();

        // and check it
        checkHeader( recPacket->getHeader(), header);
        checkByteBuffer( recPacket->getPayload(), buffer );
      }
      {
        // packet size is too small (smaller than header size)-> overflow?
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            22, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet size too small exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="packet size in received header is too small!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // packet size is too large > 4MB -> throw
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            1024*1024*4+24, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet size too large exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="packet size in received header is too large!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // checksum is wrong
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            123, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet wrong checksum exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="payload checksum in received header is wrong!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // payload size is wrong (too large) -> checksum exception expected
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            154, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );
        // fill some garbage to make sure we can receive enough data
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet wrong checksum exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="payload checksum in received header is wrong!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // payload size is wrong (too small) -> checksum exception expected
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            100, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );
        // fill some garbage to make sure we can receive enough data
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet wrong checksum exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="payload checksum in received header is wrong!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // header is missing -> wrong magic word exception
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            123, /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet wrong magic word exception");

        }
        catch( CTLMagicWordException & e) {
        }
      }
      {
        // header has wrong version -> exception
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(CTLOK, 
            CTLPACKET_HEADER_SIZE + buffer.usedSize(), /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        unsigned char charHeader[CTLPACKET_HEADER_SIZE];
        ::memcpy( &charHeader[0], header.data(), CTLPACKET_HEADER_SIZE );
        // manipulate the version
        charHeader[2] = 0x1 | 0x00;

        // write it to the one end
        src->writeAll( &charHeader[0], CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected packet wrong version exception");

        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="received packet with wrong version number!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // status is not CTLOK
        // this checks also that the status is checked before the packet size
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(
            CTLWrongVersion,
            5, /* wrong packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );
        // fill some garbage to make sure we can receive enough data
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected status wrong version");

        }
        catch( PacketStatusNotOkException & e) {
          std::string expMsg="Received a packet with status WrongVersion";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // status is not CTLOK
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(
            CTLWrongMagicWord,
            CTLPACKET_HEADER_SIZE+buffer.usedSize(), /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );
        // fill some garbage to make sure we can receive enough data
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected status wrong magic word");

        }
        catch( PacketStatusNotOkException & e) {
          std::string expMsg="Received a packet with status WrongMagicWord";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
      {
        // status is not CTLOK
        createSockets( src, dst );
        PacketSocket pSocket( dst );
        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        CTLPacketHeader header(
            CTLWrongChecksum,
            CTLPACKET_HEADER_SIZE+buffer.usedSize(), /*packet size*/
            0, /*request id*/
            0, /* client id*/
            0, /* segment number */
            false, /* more segments */
            CTLPacketHeader::computeChecksum( buffer.data(), buffer.usedSize() )+40
            );

        // write it to the one end
        src->writeAll( header.data(), CTLPACKET_HEADER_SIZE );
        src->writeAll( buffer.data(), buffer.usedSize() );
        // fill some garbage to make sure we can receive enough data
        src->writeAll( buffer.data(), buffer.usedSize() );

        // receive the packet on the other end
        try {
          PacketPtr recPacket=pSocket.receivePacket();
          CPPUNIT_FAIL("expected status wrong checksum");

        }
        catch( PacketStatusNotOkException & e) {
          std::string expMsg="Received a packet with status WrongChecksum";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }


    };

    void test_sendPackets()
    {
      ISocketPtr src;
      ISocketPtr dst;

      {
        // check with correct packet
        createSockets( src, dst );
        PacketSocket dSocket( dst );
        PacketSocket sSocket( src );

        ByteBuffer buffer(100);
        for (unsigned int i=0; i<buffer.maxSize(); i++)
          *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer.setUsedSize( buffer.maxSize() );

        PacketPtr packet( new PacketSLAC(
              0 /* ctlRequestID */,
              0 /* clientID */,
              0 /* ctlSegmentNumber */,
              false /* ctlMoreSegments */,
              buffer) );

        sSocket.sendPacket( *packet );

        PacketPtr recPacket=dSocket.receivePacket();

        checkPacket( recPacket->getHeader(), recPacket->getPayload(),
            0 /* ctlRequestID */,
            0 /* clientID */,
            0 /* ctlSegmentNumber */,
            false /* ctlMoreSegments */,
            buffer);

        checkByteBuffer( recPacket->getPayload(), buffer );
      }
      {
        // check with too large packet
        createSockets( src, dst );
        PacketSocket dSocket( dst );
        PacketSocket sSocket( src );

        std::auto_ptr<ByteBuffer> buffer( new ByteBuffer(1024*1024*5) );
        //for (unsigned int i=0; i<buffer.maxSize(); i++)
        //  *(buffer.data()+i)=(unsigned char)(i&0xff);
        buffer->setUsedSize( buffer->maxSize() );

        PacketPtr packet( new PacketSLAC(
              0 /* ctlRequestID */,
              0 /* clientID */,
              0 /* ctlSegmentNumber */,
              false /* ctlMoreSegments */,
              *buffer) );

        try {
          sSocket.sendPacket( *packet );
          CPPUNIT_FAIL("expected packet size is too large exception");
        }
        catch( ErrorInHeaderException & e) {
          std::string expMsg="packet size is too large!";
          CPPUNIT_ASSERT_EQUAL_MESSAGE( "exception message",
              expMsg, std::string( e.what(), expMsg.size() ) );
          // cut CORAL info
        }
      }
   };


  };

  CPPUNIT_TEST_SUITE_REGISTRATION( PacketSocketTest );
  
}

CORALSERVERTEST_MAIN( PacketSocketTest )

