
//typedef std::auto_ptr<ByteBuffer> ByteBufferPtr;
#define ByteBufferPtr std::auto_ptr<ByteBuffer>

// port used for testing
const int testPort = 50007;


std::auto_ptr<RingBufferPipes> pipes;
TcpSocketPtr listenSocket;

// ---------------------------------------------------
// helpers

void createSockets( ISocketPtr &src, ISocketPtr &dst)
{
#if 1 
  pipes = std::auto_ptr<RingBufferPipes>(
      new RingBufferPipes( 10000, "test pipes") );

  src=pipes->getSrc();
  dst=pipes->getDst();
#else
  listenSocket = TcpSocketPtr( (TcpSocket*)0);

  while ( listenSocket.get() == 0 )
  {
    try {
      listenSocket = TcpSocket::listenOn( "localhost", testPort );
    }
    catch (GenericSocketException &e) {
      std::string expMsg="Error binding socket";
      if (expMsg == std::string( e.what(), expMsg.size() ) ) {
        std::cout << "Caught exeption: '" << e.what() 
          << "' sleeping."<< std::endl;
        sleep( 10 );
        continue;
      }
      throw;
    }
  };


  src = TcpSocket::connectTo( "localhost", testPort );
  dst = listenSocket->accept();
#endif
}

ByteBufferPtr createBuffer( int num ) {
  ByteBufferPtr buf( new ByteBuffer( sizeof( int ) ) );
  *((int*)buf->data()) = num;
  buf->setUsedSize( sizeof( int ) );
  return buf;
}

int getBufferNum( ByteBufferPtr buf ) {
  return *(int*)buf->data();
}

int getBufferNum( const ByteBuffer& buf ) {
  return *(int*)buf.data();
}

ByteBufferPtr  string2ByteBuffer( std::string str ) {
  ByteBufferPtr  result( new ByteBuffer( str.length() ) );
  const char * cstr=str.c_str();

  for (unsigned int i=0; i<str.length(); i++ )
    *(result->data()+i) = (unsigned char)*(cstr+i);

  result->setUsedSize( str.length() );
  return result;
}

std::string byteBuffer2String( const ByteBuffer& buf ) {
  return std::string( (char*)buf.data(), buf.usedSize() );
}

#define  checkByteBuffer( mesg, buf1, buf2 )                    \
do {                                                            \
  CPPUNIT_ASSERT_EQUAL_MESSAGE( mesg " size",                   \
      buf1.usedSize(), buf2.usedSize() );                       \
                                                                \
  for (unsigned int i = 0; i< buf1.usedSize(); i++) {           \
    if ( *(buf1.data()+i) != *(buf2.data()+i) )                 \
      CPPUNIT_FAIL(mesg " content not equal");                  \
  };                                                            \
} while (0)



