#include "ClientUtils.h"

//CoralStubs includes
#include "Exceptions.h"
#include "StreamBuffer.h"

//CoralServerBase includes
#include "CoralServerBase/ByteBuffer.h"
#include "CoralServerBase/IByteBufferIterator.h"


//C++ includes
#include <iostream>
#include <sstream>

namespace coral { namespace CoralStubs {

ByteBuffer*
prepareRequestMessage( coral::CALOpcode opcode )
{
  coral::ByteBuffer* requestBuffer = new ByteBuffer(10000);
  //create a new request header
  ///FIXME setupt the proxy flag
  coral::CALPacketHeader header_request(opcode, false, false, 0);
  //copy the header into the buffer
  memcpy(requestBuffer->data(), header_request.data(), coral::CALPACKET_HEADER_SIZE);  
  //reset the write position
  requestBuffer->setUsedSize(CALPACKET_HEADER_SIZE);
  
  return requestBuffer;
}

void
prepareExceptionMessage(const ByteBuffer& replyBuffer)
{
  //create a streambuffer for in
  RStreamBuffer stream_in(replyBuffer, CALPACKET_HEADER_SIZE);

  coral::CALOpcode fl01;
  coral::CALOpcode fl02;
 
  stream_in.read(fl01);
  stream_in.read(fl02);
  
  if(fl01 == 0x00){
    //all CoralExceptions here
    if(fl02 == 0x01){
      //std::exception
      std::string s1;
      stream_in.read(s1);
      std::string s2;
      stream_in.read(s2);
      //throw it
      throw Exception("C++ exception: " + s1, s2, "coral::CoralStubs");
    }else if(fl02 == 0x02){
      //coral exception
      std::string part01;
      stream_in.read(part01);
      std::string part02;
      stream_in.read(part02);
      std::string part03;
      stream_in.read(part03);
      //throw it
      throw Exception(part01, part02, part03);
    }else if(fl02 == 0x10){
      //stubs exception
      std::string s1;
      stream_in.read(s1);
      std::string s2;
      stream_in.read(s2);
      //throw it
      throw InvalidMessageException(s1, s2);
    }
  }else if(fl01 == 0x01){
    //andy's exceptions (exceptions from the dbproxy)
    //get the size of the message inside the rest of the payload
    size_t maxsize = replyBuffer.usedSize() - CALPACKET_HEADER_SIZE - 2;
    //create a buffer
    char* buffer = (char*)malloc(sizeof(char) * (maxsize + 1));
    //copy the string into the buffer
    memcpy((void*)buffer, replyBuffer.data() + CALPACKET_HEADER_SIZE + 2, maxsize);
    //throw the exception
    buffer[maxsize] = 0;
    
    throw DBProxyException(buffer);
  }
  std::ostringstream s;
  s << "Unknown exception (" << hexstring(&fl01,1) << ")(" << hexstring(&fl02,1) << ")";
  throw Exception(s.str(), "ClientStub::prepareExceptionMessage", "coral::CoralStubs");
}

const ByteBuffer&
prepareReplyMessageFP(IByteBufferIterator& reply, coral::CALOpcode opcode, bool& fromProxy)
{
  //should be only one reply message
  if(!reply.next()){
    throw ReplyIteratorException("The reply was empty", "ClientStub::prepareReplyMessage");
  }
  //should be the last reply
  if(!reply.isLast()){
    throw ReplyIteratorException("The reply has more entries, one expected", "ClientStub::prepareReplyMessage");
  }
  //get the reply buffer form the reply
  const ByteBuffer& replyBuffer = reply.currentBuffer();
  //create a header form the buffer
  CALPacketHeader header_reply( replyBuffer.data(), CALPACKET_HEADER_SIZE );  
  //get the opcode
  coral::CALOpcode opcode_reply = header_reply.opcode();
  //get the real opcode from the reply
  coral::CALOpcode opcode_reply2 = CALOpcodes::getCALRequest(opcode_reply);
  //check if we received the same opcode as we sent 
  if(opcode != opcode_reply2){
    std::ostringstream s;
    s << "Reply opcode (" << hexstring(&opcode_reply2,1) << ") is different from request opcode (" << hexstring(&opcode,1) <<")";
    throw InvalidMessageException(s.str(), "ClientStub::prepareReplyMessage");  
  }
  //check if we have an exception
  if(CALOpcodes::isCALReplyException(opcode_reply)){
    //we have some kind of exception here
    prepareExceptionMessage(replyBuffer);
  }
  //check if we have a valid reply ok message
  if(!CALOpcodes::isCALReplyOK(opcode_reply)){
    throw InvalidMessageException("Message is not a reply", "ClientStub::prepareReplyMessage");
  }
  
  fromProxy = header_reply.fromProxy();
  //everything is fine continue
  return replyBuffer;
}

const ByteBuffer&
prepareReplyMessage(IByteBufferIterator& reply, coral::CALOpcode opcode)
{
  //should be only one reply message
  if(!reply.next()){
    throw ReplyIteratorException("The reply was empty", "ClientStub::prepareReplyMessage");
  }
  //should be the last reply
  if(!reply.isLast()){
    throw ReplyIteratorException("The reply has more entries, one expected", "ClientStub::prepareReplyMessage");
  }
  //get the reply buffer form the reply
  const ByteBuffer& replyBuffer = reply.currentBuffer();
  //create a header form the buffer
  CALPacketHeader header_reply( replyBuffer.data(), CALPACKET_HEADER_SIZE );  
  //get the opcode
  coral::CALOpcode opcode_reply = header_reply.opcode();
  //get the real opcode from the reply
  coral::CALOpcode opcode_reply2 = CALOpcodes::getCALRequest(opcode_reply);
  //check if we received the same opcode as we sent 
  if(opcode != opcode_reply2){
    std::ostringstream s;
    s << "Reply opcode (" << hexstring(&opcode_reply2,1) << ") is different from request opcode (" << hexstring(&opcode,1) <<")";
    throw InvalidMessageException(s.str(), "ClientStub::prepareReplyMessage");  
  }
  //check if we have an exception
  if(CALOpcodes::isCALReplyException(opcode_reply)){
    //we have some kind of exception here
    prepareExceptionMessage(replyBuffer);
  }
  //check if we have a valid reply ok message
  if(!CALOpcodes::isCALReplyOK(opcode_reply)){
    throw InvalidMessageException("Message is not a reply", "ClientStub::prepareReplyMessage");
  }
  //everything is fine continue
  return replyBuffer;
}

}  }

