#include "DummyFacade.h"

//CoralExceptions
#include "RelationalAccess/ConnectionServiceException.h"
#include "RelationalAccess/AuthenticationServiceException.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/SchemaException.h"
#include "RelationalAccess/SessionException.h"
#include "RelationalAccess/MonitoringException.h"
#include "RelationalAccess/WebCacheControlException.h"
#include "RelationalAccess/LookupServiceException.h"
#include "CoralBase/AttributeException.h"
#include "CoralBase/AttributeListException.h"

#include <stdexcept>
#include <iostream>

#include "RowIteratorAll.h"
#include "CoralBase/Attribute.h"

#include "CoralBase/Blob.h"
#include "CoralBase/Date.h"
#include "CoralBase/TimeStamp.h"

#include "Exceptions.h"

namespace coral { namespace CoralStubs {

DummyFacade::DummyFacade()
{
}

DummyFacade::~DummyFacade()
{
}

coral::Token
DummyFacade::connect( const std::string& dbUrl, const coral::AccessMode mode, bool& /*fromProxy*/ ) const
{
  if(mode == coral::ReadOnly) return 42;
  throw ConnectionNotAvailableException(dbUrl, "DummyFacade::connect");
}

void
DummyFacade::releaseSession( coral::Token /*sessionID*/ ) const
{
}

void
DummyFacade::startTransaction( coral::Token /*sessionID*/, bool /*readOnly*/ ) const
{
 // if(!readOnly) throw TransactionException("coral::CoralStubs","Can't open read-write transaction","DummyFacade::startTransaction");
}

void
DummyFacade::commitTransaction( coral::Token /*sessionID*/ ) const
{
 // throw std::runtime_error("May the force be with you");
}

const std::vector<std::string>
DummyFacade::sessionProperties( Token /*sessionID*/ ) const
{
  std::vector<std::string> h;
  h.push_back("Test01");
  return h;
}

IRowIteratorPtr
DummyFacade::fetchRows( Token /*sessionID*/, const QueryDefinition& /*qd*/, AttributeList* /*pRowBuffer*/, size_t /*cacheSize*/, bool /*cacheSizeInMB*/ ) const
{
  RowIteratorAll* tall = new RowIteratorAll;
  
  for(size_t i = 0; i < 30; i++ )
  {

    AttributeList& attr = tall->addRow();

    attr.extend("A", typeid(coral::TimeStamp));
    attr.extend("B", typeid(double));
    attr.extend("C", typeid(std::string));
    attr.extend("D", typeid(int));

   coral::TimeStamp time(2006, 4, 23, 12, i, 6, 23); 
    
   attr[0].data<coral::TimeStamp>() = time;
   attr[1].data<double>() = 23.54 * i;
   attr[2].data<std::string>() = "huhu";
   attr[3].data<int>() = i;

  }
  
  return std::auto_ptr<IRowIterator>(tall);
}

IRowIteratorPtr
DummyFacade::fetchAllRows( Token /*sessionID*/, const QueryDefinition& /*qd*/, AttributeList* rowBuffer ) const
{
  if(rowBuffer != NULL){
    rowBuffer->toOutputStream(std::cout) << std::endl;
  
  }

  RowIteratorAll* tall = new RowIteratorAll;

  
  for(size_t i = 0; i < 30; i++ )
  {

    AttributeList& attr = tall->addRow();

    attr.extend("A", typeid(coral::TimeStamp));
    attr.extend("B", typeid(double));
    attr.extend("C1", typeid(std::string));
    attr.extend("C2", typeid(std::string));
    attr.extend("D", typeid(int));
    attr.extend("E", typeid(coral::Blob));

   coral::TimeStamp time(2006, 4, 23, 12, i, 6, 23); 
    
   attr[0].data<coral::TimeStamp>() = time;
   attr[1].data<double>() = 23.54 * i;
   attr[2].data<std::string>() = "2009-02-05_09:44:08.269264000 GMT";
   attr[3].data<std::string>() = "2009-02-05_09:44:08.269264000 GMT";
   attr[4].data<int>() = i;
   coral::Blob& myblob = attr[5].data<coral::Blob>();
   
   std::cout << "Initialise the blob ... " << std::endl;
   size_t maxc = 1000000; //10MB
   myblob.resize( maxc + 1 );
   unsigned char* p = static_cast<unsigned char*>( myblob.startingAddress() );
   for ( size_t j = 0; j < maxc; ++j, ++p ) *p = rand() % 256;
   std::cout << "Done" << std::endl;
   
   

  }
  
  return std::auto_ptr<IRowIterator>(tall);
}

const std::set<std::string>
DummyFacade::listTables( Token /*sessionID*/, const std::string& /*schemaName*/ ) const
{
  std::set<std::string> myset;
  
  myset.insert("the_big_table");
  myset.insert("the_small_table");

  return myset;
}

const TableDescription
DummyFacade::tableDescription( Token /*sessionID*/, const std::string& /*schemaName*/, const std::string& /*tableName*/ ) const
{
  TableDescription td;
  
  td.setName("MyGreatTable");
  td.insertColumn("Column01", "string", 0, true, "Column01_SPName");
  
  return td;
}

const TableDescriptionMap
DummyFacade::tableDescriptionMap( Token /*sessionID*/, const std::string& /*schemaName*/ ) const
{
  TableDescriptionMap tdm;
  
  TableDescription td01;
  
  td01.setName("MyGreatTable");
  td01.insertColumn("Column01", "string", 0, true, "Column01_SPName");
  td01.insertColumn("Column02", "float", 0, true, "Column02_SPName");

  TableDescription td02;
  
  td02.setName("MyTinyTable");
  td02.insertColumn("Col01", "short", 0, true, "Col01_SPName");
  
  tdm.insert(std::pair< std::string, TableDescription >("T01", td01));
  tdm.insert(std::pair< std::string, TableDescription >("T02", td02));
  
  return tdm;

}

int
DummyFacade::deleteTableRows( Token /*sessionID*/, const std::string& /*schemaName*/, const std::string& /*tableName*/, const std::string& /*whereClause*/, const std::string& /*whereData*/ ) const
{
 return 0;
}

const std::string
DummyFacade::formatRowBufferAsString( Token /*sessionID*/, const std::string& /*schemaName*/, const std::string& /*tableName*/ ) const
{
  return std::string("");
}

void
DummyFacade::insertRowAsString( Token /*sessionID*/, const std::string& /*schemaName*/, const std::string& /*tableName*/, const std::string& /*rowBufferAS*/ ) const
{
  throw StubsException("This function is not available");

}

Token
DummyFacade::bulkInsertAsString( Token /*sessionID*/, const std::string& /*schemaName*/, const std::string& /*tableName*/, const std::string& /*rowBufferAS*/, int /*rowCacheSizeDb*/ ) const
{
  throw StubsException("This function is not available");
}

void
DummyFacade::releaseBulkOp( Token /*bulkOpID*/ ) const
{
  throw StubsException("This function is not available");

}

void
DummyFacade::processRows( Token /*bulkOpID*/, const std::vector<coral::AttributeList>& /*rowsAS*/ ) const
{
  throw StubsException("This function is not available");

}

void
DummyFacade::flush( Token /*bulkOpID*/ ) const
{
  throw StubsException("This function is not available");

}
} }
