#include "RowIteratorFetch.h"

//CoralStubs includes
#include "ClientUtils.h"
#include "CALProtocol.h"
#include "StreamBuffer.h"

//CoralServerBase includes


//C++ includes
#include <iostream>
#include <sstream>

namespace coral { namespace CoralStubs {     
      
RowIteratorFetch::RowIteratorFetch(IRequestHandler& requestHandler, uint32_t cursorID, size_t maxrows, AttributeList* rowBuffer)
  : m_obuffer( rowBuffer )
  , m_requestHandler( requestHandler )
  , m_cursor( cursorID )
  , m_maxrows( maxrows )
  , m_attrlist( rowBuffer )
{
}
      
RowIteratorFetch::~RowIteratorFetch()
{
  coral::ByteBuffer* requestBuffer = prepareRequestMessage( coral::CALOpcodes::ReleaseCursor );
  //create the stream out buffer
  WStreamBuffer* stream_out = new WStreamBuffer(*requestBuffer);
  //encode here
  stream_out->write(m_cursor);
  //delete the stream buffer and set the buffer position = stream position 
  delete stream_out;
  //call the request handler and keep the reply interator
  coral::IByteBufferIteratorPtr reply = m_requestHandler.replyToRequest( *requestBuffer );
  //release the request buffer
  delete requestBuffer;
  //receive success
  prepareReplyMessage( *(reply.get()), coral::CALOpcodes::ReleaseCursor );
}
      
bool
RowIteratorFetch::next()
{
  coral::ByteBuffer* requestBuffer = prepareRequestMessage( coral::CALOpcodes::FetchRowsNext );
  //create the stream out buffer
  WStreamBuffer* stream_out = new WStreamBuffer(*requestBuffer);
  //encode here
  stream_out->write(m_cursor);
  //delete the stream buffer and set the buffer position = stream position 
  delete stream_out;
  //call the request handler and keep the reply interator
  coral::IByteBufferIteratorPtr reply = m_requestHandler.replyToRequest( *requestBuffer );
  //release the request buffer
  delete requestBuffer;
  
  const coral::ByteBuffer& replyBuffer = prepareReplyMessage( *(reply.get()), coral::CALOpcodes::FetchRowsNext );
  
  //create a streambuffer for in
  RStreamBuffer stream_in(replyBuffer, CALPACKET_HEADER_SIZE);
  
  stream_in.print();
  
  bool nextrow = false;
  
  stream_in.read(nextrow);
  if(nextrow){
    if(m_obuffer == NULL){
      if(m_attrlist != NULL) delete m_attrlist;
      m_attrlist = new AttributeList;
      stream_in.read(*m_attrlist);
    }else{
      AttributeList attr;
      stream_in.read(attr);
      *m_attrlist = attr;
    }
    return true;
  }else{
    return false;
  }
}
    
const AttributeList&
RowIteratorFetch::currentRow() const
{
  return *m_attrlist;
}
      
} }
