#include "CoralStubs/ServerStub.h"

#include "CoralServerBase/CALPacketHeader.h"
#include "CALProtocol.h"
#include "StreamBuffer.h"

#include "CoralServerBase/ByteBuffer.h"
#include "CoralServerBase/IRowIterator.h"

#include "SingleReplyIterator.h"

#include "RelationalAccess/AccessMode.h"
#include "Exceptions.h"

//CoralExceptions
#include "RelationalAccess/ConnectionServiceException.h"
#include "RelationalAccess/AuthenticationServiceException.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/SchemaException.h"
#include "RelationalAccess/SessionException.h"
#include "RelationalAccess/MonitoringException.h"
#include "RelationalAccess/WebCacheControlException.h"
#include "RelationalAccess/LookupServiceException.h"
#include "CoralBase/AttributeException.h"
#include "CoralBase/AttributeListException.h"

#include <sstream>

#include "CppMutex.h"
#include "CppMutexFct.h"
#include "CppTypes.h"

//#define LOGGER_NAME "CoralStubs::ServerStub"
//#include "logger.h"

namespace coral { namespace CoralStubs {

#include "StreamBuffer.h"


struct rowitermgr {
    
  std::vector<IRowIterator*> maps;
      
  pmutex_t* mutex;
      
};

bool
request_ConnectRO( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool fromProxy, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  //create a string for url
  std::string url;
  stream_in.read(url);
  //call the facade method
  Token sessionID = facade.connect( url, coral::ReadOnly, fromProxy);
  //create a streambuffer for out
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(sessionID);
  //return not cacheable
  return false;
}

bool
request_ConnectRW( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool fromProxy, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  //create a string for url
  std::string url;
  stream_in.read(url);
  //call the facade method
  Token sessionID = facade.connect( url, coral::Update, fromProxy);
  //create a streambuffer for out
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(sessionID);
  //return not cacheable
  return false;
}

bool
request_Disconnect( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
  
  stream_in.read(sessionID);
  
  facade.releaseSession( sessionID );

  return false;
}

bool
request_TransactionStartRO( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
  
  stream_in.read(sessionID);
  
  facade.startTransaction( sessionID, true );

  return false;
}

bool
request_TransactionStartRW( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
  
  stream_in.read(sessionID);
  
  facade.startTransaction( sessionID, false );

  return false;
}

bool
request_TransactionCommit( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
  
  stream_in.read(sessionID);
  
  facade.commitTransaction( sessionID );

  return false;
}

bool
request_TransactionRollback( ICoralFacade& /*facade*/, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
  
  stream_in.read(sessionID);
  
//  std::cout << "Rollback with sessionID " << sessionID << std::endl;
  ///FIXME call the CoralFacade TransactionStart method, which is missing!!
  return false;
}

bool
request_FetchRows( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& mgri )
{
//  std::cout << "Execute Query" << std::endl;
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
      
  stream_in.read(sessionID);
  
  QueryDefinition qd;
  
  stream_in.read(qd);

  bool cacheSizeInMB;
  stream_in.read(cacheSizeInMB);
  
  uint32_t cacheSize;
  stream_in.read(cacheSize);
  
  //check if we have an empty rowbuffer
  AttributeList* pRowBuffer = NULL;
  
  bool isempty;
  stream_in.read(isempty);
  //row buffer is empty
  if(isempty){
    //create a new attributelist
    pRowBuffer = new AttributeList;
    //get the list
    stream_in.read(*pRowBuffer);
  }
  
  stream_in.print();
   
  IRowIteratorPtr rowi = facade.fetchRows( sessionID, qd, pRowBuffer, cacheSize, cacheSizeInMB );
  
  WStreamBuffer stream_out(replyBuffer);
  //return cacheable
  lockMutex(mgri.mutex);

  mgri.maps.push_back(rowi.release());  
  //write the session id to the stream
  stream_out.write((uint32_t)(mgri.maps.size() - 1));
  //return cacheable
  unlockMutex(mgri.mutex);

  return false;
}

bool
request_FetchRowsNext( ICoralFacade& /*facade*/, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& mgri )
{
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  uint32_t cursor;
      
  stream_in.read(cursor);
  
  lockMutex(mgri.mutex);
  
  IRowIterator* rowi = mgri.maps[cursor];

  unlockMutex(mgri.mutex);  
  
  WStreamBuffer stream_out(replyBuffer);
  //get the next rows
  if(rowi->next()){
    //write true
    stream_out.write(true);
    //get the attribute list
    stream_out.write(rowi->currentRow());
  }else{
    //only write false
    stream_out.write(false);  
  }
  stream_out.print();
  
  return true;
}

bool
request_ReleaseCursor( ICoralFacade& /*facade*/, const ByteBuffer& requestBuffer, ByteBuffer& /*replyBuffer*/, bool /*fromProxy*/, rowitermgr& mgri )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  
  uint32_t cursorID;
      
  stream_in.read(cursorID);

  lockMutex(mgri.mutex);
  
  IRowIterator* rowi = mgri.maps[cursorID];
  
  delete rowi;
  
  mgri.maps.erase( mgri.maps.begin() + cursorID );
  
  unlockMutex(mgri.mutex);
  
  return false;
}

bool
request_FetchAllRows( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
//  std::cout << "Execute Query" << std::endl;
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
      
  stream_in.read(sessionID);
  //check if we have an empty rowbuffer
  AttributeList* rowBuffer = NULL;
  
  bool isempty;
  stream_in.read(isempty);
  //row buffer is empty
  if(isempty){
    //create a new attributelist
    rowBuffer = new AttributeList;
    //get the list
    stream_in.read(*rowBuffer);
  }  
  QueryDefinition qd;
  
  stream_in.read(qd);
  
  stream_in.print();

  IRowIteratorPtr rowptr = facade.fetchAllRows( sessionID, qd, rowBuffer );
    
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(*rowptr);
  //return cacheable
///  stream_out.print();
  //dstroy the buffer at the end to avoid segfault
  if(rowBuffer != NULL) delete rowBuffer;
    
  return true;
}

bool
request_ListTables( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);

  coral::Token sessionID;
      
  stream_in.read(sessionID);

  std::string schemaName;
  
  stream_in.read(schemaName);
  
  const std::set<std::string> myset = facade.listTables( sessionID, schemaName );
  
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(myset);  
  
  return true;
}

bool
request_TableDescription( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  
  coral::Token sessionID;
      
  stream_in.read(sessionID);

  std::string schemaName;
  
  stream_in.read(schemaName);
  
  std::string tableName;
  
  stream_in.read(tableName);
  
  const TableDescription& desc = facade.tableDescription( sessionID, schemaName, tableName );
  
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(desc);  
  
  return true;
}

bool
request_TableDescriptionMap( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  
  coral::Token sessionID;
      
  stream_in.read(sessionID);

  std::string schemaName;
  
  stream_in.read(schemaName);
    
  const TableDescriptionMap& descmap = facade.tableDescriptionMap( sessionID, schemaName );
  
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(descmap);  
  
  return true;
}

bool
request_SessionProperties( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool /*fromProxy*/, rowitermgr& /*mgri*/ )
{
  //create a streambuffer for in
  RStreamBuffer stream_in(requestBuffer, CALPACKET_HEADER_SIZE);
  
  coral::Token sessionID;
      
  stream_in.read(sessionID);

  const std::vector<std::string>& sessionp = facade.sessionProperties( sessionID );
  
  WStreamBuffer stream_out(replyBuffer);
  //write the session id to the stream
  stream_out.write(sessionp);  
  
  return false;
}

typedef bool (*request_t)( ICoralFacade& facade, const ByteBuffer& requestBuffer, ByteBuffer& replyBuffer, bool fromProxy, rowitermgr& mgri );

//the array for calling the function by the opcode
static request_t request_matrix[25] = {
  NULL,                                 //0x00
  &request_ConnectRO,                   //0x01
  &request_ConnectRW,                   //0x02
  &request_Disconnect,                  //0x03
  &request_TransactionStartRO,          //0x04
  &request_TransactionStartRW,          //0x05
  &request_TransactionCommit,           //0x06
  &request_TransactionRollback,         //0x07
  NULL,                                 //0x08
  NULL,                                 //0x09
  NULL,                                 //0x0a
  NULL,                                 //0x0b
  NULL,                                 //0x0c
  NULL,                                 //0x0d
  NULL,                                 //0x0e
  NULL,                                 //0x0f
  &request_FetchRows,                   //0x10
  &request_FetchRowsNext,               //0x11
  &request_ReleaseCursor,               //0x12
  &request_FetchAllRows,                //0x13
  &request_ListTables,                  //0x14
  &request_TableDescription,            //0x15
  &request_TableDescriptionMap,         //0x16
  &request_SessionProperties            //0x17
};

//the array to determine which call is available, go from 0 to 255
static bool request_table[255] = {
  false,				//0x00
  true,					//0x01
  true,					//0x02
  true,					//0x03
  true,					//0x04
  true,					//0x05
  true,					//0x06
  true,					//0x07
  false,				//0x08
  false,				//0x09
  false,				//0x0a
  false,				//0x0b
  false,				//0x0c
  false,				//0x0d
  false,				//0x0e
  false,				//0x0f
  true ,                                //0x10
  true ,                                //0x11
  true ,                                //0x12
  true ,                                //0x13
  true ,                                //0x14
  true ,                                //0x15
  true ,                                //0x16
  true ,                                //0x17
  false ,                               //0x18
  false,                                //0x19
  false,                                //0x1a
  false,                                //0x1b
  false,                                //0x1c
  false,                                //0x1d
  false,                                //0x1e
  false                                 //0x1f
};

inline
IByteBufferIteratorPtr
prepareExceptionMessage( StaticBufferPool& replyBufferPool, coral::CALOpcode opcode, coral::CALOpcode exception_opcode, std::string s1, std::string s2 )
{
  //get a new bytebuffer from the pool
  ByteBuffer& replyBuffer = replyBufferPool.allocateBuffer();
  //set the used amount of bytes to the write pos
  //later we can add the header there
  replyBuffer.setUsedSize(CALPACKET_HEADER_SIZE);
  //add the exception string
  WStreamBuffer stream_out(replyBuffer);
  //write the encoding for exceptions to the buffer  
  stream_out.write((coral::CALOpcode)0x00);
  stream_out.write(exception_opcode);
  //write the exception string to the buffer
  stream_out.write(s1);
  stream_out.write(s2);
  //flush the stream
  stream_out.flush();
  //set the exception bit
  opcode = CALOpcodes::getCALReplyException( opcode );
  //create the header
  //calculate the cachesize, set the cacheable flag to false
  ///FIXME setupt the proxy flag
  CALPacketHeader header_out(opcode, false, false, (replyBuffer.usedSize() - CALPACKET_HEADER_SIZE));
  //copy the header to the bytebuffer
  memcpy(replyBuffer.data(), header_out.data(), CALPACKET_HEADER_SIZE);
  //setup the iterator for return
  std::auto_ptr<IByteBufferIterator> ret(new SingleReplyIterator(replyBuffer, replyBufferPool));
  return ret;
}

inline
IByteBufferIteratorPtr
prepareCoralException( StaticBufferPool& replyBufferPool, coral::CALOpcode opcode, coral::Exception& e, coral::CALOpcode exception_opcode )
{
  //copy the exception string to a local string
  std::string ex(e.what());
  //find the first position
  size_t pos01 = ex.find("( CORAL");
  //create a substring until the end of the exception string
  std::string ez(ex.substr(pos01));
  //find the second position  
  size_t pos02 = ez.find(" from ");
  //create the original three parts
  std::string part01(ex.substr(0, pos01 - 1));
  std::string part02(ez.substr(11, pos02 - 12));
  std::string part03(ez.substr(pos02 + 7, ez.size() - pos02 - 10));
  //get a new bytebuffer from the pool
  ByteBuffer& replyBuffer = replyBufferPool.allocateBuffer();
  //set the used amount of bytes to the write pos
  //later we can add the header there
  replyBuffer.setUsedSize(CALPACKET_HEADER_SIZE);
  //add the exception string
  WStreamBuffer stream_out(replyBuffer);
  //write the encoding for exceptions to the buffer  
  stream_out.write((coral::CALOpcode)0x00);
  stream_out.write(exception_opcode);
  //write the exception string to the buffer
  stream_out.write(part01);
  stream_out.write(part02);
  stream_out.write(part03);
  //flush the stream
  stream_out.flush();
  //set the exception bit
  opcode = CALOpcodes::getCALReplyException( opcode );
  //create the header
  //calculate the cachesize, set the cacheable flag to false
  ///FIXME setupt the proxy flag
  CALPacketHeader header_out(opcode, false, false, (replyBuffer.usedSize() - CALPACKET_HEADER_SIZE));
  //copy the header to the bytebuffer
  memcpy(replyBuffer.data(), header_out.data(), CALPACKET_HEADER_SIZE);
  //setup the iterator for return
  std::auto_ptr<IByteBufferIterator> ret(new SingleReplyIterator(replyBuffer, replyBufferPool));
  return ret;
}

inline
IByteBufferIteratorPtr
prepareOKMessage( ICoralFacade& facade, const ByteBuffer& requestBuffer, StaticBufferPool& replyBufferPool, coral::CALOpcode opcode, bool fromProxy, rowitermgr& mgri)
{
//  std::cout << "Execute : " << hexstring(&opcode,1) << std::endl;
  //get a new bytebuffer from the pool
  ByteBuffer& replyBuffer = replyBufferPool.allocateBuffer();
  //set the used amount of bytes to the write pos
  //later we can add the header there
  replyBuffer.setUsedSize(CALPACKET_HEADER_SIZE);
  //call the encode/decode method by the opcode
  bool cacheable = false;
  //start the encoding and decoding part
  //followed by the coral facade part
  try
  {
    //execute
    cacheable = request_matrix[opcode](facade, requestBuffer, replyBuffer, fromProxy, mgri);  
  }
/*
  catch ( coral::ConnectionServiceException& e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
    if(typeid(e) == typeid(ConnectionNotAvailableException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x35 );    
    }else if(typeid(e) == typeid(ReplicaNotAvailableException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x34 );    
    }else{
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x33 );    
    }
  }
  catch ( coral::AuthenticationServiceException& e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
    if(typeid(e) == typeid(UnknownConnectionException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x32 );    
    }else if(typeid(e) == typeid(UnknownRoleException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x31 );    
    }else if(typeid(e) == typeid(InvalidCredentialItemException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x30 );    
    }else if(typeid(e) == typeid(CredentialItemIndexOutOfRangeException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x2f );    
    }else{
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x2e );    
    }
  }
  catch ( coral::SchemaException & e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
    if(typeid(e) == typeid(TableAlreadyExistingException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x2c );    
    }else if(typeid(e) == typeid(TableNotExistingException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x2b );    
    }else if(typeid(e) == typeid(ViewAlreadyExistingException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x2a );    
    }else if(typeid(e) == typeid(ViewNotExistingException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x29 );    
    }else if(typeid(e) == typeid(QueryException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x28 );    
    }else if(typeid(e) == typeid(QueryExecutedException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x27 );    
    }else if(typeid(e) == typeid(InvalidUniqueConstraintIdentifierException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x26 );    
    }else if(typeid(e) == typeid(InvalidForeignKeyIdentifierException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x25 );    
    }else if(typeid(e) == typeid(InvalidIndexIdentifierException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x24 );    
    }else if(typeid(e) == typeid(InvalidColumnIndexException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x23 );    
    }else if(typeid(e) == typeid(InvalidColumnNameException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x22 );    
    }else if(typeid(e) == typeid(NoPrimaryKeyException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x21 );    
    }else if(typeid(e) == typeid(ExistingPrimaryKeyException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x20 );    
    }else if(typeid(e) == typeid(UniqueConstraintAlreadyExistingException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1f );    
    }else if(typeid(e) == typeid(DataEditorException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1e );    
    }else if(typeid(e) == typeid(DuplicateEntryInUniqueKeyException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1d );
    }else{
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1c );    
    }
  }
  catch ( coral::SessionException & e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
    if(typeid(e) == typeid(MaximumNumberOfSessionsException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1b );    
    }else if(typeid(e) == typeid(StartSessionException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x1a );    
    }else if(typeid(e) == typeid(InvalidOperationInReadOnlyModeException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x19 );    
    }else if(typeid(e) == typeid(InvalidSchemaNameException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x18 );    
    }else if(typeid(e) == typeid(NoAuthenticationServiceException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x17 );    
    }else if(typeid(e) == typeid(AuthenticationFailureException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x16 );    
    }else if(typeid(e) == typeid(MonitoringServiceNotFoundException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x15 );    
    }else if(typeid(e) == typeid(ServerException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x14 );    
    }else if(typeid(e) == typeid(DatabaseNotAccessibleException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x13 );    
    }else if(typeid(e) == typeid(ConnectionException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x12 );    
    }else if(typeid(e) == typeid(ConnectionNotActiveException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x11 );    
    }else if(typeid(e) == typeid(TransactionException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x10 );    
    }else if(typeid(e) == typeid(TransactionNotStartedException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0f );    
    }else if(typeid(e) == typeid(TransactionNotCommittedException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0e );    
    }else if(typeid(e) == typeid(TransactionNotActiveException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0d );    
    }else if(typeid(e) == typeid(TypeConverterException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0c );    
    }else if(typeid(e) == typeid(UnSupportedSqlTypeException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0b );    
    }else if(typeid(e) == typeid(UnSupportedCppTypeException)){    
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x0a );
    }else{
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x09 );
    }
  }
  catch ( coral::RelationalServiceException & e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
    if(typeid(e) == typeid(NonExistingDomainException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x08 );    
    }else if(typeid(e) == typeid(InvalidOperationException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x07 );
    }else{
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x06 );
    }
  }*/
  catch ( coral::Exception& e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //check all kinds of excpetions
/*    if(typeid(e) == typeid(coral::AttributeException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x05 );    
    }else if(typeid(e) == typeid(coral::AttributeListException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x04 );    
    }else if(typeid(e) == typeid(coral::MonitoringException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x03 );    
    }else if(typeid(e) == typeid(coral::WebCacheControlException)){
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x02 );    
    }else{*/
      //create new exception message
      return prepareCoralException( replyBufferPool, opcode, e, 0x02 );
//    }
  }
  catch ( std::exception& e )
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //create new exception message
    return prepareExceptionMessage( replyBufferPool, opcode, 0x01, e.what(), "ServerStub::replyToRequest" );
  }
  catch (...)
  {
    //release the byte buffer and return an exception message
    replyBufferPool.releaseBuffer(replyBuffer);
    //create new exception message
    return prepareExceptionMessage( replyBufferPool, opcode, 0x00, "Unkown exception caught", "ServerStub::replyToRequest" );
  }
  //set the reply ok bit
  opcode = CALOpcodes::getCALReplyOK( opcode );
  //create a expection message
//  std::cout << "Reply with : " << hexstring(&opcode,1) << std::endl;
  
  ///FIXME setupt the proxy flag
  CALPacketHeader header_out(opcode, false, cacheable, (replyBuffer.usedSize() - CALPACKET_HEADER_SIZE));
  //copy the header to the bytebuffer
  memcpy(replyBuffer.data(), header_out.data(), CALPACKET_HEADER_SIZE);
  //setup the iterator for return
  std::auto_ptr<IByteBufferIterator> ret(new SingleReplyIterator(replyBuffer, replyBufferPool));
  return ret;
}

ServerStub::ServerStub(ICoralFacade& coralFacade)
  : m_facade( coralFacade )
  , m_bufferpool( new StaticBufferPool )
{
  //logger << "Create ServerStub" << endlog;
  m_rowiterators = new rowitermgr;
  
  m_rowiterators->mutex = new pmutex_t;
  
  initMutex(m_rowiterators->mutex);
}

ServerStub::~ServerStub()
{
  //logger << "Delete ServerStub..." << endlog;  
  delete m_bufferpool;
  //logger << "Delete ServerStub... 1" << endlog;  
  std::vector<IRowIterator*>::iterator i;
  for(i = m_rowiterators->maps.begin(); i != m_rowiterators->maps.end(); i++ )
  {
    delete *i;
  }
  //logger << "Delete ServerStub... 2" << endlog;  
  
  delete m_rowiterators->mutex;
  //logger << "Delete ServerStub... 3" << endlog;  
  delete m_rowiterators;
  //logger << "Delete ServerStub... DONE" << endlog;  
}

IByteBufferIteratorPtr
ServerStub::replyToRequest( const ByteBuffer& requestBuffer )
{
  //decode the header of the received bytebuffer
  CALPacketHeader header_in( requestBuffer.data(), CALPACKET_HEADER_SIZE );
  //retrieve the opcode
  coral::CALOpcode opcode = header_in.opcode();
  
  if(!CALOpcodes::isCALRequest(opcode))
  {
    return prepareExceptionMessage( *m_bufferpool, opcode, 0x10, "Message is not a request", "ServerStub::replyToRequest" );
  }
  //check in the array if we have a valid function for the opcode
  if(!request_table[opcode])
  {
    std::ostringstream s;
    s << "Message with opcode (" << hexstring(&opcode,1) <<") can't be handled";
    return prepareExceptionMessage( *m_bufferpool, opcode, 0x10, s.str(), "ServerStub::replyToRequest" );
  }
  //we have a valid opcode here
  return prepareOKMessage( m_facade, requestBuffer, *m_bufferpool, opcode, header_in.fromProxy(), *m_rowiterators );
}


} }

