// $Id: SingleReplyIterator.h,v 1.3 2009/02/06 17:52:26 kalkhof Exp $
#ifndef CORALSTUBS_SINGLEREPLYITERATOR_H
#define CORALSTUBS_SINGLEREPLYITERATOR_H 1

// Include files
#include "CoralServerBase/IByteBufferIterator.h"
#include "StaticBufferPool.h"
#include "CoralServerBase/ByteBuffer.h"

namespace coral 
{

  namespace CoralStubs
  {
    
    /** @class SingleReplyIterator
     *
     *  Simple implementation of a reply iterator with only one reply.
     *
     *  @author Andrea Valassi
     *  @date   2009-01-23
     */
    
    class SingleReplyIterator : virtual public IByteBufferIterator
    { 
      
    public:
      
      /// Constructor from a reply buffer and a buffer pool.
      /// The new iterator is positioned before the start of the loop.
      /// The input buffer containing the reply is kept as a reference
      /// (instead of being copied): its ownership is passed over to the 
      /// iterator, which becomes reponsible to deallocate it in the dtor.
      SingleReplyIterator( ByteBuffer& buffer, StaticBufferPool& pool );
      
      /// Destructor.
      virtual ~SingleReplyIterator();
      
      /// Get the next reply.
      bool next();
      
      /// Is the current row the last one?
      /// Throws an exception if next() was never called (<first reply).
      /// Throws an exception if next() failed to get a new reply (>last reply).
      bool isLast() const;
      
      /// Get a reference to the current reply buffer.
      /// Throws an exception if next() was never called (<first reply).
      /// Throws an exception if next() failed to get a new reply (>last reply).
      const ByteBuffer& currentBuffer() const;
      
    private:
      
      /// Standard constructor is private.
      SingleReplyIterator();
      
      /// Copy constructor is private. 
      SingleReplyIterator( const SingleReplyIterator& rhs ); 
      
      /// Assignment operator is private.
      SingleReplyIterator& operator=( const SingleReplyIterator& rhs );
      
    private:
      
      /// The next call count.
      unsigned int m_next;
      
      /// The reply buffer.
      coral::ByteBuffer& m_buffer;
      
      /// The buffer pool (which must be used to release the buffer).
      StaticBufferPool& m_pool;
      
    };

  }
  
}
#endif // CORALSTUBS_SINGLEREPLYITERATOR_H
