#ifndef CORAL_CORALSTUBS_STREAMBUFFER_H
#define CORAL_CORALSTUBS_STREAMBUFFER_H

#include "CoralServerBase/ByteBuffer.h"
//Includes some types
#include "CoralServerBase/QueryDefinition.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "RelationalAccess/IQueryDefinition.h"
#include "RelationalAccess/TableDescription.h"
#include "RowIteratorAll.h"

#include "CoralServerBase/CALOpcode.h"

//C++ includes
#include <string>
#include <vector>
#include <set>

#include "CppTypes.h"

namespace coral {

  //forward declaration
  class Blob;
  class Date;
  class TimeStamp;
  
  namespace CoralStubs {
  
    /** @class WStreamBuffer
    *
    *  A write only stream buffer transforming several types to a ByteBuffer
    *
    *  @author Alexander Kalkhof
    *  @date   2009-02-09
    */
 
    class WStreamBuffer {
    public:

      WStreamBuffer(coral::ByteBuffer& bytebuffer);

      ~WStreamBuffer();

      ///write methods
      //add something to the stream and gives the position
      //  for types of:
      //  char
      //  uint32_t (using for 4bytes for all integer types)
      //  bool
      //  std::string
      //  std::vector<std::string>
      //  std::vector< std::pair<std::string, std::string> >
      template<class T> inline void write(const T&);
      //special case
      void write(IRowIterator& data);
      //set the write pos in the ByteBuffer to the internal write pos
      void flush();
      
      ///debug methods
      void print();

    private:
    
      template<class T> inline void writeAttribute(const coral::CALOpcode& opcode, const bool isnull, const Attribute& attr);

      coral::ByteBuffer& m_bytebuffer;

      size_t m_write_pos;

    };
    
    /** @class RStreamBuffer
    *
    *  A read only stream buffer transforming from ByteBuffer to several types
    *
    *  @author Alexander Kalkhof
    *  @date   2009-02-09
    */

    class RStreamBuffer {
    public:
      
      RStreamBuffer(const coral::ByteBuffer& bytebuffer, size_t offset);

      ~RStreamBuffer();

      ///read methods (all const)
      //get a part of the stream
      //  for types of:
      //  char
      //  uint32_t (using for 4bytes for all integer types)
      //  bool
      //  std::string
      //  std::vector<std::string>
      //  std::vector< std::pair<std::string, std::string> >
      template<class T> inline void read(T&);
      
      ///debug methods
      void print();

    private:
    
      template<class T> inline void readAttribute(AttributeList& data, const bool isnull, const std::string& name, const std::type_info& type);

      const coral::ByteBuffer& m_bytebuffer;

      size_t m_read_pos;

    };
    
    ///inline the methods
    
    template<> void WStreamBuffer::write(unsigned char const& data);
    
    template<> void WStreamBuffer::write(bool const& data);
    
    template<> void WStreamBuffer::write(uint16_t const& data);
    
    template<> void WStreamBuffer::write(uint32_t const& data);

    template<> void WStreamBuffer::write(uint64_t const& data);
    
    template<> void WStreamBuffer::write(const std::string& data);
    
    template<> void WStreamBuffer::write(const std::vector<std::string>& data);
    
    template<> void WStreamBuffer::write(const std::set<std::string>& data);
    
    template<> void WStreamBuffer::write(const std::vector< std::pair<std::string, std::string> >& data);
    
    template<> void WStreamBuffer::write(const TableDescription& data);
    
    template<> void WStreamBuffer::write(const std::map< std::string, TableDescription >& data);
    
    template<> void WStreamBuffer::write(const QueryDefinition& data);
    
    template<> void WStreamBuffer::write(const IQueryDefinition::SetOperation& data);
    
    template<> void WStreamBuffer::write(const AttributeList& data);
    
    template<> void WStreamBuffer::write(const coral::Blob& data);
    
    template<> void WStreamBuffer::write(const coral::Date& data);
    
    template<> void WStreamBuffer::write(const coral::TimeStamp& data);
    
    
    
    template<> void RStreamBuffer::read(unsigned char& data);
    
    template<> void RStreamBuffer::read(bool& data);
    
    template<> void RStreamBuffer::read(uint16_t& data);
    
    template<> void RStreamBuffer::read(uint32_t& data);

    template<> void RStreamBuffer::read(uint64_t& data);
    
    template<> void RStreamBuffer::read(std::string& data);
    
    template<> void RStreamBuffer::read(std::vector<std::string>& data);
    
    template<> void RStreamBuffer::read(std::set<std::string>& data);
    
    template<> void RStreamBuffer::read(std::vector< std::pair<std::string, std::string> >& data);

    template<> void RStreamBuffer::read(TableDescription& data);
    
    template<> void RStreamBuffer::read(std::map< std::string, TableDescription >& data);
    
    template<> void RStreamBuffer::read(QueryDefinition& data);
    
    template<> void RStreamBuffer::read(IQueryDefinition::SetOperation& data);    
    
    template<> void RStreamBuffer::read(AttributeList& data);
    
    template<> void RStreamBuffer::read(coral::Blob& data);
    
    template<> void RStreamBuffer::read(coral::Date& data);
    
    template<> void RStreamBuffer::read(coral::TimeStamp& data);
  
  }
}


#endif
