#include "SessionHandle.h"

#include "RelationalAccess/AuthenticationServiceException.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/SessionException.h"

/// default constructor
coral::ConnectionService::SessionHandle::SessionHandle():
  m_session(),
  m_info( new SessionSharedInfo()),
  m_connection()
{
}

/// constructor
coral::ConnectionService::SessionHandle::SessionHandle( const ConnectionHandle& connection ):
  m_session(),
  m_info( new SessionSharedInfo() ),
  m_connection(connection){
}

/// destructor
coral::ConnectionService::SessionHandle::~SessionHandle(){
}

coral::ConnectionService::SessionHandle::SessionHandle(const SessionHandle& rhs):m_session(rhs.m_session),
                                                                                 m_info(rhs.m_info),
                                                                                 m_connection(rhs.m_connection){}

coral::ConnectionService::SessionHandle&
coral::ConnectionService::SessionHandle::operator=(const SessionHandle& rhs){
  m_session = rhs.m_session;
  m_info = rhs.m_info;
  m_connection = rhs.m_connection;
  return *this;
}

/// returns 
coral::ConnectionService::SessionHandle::operator bool() const{
  return m_session.get()!=0;
}

/// initialize the session
bool coral::ConnectionService::SessionHandle::open( const std::string& schemaName,
                                                    const std::string& userName,
                                                    const std::string& password,
                                                    coral::AccessMode accessMode){
  coral::ISession* sess = m_connection.newSession(schemaName,userName,password,accessMode);
  if(sess){
    m_session = boost::shared_ptr<ISession>(sess);
    m_info->m_open = true;
  }
  return m_info->m_open;
}


void coral::ConnectionService::SessionHandle::close(){
  if(m_session) {
    if(m_info->m_open) {
      m_session->endUserSession();
      m_info->m_open = false;
      m_session.reset();
    }
  }
}

bool coral::ConnectionService::SessionHandle::isOpen() const {
  bool open = false;
  if(m_session && m_info->m_open) open = true;
  return open;
}

bool coral::ConnectionService::SessionHandle::isValid() const {
  bool valid = false;
  if(isOpen() && m_connection.isValid()) valid = true;
  return valid;
}

coral::ISession* coral::ConnectionService::SessionHandle::physicalSession() const{
  return m_session.get();
}

coral::ConnectionService::ConnectionHandle& coral::ConnectionService::SessionHandle::connection() {
  return m_connection;
}

const coral::ConnectionService::ConnectionHandle& coral::ConnectionService::SessionHandle::connection() const {
  return m_connection;
}
