#ifndef CONNECTIONSERVICE_SESSIONHANDLE_H
#define CONNECTIONSERVICE_SESSIONHANDLE_H

#include "ConnectionHandle.h"
#include "RelationalAccess/AccessMode.h"

namespace coral{

  class ISession;

  namespace ConnectionService {
    
    /// Placeholder structure for bookkeping of Connection instances allocation 
    class SessionHandle {

      public:

      /// convenience object to store the info shared on all the handles
      class SessionSharedInfo {
        public:
        /// empty constructor
        SessionSharedInfo():
         m_open(false){}
        /// destructor
        virtual ~SessionSharedInfo(){}
        /// the connection open flag
        bool m_open;
      };

      /// default constructor
      SessionHandle();
      
      explicit SessionHandle( const ConnectionHandle& connection );

      SessionHandle(const SessionHandle& rhs);

      /// destructor
      virtual ~SessionHandle();

      SessionHandle& operator=(const SessionHandle& rhs);

      /// 
      operator bool() const;

      /// initialize the session
      bool open( const std::string& schemaName,
                 const std::string& userName,
                 const std::string& password,
                 coral::AccessMode accessMode );
      
      void close();

      bool isOpen() const;

      bool isValid() const;
      
      ISession* physicalSession() const;
      
      ConnectionHandle& connection();
      
      const ConnectionHandle& connection() const;

      private:

      /// handle of the ISession object
      boost::shared_ptr<ISession> m_session;
      /// stored session (shared) info
      boost::shared_ptr<SessionSharedInfo> m_info;
      
      /// the underlying connection
      ConnectionHandle m_connection;

    };

  }
  
}

#endif
