#include "RelationalAccess/ISession.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/AccessMode.h"
#include "RelationalAccess/IRelationalService.h"
#include "RelationalAccess/ITypeConverter.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"

#include "CoralKernel/Context.h"

#include "CoralBase/Exception.h"

#include <exception>
#include <string>
#include <iostream>
#include <stdexcept>

using std::cout;
using std::endl;

int main( int, char** )
{   
   try
   {
     coral::Context& m_context = coral::Context::instance();
     m_context.loadComponent("CORAL/Services/ConnectionService");
     coral::IHandle<coral::IConnectionService> iHandle = m_context.query<coral::IConnectionService>();

     if ( iHandle.isValid() ) 
     {
        /** Set configuration parameters */
        coral::IConnectionServiceConfiguration& config = iHandle->configuration();
        config.setConnectionRetrialPeriod(1);
        config.setConnectionRetrialTimeOut(10);

        /** Get proxy to session providing 
            physical connection string
            access mode */
        coral::ISessionProxy* proxy = iHandle->connect("oracle://devdb10/ral_reader",coral::ReadOnly );

        proxy->transaction().start();
        std::cout << "Tables in the schema: " << std::endl;
        std::set< std::string > listOfTables = proxy->schema("ral_writer").listTables();
        for ( std::set< std::string >::const_iterator iTable = listOfTables.begin();
            iTable != listOfTables.end(); ++iTable ) std::cout << *iTable << std::endl;

        /** get proxy providing logical service name 
           connection with same credentials exists and access mode is ReadOnly,
           new physical connection will not be opened*/

        coral::ISessionProxy* proxy_read = iHandle->connect("reader_test",coral::ReadOnly );
        proxy_read->transaction().start();
        proxy->transaction().commit();
        ///** physical connection is shared */
        //cout <<"connection shared: "<<proxy_read->isConnectionShared()<<endl;
        delete proxy;
        delete proxy_read;

       /** New connection will be opened since the access mode of original
           physical connection is ReadOnly */  

       coral::ISessionProxy* proxy1 = iHandle->connect("write_test",coral::Update);
       proxy1->transaction().start();
       listOfTables = proxy1->nominalSchema().listTables();
       for ( std::set< std::string >::const_iterator iTable = listOfTables.begin();
            iTable != listOfTables.end(); ++iTable ) std::cout << *iTable << std::endl;

       std::set<std::string> cppTypes = proxy1->typeConverter().supportedCppTypes();
       for ( std::set<std::string>::const_iterator iType = cppTypes.begin();
           iType != cppTypes.end(); ++iType ) {
           std::cout << *iType << std::endl;
           std::cout << "                      -> "
                << proxy1->typeConverter().sqlTypeForCppType( *iType ) << std::endl;
       }
       proxy1->transaction().commit();
       /** New physical connection will be opened because both sessions are in 'Update'
           mode */ 
       coral::ISessionProxy* proxy_write = iHandle->connect("write_test",coral::Update);

       delete proxy1;
       delete proxy_write;
    }
    else
    {
      throw std::runtime_error( "Could not load CORAL ConnectionService!" );
    } 
  }
  catch ( coral::Exception& e )
  {
    std::cerr << "CORAL exception " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e )
  {
    std::cerr << "Standard C++ exception " << e.what() << std::endl;
    return 2;
  }
  catch( ... )
  {
    std::cerr << "Unknown exception ..." << std::endl;
    return 3;
  }
  
  return 0;
}
