/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_CONNECTION_H
#define CORALACCESS_CONNECTION_H

#include "base.h"

#include <boost/scoped_ptr.hpp>

#include "RelationalAccess/AccessMode.h"
#include "CoralBricks/MonitorObject.h"
#include "CoralCommon/IDevConnection.h"

#include "Session.h"
#include "TypeConverter.h"

START_CORALACCESS_NAMESPACE

class IConnectionStub;
class Domain;
class IConnectionStubFactory;

class Connection : virtual public coral::IDevConnection, public coral::MonitorObject, public boost::noncopyable
{
public:
	Connection(Domain& domain, const std::string& url, const uint32_t clientID = 0);

	virtual ~Connection();

	// From coral::IDevConnection
	virtual void connect();

	virtual bool isConnected(bool probePhysicalConnection = false);

	virtual void disconnect();

	virtual std::string serverVersion() const;

	virtual ITypeConverter& typeConverter();

	virtual const std::string& serverName() const;

	virtual coral::ISession* newSession(const std::string& dbUri, coral::AccessMode mode) const;

	// -------------------------------------------------------------------------
	IConnectionStub& getStub();

	Domain& domain();

	template<class FACTORY> void resetFactory()
	{
		_stubFactory.reset(new FACTORY);
	}

private:
	bool _isConnected(bool probePhysicalConnection = false) const;

	// Session can manipulate the Connection state via the following methods
	friend class Session;
	void _setServerVersion(const std::string& serverVersion);
	void _sessionClosed();

	Domain& _domain;
	TypeConverter _typeConverter;
	std::pair<bool, std::string> _serverVersion; // the first means if we obtained the server version
	                                             // We get it after starting the session (the first message exchange)
	std::string _hostName;
	unsigned int _port;
	std::string _serverName;
	uint32_t _clientID;
	boost::scoped_ptr<IConnectionStub> _stub;
	boost::scoped_ptr<IConnectionStubFactory> _stubFactory;
	mutable bool _isSessionOpen;
};

END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_CONNECTION_H
