/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include "CoralMessaging/Tcp/TcpSocketFactory.h"
#include "CoralMessaging/IMessage.h"
#include "CoralMessaging/CoralSocket.h"
#include "CoralMessaging/MessageReceiver.h"
#include "CoralMessaging/Timeout.h"
#include "CoralMessaging/CoralException.h"
#include "CoralMessaging/Insulation/SessionException.h"
#include "CoralMessaging/Utils/Error.h"
#include "CoralProtocol/Opcode.h"

#include "ConnectionStub.h"

START_CORALACCESS_NAMESPACE

ConnectionStub::ConnectionStub(const std::string& domainName, const uint32_t cid, ConnectionStub::_HandterPtrType& ioh)
	: _iohandler(ioh),
	  _clientid(cid),
	  _reqid(0),
	  _domainName(domainName)
{
	// EMPTY
}

ConnectionStub::~ConnectionStub()
{
MONITOR_START_CRITICAL
	// EMPTY
MONITOR_END_CRITICAL
}

uint32_t ConnectionStub::send(const coral::CoralMessaging::IMessage& msg)
{
MONITOR_START_CRITICAL
	coral::CoralMessaging::CoralSocket csock(_iohandler, _clientid, _reqid, msg);
	msg.write(csock);
	return _reqid++;
MONITOR_END_CRITICAL
}

IMessagePtr ConnectionStub::recv(uint32_t requestID, uint8_t opcode)
{
MONITOR_START_CRITICAL
	coral::CoralMessaging::MessageReceiver receiver(_iohandler);
	coral::CoralMessaging::InfiniteTimeout to;

	while (receiver.receive(to)) {
		IMessagePtr reply = receiver.message();
		DEBUG_ASSERT_PTR(reply);
		CORALMESSAGING_NAMESPACE_NAME::CoralException::checkAndThrow(*reply);

		if (reply->opcode() == opcode && receiver.from()->requestID() == requestID) {
			return reply;
		} else {
			std::string msg("Unexpected reply message. Expected " + CoralProtocol::Opcode::name(opcode)
				+ ", got " + reply->name());
			throw coral::ServerException(_domainName, msg, __FUNCTION__);
			return IMessagePtr();
		}
	}

	throw coral::ServerException(_domainName, "Timeout occured", __FUNCTION__);
	return IMessagePtr();
MONITOR_END_CRITICAL
}

bool ConnectionStub::connected() const
{
MONITOR_START_CRITICAL
	return !_iohandler->isClosed();
MONITOR_END_CRITICAL
}

IMessagePtr ConnectionStub::sendRecv(const IMessage& msg, uint8_t opcode)
{
	return recv(send(msg), opcode);
}

END_CORALACCESS_NAMESPACE
