/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_CONNECTIONSTUB_H
#define CORALACCESS_CONNECTIONSTUB_H

#include "base.h"

#include <string>
#include <vector>
#include <boost/noncopyable.hpp>

#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/IMessage.h"
#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/Pointers.h"

#include "IConnectionStub.h"

START_CORALACCESS_NAMESPACE

class ConnectionStub : public coral::CoralAccess::IConnectionStub, public coral::MonitorObject, public boost::noncopyable 
{
	typedef coral::CoralMessaging::Pointer<coral::CoralMessaging::ICoralIOHandler>::Shared _HandterPtrType;

public:
	ConnectionStub(const std::string& domainName, const uint32_t cid, _HandterPtrType& ioh);
	
    virtual ~ConnectionStub();

    virtual uint32_t send(const coral::CoralMessaging::IMessage& msg);

    virtual IMessagePtr recv(uint32_t request, uint8_t opcode);
    
    virtual IMessagePtr sendRecv(const IMessage& msg, uint8_t opcode);
    
    virtual bool connected() const;
    
private:    
	_HandterPtrType _iohandler;
     uint32_t _clientid;
     uint32_t _reqid;
     std::string _domainName;
};

END_CORALACCESS_NAMESPACE

#endif // #include CORALACCESS_CONNECTIONSTUB_H
