
#include "Domain.h"
#include "Connection.h"
#include "CoralCommon/URIParser.h"
#include "NotImplemented.h"

#include <sstream>
#include <locale>
#include <iostream>
#include <cstdlib>

#ifdef CORAL_SEALED

#include "SealKernel/PropertyManager.h"

DEFINE_SEAL_COMPONENT ( coral::CoralAccess::Domain, "CORAL/RelationalPlugins/coral" )

START_CORALACCESS_NAMESPACE

coral::CoralAccess::Domain::Domain( seal::Context* c )
	: seal::Service( c, classContextKey() )
	, m_flavorName( "Coral" )
	, m_implementationName( "coral" )
	, m_implementationVersion( "1.1.0" )
	, m_properties(this)
{
	// EMPTY
}

coral::CoralAccess::Domain::Domain( seal::Context* c, const std::string& key )
	: seal::Service( c, key )
	, m_flavorName( "Coral" )
	, m_implementationName( "coral" )
	, m_implementationVersion( "1.1.0" )
	, m_properties(this)
{
	// EMPTY
}

END_CORALACCESS_NAMESPACE

#else // #ifdef CORAL_SEALED

START_CORALACCESS_NAMESPACE

Domain::Domain( const std::string& componentName )
	: coral::Service( componentName )
	, m_flavorName( "coral" )
	, m_implementationName( "coral" )
	, m_implementationVersion( "1.1.0" )
	, m_properties(this)
{
	// EMPTY
}

END_CORALACCESS_NAMESPACE

#endif // #ifdef CORAL_SEALED

START_CORALACCESS_NAMESPACE

Domain::~Domain()
{
	// EMPTY
}

std::string
Domain::flavorName() const
{
	return m_flavorName;
}

std::string
Domain::implementationName() const
{
	return m_implementationName;
}

std::string
Domain::implementationVersion() const
{
	return m_implementationVersion;
}

coral::IConnection*
Domain::newConnection(const std::string& uriString) const
{
	return new Connection(const_cast<Domain&>(*this), uriString, _getClientID());
}

void
Domain::setTableSpaceForTables( const std::string& /*tableSpace*/)
{
	//m_properties->setTableSpaceForTables( tableSpace );
	throw NotImplemented("Domain::setTableSpaceForTables");

}

void
Domain::setTableSpaceForIndices( const std::string& /*tableSpace*/)
{
	//m_properties->setTableSpaceForIndices( tableSpace );
	throw NotImplemented("Domain::setTableSpaceForTables");
}

void
Domain::setTableSpaceForLobs( const std::string& /*tableSpace*/)
{
	//m_properties->setTableSpaceForLobs( tableSpace );
	throw NotImplemented("Domain::setTableSpaceForTables");
}

void
Domain::setLobChunkSize( int /*lobChunkSize*/)
{
	//m_properties->setLobChunkSize( lobChunkSize );
	throw NotImplemented("Domain::setTableSpaceForTables");
}

std::pair<std::string, std::string>
Domain::decodeUserConnectionString( const std::string& userConnectionString ) const
{
	// Processing the connection string
	size_t startPos = userConnectionString.find("://");
	std::string h(userConnectionString.substr(startPos + 3));
	size_t endPos = h.find('|');
	if(endPos == std::string::npos) {
		//try to find the old seperator
		endPos = h.find('&');
		if(endPos == std::string::npos) throw NotImplemented("Domain::decodeUserConnectionString::wrongConnectionString");
	}

	//std::cout << userConnectionString << "|" << endPos << "|" << h << "|" << h.substr(0, endPos) << "|" << h.substr(endPos + 1) << std::endl;

	/*
	 coral::URIParser parser;
	 parser.setURI( userConnectionString );
	 std::ostringstream osDatabase;
	 osDatabase << parser.hostName();
	 int portNumber = parser.portNumber();
	 if ( portNumber != 0 ) osDatabase << ":" << portNumber;

	 std::string tempSchema = parser.databaseOrSchemaName();
	 std::string::size_type slashPosition = tempSchema.find( "/" );
	 if ( slashPosition != std::string::npos ) {
	 osDatabase << "/" << tempSchema.substr( 0, slashPosition );
	 tempSchema = tempSchema.substr( slashPosition + 1 );
	 }

	 std::string schemaName = "";
	 for ( std::string::size_type i = 0; i < tempSchema.size(); ++i ) {
	 schemaName += std::toupper( tempSchema[i], std::locale::classic() );
	 }
	 */
	return std::make_pair( h.substr(0, endPos) , h.substr(endPos + 1) );
}

uint32_t Domain::_getClientID() const
{
	return 0;
}

END_CORALACCESS_NAMESPACE
