#ifndef CORALACCESS_DOMAIN_H
#define CORALACCESS_DOMAIN_H

#include "RelationalAccess/IRelationalDomain.h"
#include "DomainProperties.h"

START_CORALACCESS_NAMESPACE

/** @class Domain
 *
 *  Stripped-down version of the OracleAccess Domain class.
 */
class Domain : public DomainBase, virtual public coral::IRelationalDomain
{
#ifdef CORAL_SEALED
    DECLARE_SEAL_COMPONENT;
public:
    explicit Domain( seal::Context* c );
	Domain( seal::Context* c, const std::string& key);
#else // #ifdef CORAL_SEALED
public:
    Domain( const std::string& componentName );
#endif  // #ifdef CORAL_SEALED

	virtual ~Domain();

	/// Returns the name of the RDBMS technology
	std::string flavorName() const;

	/// Returns the name of the software implementation library
	std::string implementationName() const;

	/// Returns the version of the software implementation library
	std::string implementationVersion() const;

	/// Creates a new session object. The user acquires ownership of this object.
	virtual coral::IConnection* newConnection(const std::string& uriString) const;

	/// Sets the table space for the tables
	void setTableSpaceForTables( const std::string& tableSpace );

	/// Sets the table space for the indices
	void setTableSpaceForIndices( const std::string& tableSpace );

	/// Sets the table space for the lobs
	void setTableSpaceForLobs( const std::string& tableSpace );

	/// Sets the lob chunk size
	void setLobChunkSize( int lobChunkSize );

	/**
	 * Decodes the user connection string into the real connection string that should be passed to the Connection class
	 * and the schema name that should be passed to the Session class
	 */
	std::pair<std::string, std::string> decodeUserConnectionString( const std::string& userConnectionString ) const;

	/**
	 * Returns true if credentials have to be provided to start a session on the specified connection.
	 * For the oracle plugin this is always true.
	 */
	bool isAuthenticationRequired( const std::string& ) const {return false;}

private:
	uint32_t _getClientID() const;

private:
	std::string m_flavorName;

	/// The name of the software implementation library
	std::string m_implementationName;

	/// The version of the software implementation library
	std::string m_implementationVersion;

	DomainProperties m_properties;
};

END_CORALACCESS_NAMESPACE

#endif // #define CORALACCESS_DOMAIN_H
