/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_PRIVILEGEMANAGER_H
#define CORALACCESS_PRIVILEGEMANAGER_H

#include "RelationalAccess/ITablePrivilegeManager.h"
#include "base.h"

START_CORALACCESS_NAMESPACE

class Session;

class PrivilegeManager : virtual public coral::ITablePrivilegeManager
{
public:
	/// Constructor
	PrivilegeManager(const Session& s, const std::string& tableName);

	/// Destructor
	virtual ~PrivilegeManager();
	
	/// Grants an access right to a specific user. 
	virtual void grantToUser(const std::string& userName, coral::ITablePrivilegeManager::Privilege right);
	
	/// Revokes a right from the specified user.
	virtual void revokeFromUser(const std::string& userName, coral::ITablePrivilegeManager::Privilege right);
	
	/// Grants the specified right to all users.
	virtual void grantToPublic(coral::ITablePrivilegeManager::Privilege right);
	
	/// Revokes the specified right from all users.
	virtual void revokeFromPublic(coral::ITablePrivilegeManager::Privilege right);

private:
	const Session& _session;
	std::string _tableName;
};

END_CORALACCESS_NAMESPACE

#endif // CORALACCESS_PRIVILEGEMANAGER_H
