/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/CAL/MsgObj/QueryRepresentation.h"
#include "CoralMessaging/CAL/CALMessages.h"

#include "Session.h"
#include "ConnectionStub.h"
#include "Connection.h"
#include "Query.h"

START_CORALACCESS_NAMESPACE

using CORALMESSAGING_NAMESPACE_NAME::CAL::MsgObj::QueryRepresentationTypes;
using CORALMESSAGING_NAMESPACE_NAME::CAL::MsgObj::AttributeListRepresentation;

Query::Query(Session& session, const std::string& schemaName)
	: QueryDefinition(session),
	  _outputBuffer(new AttributeList)
{
	_eqr.setSchemaName(schemaName);
}

// replace
void Query::setForUpdate()
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	typedef QueryRepresentationTypes::setForUpdate repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type;
	_eqr.replaceMethodCall(repr);
MONITOR_END_CRITICAL
}

// replace
void Query::setRowCacheSize(int numberOfCachedRows)
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	typedef QueryRepresentationTypes::setRowCacheSize repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(numberOfCachedRows);
	_eqr.replaceMethodCall(repr);
MONITOR_END_CRITICAL
}

// replace
void Query::setMemoryCacheSize(int sizeInMB)
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	typedef QueryRepresentationTypes::setMemoryCacheSize repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(sizeInMB);
	_eqr.replaceMethodCall(repr);
MONITOR_END_CRITICAL
}

// insert
void Query::defineOutputType(const std::string& outputIdentifier, const std::string& cppTypeName)
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	typedef QueryRepresentationTypes::defineOutputType repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(outputIdentifier, cppTypeName);
	_eqr.insertMethodCall(repr);
MONITOR_END_CRITICAL
}

// replace
void Query::defineOutput(AttributeList& outputDataBuffer)
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	typedef QueryRepresentationTypes::defineOutput repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(AttributeListRepresentation(outputDataBuffer));
	_eqr.replaceMethodCall(repr);
	_outputBuffer.reset(new AttributeList(outputDataBuffer));
	const unsigned int numberOfVariables = outputDataBuffer.size();

	for (unsigned int i = 0; i < numberOfVariables; ++i)
	    (*_outputBuffer)[i].shareData(outputDataBuffer[i]);
MONITOR_END_CRITICAL
}

ICursor& Query::execute()
{
MONITOR_START_CRITICAL
	_checkIfExecuted();
	// Add the subqueries
	_executeQueryDefinition();
	CORALMESSAGING_NAMESPACE_NAME::CAL::ExecuteQuery msg(_eqr);
	DEBUG_ASSERT_PTR(_session);
	IMessagePtr reply = _session->connection().getStub().sendRecv(msg, coral::CoralProtocol::Opcode::E_ExecuteQueryReply);
	_cursor.reset(new Cursor(reply, *_outputBuffer));
	_setExecuted();
	return *_cursor;
MONITOR_END_CRITICAL
}

SingleTableQuery::SingleTableQuery(Session& session, const std::string& schemaName, const std::string& tableName)
	: QueryDefinition(session),
	  Query(session, schemaName),
	  SingleTableQueryDefinition(session, tableName)
{
	// EMPTY
}

MultiTableQuery::MultiTableQuery(Session& session, const std::string& schemaName)
	: QueryDefinition(session),
	  Query(session, schemaName),
      MultiTableQueryDefinition(session)
{
	// EMPTY
}

END_CORALACCESS_NAMESPACE
