/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_QUERY_H
#define CORALACCESS_QUERY_H

#include "base.h"

#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/IQuery.h"
#include "CoralBase/AttributeList.h"

#include "Cursor.h"
#include "QueryDefinition.h"
#include "Pointers.h"

START_CORALACCESS_NAMESPACE

class Session;

class Query : virtual public coral::IQuery, virtual public QueryDefinition
{
public:
    // From IQuery
    virtual void setForUpdate();

    virtual void setRowCacheSize(int numberOfCachedRows);

    virtual void setMemoryCacheSize(int sizeInMB);

    virtual void defineOutputType(const std::string& outputIdentifier, const std::string& cppTypeName);

    virtual void defineOutput(AttributeList& outputDataBuffer);

    virtual ICursor& execute();

protected:
	Query(Session& session, const std::string& schemaName);

private:
	Pointer<AttributeList>::Scoped _outputBuffer;
	Pointer<Cursor>::Scoped _cursor;
};

struct SingleTableQuery : virtual public Query,  virtual public SingleTableQueryDefinition
{
	SingleTableQuery(Session& session, const std::string& schemaName, const std::string& tableName);
};

struct MultiTableQuery : virtual public Query, virtual public MultiTableQueryDefinition
{
	MultiTableQuery(Session& session, const std::string& schemaName);
};

END_CORALACCESS_NAMESPACE

#endif // #define CORALACCESS_QUERY_H
