/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_QUERYDEFINITION_H
#define CORALACCESS_QUERYDEFINITION_H

#include <vector>

#include "CoralMessaging/CAL/MsgObj/QueryRepresentation.h"
#include "RelationalAccess/IQueryDefinition.h"
#include "Pointers.h"

START_CORALACCESS_NAMESPACE

class Session;
class Query;

using CORALMESSAGING_NAMESPACE_NAME::CAL::MsgObj::QueryRepresentation;

class QueryDefinition : virtual public IQueryDefinition, virtual public coral::MonitorObject, virtual public boost::noncopyable
{
public:
	virtual void setDistinct();

    //insert
    virtual void addToOutputList(const std::string& expression, std::string alias = "");

    virtual void setCondition(const std::string& condition, const AttributeList& inputData);

    virtual void groupBy(const std::string& expression);

    // insert
    virtual void addToOrderList(const std::string& expression);

    virtual void limitReturnedRows(int maxRows = 1, int offset = 0);

    virtual IQueryDefinition& applySetOperation(SetOperation operationType);

    bool operator == (const QueryDefinition&) const;

protected:
	QueryDefinition(Session& session);

	virtual QueryDefinition* _getSimilar() const = 0;

	IQueryDefinition& _defineSubQuery(const std::string& alias, QueryDefinition*);

	void _addToTableList(const std::string& tableName, std::string alias = "");

protected:
	void _checkIfExecuted() const;
	void _setExecuted();
	void _executeQueryDefinition();

	Session* _session;
    QueryRepresentation _eqr;

private:
    typedef std::vector<std::pair<std::string, Pointer<QueryDefinition>::Shared> > _SubQueriesType;
    _SubQueriesType _subQueries;

    std::pair<SetOperation, Pointer<QueryDefinition>::Shared> _setQuery;
	bool _isExecuted;
};

class SingleTableQueryDefinition : virtual public QueryDefinition
{
public:
	SingleTableQueryDefinition(Session& session, const std::string& tableName);

    //insert
    virtual IQueryDefinition& defineSubQuery(const std::string& alias);

    //insert
    virtual void addToTableList(const std::string& tableName, std::string alias = "");

private:
	virtual QueryDefinition* _getSimilar() const;

	const std::string _tableName;
};

class MultiTableQueryDefinition : virtual public QueryDefinition
{
public:
	MultiTableQueryDefinition(Session& session);

	//insert
    virtual IQueryDefinition& defineSubQuery(const std::string& alias);

    //insert
    virtual void addToTableList(const std::string& tableName, std::string alias = "");

private:
	virtual QueryDefinition* _getSimilar() const;

	const std::string _singleTableName;
};

END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_QUERYDEFINITION_H
