/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_SCHEMA_H
#define CORALACCESS_SCHEMA_H

#include "base.h"

#include <boost/scoped_ptr.hpp>

#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/SchemaException.h"
#include "CoralBricks/MonitorObject.h"

#include "SchemaProperties.h"
#include "SchemaEditor.h"

START_CORALACCESS_NAMESPACE

class Session;

class Schema: virtual public coral::ISchema, public coral::MonitorObject, public boost::noncopyable
{
	typedef SchemaProperties::NameListType NameListType;

public:
	Schema(Session& session, const std::string& schemaName);

	virtual ~Schema();

	// From coral::ISchema
	virtual std::string schemaName() const;

	virtual NameListType listTables() const;

	virtual void setTableListStale();

	virtual bool existsTable(const std::string& tableName) const;

	virtual void dropTable( const std::string& tableName );

	virtual void dropIfExistsTable( const std::string& tableName );

	virtual coral::ITable& createTable(const coral::ITableDescription& description);

	virtual coral::ITable& tableHandle(const std::string& tableName);

	virtual void truncateTable(const std::string& tableName);

	virtual void callProcedure(const std::string&, const coral::AttributeList& inputArguments);

	virtual bool existsView(const std::string& viewName) const;

	virtual void dropView(const std::string& viewName);

	virtual void dropIfExistsView( const std::string& viewName );

	virtual NameListType listViews() const;

	virtual coral::IView& viewHandle(const std::string&);

	virtual IQuery* newQuery() const;

	virtual IViewFactory* viewFactory();

private:
	void _dropExistingTable(const std::string& tableName);

	NameListType _listTables() const;

	bool _existsTable(const std::string& tableName) const;

	NameListType _listViews() const;

	coral::ITable& _tableHandle(const std::string& tableName);

	void _throwIfTableExists(const std::string& tableName);

	void _throwIfTableNotExists(const std::string& tableName);

	void _throwIfViewNotExists(const std::string& viewName);

	void _dropExistingView(const std::string& viewName);

	bool _existsView(const std::string& viewName) const;

private:
	Session& _session;
	mutable SchemaProperties _schemaProperties;
	SchemaEditor _sb;
	mutable bool _tableListFresh;
	mutable bool _viewListFresh;
};

END_CORALACCESS_NAMESPACE

#endif // CORALACCESS_SCHEMA_H
