/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#ifndef CORALACCESS_SCHEMAEDITOR_H
#define CORALACCESS_SCHEMAEDITOR_H

#include "base.h"

#include <string>
#include <set>

#include "RelationalAccess/IView.h"
#include "RelationalAccess/ITableDescription.h"

START_CORALACCESS_NAMESPACE

class Session;

class SchemaEditor
{
public:
	typedef std::set<std::string> NameListType;
	
	SchemaEditor(Session& session);

	virtual ~SchemaEditor();
	/// Get the list of table for the current schema from the db server
	std::set<std::string> listTables() const;
	/// Wipe out all the rows in the table in the current schema
	void truncateTable( const std::string& );
	/// Drop the table in the current schema
	void dropTable( const std::string& );
	/// Create the table in the current schema
	void createTable(const coral::ITableDescription& description);
	/// Checks the existence of a view with the specified name.
	bool existsView( const std::string& viewName ) const;
	/// Drops the view with the specified name.
	/// If the view does not exist a ViewNotExistingException is thrown.
	void dropView( const std::string& viewName );
	/// Returns the names of all views in the schema.
	std::set<std::string> listViews() const;
	/// Returns a reference to an IView object corresponding to the view with the specified name.
	/// In case no view with such a name exists, a ViewNotExistingException is thrown.
	IView& viewHandle( const std::string& viewName );

private:
	Session& _session;
};

END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_SCHEMAEDITOR_H
