/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralCommon/URIParser.h"

#include "SchemaProperties.h"

START_CORALACCESS_NAMESPACE

SchemaProperties::SchemaProperties(const std::string& schemaName)
	: _schemaName(schemaName)
{
	// EMPTY
}

SchemaProperties::~SchemaProperties()
{
	// EMPTY
}

const std::string& SchemaProperties::schemaName() const
{
	return _schemaName;
}

void SchemaProperties::eraseTable(const std::string& tableName)
{
	_tables.erase(tableName);
	_tableNames.erase(tableName);
}

bool SchemaProperties::isTableExists(const std::string& tableName) const
{
	return _tableNames.find(tableName) != _tableNames.end();
}

void SchemaProperties::refreshTableNameList(const SchemaProperties::NameListType& list)
{
	_tableNames = list;
}

void SchemaProperties::refreshViewNameList(const SchemaProperties::NameListType& list)
{
	_viewNames = list;
}

Table* SchemaProperties::findTable(const std::string& tableName)
{
	return _tables.find(tableName);
}

Table* SchemaProperties::newTable(const std::string& tableName, Session& s)
{
	Table* t = new Table(_schemaName, tableName, s);
	_tables.add(tableName, t);
	return t;
}

bool SchemaProperties::isViewExists(const std::string& viewName) const
{
	return _viewNames.find(viewName) != _viewNames.end();
}

void SchemaProperties::eraseView(const std::string& viewName)
{
	_viewNames.erase(viewName);
}

END_CORALACCESS_NAMESPACE
