/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_SCHEMAPROPERTIES_H
#define CORALACCESS_SCHEMAPROPERTIES_H

#include "base.h"

#include <string>
#include <map>
#include <set>
#include <boost/noncopyable.hpp>

#include "Table.h"

START_CORALACCESS_NAMESPACE

class Session;

/** Implements the registry of schema properties and its schema objects */
class SchemaProperties : public boost::noncopyable
{
private:
	template <typename T>
	class _RegistryType
	{
	public:
		T* find(const std::string& name) const
		{
			typename _ContainerType::const_iterator tdri = _cont.find(name);
			return tdri == _cont.end() ? NULL : tdri->second.get();
		}

		void erase(const std::string& name)
		{
			_cont.erase(name);
		}

		const T& get(const std::string& name) const
		{
			const T* t = this->find(name);
			CORALPTL_DEBUG_ASSERT(t);
			return *t;
		}

		void add(const std::string& name, T* t)
		{
			CORALACCESS_DEBUG_ASSERT(find(name) == NULL);
			CORALACCESS_DEBUG_ASSERT(t != NULL);
			_cont[name] = boost::shared_ptr<T>(t);
		}

	private:
		typedef std::map<std::string, typename boost::shared_ptr<T> > _ContainerType;
		_ContainerType _cont;
	};

	typedef _RegistryType<Table> TableRegistryType;

public:
	typedef std::set<std::string> NameListType;

	SchemaProperties(const std::string& schemaName);

	virtual ~SchemaProperties();

	const std::string& schemaName() const;

	const std::string& connectionString() const;

	void eraseTable(const std::string& tableName);

	bool isTableExists(const std::string& tableName) const;

	void refreshTableNameList(const NameListType& list);

	void refreshViewNameList(const NameListType& list);

	Table* findTable(const std::string& tableName);

	Table* newTable(const std::string& tableName, Session& s);

	bool isViewExists(const std::string& viewName) const;

	void eraseView(const std::string& viewName);

private:
	/// Current schema name
	std::string _schemaName;
	/// Cached view names
	NameListType _viewNames;
	/// Cached table names
	NameListType _tableNames;
	/// Currently opened table handles
	TableRegistryType _tables;
};

END_CORALACCESS_NAMESPACE

#endif // CORALACCESS_SCHEMAPROPERTIES_H
