/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_SESSION_H
#define CORALACCESS_SESSION_H

#include "base.h"

#include <string>
#include <boost/ptr_container/ptr_map.hpp>

#include "RelationalAccess/AccessMode.h"
#include "CoralCommon/IDevSession.h"
#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/Insulation/SessionException.h"

#include "Transaction.h"
#include "MonitorController.h"
#include "SchemaLoader.h"
#include "Schema.h"

START_CORALACCESS_NAMESPACE

class Connection;

class Session : public coral::IDevSession, public coral::MonitorObject, public boost::noncopyable
{
public:
    Session(Connection& connection, const std::string& dbUri, coral::AccessMode mode);

    virtual ~Session();

    Connection& connection();

    // From coral::IDevSession
    coral::IMonitoring& monitoring();

    std::string nominalSchemaNameForConnection(const std::string& connectionName) const;

    bool isUserSessionActive() const;

    coral::ITransaction& transaction();

    coral::ISchema& nominalSchema();

    coral::ISchema& schema(const std::string& schemaName);

	virtual void startUserSession(const std::string& userName, const std::string& password);

	virtual void endUserSession();

      /**
       * Returns the technology name for the remote session.
       * For plugins establishing a database connection through a middle-tier 
       * (e.g. CoralAccess), this is discovered when establishing the remote session.
       */
      std::string remoteTechnologyName() const; 

      /**
       * Returns the technology name for the remote session.
       * For plugins establishing a database connection through a middle-tier 
       * (e.g. CoralAccess), this is discovered when establishing the remote session.
       */
      std::string remoteServerVersion() const;

private:
	void _endUserSession();
	coral::ISchema& _nominalSchema();

	class _SchemaRegistry {
	public:
		Schema* find(const std::string& schemaName) const;

		void add(const std::string& schemaName, Schema* sch);

	private:
		typedef boost::ptr_map<std::string, Schema> _ContainerType;
		_ContainerType _cont;
	};

	Connection& _connection;
    coral::AccessMode _mode;
    bool _isUserSessionActive;
    _SchemaRegistry _schreg;
    /// The nominal schema object
    Transaction _transaction;
    MonitorController _monitorController;
    SchemaLoader _sl;
    std::string _dbUri;
	Pointer<Schema>::Scoped _schema;

  /// The remote technology name
  std::string m_remoteTechnologyName;
  /// The remote server version
  std::string m_remoteServerVersion;
};

END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_SESSION_H
