/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/CAL/CALMessages.h"

#include "NotImplemented.h"
#include "ConnectionStub.h"
#include "Connection.h"
#include "Transaction.h"

START_CORALACCESS_NAMESPACE

Transaction::Transaction(Connection& connection)
  : coral::MonitorObject(),
    _connection(connection),
    _isActive(false),
    _isReadOnly(false)
{
	// EMPTY
}

//-----------------------------------------------------------------------------

Transaction::~Transaction()
{
MONITOR_START_CRITICAL
	// EMPTY
MONITOR_END_CRITICAL
}

//-----------------------------------------------------------------------------

void Transaction::start(bool readOnly)
{
MONITOR_START_CRITICAL
	coral::CoralMessaging::CAL::MsgObj::StartTransaction obj(readOnly);
	_connection.getStub().sendRecv(coral::CoralMessaging::CAL::StartTransaction(obj), coral::CoralProtocol::Opcode::E_OK);
	_isActive = true;
	_isReadOnly = readOnly;
MONITOR_END_CRITICAL
}

//-----------------------------------------------------------------------------

void Transaction::commit()
{
MONITOR_START_CRITICAL
	_connection.getStub().sendRecv(coral::CoralMessaging::CAL::CommitTransaction(), coral::CoralProtocol::Opcode::E_OK);
	_isActive = false;
MONITOR_END_CRITICAL
}

//-----------------------------------------------------------------------------

void Transaction::rollback()
{
	// TODO
	throw NotImplemented(__FUNCTION__);
}

//-----------------------------------------------------------------------------

bool Transaction::isActive() const
{
MONITOR_START_CRITICAL
	return _isActive;
MONITOR_END_CRITICAL
}

//-----------------------------------------------------------------------------

bool Transaction::isReadOnly() const
{
MONITOR_START_CRITICAL
	return _isReadOnly;
MONITOR_END_CRITICAL
}

END_CORALACCESS_NAMESPACE
