/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_TRANSACTION_H
#define CORALACCESS_TRANSACTION_H

#include "base.h"

#include "RelationalAccess/ITransaction.h"

#include "ConnectionStub.h"

START_CORALACCESS_NAMESPACE

class Connection;

class Transaction : virtual public coral::ITransaction, public coral::MonitorObject, public boost::noncopyable
{

public:
	Transaction(Connection& connection);

	virtual ~Transaction();
	/*
	 * Starts a new transaction.
	 * In case of failure a TransactionNotStartedException is thrown.
	 * 
	 */
	void start(bool readOnly = false);
	/**
	 * Commits the transaction.
	 * In case of failure a TransactionNotCommittedException is thrown.
	 */
	void commit();
	/**
	 * Aborts and rolls back a transaction.
	 */
	void rollback();
	/**
	 * Returns the status of the transaction (if it is active or not)
	 */
	bool isActive() const;
	/**
	 * Returns the mode of the transaction (if it is read-only or not)
	 */
	bool isReadOnly() const;

private:
	Connection& _connection;
	bool _isActive;
	bool _isReadOnly;
};
    
END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_TRANSACTION_H
