/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_VIEWFACTORY_H
#define CORALACCESS_VIEWFACTORY_H

#include "base.h"

#include "RelationalAccess/IViewFactory.h"

#include "QueryDefinition.h"
#include "Session.h"

START_CORALACCESS_NAMESPACE

class Session;

class ViewFactory : public QueryDefinition, virtual public coral::IViewFactory
{
public:
	ViewFactory(Session& session);

	virtual ~ViewFactory();
	/**
	 * Creates a new view with the specified
	 * name and the current query definition.
	 * In case the view already exists a ViewAlreadyExistingException is thrown.
	 */
	virtual coral::IView& create(const std::string& viewName);
	/**
	 * Creates or replaces in case it exists a view with the specified
	 * name and the current query definition.
	 */
	virtual coral::IView& createOrReplace(const std::string& viewName);

	virtual void addToTableList(const std::string& tableName, std::string alias = "");

	virtual IQueryDefinition& defineSubQuery(const std::string& alias);

	virtual QueryDefinition* _getSimilar() const;
};

END_CORALACCESS_NAMESPACE

#endif // #ifndef CORALACCESS_VIEWFACTORY_H
