/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ISessionProperties.h"
#include "RelationalAccess/ISessionProxy.h"

#include "../../src/Connection.h"
#include "../../src/Domain.h"
#include "../../src/Session.h"
#include "../../src/Error.h"
#include "../../src/Query.h"

#include "../UnitTestSuite/OracleSessionCreator.h"
#include "StressTest.h"

START_CORALACCESS_NAMESPACE

void StressTest::testExecuteManyQueries()
{
	coral::ConnectionService connService;
	Pointer<coral::ISessionProxy>::Scoped s(
			connService.connect(TestData::WithServer ? TestData::CoralServiceName : TestData::DbServiceName, coral::ReadOnly));
	CPPUNIT_ASSERT(s.get());
	ISessionProxy& session = *s;
	session.transaction().start(true);
	TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds();

	for (int i = 0; i < QueryNum(); ++i) {
		coral::ISchema& schema = session.nominalSchema();
		Pointer<coral::IQuery>::Scoped query(schema.tableHandle(TableName()).newQuery());
		ICursor& cursor = query->execute();

		if (!TestData::WithServer)
			while (cursor.next()) {}

		if (!(i % 100))
			std::cout << i << "; " << std::flush;
	}
	std::cout << "\n\nExecution time of " << QueryNum() << " queries with server: "
		<< (TimeStamp::now().total_nanoseconds() - start) / 1000 / 1000 / 1000 << " sec" << std::endl;

	session.transaction().commit();
}

END_CORALACCESS_NAMESPACE
