/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/program_options.hpp>

#include "../../src/Logger.h"
#include "../UnitTestSuite/TestTableCreator.h"
#include "../UnitTestSuite/TestBase.h"

#include "StressTest.h"

START_CORALACCESS_NAMESPACE

// Registered tests
// comment out the test that you do not want to run
CPPUNIT_TEST_SUITE_REGISTRATION (StressTest);

namespace TestData
{
	bool WithServer = true;
	std::string DbServiceName =  "/tests/db_ora/admin";
	std::string CoralServiceName = "/tests/db_cor/admin";
	std::string CoralServerUrl = "localhost:8080";
	std::string ExistingSchemaName = "CORAL_WRITER";
	std::string OtherExistingSchemaName = "CORAL_READER";
	std::string NotExistingSchemaName = "CORAL_WRITERCORAL_WRITER";
	std::string ExistingDbUri = "oracle://lcg_coral_nightly/CORAL_WRITER";
	std::string ServerVersion = "0.1.8";
}

END_CORALACCESS_NAMESPACE

using namespace CORALACCESS_NAMESPACE_NAME;

int main (int argc, char** argv)
{
	namespace po = boost::program_options;
	po::options_description desc("Allowed options");

	desc.add_options()
	    ("help", "Display this help page")
	    ("ct", "Create the table")
		("no_server", "Omit the CORAL server, execute directly on Oracle server")
	;

	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);

	if (vm.count("help")) {
	    std::cout << desc << "\n";
	    return 1;
	}

	if (vm.count("ct")) {
		std::cout << "Creating tables..." << std::endl;
		TestTableCreator::Create(StressTest::TableName());
	}

	if (vm.count("no_server")) {
		std::cout << "Omitting CORAL server..." << std::endl;
		TestData::WithServer = false;
	}

	theLogger().setLogOff();

	return coral::UnitTests::run();
}
