/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_TESTS_QUERYTEST_H_
#define CORALACCESS_TESTS_QUERYTEST_H_

#include "../../src/IConnectionStubFactory.h"
#include "../../src/IConnectionStub.h"

#include "TestBase.h"

START_CORALACCESS_NAMESPACE

class Session;
class Query;

#define TEST_NAMES (testSimpleQueryWithServer)(testQueryConstruction)(testQueryEquality)(testStressWithQuery)

class QueryTest: public CPPUNIT_NS :: TestFixture
{
    CPPUNIT_TEST_SUITE (QueryTest);
    BOOST_PP_SEQ_FOR_EACH(TESTBASE_TESTDEF, ~, TEST_NAMES);
    CPPUNIT_TEST_SUITE_END ();

public:
    void setUp (void) {};
    void tearDown (void) {};

	static void SetupQuery_msg_2(Query& q);

protected:
	BOOST_PP_SEQ_FOR_EACH(TESTBASE_METHODEF, ~, TEST_NAMES)

private:
	static std::string _TableName() {return "T";}
	static int _RowNum() {return 6;}
	static int _QueryNumber() {return 1000;}

	void _testSimpleQuery(Session& connection, const std::vector<AttributeList>& fixture);
	std::vector<AttributeList> _fillData(const std::string& serviceName);
};

#undef TEST_NAMES

template<int MSG_SET>
class QueryTestConnectionFactory : public IConnectionStubFactory
{
public:
	IConnectionStub* create(const std::string& domainName, const std::string& host, uint32_t port, const uint32_t cid) const;
};

template<int MSG_SET>
class QueryTestConnectionStub : public IConnectionStub
{
public:
    virtual uint32_t send(const IMessage&) {return 0;};

    virtual IMessagePtr recv(uint32_t, uint8_t) {return IMessagePtr();};

    virtual IMessagePtr sendRecv(const IMessage& msg, uint8_t opcode);

    virtual bool connected() const {return true;}

private:
	static int _msgExchange;
};

END_CORALACCESS_NAMESPACE

#endif /*CORALACCESS_TESTS_QUERYTEST_H_*/
