/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Insulation/Attribute.h"
#include "include_RelationalAccess.h"

#include "../../src/Pointers.h"
#include "RegressionTestCases.h"

START_CORALACCESS_NAMESPACE

void RegressionTestCases::testQueryWithString()
{
	if (!TestData::WithServer)
		return;

	const int rowNum = 1000;
	_createTableWithStrings("OFLP200_NODES", rowNum);

	// Execute the query against the db server directly
	coral::ConnectionService connService;
	Pointer<coral::ISessionProxy>::Scoped s_db(connService.connect(TestData::DbServiceName, coral::ReadOnly));
	CPPUNIT_ASSERT(s_db.get());
	_executeQuery("OFLP200_NODES", *s_db);
	// Execute the query against the db server directly
	Pointer<coral::ISessionProxy>::Scoped s_coral(connService.connect(TestData::CoralServiceName));
	CPPUNIT_ASSERT(s_coral.get());
	_executeQuery("OFLP200_NODES", *s_coral);
}


template <class T>
void RegressionTestCases::_addToDesc(coral::TableDescription& desc, const std::string& name)
{
	desc.insertColumn(name, AttributeSpecification::typeNameForId(typeid(T)));
}


template <class T>
void RegressionTestCases::_addToRow(coral::AttributeList& rowBuffer, const std::string& name)
{
	rowBuffer[name].data<T>() = T();
}


void RegressionTestCases::_createTableWithStrings(const std::string& tableName, const int rowNum)
{
	coral::ConnectionService connService;
	Pointer<coral::ISessionProxy>::Scoped session(connService.connect(TestData::DbServiceName, coral::Update));
	session->transaction().start();
	coral::ISchema& schema = session->nominalSchema();
	schema.dropIfExistsTable(tableName);
	coral::TableDescription description;
	description.setName(tableName);

	_addToDesc<unsigned int>(description, "NODE_ID");
	_addToDesc<unsigned int>(description, "NODE_PARENTID");
	_addToDesc<std::string>(description, "NODE_NAME");
	_addToDesc<std::string>(description, "NODE_FULLPATH");
	_addToDesc<std::string>(description, "NODE_DESCRIPTION");
	_addToDesc<std::string>(description, "NODE_SCHEMA_VERSION");
	_addToDesc<std::string>(description, "NODE_INSTIME");
	_addToDesc<std::string>(description, "LASTMOD_DATE");
	_addToDesc<int>(description, "FOLDER_VERSIONING");
	_addToDesc<std::string>(description, "FOLDER_PAYLOADSPEC");
	_addToDesc<unsigned short>(description, "FOLDER_PAYLOAD_INLINE");
	_addToDesc<std::string>(description, "FOLDER_PAYLOAD_EXTREF");
	_addToDesc<std::string>(description, "FOLDER_CHANNELSPEC");
	_addToDesc<std::string>(description, "FOLDER_CHANNEL_EXTREF");
	_addToDesc<std::string>(description, "FOLDER_IOVTABLENAME");
	_addToDesc<std::string>(description, "FOLDER_TAGTABLENAME");
	_addToDesc<std::string>(description, "FOLDER_IOV2TAGTABLENAME");
	_addToDesc<std::string>(description, "FOLDER_CHANNELTABLENAME");

	coral::ITable& table = schema.createTable( description );
	coral::AttributeList rowBuffer;
	table.dataEditor().rowBuffer(rowBuffer);

	for (int i = 0; i < rowNum; ++i) {
		_addToRow<int>(rowBuffer, "NODE_ID");
		_addToRow<int>(rowBuffer, "NODE_PARENTID");
		_addToRow<std::string>(rowBuffer, "NODE_NAME");
		_addToRow<std::string>(rowBuffer, "NODE_FULLPATH");
		_addToRow<std::string>(rowBuffer, "NODE_DESCRIPTION");
		_addToRow<std::string>(rowBuffer, "NODE_SCHEMA_VERSION");
		_addToRow<std::string>(rowBuffer, "NODE_INSTIME");
		_addToRow<std::string>(rowBuffer, "LASTMOD_DATE");
		_addToRow<int>(rowBuffer, "FOLDER_VERSIONING");
		_addToRow<std::string>(rowBuffer, "FOLDER_PAYLOADSPEC");
		_addToRow<short>(rowBuffer, "FOLDER_PAYLOAD_INLINE");
		_addToRow<std::string>(rowBuffer, "FOLDER_PAYLOAD_EXTREF");
		_addToRow<std::string>(rowBuffer, "FOLDER_CHANNELSPEC");
		_addToRow<std::string>(rowBuffer, "FOLDER_CHANNEL_EXTREF");
		_addToRow<std::string>(rowBuffer, "FOLDER_IOVTABLENAME");
		_addToRow<std::string>(rowBuffer, "FOLDER_TAGTABLENAME");
		_addToRow<std::string>(rowBuffer, "FOLDER_IOV2TAGTABLENAME");
		_addToRow<std::string>(rowBuffer, "FOLDER_CHANNELTABLENAME");

		table.dataEditor().insertRow(rowBuffer);
	}

	session->transaction().commit();
	::sleep(1);
}

void RegressionTestCases::_executeQuery(const std::string& tableName, ISessionProxy& session)
{
	coral::ISchema& schema = session.nominalSchema();
	session.transaction().start(true);
	Pointer<coral::IQuery>::Scoped query(schema.tableHandle(tableName).newQuery());

	query->setRowCacheSize(64);
	query->addToOrderList("NODE_FULLPATH ASC");
	query->addToOutputList("NODE_ID");
	query->addToOutputList("NODE_PARENTID");
	query->addToOutputList("NODE_NAME");
	query->addToOutputList("NODE_FULLPATH");
	query->addToOutputList("NODE_DESCRIPTION");
	query->addToOutputList("NODE_SCHEMA_VERSION");
	query->addToOutputList("NODE_INSTIME");
	query->addToOutputList("LASTMOD_DATE");
	query->addToOutputList("FOLDER_VERSIONING");
	query->addToOutputList("FOLDER_PAYLOADSPEC");
	query->addToOutputList("FOLDER_PAYLOAD_INLINE");
	query->addToOutputList("FOLDER_PAYLOAD_EXTREF");
	query->addToOutputList("FOLDER_CHANNELSPEC");
	query->addToOutputList("FOLDER_CHANNEL_EXTREF");
	query->addToOutputList("FOLDER_IOVTABLENAME");
	query->addToOutputList("FOLDER_TAGTABLENAME");
	query->addToOutputList("FOLDER_IOV2TAGTABLENAME");
	query->addToOutputList("FOLDER_CHANNELTABLENAME");

	AttributeList attList;
	attList.extend<unsigned int>("NODE_ID");
	attList.extend<unsigned int>("NODE_PARENTID");
	attList.extend<std::string>("NODE_NAME");
	attList.extend<std::string>("NODE_FULLPATH");
	attList.extend<std::string>("NODE_DESCRIPTION");
	attList.extend<std::string>("NODE_SCHEMA_VERSION");
	attList.extend<std::string>("NODE_INSTIME");
	attList.extend<std::string>("LASTMOD_DATE");
	attList.extend<int>("FOLDER_VERSIONING");
	attList.extend<std::string>("FOLDER_PAYLOADSPEC");
	attList.extend<unsigned short>("FOLDER_PAYLOAD_INLINE");
	attList.extend<std::string>("FOLDER_PAYLOAD_EXTREF");
	attList.extend<std::string>("FOLDER_CHANNELSPEC");
	attList.extend<std::string>("FOLDER_CHANNEL_EXTREF");
	attList.extend<std::string>("FOLDER_IOVTABLENAME");
	attList.extend<std::string>("FOLDER_TAGTABLENAME");
	attList.extend<std::string>("FOLDER_IOV2TAGTABLENAME");
	attList.extend<std::string>("FOLDER_CHANNELTABLENAME");

	query->defineOutput(attList);
	ICursor& cursor = query->execute();
	while (cursor.next()) {}
	session.transaction().commit();
}

END_CORALACCESS_NAMESPACE
