/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Insulation/SchemaException.h"
#include "CoralMessaging/CAL/CALMessages.h"

#include "../../src/SchemaLoader.h"
#include "../../src/Domain.h"
#include "../../src/Connection.h"
#include "../../src/Pointers.h"
#include "../../src/Session.h"

#include "OracleSessionCreator.h"
#include "DomainHandle.h"
#include "SchemaLoaderTest.h"

START_CORALACCESS_NAMESPACE

void SchemaLoaderTest::testSchemaExistsWithoutServer()
{
	DomainHandle dh;
	Domain& domain = dh.get();
	Pointer<Connection>::Scoped connection(dynamic_cast<Connection*>(domain.newConnection(TestData::CoralServerUrl)));
	connection->resetFactory<SchemaLoaderTestConnectionFactory>();
	connection->connect();
	Pointer<Session>::Scoped session(dynamic_cast<Session*>(connection->newSession(TestData::ExistingDbUri, coral::ReadOnly)));
	SchemaLoader sl;
	// Test existing schema
	CPPUNIT_ASSERT(sl.schemaExists(*session, TestData::ExistingSchemaName));
	// Test non-existing schema
	try {
		sl.schemaExists(*session, NotExistingSchemaName());
	} catch(coral::InvalidSchemaNameException&) {
		return;
	}
	// Should not get here...
	CPPUNIT_ASSERT(0);
}

void SchemaLoaderTest::testSchemaExistsWithServer()
{
	if (!TestData::WithServer)
		return;

	OracleSessionCreator osc(coral::ReadOnly);
	Session& session = osc.session();

	SchemaLoader sl;
	// Test existing schema
	CPPUNIT_ASSERT(sl.schemaExists(session, TestData::ExistingSchemaName));

	//The test fails due to OracleAccess bug
#if 0
	// Test non-existing schema
	bool catched = false;

	try {
		sl.schemaExists(session, NotExistingSchemaName());
	} catch(coral::InvalidSchemaNameException&) {
		catched = true;
	}

	CPPUNIT_ASSERT(catched);
#endif
}

IConnectionStub* SchemaLoaderTestConnectionFactory::create(const std::string&, const std::string&, uint32_t, const uint32_t) const
{
	return new SchemaLoaderTestConnectionStub;
}

IMessagePtr SchemaLoaderTestConnectionStub::sendRecv(const IMessage& m, uint8_t opcode)
{
	CPPUNIT_ASSERT_EQUAL(static_cast<int>(coral::CoralProtocol::Opcode::E_TableSchemaExists), static_cast<int>(m.opcode()));
	CPPUNIT_ASSERT_EQUAL(static_cast<int>(coral::CoralProtocol::Opcode::E_OK), static_cast<int>(opcode));
	using coral::CoralMessaging::CAL::TableSchemaExists;
	using coral::CoralMessaging::CAL::OK;
	const TableSchemaExists& msg = dynamic_cast<const TableSchemaExists&>(m);

	if (msg.schemaName() == TestData::ExistingSchemaName)
		return IMessagePtr(new OK);
	else if (msg.schemaName() == SchemaLoaderTest::NotExistingSchemaName())
		throw coral::InvalidSchemaNameException();

	// Should not get here...
	CPPUNIT_ASSERT(0);
	return IMessagePtr();
}

END_CORALACCESS_NAMESPACE
