/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_TESTS_SESSIONTEST_H_
#define CORALACCESS_TESTS_SESSIONTEST_H_

#include <string>
#include <set>

#include "../../src/Pointers.h"
#include "../../src/Domain.h"
#include "../../src/Connection.h"
#include "../../src/Session.h"

#include "TestBase.h"

START_CORALACCESS_NAMESPACE

#define TEST_NAMES (testSuccessfullConnection)

class SessionTest: public CPPUNIT_NS :: TestFixture
{
    CPPUNIT_TEST_SUITE (SessionTest);
    BOOST_PP_SEQ_FOR_EACH(TESTBASE_TESTDEF, ~, TEST_NAMES);
    CPPUNIT_TEST_SUITE_END ();

public:
    void setUp (void) {};
    void tearDown (void) {};

    static const std::string& UserName()
    {
    	static const std::string s("UserName");
    	return s;
    }

    static const std::string& Password()
    {
    	static const std::string s("Password");
    	return s;
    }

    static const std::string& ConnectionString()
    {
    	static const std::string s = "coral://" + TestData::CoralServerUrl+ "|" + TestData::ExistingDbUri;
    	return s;
    }

protected:
	BOOST_PP_SEQ_FOR_EACH(TESTBASE_METHODEF, ~, TEST_NAMES)

private:
	void _testSuccessfullConnection(Connection& connection);
};

#undef TEST_NAMES

class SessionTestConnectionFactory : public IConnectionStubFactory
{
public:
	IConnectionStub* create(const std::string& domainName, const std::string& host, uint32_t port, const uint32_t cid) const;
};

class SessionTestConnectionStub : public IConnectionStub
{
public:
    virtual uint32_t send(const IMessage&) {return 0;};

    virtual IMessagePtr recv(uint32_t, uint8_t) {return IMessagePtr();};

    virtual IMessagePtr sendRecv(const IMessage& msg, uint8_t opcode);

    virtual bool connected() const {return true;}

private:
	static int _msgExchange;
};

END_CORALACCESS_NAMESPACE

#endif /*CORALACCESS_TESTS_SESSIONTEST_H_*/
