/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <unistd.h>
#include <boost/bind.hpp>
#include <boost/ptr_container/ptr_vector.hpp>

#include "CoralBricks/ThreadTraits.h"
#include "CoralMessaging/CAL/CALMessages.h"

#include "../../src/Domain.h"
#include "../../src/Connection.h"
#include "../../src/Session.h"
#include "../../src/IConnectionStub.h"

#include "TestTableCreator.h"
#include "OracleSessionCreator.h"
#include "StressTest.h"

START_CORALACCESS_NAMESPACE

void StressTest::testQueriesWithConnectionDrop()
{
	if (!TestData::WithServer || !TestData::ExecuteStressTest)
		return;

	try {
		Pointer<OracleSessionCreator>::Scoped osc(new OracleSessionCreator(coral::ReadOnly));
		// Execute many queries from a thread.
		ThreadTraits::THREAD_GROUP tg;

		for (int i = 0; i < 100; ++i) {
			tg.create_thread(boost::bind(&StressTest::_launchQuery, this, osc.get()));
			std::cout << i << " scheduled" << std::endl;
		}

		tg.join_all();
		// When all the threads launched the query, drop the connection.
		osc.reset(NULL);
		// The server should survive this. Wait 10 seconds, and try to reconnect. It must be successfull.
		::sleep(10);
		OracleSessionCreator osc2(coral::ReadOnly);
	} catch (...) {
		CPPUNIT_ASSERT(0);
	}
}

void StressTest::testManyMultipleConnections()
{
	if (!TestData::WithServer || !TestData::ExecuteStressTest)
		return;

	try {
		boost::ptr_vector<OracleSessionCreator> connections;

		for (int i = 0; i < 5; ++i) {
			connections.push_back(new OracleSessionCreator(coral::ReadOnly));
			std::cout << i << " connected" << std::endl;
		}
	} catch (...) {
		CPPUNIT_ASSERT(0);
	}
}

void StressTest::setUp (void)
{
	if (!TestData::WithServer || !TestData::ExecuteStressTest)
		return;

	TestTableCreator::Create(_TableName());
}


void StressTest::_launchQuery(OracleSessionCreator* osc)
{
	// We do not care about any exceptions here comint from the dropped connection
	try {
		coral::ISchema& schema = osc->session().nominalSchema();
		Pointer<coral::IQuery>::Scoped query(schema.tableHandle(_TableName()).newQuery());
		query->execute();
	} catch (...) {}
}

END_CORALACCESS_NAMESPACE
