/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALACCESS_TESTS_TABLESCHEMALOADERTEST_H_
#define CORALACCESS_TESTS_TABLESCHEMALOADERTEST_H_

#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/TableDescription.h"
#include "CoralMessaging/CAL/MsgObj/Objects.h"

#include "../../src/IConnectionStubFactory.h"
#include "../../src/IConnectionStub.h"

#include "TestBase.h"

START_CORALACCESS_NAMESPACE

class Session;

#define TEST_NAMES (testSchemaExists)

using namespace coral::CoralMessaging::CAL;

class TableSchemaLoaderTest: public CPPUNIT_NS :: TestFixture
{
    CPPUNIT_TEST_SUITE (TableSchemaLoaderTest);
    BOOST_PP_SEQ_FOR_EACH(TESTBASE_TESTDEF, ~, TEST_NAMES);
    CPPUNIT_TEST_SUITE_END ();

public:
    void setUp (void) {};
    void tearDown (void) {};

	static MsgObj::ColumnDescription& ColDesc_1()
	{
		static MsgObj::ColumnDescription obj("columnName_1",  coral::AttributeSpecification::typeNameForId(typeid(std::string)), 32, true, true, TableSpaceName());
		return obj;
	}

	static MsgObj::ColumnDescription& ColDesc_2()
	{
		static MsgObj::ColumnDescription obj("columnName_2", coral::AttributeSpecification::typeNameForId(typeid(int)), 32, true, false, TableSpaceName());
		return obj;
	}

	static MsgObj::IndexDescription& IndexDesc()
	{
		static MsgObj::IndexDescription obj("indexName", std::vector<std::string>(1, "columnName_1"), false, TableSpaceName());
		return obj;
	}

	static MsgObj::ForeignKeyDescription& FKDesc()
	{
		static MsgObj::ForeignKeyDescription obj("keyName", std::vector<std::string>(1, "columnName_2"), "referencedTableName", std::vector<std::string>(1, "ReferencedColumnName"));
		return obj;
	}

	static MsgObj::UniqueConstraintDescription& UCDesc()
	{
		static MsgObj::UniqueConstraintDescription obj("C_NAME", std::vector<std::string>(1, "columnName_2"), TableSpaceName());
		return obj;
	}

	static MsgObj::PrimaryKeyDescription& PKDesc()
	{
		static MsgObj::PrimaryKeyDescription obj(std::vector<std::string>(1, "columnName_1"), TableSpaceName());
		return obj;
	}

	static std::string& TableName()
	{
		static std::string t("TableName");
		return t;
	}

	static std::string& TableSpaceName()
	{
		static std::string t("CORAL_DATA01");
		return t;
	}

	static std::string& Type()
	{
		static std::string t("");
		return t;
	}

protected:
	BOOST_PP_SEQ_FOR_EACH(TESTBASE_METHODEF, ~, TEST_NAMES)

private:
	void _testSchemaExists(coral::TableDescription&);
};

class TableSchemaLoaderTestConnectionFactory : public IConnectionStubFactory
{
public:
	IConnectionStub* create(const std::string& domainName, const std::string& host, uint32_t port, const uint32_t cid) const;
};

class TableSchemaLoaderTestConnectionStub : public IConnectionStub
{
public:
    virtual uint32_t send(const IMessage&) {return 0;};

    virtual IMessagePtr recv(uint32_t, uint8_t) {return IMessagePtr();};

    virtual IMessagePtr sendRecv(const IMessage& msg, uint8_t opcode);

    virtual bool connected() const {return true;}

private:
	static int _msgExchange;
};

#undef TEST_NAMES

END_CORALACCESS_NAMESPACE

#endif /*#ifndef CORALACCESS_TESTS_TABLESCHEMALOADERTEST_H_*/
