/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/CAL/CALMessages.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ISessionProperties.h"
#include "RelationalAccess/ISessionProxy.h"
#include "CoralBase/Blob.h"

#include "../../src/Connection.h"
#include "../../src/Domain.h"
#include "../../src/Session.h"
#include "../../src/Error.h"
#include "../../src/Query.h"

#include "TestTableCreator.h"

START_CORALACCESS_NAMESPACE

void TestTableCreator::Create(const std::string& tableName)
{
	coral::ConnectionService connService;
	Pointer<coral::ISessionProxy>::Scoped session(connService.connect(TestData::DbServiceName, coral::Update));
	session->transaction().start();
	coral::ISchema& schema = session->nominalSchema();
	schema.dropIfExistsTable(tableName);

	coral::TableDescription description;
	description.setName(tableName);
	description.insertColumn("ID", AttributeSpecification::typeNameForId(typeid(int)));
	description.setPrimaryKey("ID");
	description.insertColumn("x", coral::AttributeSpecification::typeNameForId(typeid(float)));
	description.setNotNullConstraint( "x" );
	description.insertColumn("Y", coral::AttributeSpecification::typeNameForId(typeid(double)));
	description.insertColumn("Z", coral::AttributeSpecification::typeNameForId(typeid(double)));
	description.insertColumn( "data1", coral::AttributeSpecification::typeNameForId(typeid(std::string)), 100, false);
	description.insertColumn( "data2", coral::AttributeSpecification::typeNameForId(typeid(coral::Blob)));

	coral::ITable& table = schema.createTable( description );
	coral::AttributeList rowBuffer;
	table.dataEditor().rowBuffer(rowBuffer);

	for (int i = 0; i < _RowNum(); ++i) {
		rowBuffer["ID"].data<int>() = i + 10;
		rowBuffer["x"].data<float>() = i + 0.1 * i;
		rowBuffer["Y"].setNull(false);
		rowBuffer["Y"].data<double>() = i + 0.01 * i;
		rowBuffer["Y"].setNull(i%2);
		rowBuffer["Z"].data<double>() = i + 0.001 * i;
		rowBuffer["data1"].data<std::string>() = "Data";

		rowBuffer["data2"].setNull(false);
		coral::Blob& blob = rowBuffer["data2"].data<coral::Blob>();
		int blobSize = ( i + 1 ) * 1000;
		blob.resize( blobSize );
		unsigned char* p = static_cast<unsigned char*>( blob.startingAddress() );

		for ( int j = 0; j < blobSize; ++j, ++p )
			*p = j%256;

		rowBuffer["data2"].setNull(i%2);
		table.dataEditor().insertRow( rowBuffer );
	}

	session->transaction().commit();
	::sleep(1);
}

END_CORALACCESS_NAMESPACE
