/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Insulation/SchemaException.h"

#include "../../src/SchemaLoader.h"
#include "../../src/Domain.h"
#include "../../src/Connection.h"
#include "../../src/Pointers.h"
#include "../../src/Transaction.h"

#include "DomainHandle.h"
#include "TransactionTest.h"

START_CORALACCESS_NAMESPACE

void TransactionTest::testTransaction()
{
	DomainHandle dh;
	Domain& domain = dh.get();
	Pointer<IConnection>::Scoped connection(domain.newConnection("dummy"));

	Transaction tr(dynamic_cast<Connection&>(*connection));
	CPPUNIT_ASSERT(!tr.isActive());


	bool catched = false;
	// Must throw an exception, because transaction has not been started yet.
	try {
		tr.isReadOnly();
	} catch (coral::TransactionException&) {
		catched = true;
	}

	CPPUNIT_ASSERT(catched);

	// Must throw an exception, because it's not connected
	catched = false;

	try {
		tr.commit();
	} catch (...) {
		catched= true;
	}

	CPPUNIT_ASSERT(catched);


}

END_CORALACCESS_NAMESPACE
