/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/program_options.hpp>

#include "TestBase.h"
#include "StressTest.h"
#include "SchemaLoaderTest.h"
#include "SchemaEditorTest.h"
#include "ConnectionStubTest.h"
#include "SessionTest.h"
#include "TableSchemaLoaderTest.h"
#include "ConnectionTest.h"
#include "QueryTest.h"
#include "CursorTest.h"
#include "RegressionTestCases.h"

START_CORALACCESS_NAMESPACE

// Registered tests
// comment out the test that you do not want to run
CPPUNIT_TEST_SUITE_REGISTRATION (StressTest);
CPPUNIT_TEST_SUITE_REGISTRATION (SchemaLoaderTest);
CPPUNIT_TEST_SUITE_REGISTRATION (SchemaEditorTest);
CPPUNIT_TEST_SUITE_REGISTRATION (ConnectionStubTest);
CPPUNIT_TEST_SUITE_REGISTRATION (SessionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TableSchemaLoaderTest);
CPPUNIT_TEST_SUITE_REGISTRATION (ConnectionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (QueryTest);
CPPUNIT_TEST_SUITE_REGISTRATION (CursorTest);
//CPPUNIT_TEST_SUITE_REGISTRATION (RegressionTestCases);

namespace TestData
{
	bool WithServer = false;
	bool ExecuteStressTest = false;
	std::string DbServiceName = "/tests/db_ora/admin";
	std::string CoralServiceName = "/tests/db_cor_ora/admin";
	std::string CoralServerUrl = "localhost:8080";
	std::string ExistingSchemaName = "CORAL_WRITER";
	std::string OtherExistingSchemaName = "CORAL_READER";
	std::string NotExistingSchemaName = "CORAL_WRITERCORAL_WRITER";
	std::string ExistingDbUri = "oracle://lcg_coral_nightly/CORAL_WRITER";
	std::string ServerVersion = "0.1.11";
}

END_CORALACCESS_NAMESPACE

bool WithServer = false;

int main (int argc, char** argv)
{
	namespace po = boost::program_options;
	po::options_description desc("Allowed options");

	desc.add_options()
	    ("help", "Display this help page")
	    ("ws", "run the tests with also the server")
	    ("cs", po::value<std::string>(), "CORAL server url in the form of hostname:port")
	    ("sn", po::value<std::string>(), "Schema name used during the tests")
	    ("st", "Run also the stress tests")
	    ("backend", po::value<std::string>(), "Backend type: mysql, oracle, sqlite. Default is oracle.")
	;

	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);

	if (vm.count("help")) {
	    std::cout << desc << "\n";
	    return 1;
	}

	using namespace CORALACCESS_NAMESPACE_NAME;

	if (vm.count("cs"))
		TestData::CoralServerUrl = vm["cs"].as<std::string>();

	if (vm.count("sn"))
		TestData::ExistingSchemaName = vm["sn"].as<std::string>();

	if (vm.count("backend")) {
		std::string bk = vm["backend"].as<std::string>();

		if (bk == "oracle") {}
		else if (bk == "mysql") {
			TestData::ExistingDbUri = "mysql://itrac424.cern.ch/RAL_WRITER";
			TestData::DbServiceName = "/tests/db_mys/admin";
			TestData::CoralServiceName = "/tests/db_cor_mys/admin";
			std::string ExistingSchemaName = "RAL_WRITER";
			std::string OtherExistingSchemaName = "CORAL_WRITER";
		} else if (bk == "sqlite") {
			TestData::ExistingDbUri = "sqlite_file:ral.db";
			TestData::DbServiceName = "/tests/db_sql/admin";
			TestData::CoralServiceName = "/tests/db_cor_sql/admin";
			std::string ExistingSchemaName = "ral.db";
			std::string OtherExistingSchemaName = "ral.db";
		} else {
			std::cout << "Unsupported backend\n;";
			std::cout << desc << "\n" << std::endl;
			exit(-1);
		}
	}

	TestData::WithServer = vm.count("ws");
	TestData::ExecuteStressTest = vm.count("st");

	return coral::UnitTests::run();
}
