#include "CoralBricks/TimeTraits.h"
#include "CoralBase/TimeStamp.h"

const boost::posix_time::ptime& coral::TimeStamp::Epoch = boost::posix_time::ptime( boost::gregorian::date(1970, 1, 1 ) );

coral::TimeStamp::TimeStamp( int year, int month,  int day, int hour, int minute, int second, long nanosecond, bool isLocalTime /*=true*/ )
: m_time( boost::gregorian::date( year, month, day ), boost::posix_time::time_duration( hour, minute, second, nanosecond ) )
, m_isLocal( isLocalTime )
{
  if( m_isLocal )
    m_localTime = std::auto_ptr<boost::posix_time::ptime>( 
                      new boost::posix_time::ptime( coral::time::local_adj::utc_to_local( m_time ) ) );
}

coral::TimeStamp::TimeStamp( const boost::posix_time::ptime& pT, bool isLocalTime )
: m_time( pT ), m_isLocal(isLocalTime), m_localTime( 0 )
{
  if( m_isLocal )
    m_localTime = std::auto_ptr<boost::posix_time::ptime>( 
                      new boost::posix_time::ptime( coral::time::local_adj::utc_to_local( m_time ) ) );
}

coral::TimeStamp::TimeStamp( coral::TimeStamp::ValueType& nsecs )
: m_time(), m_isLocal( false ), m_localTime( 0 )
{
   // This contructs the timestamp only assuming UTC
   unsigned long int lSecs  = nsecs/1000000000; // extract seconds     part
   unsigned long int lNsecs = nsecs%1000000000; // extract nanoseconds part
   
   // Little precision check
   long int precision = boost::posix_time::time_duration::num_fractional_digits();
   
   boost::posix_time::time_duration since_epoch;
   
   if( precision > 6 )
     since_epoch = boost::posix_time::seconds( lSecs ) + coral::time::nanoseconds( lNsecs );
   else
     since_epoch = boost::posix_time::seconds( lSecs ) + boost::posix_time::microseconds( lNsecs/1000 );
     
   boost::posix_time::ptime time_since_epoch( boost::gregorian::date( 1970, 1, 1 ), since_epoch );
   
   m_time = time_since_epoch;
}

coral::TimeStamp::ValueType coral::TimeStamp::total_nanoseconds() const
{
   // This is always returned in UTC, keeps the former SEAL semantics
   coral::TimeStamp::ValueType nsecs = 0L;
  
   boost::posix_time::time_duration since_epoch = m_time - coral::TimeStamp::Epoch;
   nsecs = since_epoch.total_nanoseconds();

   return nsecs;
}

coral::TimeStamp coral::TimeStamp::now( bool isLocalTime /*=false*/ )
{
  boost::posix_time::ptime timeNow;
  
  if( isLocalTime )
  {
    timeNow = boost::posix_time::microsec_clock::local_time();
  }
  else
  {
    timeNow = boost::posix_time::microsec_clock::universal_time();
  }
  
  return coral::TimeStamp( timeNow, isLocalTime );
}

std::string coral::TimeStamp::toString() const
{
  std::stringstream ss;
  ss << day()  << "/" << month()  << "/" << year()   << " "
     << hour() << ":" << minute() << ":" << second() << "." << nanosecond();
  return ss.str();
}
