/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALBRICKS_BOOSTTHREADTRAITS_H_
#define CORALBRICKS_BOOSTTHREADTRAITS_H_

/**
 * Include this file only from ThreadTraits. If you include it from elsewhere,
 * you will get compilation error.
 */

#ifndef __THREADTRAITS_H_GUARD__
  #error File is included from elsewhere that ThreadTraits.h
#endif

// Disable warnings triggered by the CppUnit headers
// See http://wiki.services.openoffice.org/wiki/Writing_warning-free_code
// See also http://www.artima.com/cppsource/codestandards.html
#if defined __GNUC__
#  pragma GCC system_header
#elif defined __SUNPRO_CC
#  pragma disable_warn
#elif defined _MSC_VER
#  pragma warning(push, 1)
#endif

#include <boost/thread.hpp>
#include <boost/thread/condition.hpp>

#ifndef BOOST_HAS_THREADS
  #error Threaded version of Boost is required.
#endif

#include <boost/version.hpp>

namespace coral {

struct ThreadTraits {
	typedef boost::mutex              MUTEX;
	typedef boost::condition          CONDITION;
	typedef boost::mutex::scoped_lock LOCK;
	typedef boost::thread_group       THREAD_GROUP;
	typedef boost::thread             THREAD;

#if (BOOST_VERSION >= 103500)
	static boost::thread::id get_id()
	{
		return boost::this_thread::get_id();
	}
#else
    static int get_id()
    {
        return 0;
    }
#endif

};

} // namespace coral

#if defined __SUNPRO_CC
#  pragma enable_warn
#elif defined _MSC_VER
#  pragma warning(pop)
#endif

#endif /*CORALBRICKS_BOOSTTHREADTRAITS_H_*/
