/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALBRICKS_TIMETRAITS_H_
#define CORALBRICKS_TIMETRAITS_H_

/**
 * Define the time/date strategy. Include the file that implements the
 * traits. The rest of the code should include only this file, and
 * use the ThreadTraits type as a template parameter, typedef, etc. 
 * 
 * The file implementing the strategy should check if the __TIMETRAITS_H_GUARD__
 * is defined or not. If not defined, it indicates that the file was included
 * from elsewhere than this file. It is normally an unwanted thing.
 *  
 * For example, see the BoostTimeTraits.h
 */

#ifdef __TIMETRAITS_H_GUARD__
  #error __TIMETRAITS_H_GUARD__ should not be defined here!
#endif

#define __TIMETRAITS_H_GUARD__
// include the file defining the time strategy here. Including it 
// elsewhere should result in compilation error.
#include "BoostTimeTraits.h"
#undef __TIMETRAITS_H_GUARD__

#endif /*CORALBRICKS_TIMETRAITS_H_*/
