/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <boost/tuple/tuple_io.hpp>

#include "CoralCommon/URIParser.h"
#include "UriParserTest.h"

namespace coral { namespace CoralCommon {

void UriParserTest::test()
{
	CPPUNIT_ASSERT_EQUAL(_testParser("oracle://dbhost/D9/user"),
			URIParserTest_Results("oracle://dbhost/D9/user", "oracle", "", "dbhost", 0, "D9/user"));

	CPPUNIT_ASSERT_EQUAL(_testParser("oracle://dbhost/user"),
			URIParserTest_Results("oracle://dbhost/user", "oracle", "", "dbhost", 0, "user"));

	CPPUNIT_ASSERT_EQUAL(_testParser("mysql://myhost/userdb"),
			URIParserTest_Results("mysql://myhost/userdb", "mysql", "", "myhost", 0, "userdb"));

	CPPUNIT_ASSERT_EQUAL(_testParser("mysql://myhost:1105/userdb"),
			URIParserTest_Results("mysql://myhost:1105/userdb", "mysql", "", "myhost", 1105, "userdb"));

	CPPUNIT_ASSERT_EQUAL(_testParser("xml_file:mylocalFile.xml"),
			URIParserTest_Results("xml_file:mylocalFile.xml", "xml", "file", "", 0, "mylocalFile.xml"));

	CPPUNIT_ASSERT_EQUAL(_testParser("xml_file:/absolutePath.xml"),
			URIParserTest_Results("xml_file:/absolutePath.xml", "xml", "file", "", 0, "/absolutePath.xml"));

	CPPUNIT_ASSERT_EQUAL(_testParser("xml_file:///absolutePath.xml"),
			URIParserTest_Results("xml_file:///absolutePath.xml", "xml", "file", "", 0, "/absolutePath.xml"));

	CPPUNIT_ASSERT_EQUAL(_testParser("xml_http://myhost/absolutePath.xml"),
			URIParserTest_Results("xml_http://myhost/absolutePath.xml", "xml", "http", "myhost", 0, "/absolutePath.xml"));
}

UriParserTest::URIParserTest_Results UriParserTest::_testParser(const std::string& uri)
{
	coral::URIParser parser;
	parser.setURI(uri);

	return URIParserTest_Results(
		parser.getURI(),
		parser.technology(),
		parser.protocol(),
		parser.hostName(),
		parser.portNumber(),
		parser.databaseOrSchemaName()
	);
}

} } // namespace coral, CoralCommon
