/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <iterator>
#include <boost/assign/list_of.hpp>

#include "CoralKernel/Context.h"
#include "CoralKernel/ILoadableComponent.h"
#include "ContextTest.h"

namespace coral { namespace CoralKernel {

void ContextTest::testLoadedComponentsAfterAddingNewPluginManager()
{
	Context& ctx = Context::instance();
	// Record the component names known by the default plugin manager
	std::set<std::string> knownComp(ctx.knownComponents());
	CPPUNIT_ASSERT(!knownComp.empty());
	// No plugin is loaded yet, so the list of known plugins must be empty
	CPPUNIT_ASSERT(ctx.loadedComponents().empty());
	// Load a plugin, by using the default plugin manager
	ctx.loadComponent(*ComponentNameFixture().begin());
	std::set<std::string> loadedComp(ctx.loadedComponents());
	// The component name must be known here, but only this
	CPPUNIT_ASSERT_EQUAL(size_t(1), loadedComp.size());
	CPPUNIT_ASSERT_EQUAL(*loadedComp.begin(), *ComponentNameFixture().begin());
	// Now load a component, by using a different plugin manager
	ctx.loadComponent(*(++(ComponentNameFixture().begin())), new TestPluginManager);
	loadedComp = ctx.loadedComponents();
	// Both the components name must be visible
	CPPUNIT_ASSERT(loadedComp == ComponentNameFixture());
	// Check if the known components did not altered
	CPPUNIT_ASSERT(knownComp == ctx.knownComponents());
}

const std::set<std::string>& ContextTest::ComponentNameFixture()
{
	// The first component is from PluginManager, the second is from TestPluginManager
	static const std::set<std::string> f = boost::assign::list_of
		("CORAL/RelationalPlugins/frontier")("TestComponent");
	return f;
}

std::set<std::string> ContextTest::TestPluginManager::knownPlugins() const
{
	// The first component is from PluginManager, the second is from TestPluginManager
	static const std::set<std::string> f = boost::assign::list_of
		(*(++(ComponentNameFixture().begin())));

	return f;
}

ILoadableComponent* ContextTest::TestPluginManager::newComponent(const std::string&)
{
	return new TestLoadableComponent;
}

ContextTest::TestLoadableComponent::TestLoadableComponent()
	: ILoadableComponent("Dummy")
{
	// EMPTY
}

} } // namespace coral::CoralKernel
