/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/bind.hpp>

#include "PropertyManagerTest.h"

namespace coral { namespace CoralKernel {

void PropertyManagerTest::setUp (void)
{
	pm = &coral::Context::instance().PropertyManager();
}

void PropertyManagerTest::testDefaults()
{
	coral::IProperty* p1 = pm->property("AuthenticationFile");
	CPPUNIT_ASSERT(p1);
	CPPUNIT_ASSERT_EQUAL(std::string("authentication.xml"), p1->get());
	p1 = pm->property("DBLookupFile");
	CPPUNIT_ASSERT(p1);
	CPPUNIT_ASSERT_EQUAL(std::string("dblookup.xml"), p1->get());
	// For the rest, check only the existence of the properties
	CPPUNIT_ASSERT(pm->property("Server_Hostname"));
	CPPUNIT_ASSERT(pm->property("Server_Port"));
}

void PropertyManagerTest::testPropertySetting()
{
	coral::IProperty* p1 = pm->property("AuthenticationFile");
	CPPUNIT_ASSERT(p1);
	CPPUNIT_ASSERT(p1->set("newValue"));
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), p1->get());
	// New search
	coral::IProperty* p2 = pm->property("AuthenticationFile");
	CPPUNIT_ASSERT(p2);
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), p2->get());
}

void PropertyManagerTest::testMissingPropertySetting()
{
	coral::IProperty* p1 = pm->property("SurelyMissingProperty");
	CPPUNIT_ASSERT(!p1);
}

void PropertyManagerTest::testCallback()
{
	coral::Property* p1 = dynamic_cast<coral::Property*>(pm->property("AuthenticationFile"));
	CPPUNIT_ASSERT(p1);
	coral::Property::CallbackID cid1 = -1, cid2 = -1;
	TestCallback obj1, obj2;
	boost::function1<void, std::string> cb1(boost::bind(&TestCallback::set, &obj1, _1));
	boost::function1<void, std::string> cb2(boost::bind(&TestCallback::set, &obj2, _1));
	cid1 = p1->registerCallback(cb1);
	CPPUNIT_ASSERT_EQUAL((coral::Property::CallbackID) 0, cid1);
	cid2 = p1->registerCallback(cb2);
	CPPUNIT_ASSERT_EQUAL((coral::Property::CallbackID) 1, cid2);
	CPPUNIT_ASSERT(p1->set("newValue"));
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), obj1.val);
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), obj2.val);
	CPPUNIT_ASSERT(p1->unregisterCallback(cid1));
	CPPUNIT_ASSERT(p1->set("newValue2"));
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), obj1.val);
	CPPUNIT_ASSERT_EQUAL(std::string("newValue2"), obj2.val);
	CPPUNIT_ASSERT(!p1->unregisterCallback(cid1));
	CPPUNIT_ASSERT(p1->unregisterCallback(cid2));
	CPPUNIT_ASSERT(p1->set("newValue2"));
	CPPUNIT_ASSERT_EQUAL(std::string("newValue"), obj1.val);
	CPPUNIT_ASSERT_EQUAL(std::string("newValue2"), obj2.val);
}

} } // namespace coral::CoralKernel
