/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CAL_PAR_NUM

#ifndef CORALMESSAGING_CAL_CALMESSAGEGENERATOR_H_
#define CORALMESSAGING_CAL_CALMESSAGEGENERATOR_H_

#include "CoralProtocol/Opcode.h"

#include "CoralMessaging/Insulation/TimeStamp.h"
#include "CoralMessaging/Utils/StatisticsCollector.h"
#include "CoralMessaging/Message/ClassGenerator.h"
#include "CoralMessaging/IMessage.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

#define CAL_MESSAGE_ENUM(classname) coral::CoralProtocol::Opcode::BOOST_PP_CAT(E_, classname)

#define CAL_MESSAGE_CLASSGENERATOR_0(CLASSNAME) \
	typedef gen::CLASSNAME::CLASSNAME<CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_1(CLASSNAME, TYPE1) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_2(CLASSNAME, TYPE1, TYPE2) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_3(CLASSNAME, TYPE1, TYPE2, TYPE3) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, TYPE3, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_4(CLASSNAME, TYPE1, TYPE2, TYPE3, TYPE4) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, TYPE3, TYPE4, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_5(CLASSNAME, TYPE1, TYPE2, TYPE3, TYPE4, TYPE5) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_6(CLASSNAME, TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, TYPE6) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, TYPE6, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME

#define CAL_MESSAGE_CLASSGENERATOR_7(CLASSNAME, TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, TYPE6, TYPE7) \
	typedef gen::CLASSNAME::CLASSNAME<TYPE1, TYPE2, TYPE3, TYPE4, TYPE5, TYPE6, TYPE7, CAL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME
} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /*#ifndef CORALMESSAGING_CAL_CALMESSAGEGENERATOR_H_ */

#else // #ifndef CAL_PAR_NUM

#include <boost/preprocessor/control/iif.hpp>
#include <boost/preprocessor/repetition.hpp>
#include <boost/preprocessor/punctuation/comma_if.hpp>

#define PAR_NUM CAL_PAR_NUM

#ifdef CAL_PAR_NAME0
  #define PAR_NAME0 CAL_PAR_NAME0
#endif

#ifdef CAL_PAR_NAME1
  #define PAR_NAME1 CAL_PAR_NAME1
#endif

#ifdef CAL_PAR_NAME2
  #define PAR_NAME2 CAL_PAR_NAME2
#endif

#ifdef CAL_PAR_NAME3
  #define PAR_NAME3 CAL_PAR_NAME3
#endif

#ifdef CAL_PAR_NAME4
  #define PAR_NAME4 CAL_PAR_NAME4
#endif

#ifdef CAL_PAR_NAME5
  #define PAR_NAME5 CAL_PAR_NAME5
#endif

#ifdef CAL_PAR_NAME6
  #define PAR_NAME6 CAL_PAR_NAME6
#endif

#ifndef CACHEABLE
  #define CACHEABLE 0
#endif

#define CLASSNAME BOOST_PP_CAT(CAL_CLASSNAME, _CAL)
#define CLASSNAME_STR CAL_CLASSNAME_STR
#include "CoralMessaging/Message/ClassGenerator.h"

namespace gen { namespace CAL_CLASSNAME {

#if CAL_PAR_NUM
#  define BASECLASS BOOST_PP_CAT(CAL_CLASSNAME, _CAL)::BOOST_PP_CAT(CAL_CLASSNAME, _CAL) < BOOST_PP_ENUM_PARAMS(CAL_PAR_NUM, T) >
#else
#  define BASECLASS BOOST_PP_CAT(CAL_CLASSNAME, _CAL)::BOOST_PP_CAT(CAL_CLASSNAME, _CAL)
#endif

#define TEMPLATE_EXPRESSION BOOST_PP_ENUM_PARAMS(CAL_PAR_NUM, class T) BOOST_PP_COMMA_IF(CAL_PAR_NUM) int OPCODE

template < TEMPLATE_EXPRESSION >
	class CAL_CLASSNAME : public BASECLASS, public CORALMESSAGING_NAMESPACE_NAME::IMessage
{
public:
	#define GENERATOR_BASE_INIT(z, n, unused) par##n
	#define GENERATOR_PARAM(z, n, unused) typename boost::call_traits<T##n>::param_type par##n = T##n()

	explicit CAL_CLASSNAME(BOOST_PP_ENUM(CAL_PAR_NUM, GENERATOR_PARAM, @) BOOST_PP_COMMA_IF(CAL_PAR_NUM) coral::CoralProtocol::t_endianness e = coral::CoralProtocol::_system_endian)
		: BASECLASS(BOOST_PP_ENUM(CAL_PAR_NUM, GENERATOR_BASE_INIT, @) BOOST_PP_COMMA_IF(CAL_PAR_NUM) e)
	{
		// EMPTY
	}

	#undef 	GENERATOR_BASE_INIT
	#undef 	GENERATOR_PARAM

	virtual coral::CoralProtocol::Opcode opcode() const
	{
		return coral::CoralProtocol::Opcode_gen<OPCODE>();
	}

	virtual bool cacheable() const
	{
		return CACHEABLE;
	}

	virtual bool write(CoralSocket& out) const
	{
		TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds();
		bool res = this->BASECLASS::write(&out);
		StatisticsCollector::Instance().addMessageWriteTime(*this, TimeStamp::now().total_nanoseconds() - start);

		if (res)
		{
			StatisticsCollector::Instance().addSentMessage(*this);
		}

		return res;
	}

	virtual const std::string& name() const
	{
		return BASECLASS::name();
	}
};

}} // namespace gen

#undef BASECLASS
#undef TEMPLATE_EXPRESSION

#undef CAL_PAR_NUM

#ifdef CAL_PAR_NAME0
#  undef CAL_PAR_NAME0
#endif

#ifdef CAL_PAR_NAME1
#  undef CAL_PAR_NAME1
#endif

#ifdef CAL_PAR_NAME2
#  undef CAL_PAR_NAME2
#endif

#ifdef CAL_PAR_NAME3
#  undef CAL_PAR_NAME3
#endif

#ifdef CAL_PAR_NAME4
#  undef CAL_PAR_NAME4
#endif

#ifdef CAL_PAR_NAME5
#  undef CAL_PAR_NAME5
#endif

#ifdef CAL_PAR_NAME6
#  undef CAL_PAR_NAME6
#endif

#undef CAL_CLASSNAME
#undef CAL_CLASSNAME_STR
#undef CAL_CLASSNAME_GEN
#undef CACHEABLE

#endif // #ifndef CAL_PAR_NUM

