/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_CALMESSAGES_H_
#define CORALMESSAGING_CAL_CALMESSAGES_H_

#include "MsgObj/QueryRepresentation.h"
#include "MsgObj/AttributeListRepresentation.h"
#include "CoralMessaging/CAL/CALMessageGenerator.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

// Connect
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Connect
#define CAL_CLASSNAME_STR "CAL::Connect"
#define CAL_PAR_NUM 2
#define CAL_PAR_NAME0 connection
#define CAL_PAR_NAME1 monitoring
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_2(Connect, MsgObj::Connection, MsgObj::MonitoringInformation);


// ConnectReply
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME ConnectReply
#define CAL_CLASSNAME_STR "CAL::ConnectReply"
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 serverVersion
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(ConnectReply, std::string);


// CloseSession
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME CloseSession
#define CAL_CLASSNAME_STR "CAL::CloseSession"
#define CAL_PAR_NUM 0
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_0(CloseSession);


// StartTransaction
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME StartTransaction
#define CAL_CLASSNAME_STR "CAL::StartTransaction"
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 access
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(StartTransaction, MsgObj::StartTransaction);


// CommitTransaction
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME CommitTransaction
#define CAL_CLASSNAME_STR "CAL::CommitTransaction"
#define CAL_PAR_NUM 0
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_0(CommitTransaction);


// ExecuteQuery
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME ExecuteQuery
#define CAL_CLASSNAME_STR "CAL::ExecuteQuery"
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 query
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(ExecuteQuery, MsgObj::QueryRepresentation);


// ExecuteQueryReply
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME ExecuteQueryReply
#define CAL_CLASSNAME_STR "CAL::ExecuteQueryReply"
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 rows
#define CACHEABLE 1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(ExecuteQueryReply, std::vector<MsgObj::AttributeListRepresentation>);


// OK
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME OK
#define CAL_PAR_NUM 0
#define CACHEABLE 1
#define CAL_CLASSNAME_STR "CAL::OK"
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_0(OK);


// Exception
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Exception
#define CAL_CLASSNAME_STR "CAL::Exception"
#define CAL_PAR_NUM 2
#define CAL_PAR_NAME0 id
#define CAL_PAR_NAME1 data
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_2(Exception, uint32_t, std::string);


// TableSchemaExists
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME TableSchemaExists
#define CAL_CLASSNAME_STR "CAL::TableSchemaExists"
#define CACHEABLE 1
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 schemaName
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(TableSchemaExists, std::string);


// TableSchemaRead
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME TableSchemaRead
#define CAL_CLASSNAME_STR "CAL::TableSchemaRead"
#define CACHEABLE 1
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 schema
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(TableSchemaRead, MsgObj::TableSchemaRead);


// TableSchemaReadReply
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME TableSchemaReadReply
#define CAL_CLASSNAME_STR "CAL::TableSchemaReadReply"
#define CACHEABLE 1
#define CAL_PAR_NUM 7
#define CAL_PAR_NAME0 columnDescription
#define CAL_PAR_NAME1 indexDescription
#define CAL_PAR_NAME2 foreignKeyDescription
#define CAL_PAR_NAME3 uniqueConstraintDescription
#define CAL_PAR_NAME4 primaryKeyDescription
#define CAL_PAR_NAME5 type
#define CAL_PAR_NAME6 tableSpaceName
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_7(TableSchemaReadReply, std::vector<MsgObj::ColumnDescription>, \
		std::vector<MsgObj::IndexDescription>, std::vector<MsgObj::ForeignKeyDescription>, \
		std::vector<MsgObj::UniqueConstraintDescription>, MsgObj::PrimaryKeyDescription, \
		std::string, std::string);


// ListTables
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME ListTables
#define CAL_CLASSNAME_STR "CAL::ListTables"
#define CACHEABLE 0
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 schemaName
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(ListTables, std::string);

// ListTablesReply
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME ListTablesReply
#define CAL_CLASSNAME_STR "CAL::ListTablesReply"
#define CACHEABLE 0
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 tableList
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(ListTablesReply, std::vector<std::string>);

// Ping0
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Ping0
#define CAL_CLASSNAME_STR "CAL::Ping0"
#define CACHEABLE 0
#define CAL_PAR_NUM 0
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_0(Ping0);

// Ping1
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Ping1
#define CAL_CLASSNAME_STR "CAL::Ping1"
#define CACHEABLE 0
#define CAL_PAR_NUM 1
#define CAL_PAR_NAME0 data1
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_1(Ping1, std::string);

// Ping2
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Ping2
#define CAL_CLASSNAME_STR "CAL::Ping2"
#define CACHEABLE 0
#define CAL_PAR_NUM 2
#define CAL_PAR_NAME0 data1
#define CAL_PAR_NAME1 data2
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_2(Ping2, std::string, std::string);


// Ping3
// -----------------------------------------------------------------------------
#define CAL_CLASSNAME Ping3
#define CAL_CLASSNAME_STR "CAL::Ping3"
#define CACHEABLE 0
#define CAL_PAR_NUM 3
#define CAL_PAR_NAME0 data1
#define CAL_PAR_NAME1 data2
#define CAL_PAR_NAME2 data3
#include "CoralMessaging/CAL/CALMessageGenerator.h"
CAL_MESSAGE_CLASSGENERATOR_3(Ping3, std::string, std::string, std::string);

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* CORALMESSAGING_CAL_CALMESSAGES_H_ */
