/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_CORALEXCEPTIONIMPL_H_
#define CORALMESSAGING_CAL_CORALEXCEPTIONIMPL_H_

#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/CAL/CppType_ProtocolID_Map.h"

#include <map>

START_CORALMESSAGING_NAMESPACE

class IMessage;

namespace CAL {

/** Implementation of the CORAL exception messaging protocol. */
struct CoralExceptionImpl
{
public:
	/** Check if the given message conveys a CORAL exception. If so, reproduce the exception
	 * and throw it. */
	static void checkAndThrow(const IMessage& msg);

	/** Map the CORAL exception identifier to the CORAL exception type. */
	static uint32_t getID(const std::type_info& ti);

private:
	/** Reproduce the CORAL exception from the Exception message and throw it. */
	template<int ID>
	static void _createAndThrow(const CORALMESSAGING_NAMESPACE_NAME::CAL::Exception&);

	struct _Map : public CppType_ProtocolID_Map
	{
		_Map();
	};

	static _Map _map;
};

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* CORALMESSAGING_CAL_CORALEXCEPTIONIMPL_H_ */
