/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_CPPTYPE_PROTOCOLID_MAP_H
#define CORALMESSAGING_CAL_CPPTYPE_PROTOCOLID_MAP_H

#include "CoralMessaging/CAL/TypeInfoMap.h"
#include "CoralMessaging/Insulation/TimeStamp.h"
#include "CoralBase/Date.h"
#include "CoralBase/Blob.h"

#include <boost/mpl/pair.hpp>
#include <boost/mpl/int.hpp>
#include <typeinfo>
#include <map>

START_CORALMESSAGING_NAMESPACE

namespace CAL {

/** Map the some C++ types and the protocol ID-s. */
class CppType_ProtocolID_Map : public TypeInfoMap<uint32_t>
{
public:
	/// Inserter class: insert a typeinfo -> ID pair into the map */
	class Insert
	{
	public:
		Insert(CppType_ProtocolID_Map& m, const uint32_t start = 0)
			: _start(start), _map(m._map)
		{}

		template<class T> void operator ()(T)
		{
			_map[&typeid(typename T::second)] = typename T::first() + _start;
		}

	private:
		uint32_t _start;
		CppType_ProtocolID_Map::_MapType& _map;
	};
};

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_CAL_CPPTYPE_PROTOCOLID_MAP_H */
