/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_ATTRIBUTELISTREPRESENTATION_H
#define CORALMESSAGING_ATTRIBUTELISTREPRESENTATION_H

#include "CoralMessaging/base.h"

#include "CoralMessaging/Insulation/Attribute.h"
#include "CoralMessaging/Insulation/AttributeList.h"

#include "CoralMessaging/Pointers.h"
#include "CoralMessaging/Stream/DataStringStream.h"
#include "CoralMessaging/CAL/TypeInfoMap.h"
#include "CoralMessaging/CAL/MapBase.h"
#include "IMessageObjectImpl.h"
#include "Objects.h"

#include <map>
#include <typeinfo>
#include <string>

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

class AttributeListRepresentation : public IMessageObjectImpl<AttributeListRepresentation>
{
public:
	AttributeListRepresentation();

	explicit AttributeListRepresentation(const AttributeList&, const bool isShare = false);

	AttributeListRepresentation(const AttributeListRepresentation&);

	// -------- From IMessageObject --------------------------------------------
	virtual bool decode(const std::string& raw);

	virtual EncodedType encode() const;

	virtual bool operator < (const IMessageObject& x) const;

	virtual std::ostream& log(std::ostream& ost) const;
	// -------------------------------------------------------------------------

	const AttributeList& attributeList() const;

	AttributeListRepresentation& operator = (const AttributeListRepresentation&);

	bool operator == (const AttributeListRepresentation& x) const;

private:
	typedef std::string (*_GetSpecificDataStringMethodType)(const ::coral::Attribute&);
	typedef Pointer<Stream::DataStringStream>::Shared _StrType;
	typedef void (::coral::AttributeList::*_ListExtenderMethodType)(const std::string&);
	typedef void (*_DataDecoderMethodType)(Attribute&, const std::string&);

	struct _GetDataStringTypeMap : public TypeInfoMap<_GetSpecificDataStringMethodType>
	{
		_GetDataStringTypeMap();
	};

	struct _ListExtenders : public MapBase<uint32_t, _ListExtenderMethodType>
	{
		_ListExtenders();
	};

	struct _DataDecoderMap : public MapBase<uint32_t, _DataDecoderMethodType>
	{
		_DataDecoderMap();
	};

	const AttributeRepresentation& _encodeAttribute(const coral::Attribute&) const;
	void _share(const AttributeList& attList);

	template<typename T> static std::string _getSpecificDataString(const coral::Attribute&);

	template<typename T> static void _decodeData(Attribute&, const std::string&);

	static _GetDataStringTypeMap _getDataStringTypeMap;
	static _ListExtenders _listExtenders;
	static _DataDecoderMap _dataDecoderMap;

	Pointer<AttributeList>::Scoped _attList;
	bool _isShare;
};

} // namespace MsgObj

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_ATTRIBUTELISTREPRESENTATION_H */
