/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_MSGOBJ_OBJECTS_H_
#define CORALMESSAGING_CAL_MSGOBJ_OBJECTS_H_

#include "ObjectGenerator.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

// Connection
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME Connection
#define MSGOBJ_CLASSNAME_STR "MsgObj::Connection"
#define MSGOBJ_PAR_NUM 4
#define MSGOBJ_PAR_NAME0 accessMode
#define MSGOBJ_PAR_NAME1 dbUrl
#define MSGOBJ_PAR_NAME2 username
#define MSGOBJ_PAR_NAME3 password
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_4(Connection, bool, std::string, std::string, std::string);


// MonitoringInformation
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME MonitoringInformation
#define MSGOBJ_CLASSNAME_STR "MsgObj::MonitoringInformation"
#define MSGOBJ_PAR_NUM 2
#define MSGOBJ_PAR_NAME0 pid
#define MSGOBJ_PAR_NAME1 cmdline
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_2(MonitoringInformation, uint32_t, std::string);


// TimeStamp
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME TimeStamp
#define MSGOBJ_CLASSNAME_STR "MsgObj::TimeStamp"
#define MSGOBJ_PAR_NUM 7
#define MSGOBJ_PAR_NAME0 year
#define MSGOBJ_PAR_NAME1 month
#define MSGOBJ_PAR_NAME2 day
#define MSGOBJ_PAR_NAME3 hour
#define MSGOBJ_PAR_NAME4 minute
#define MSGOBJ_PAR_NAME5 second
#define MSGOBJ_PAR_NAME6 nanosecond
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_7(TimeStamp, uint16_t, uint8_t, uint8_t, uint8_t, uint8_t, uint8_t, uint64_t);


// Date
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME Date
#define MSGOBJ_CLASSNAME_STR "MsgObj::Date"
#define MSGOBJ_PAR_NUM 3
#define MSGOBJ_PAR_NAME0 year
#define MSGOBJ_PAR_NAME1 month
#define MSGOBJ_PAR_NAME2 day
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_3(Date, uint16_t, uint8_t, uint8_t);


// ColumnDescription
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME ColumnDescription
#define MSGOBJ_CLASSNAME_STR "MsgObj::ColumnDescription"
#define MSGOBJ_PAR_NUM 6
#define MSGOBJ_PAR_NAME0 columnName
#define MSGOBJ_PAR_NAME1 cppType
#define MSGOBJ_PAR_NAME2 length
#define MSGOBJ_PAR_NAME3 fixedSize
#define MSGOBJ_PAR_NAME4 notNull
#define MSGOBJ_PAR_NAME5 tableSpaceName
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_6(ColumnDescription, std::string, std::string, uint64_t, bool, bool, std::string);


// IndexDescription
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME IndexDescription
#define MSGOBJ_CLASSNAME_STR "MsgObj::IndexDescription"
#define MSGOBJ_PAR_NUM 4
#define MSGOBJ_PAR_NAME0 indexName
#define MSGOBJ_PAR_NAME1 columnNames
#define MSGOBJ_PAR_NAME2 isUnique
#define MSGOBJ_PAR_NAME3 tableSpaceName
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_4(IndexDescription, std::string, std::vector<std::string>, bool, std::string);


// ForeignKeyDescription
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME ForeignKeyDescription
#define MSGOBJ_CLASSNAME_STR "MsgObj::ForeignKeyDescription"
#define MSGOBJ_PAR_NUM 4
#define MSGOBJ_PAR_NAME0 keyName
#define MSGOBJ_PAR_NAME1 columnNames
#define MSGOBJ_PAR_NAME2 referencedTableName
#define MSGOBJ_PAR_NAME3 referencedColumnNames
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_4(ForeignKeyDescription, std::string, std::vector<std::string>, std::string, std::vector<std::string>);


// PrimaryKeyDescription
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME PrimaryKeyDescription
#define MSGOBJ_CLASSNAME_STR "MsgObj::PrimaryKeyDescription"
#define MSGOBJ_PAR_NUM 2
#define MSGOBJ_PAR_NAME0 columnNames
#define MSGOBJ_PAR_NAME1 tableSpaceName
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_2(PrimaryKeyDescription, std::vector<std::string>, std::string);


// UniqueConstraintDescription
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME UniqueConstraintDescription
#define MSGOBJ_CLASSNAME_STR "MsgObj::UniqueConstraintDescription"
#define MSGOBJ_PAR_NUM 3
#define MSGOBJ_PAR_NAME0 constraintName
#define MSGOBJ_PAR_NAME1 columnNames
#define MSGOBJ_PAR_NAME2 tableSpaceName
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_3(UniqueConstraintDescription, std::string, std::vector<std::string>, std::string);


// AttributeRepresentation
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME AttributeRepresentation
#define MSGOBJ_CLASSNAME_STR "MsgObj::AttributeRepresentation"
#define MSGOBJ_PAR_NUM 4
#define MSGOBJ_PAR_NAME0 attributeName
#define MSGOBJ_PAR_NAME1 type
#define MSGOBJ_PAR_NAME2 data
#define MSGOBJ_PAR_NAME3 isNull
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_4(AttributeRepresentation, std::string, uint32_t, std::string, bool);


// LongDoubleRepresentation - follows IEEE 854 standars
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME LongDoubleRepresentation
#define MSGOBJ_CLASSNAME_STR "MsgObj::LongDoubleRepresentation"
#define MSGOBJ_PAR_NUM 2
#define MSGOBJ_PAR_NAME0 part64
#define MSGOBJ_PAR_NAME1 part32
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_2(LongDoubleRepresentation, uint64_t, uint32_t);


// StartTransaction
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME StartTransaction
#define MSGOBJ_CLASSNAME_STR "MsgObj::StartTransaction"
#define MSGOBJ_PAR_NUM 1
#define MSGOBJ_PAR_NAME0 access
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_1(StartTransaction, bool);


// TableSchemaRead
// -----------------------------------------------------------------------------
#define MSGOBJ_CLASSNAME TableSchemaRead
#define MSGOBJ_CLASSNAME_STR "MsgObj::TableSchemaRead"
#define MSGOBJ_PAR_NUM 2
#define MSGOBJ_PAR_NAME0 schemaName
#define MSGOBJ_PAR_NAME1 tableName
#include "ObjectGenerator.h"
MSGOBJ_MESSAGE_CLASSGENERATOR_2(TableSchemaRead, std::string, std::string);

} // namespace CAL

} // namespace MsgObj

END_CORALMESSAGING_NAMESPACE

#endif /* CORALMESSAGING_CAL_MSGOBJ_OBJECTS_H_ */
