/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_MSGOBJ_OPCODE_H_
#define CORALMESSAGING_CAL_MSGOBJ_OPCODE_H_

#include "CoralMessaging/base.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

/** Represents an internal message opcode. By using this, we can implement polimorphic messges. */
template<int MAX>
class Opcode
{
public:	
	enum {E_Invalid = MAX + 1};
	
	/// Constructor
	explicit Opcode(const uint8_t opcode = E_Invalid) : _opcode(opcode) {}
	
	/// Check if the stored value represents a valid opcode. 
	///
	/// This method is called when we construct the message from the stream, and we need to check
	/// if the opcode in the message is valid or not.
	bool valid() const {return _opcode <= MAX;}
	
	/// Convert the opcode into a byte.
	operator uint8_t () const {return _opcode;}
	
private:
	uint8_t _opcode; /**< Value of the opcode (may be invalid...) */
};

/// Opcode class generator. The code is given by the template parameter. The
/// function checks in compile time if the value is valid or not.
template <int OPCODE, int MAX> inline Opode Opcode_gen()
{
	BOOST_STATIC_ASSERT((OPCODE <= MAX));
	return Opcode<MAX>(OPCODE);
}

} // namespace MsgObj

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /*CORALMESSAGING_CAL_MSGOBJ_OPCODE_H_*/
