/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_PROTOCOLCORALTYPECONVERTER_H
#define CORALMESSAGING_PROTOCOLCORALTYPECONVERTER_H

#include "CoralMessaging/base.h"

#include <boost/mpl/map.hpp>
#include <boost/mpl/has_key.hpp>
#include <boost/mpl/if.hpp>
#include <boost/mpl/at.hpp>

#include "AttributeListRepresentation.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

namespace bm = boost::mpl;

class ProtocolCoralTypeConverter
{
private:
	template <typename FROM, typename TO>
	struct _BackConverter
	{
		static TO& convert(FROM& from) {return from;}
	};

	// The type map: C++ type -> message object type
	typedef bm::map <
		bm::pair<AttributeListRepresentation, _BackConverter<AttributeListRepresentation, AttributeList> >::type
	> _BackConverterMap;

public:
	// The message object selector metafunction
	template <typename TYPE> struct GetTypeConverter
	{
		// Try to find the type in the map
		typedef typename bm::has_key<_BackConverterMap, TYPE>::type FoundInMap;

		// If the element found...
		typedef typename bm::if_<FoundInMap,
			// return the corresponding message object type
			typename bm::at<_BackConverterMap, TYPE>::type,
			// else return the type itself
			_BackConverter<TYPE, TYPE>
		>::type type;
	};
};

template<>
struct ProtocolCoralTypeConverter::_BackConverter<AttributeListRepresentation, AttributeList>
{
	static AttributeList& convert(AttributeListRepresentation& from) {return const_cast<AttributeList&>(from.attributeList());}
};

} // namespace MsgObj

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_PROTOCOLCORALTYPECONVERTER_H */
