/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_QUERYREPRESENTATION_H
#define CORALMESSAGING_QUERYREPRESENTATION_H

#include <boost/ptr_container/ptr_vector.hpp>
#include <boost/preprocessor/seq/for_each.hpp>
#include <boost/preprocessor/seq/size.hpp>
#include <boost/preprocessor/debug/assert.hpp>
#include <boost/preprocessor/comparison/less_equal.hpp>
#include <boost/preprocessor/arithmetic/add.hpp>

#include "RelationalAccess/IQuery.h"

#include "CoralMessaging/CAL/MapBase.h"
#include "CoralMessaging/Pointers.h"
#include "IMessageObjectImpl.h"
#include "QueryRepresentationTypes.h"
#include "AttributeListRepresentation.h"

#include <map>

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

class QueryRepresentation : public IMessageObjectImpl<QueryRepresentation>
{
public:
	QueryRepresentation();

	QueryRepresentation(const QueryRepresentation& x);

	virtual ~QueryRepresentation();

	QueryRepresentation& operator = (const QueryRepresentation&);

	// -------- From IMessageObject --------------------------------------------
	virtual bool decode(const std::string& raw);

	virtual IMessageObject::EncodedType encode() const;

	virtual bool operator < (const IMessageObject& x) const;

	virtual std::ostream& log(std::ostream& ost) const;
	// -------------------------------------------------------------------------

	void setSchemaName(const std::string&);

	const std::string& schemaName() const;

public:
	#define REPLACE_METHOD_CALL_GEN(r, data, ELEM) \
		void replaceMethodCall(const QueryRepresentationTypes::ELEM& params) \
		{ \
            _MethodsType::iterator i = _methods.find(QueryRepresentationTypes::BOOST_PP_CAT(E_, ELEM)); \
            if (i == _methods.end()) \
                i = _methods.insert(_MapElementType(QueryRepresentationTypes::BOOST_PP_CAT(E_, ELEM), \
                                                    _MethodsType::mapped_type(new _MethodCallsType))).first; \
			_MethodCallsType& v = *(i->second); \
			if (!v.empty()) v.clear(); \
			v.push_back(static_cast<IMessageObject*>(params.ptr)); \
		}

	BOOST_PP_SEQ_FOR_EACH(REPLACE_METHOD_CALL_GEN, ~, QRT_REPLACE_LABELS)
	#undef REPLACE_METHOD_CALL_GEN

	#define INSERT_METHOD_CALL_GEN(r, data, ELEM) \
		void insertMethodCall(const QueryRepresentationTypes::ELEM& params) \
		{ \
             _MethodsType::iterator i = _methods.find(QueryRepresentationTypes::BOOST_PP_CAT(E_, ELEM)); \
             if (i == _methods.end()) \
                 i = _methods.insert(_MapElementType(QueryRepresentationTypes::BOOST_PP_CAT(E_, ELEM), \
                                                     _MethodsType::mapped_type(new _MethodCallsType))).first; \
  		     i->second->push_back(static_cast<IMessageObject*>(params.ptr)); \
		}

	BOOST_PP_SEQ_FOR_EACH(INSERT_METHOD_CALL_GEN, ~, QRT_INSERT_LABELS)
	#undef INSERT_METHOD_CALL_GEN

	void replay(IQuery&);
	void replay(IQueryDefinition&);

	bool operator == (const QueryRepresentation&) const;

private:
	uint32_t _generateBitmap() const;

	typedef Pointer<Stream::DataStringStream>::Shared _StrType;

	struct _CloneAllocator
	{
		static IMessageObject* allocate_clone(const IMessageObject& a);
		static void deallocate_clone(const IMessageObject* a);
	};

	typedef boost::ptr_vector<IMessageObject, _CloneAllocator> _MethodCallsType;
	typedef std::pair<int, Pointer<_MethodCallsType>::Shared> _MapElementType;
	typedef std::map<_MapElementType::first_type, _MapElementType::second_type> _MethodsType;

	struct _MethodStringMap : public MapBase<int, std::string>
	{
		_MethodStringMap();
	};

	struct _MethodsElementComparator : std::binary_function<_MapElementType, _MapElementType, bool>
	{
		bool operator() (const _MapElementType& lhs, const _MapElementType& rhs) const;
	};

	static _MethodStringMap _methodStringMap;

	_MethodsType _methods;
	std::string _schemaName;
};

} // namespace MsgObj

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_QUERYREPRESENTATION_H */
