/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_QUERYREPRESENTATIONTYPES_H
#define CORALMESSAGING_QUERYREPRESENTATIONTYPES_H

#include "CoralMessaging/base.h"

#include <boost/preprocessor/seq/for_each_i.hpp>
#include <boost/preprocessor/seq/size.hpp>
#include <boost/preprocessor/debug/assert.hpp>
#include <boost/preprocessor/comparison/less_equal.hpp>
#include <boost/preprocessor/arithmetic/dec.hpp>
#include <boost/preprocessor/comparison/less.hpp>
#include <boost/mpl/map.hpp>
#include <boost/mpl/has_key.hpp>
#include <boost/mpl/if.hpp>
#include <boost/mpl/at.hpp>

#include "ParameterCodecs.h"

START_CORALMESSAGING_NAMESPACE

namespace CAL {

namespace MsgObj {

namespace bm = boost::mpl;

class QueryRepresentation;
class AttributeListRepresentation;

class QueryRepresentationTypes
{
private:
	template<class TYPE>
	struct _TypeBase
	{
		typedef TYPE type;
		TYPE* ptr;
	};

public:
	struct setDistinct : public _TypeBase<Parameter_0>  {};

	struct addToOutputList : public _TypeBase<Parameter_2<std::string, std::string> > {};

	struct addToTableList : public _TypeBase<Parameter_2<std::string, std::string> > {};

	struct defineSubQuery : public _TypeBase<Parameter_2<std::string, QueryRepresentation> > {};

	struct setCondition : public _TypeBase<Parameter_2<std::string, AttributeListRepresentation> > {};

	struct groupBy : public _TypeBase<Parameter_1<std::string> > {};

	struct addToOrderList : public _TypeBase<Parameter_1<std::string> > {};

	struct limitReturnedRows : public _TypeBase<Parameter_2<uint32_t, uint32_t> > {};

	struct applySetOperation : public _TypeBase<Parameter_2<uint8_t, QueryRepresentation> > {};

	struct setForUpdate : public _TypeBase<Parameter_0> {};

	struct setRowCacheSize : public _TypeBase<Parameter_1<uint32_t> > {};

	struct setMemoryCacheSize : public _TypeBase<Parameter_1<uint32_t> > {};

	struct defineOutputType : public _TypeBase<Parameter_2<std::string, std::string> > {};

	struct defineOutput : public _TypeBase<Parameter_1<AttributeListRepresentation> > {};

	#define QRT_REPLACE_LABELS (setDistinct)(setCondition)(groupBy)(limitReturnedRows)(defineOutput)\
							   (applySetOperation)(setForUpdate)(setRowCacheSize)(setMemoryCacheSize)

	#define QRT_INSERT_LABELS (addToOutputList)(addToTableList)(defineSubQuery)(addToOrderList)(defineOutputType)

	#define QRT_ALL_LABELS     QRT_REPLACE_LABELS QRT_INSERT_LABELS

	#define QRT_QUERYDEFINITION_LABELS (setDistinct)(addToOutputList)(addToTableList)(defineSubQuery)\
	                                   (setCondition)(groupBy)(addToOrderList)(limitReturnedRows)(applySetOperation)

	static uint32_t MethodNumber()
	{
		#define METHOD_NUM BOOST_PP_SEQ_SIZE(QRT_ALL_LABELS)
		BOOST_PP_ASSERT_MSG(BOOST_PP_LESS_EQUAL(METHOD_NUM, 32), "Too much methods, increase the bitmap size");
		return METHOD_NUM;
		#undef METHOD_NUM
	}


	#define ENUM_GEN(r, data, i, elem) BOOST_PP_CAT(E_, elem) = i BOOST_PP_COMMA_IF(BOOST_PP_LESS(i, data))

	enum {
		BOOST_PP_SEQ_FOR_EACH_I(ENUM_GEN, BOOST_PP_DEC(BOOST_PP_SEQ_SIZE(QRT_ALL_LABELS)), QRT_ALL_LABELS)
	} Method_ID;

	#undef ENUM_GEN
};

} // namespace MsgObj

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_QUERYREPRESENTATIONTYPES_H */
