/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_CAL_TYPEINFOMAP_H
#define CORALMESSAGING_CAL_TYPEINFOMAP_H

#include "CoralMessaging/base.h"

#include <boost/mpl/pair.hpp>
#include <boost/mpl/int.hpp>
#include <typeinfo>
#include <map>

START_CORALMESSAGING_NAMESPACE

namespace CAL {

template <int ID, class T>
struct ElemGen
{
	typedef typename boost::mpl::pair<typename boost::mpl::int_<ID>, T> type;
};

template <typename VALUE>
class TypeInfoMap
{
public:
	VALUE operator () (const std::type_info& ti) const
	{
		typename _MapType::const_iterator i = _map.find(&ti);
		DEBUG_ASSERT(i != _map.end());
		return i->second;
	}

private:
	struct _TypeInfoComp : std::binary_function<std::type_info*, std::type_info*, bool>
	{
		bool operator() (const std::type_info* lhs, const std::type_info* rhs) const
		{
			return lhs->before(*rhs);
		}
	};

protected:
	typedef std::map<const std::type_info*, VALUE, _TypeInfoComp> _MapType;
	_MapType _map;
};

} // namespace CAL

END_CORALMESSAGING_NAMESPACE

#endif /* #ifndef CORALMESSAGING_CAL_TYPEINFOMAP_H*/
