/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CTL_PAR_NUM

#ifndef CTLMESSAGEGENERATOR_H_
#define CTLMESSAGEGENERATOR_H_

#include <boost/preprocessor/seq/for_each_i.hpp>

#include "CoralMessaging/CAL/CALMessageGenerator.h"
#include "CoralMessaging/Message/FixedValue.h"
#include "CTLErrorCode.h"

START_CORALMESSAGING_NAMESPACE

namespace CTL {

#define CTL_MESSAGE_ENUM(classname) ErrorCode::BOOST_PP_CAT(E_, classname)

#define CTL_MESSAGE_CLASSGENERATOR_0(CLASSNAME) \
	typedef gen::CLASSNAME::CLASSNAME##_CTL<CTL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME 

#define CTL_MESSAGE_CLASSGENERATOR_1(CLASSNAME, TYPE1) \
	typedef gen::CLASSNAME::CLASSNAME##_CTL<TYPE1, CTL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME 

#define CTL_MESSAGE_CLASSGENERATOR_2(CLASSNAME, TYPE1, TYPE2) \
	typedef gen::CLASSNAME::CLASSNAME##_CTL<TYPE1, TYPE2, CTL_MESSAGE_ENUM(CLASSNAME)> CLASSNAME 
		  
} // namespace CTL

END_CORALMESSAGING_NAMESPACE

#endif // #ifndef CTLMESSAGEGENERATOR_H_

#else // #ifndef CTL_PAR_NUM

#include <boost/preprocessor/arithmetic/inc.hpp>
#include <boost/preprocessor/cat.hpp>

#define CAL_PAR_NUM BOOST_PP_INC(CTL_PAR_NUM)
#define CAL_PAR_NAME0 errorCode

#ifdef CTL_PAR_NAME0
  #define CAL_PAR_NAME1 CTL_PAR_NAME0
#endif
	  	
#ifdef CTL_PAR_NAME1
  #define CAL_PAR_NAME2 CTL_PAR_NAME1
#endif  
	  		  
#define CAL_CLASSNAME BOOST_PP_CAT(CTL_CLASSNAME, _CAL)
#define CAL_CLASSNAME_STR CTL_CLASSNAME_STR
#include "CoralMessaging/CAL/CALMessageGenerator.h"	  
#define CTL_CLASSNAME_GEN BOOST_PP_CAT(CTL_CLASSNAME, _CTL) 

namespace gen { namespace CTL_CLASSNAME {

using CORALMESSAGING_NAMESPACE_NAME::CTL::ErrorCode;

#define BASECLASS BOOST_PP_CAT(CTL_CLASSNAME, _CAL)::BOOST_PP_CAT(CTL_CLASSNAME, _CAL) < \
	CORALMESSAGING_NAMESPACE_NAME::Message::FixedByte<ERRCODE> BOOST_PP_COMMA_IF(CTL_PAR_NUM) \
	BOOST_PP_ENUM_PARAMS(CTL_PAR_NUM, T), \
	coral::CoralProtocol::Opcode::E_CTL>

template<BOOST_PP_ENUM_PARAMS(CTL_PAR_NUM, class T) BOOST_PP_COMMA_IF(CTL_PAR_NUM) int ERRCODE> class CTL_CLASSNAME_GEN
	: public BASECLASS
{
public:
	#define GENERATOR_PARAM(z, n, unused) const T##n& par##n = T##n()
	#define GENERATOR_BASE_INIT(z, n, unused) par##n

	CTL_CLASSNAME_GEN(BOOST_PP_ENUM(CTL_PAR_NUM, GENERATOR_PARAM, @) BOOST_PP_COMMA_IF(CTL_PAR_NUM) coral::CoralProtocol::t_endianness e = coral::CoralProtocol::_system_endian)
	    : BASECLASS(CORALMESSAGING_NAMESPACE_NAME::Message::FixedByte<ERRCODE>() BOOST_PP_COMMA_IF(CTL_PAR_NUM) BOOST_PP_ENUM(CTL_PAR_NUM, GENERATOR_BASE_INIT, @), e)
	{}

	#undef 	GENERATOR_BASE_INIT
	#undef 	GENERATOR_PARAM
};
	
}} // namespace gen

#undef CTL_PAR_NUM

#ifdef CTL_PAR_NAME0
#  undef CTL_PAR_NAME0
#endif

#ifdef CTL_PAR_NAME1
#  undef CTL_PAR_NAME1
#endif
#undef CLASSNAME

#undef CTL_CLASSNAME
#undef CTL_CLASSNAME_GEN
#undef CTL_CLASSNAME_STR
#undef BASECLASS

#endif // #ifndef CTL_PAR_NUM
