/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CTLMESSAGES_H_
#define CTLMESSAGES_H_

#include "CoralProtocol/SequenceNumber.h"
#include "CTLMessageGenerator.h"

START_CORALMESSAGING_NAMESPACE

namespace CTL {

// OK
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME OK
#define CTL_CLASSNAME_STR "CTL::OK"
#define CTL_PAR_NUM 0
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_0(OK);

// VersionMismatch
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME VersionMismatch
#define CTL_CLASSNAME_STR "CTL::VersionMismatch"
#define CTL_PAR_NUM 2
#define CTL_PAR_NAME0 usedVersion
#define CTL_PAR_NAME1 description
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_2(VersionMismatch, uint8_t, std::string);

// WrongOpcode
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME WrongOpcode
#define CTL_CLASSNAME_STR "CTL::WrongOpcode"
#define CTL_PAR_NUM 0
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_0(WrongOpcode);

// SegmentationError
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME SegmentationError
#define CTL_CLASSNAME_STR "CTL::SegmentationError"
#define CTL_PAR_NUM 1
#define CTL_PAR_NAME0 lastValidSegment
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_1(SegmentationError, CoralProtocol::SequenceNumber);

struct SegmentationError_First : public SegmentationError
{
	SegmentationError_First() : SegmentationError(coral::CoralProtocol::FirstSequenceNumber()) {}
};

// PermissionDenied
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME NotConnected
#define CTL_CLASSNAME_STR "CTL::NotConnected"
#define CTL_PAR_NUM 1
#define CTL_PAR_NAME0 description
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_1(NotConnected, std::string);

// SyntaxError
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME SyntaxError
#define CTL_CLASSNAME_STR "CTL::SyntaxError"
#define CTL_PAR_NUM 1
#define CTL_PAR_NAME0 description
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_1(SyntaxError, std::string);

// OpcodeMismatch
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME OpcodeMismatch
#define CTL_CLASSNAME_STR "CTL::OpcodeMismatch"
#define CTL_PAR_NUM 1
#define CTL_PAR_NAME0 lastValidSegment
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_1(OpcodeMismatch, CoralProtocol::SequenceNumber);

// GeneralError
// -----------------------------------------------------------------------------
#define CTL_CLASSNAME GeneralError
#define CTL_CLASSNAME_STR "CTL::GeneralError"
#define CTL_PAR_NUM 1
#define CTL_PAR_NAME0 description
#include "CTLMessageGenerator.h"
CTL_MESSAGE_CLASSGENERATOR_1(GeneralError, std::string);

} // namespace CTL

END_CORALMESSAGING_NAMESPACE

#endif // CTLMESSAGES_H_
