/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef _CTLPACKET_H_
#define _CTLPACKET_H_

#include "CoralProtocol/PacketHeader.h"
#include "CoralMessaging/base.h"

START_CORALMESSAGING_NAMESPACE

class ICoralIOHandler;

namespace CTL {

/** Class representing a CORAL packet.
 * 
 * The user creates a packet by a ready packet header. The packet header cannot be changed 
 * during the lifetime the object. The payload is handled as a byte string, and can be 
 * manipulated freely. The length of the payload is equal to the lenght of that string.
 */
class Packet {
public:
	/// Constructor. The packet header is equal to the provided header, and it cannot be changed later.
	explicit Packet(const CoralProtocol::PacketHeader& header);
	
	/// Destructor
	virtual ~Packet();
   
	/// Write the header into a physical stream
	bool write(ICoralIOHandler& out) const; 
    
	/// Return the packet header (readonly access)
	const CoralProtocol::PacketHeader& header() const;
	
	/// Return the packet payload
	std::string& payload();
	
private:
	CoralProtocol::PacketHeader _header; /**< The packet header */
	std::string _pload; /**< The packet payload */
};

} // namespace CTL

END_CORALMESSAGING_NAMESPACE

#endif // #ifndef _PCTLACKET_H
