/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#ifndef CORALINPUTSOCKET_H_
#define CORALINPUTSOCKET_H_

#include "CoralProtocol/PacketHeader.h"
#include "CoralMessaging/Pointers.h"

START_CORALMESSAGING_NAMESPACE

class ICoralIOHandler;

/** class CoralInputSocket
 * 
 * Descriptor of the origin of a Coral message. It encapsulates:
 * 
 * - The stream where the message comes from
 * - The client ID
 * - The reques ID
 * 
 * The CoralInputSocket remains is assigned to the (physical line - client - request) triplet
 * until the object lifetime. 
 * 
 * The three pieces of information identifies unambigouosly the sender of the message
 * and the request itself.
 */
class CoralInputSocket 
{
public:	
	/// Default constructor
	CoralInputSocket() {}
	
	/// Constructor
	CoralInputSocket(Pointer<ICoralIOHandler>::Shared str, /**< The physical origin of the data */
			         uint32_t clientID, /**< The client ID of the sender */
			         uint32_t requestID /**< The request ID of the message */);
	
	/// Constructor
	CoralInputSocket(Pointer<ICoralIOHandler>::Shared str, /**< The physical origin of the data */
				     const CoralProtocol::PacketHeader& phead /** Packet header of the message to obtain the client ID and the request ID. */);

	/// Copy constructor
	CoralInputSocket(const CoralInputSocket& csock);
	
	/// Destructor
	virtual ~CoralInputSocket() {}
	
	/// Return the client ID of the assigned client
	uint32_t clientID() const {return _clientID;}
	
	/// Return the request ID if the assigned request
	uint32_t requestID() const {return _requestID;}
	
	/// Return the assigned communication endpoint
	Pointer<ICoralIOHandler>::Shared stream();
	
	/// Ordering operator
	bool operator < (const CoralInputSocket& x) const;
	
	/// Equality operatior
	bool operator == (const CoralInputSocket& x) const;
	
	/// Nonequality operator
	bool operator != (const CoralInputSocket& x) const;

private:
	Pointer<ICoralIOHandler>::Shared _stream; /**< The associates physical I/O line */
	uint32_t _clientID; /**< The associated client ID */
	uint32_t _requestID; /**< The associated request ID */
};

END_CORALMESSAGING_NAMESPACE

#endif /*CORALINPUTSOCKET_H_*/
