/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef IMESSAGE_H_
#define IMESSAGE_H_

#include "CoralProtocol/Opcode.h"
#include "CoralMessaging/base.h"
#include "CoralMessaging/Pointers.h"
#include "CoralMessaging/CoralSocket.h"

START_CORALMESSAGING_NAMESPACE

/** Interface for the messages of the CORAL protocol. */
class IMessage
{
public:
	/// Destructor
	virtual ~IMessage() {};

	/// Return the protocol opcode of the message
	virtual coral::CoralProtocol::Opcode opcode() const = 0;
	
	/// Return the name (textual representetion) of the message for logging.
	virtual const std::string& name() const = 0;
	
	/// Return true if the message is cacheable else false.
	virtual bool cacheable() const = 0;
	
	/// Send the message to the destination
	virtual bool write(CoralSocket& out) const = 0;
};

/// Cast the abstract IMessage pointer to the real message pointer
template <class MSG_TYPE> inline
MSG_TYPE* message_cast(IMessage* msg)
{
	return dynamic_cast<MSG_TYPE*>(msg);
}

/// Cast the abstract const IMessage pointer to the real message pointer
template <class MSG_TYPE> inline
const MSG_TYPE* message_cast(const IMessage* msg)
{
	return dynamic_cast<const MSG_TYPE*>(msg);
}

/// Cast the abstract IMessage reference to the real message reference
template <class MSG_TYPE> inline 
MSG_TYPE& message_cast(IMessage& msg)
{
	return dynamic_cast<MSG_TYPE&>(msg);
}

/// Cast the abstract const IMessage reference to the real message reference
template <class MSG_TYPE> inline 
const MSG_TYPE& message_cast(const IMessage& msg)
{
	return dynamic_cast<const MSG_TYPE&>(msg);
}

/// Cast the Boost shared pointers to the abstract IMessage to the real message pointer
template <class MSG_TYPE> inline 
typename Pointer<MSG_TYPE>::Shared message_cast(const Pointer<IMessage>::Shared& msg)
{
	return boost::dynamic_pointer_cast<MSG_TYPE>(msg);
}

END_CORALMESSAGING_NAMESPACE

#endif /* IMESSAGE_H_ */
