/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_INSULATION_THREADTRAITS_H_
#define CORALMESSAGING_INSULATION_THREADTRAITS_H_

#ifdef CORAL_SEALED

/**
 * Define the synchronization/threading strategy. Include the file that implements the
 * traits. The rest of the code should include only this file, and
 * use the ThreadTraits type as a template parameter, typedef, etc. 
 * 
 * The file implementing the strategy should check if the __THREADTRAITS_H_GUARD__
 * is defined or not. If not defined, it indicates that the file was included
 * from elsewhere than this file. It is normally an unwanted thing.
 *  
 * For example, see the BoostThreadTraits.h
 */

#ifdef __THREADTRAITS_H_GUARD__
  #error __THREADTRAITS_H_GUARD__ should not be defined here!
#endif

#define __THREADTRAITS_H_GUARD__
// include the file defining the threading strategy here. Including it 
// elsewhere should result in compilation error.
#include "BoostThreadTraits.h"
#undef __THREADTRAITS_H_GUARD__

#else // #ifdef CORAL_SEALED

#include "CoralBricks/ThreadTraits.h"

#endif // #ifdef CORAL_SEALED

#endif /*CORALMESSAGING_INSULATION_THREADTRAITS_H_*/
