/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef ARRAY_H_
#define ARRAY_H_

#include <string>
#include <ostream>

#include "CoralMessaging/base.h"

START_CORALMESSAGING_NAMESPACE

namespace Message { 

/** Messages are constructed of arrays. This class encapsulates such an array. */
class Array {
public:
	/// Constructor
	explicit Array(const std::string& data = "");
	
	/// Destructor
	virtual ~Array() {}
	
	/// Assignment operator
	Array& operator = (const Array& x);
	
	/// Equality operator
	bool operator == (const Array& x) const;
	
	/// Inequality operator
	bool operator != (const Array& x) const;
	
	/// Data size of the array
	const size_t size() const;
	
	/// The array data
	const std::string& data() const {return _data;}
	
	/// Append a C character array to the end of the data
	Array& append(const char* data, /**< the C array */
			      size_t size /**< Number of bytes to be appended */);
	
	/// Append a string to the end of the data
	Array& append(const std::string& data /**< The string to be appended */); 
	
private:
	std::string _data; /**< Internal store of the array */
};

/** Wtite an array into a stream for logging purposes */
std::ostream& operator << (std::ostream& ost, const Array& a);

} // namespace Message

END_CORALMESSAGING_NAMESPACE

#endif /*ARRAY_H_*/
