/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef FIXEDVALUE_H_
#define FIXEDVALUE_H_

#include "CoralProtocol/arch.h"

#include "CoralMessaging/base.h"

START_CORALMESSAGING_NAMESPACE

namespace Message { 

/** Class encapsulating a fixed value. 
 * 
 * The purpose: for example, an opcode must be a well-specified value, determined in 
 * compile time. This template class defines separate types for bytes, so wrong opcodes
 * can be determined in compile time in certain cases.
 */
template<uint8_t VALUE>
class FixedByte
{
public:
	/// Convert the template parameter to a byte
	operator const uint8_t () const {return VALUE;}
	
	/// Check if the method parameter equals to the template parameter. Return true is they are equal.
	bool isValid(uint8_t val) const {return val == VALUE;}
	
	/// Return the byte value of the template parameter.
	uint8_t value() const {return VALUE;}
};

} // namespace Message

END_CORALMESSAGING_NAMESPACE

#endif /*FIXEDVALUE_H_*/
