/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef POINTERS_H_
#define POINTERS_H_

#include <boost/shared_ptr.hpp>
#include <boost/scoped_array.hpp>
#include <boost/scoped_ptr.hpp>
#include <boost/shared_array.hpp>

#include "base.h"

START_CORALMESSAGING_NAMESPACE

/** Type collection encapsulating different pointer policies */
template <typename T>
struct Pointer {
	typedef boost::shared_ptr<T> Shared; /**< Shared pointer behaviour: reference counted, deleted if all the users released the pointer */
	typedef boost::scoped_ptr<T> Scoped; /**< Scoped pointer behaviour. The object is deleted when the pointer gets out of scope. */
	typedef boost::scoped_array<T> ScopedArray; /**< Shared pointer for arrays */
	typedef boost::shared_array<T> SharedArray; /**< Scoped pointer for arrays */
	typedef T* Raw; /**< Raw (not smart, C-like) pointer */
};

/** Definition if null pointers. See @see Pointer. */
template <typename T>
struct NullPointer {
	static typename Pointer<T>::Shared Shared() {return typename Pointer<T>::Shared();}
	static typename Pointer<T>::Scoped Scoped() {return typename Pointer<T>::Scoped();}
	static typename Pointer<T>::ScopedArray ScopedArray() {return typename Pointer<T>::ScopedArray();}
	static typename Pointer<T>::SharedArray SharedArray() {return typename Pointer<T>::SharedArray();}
	static typename Pointer<T>::Raw Raw() {return NULL;}
};

/// Retrun true if the parameter does not hold NULL
template<class T, template <class> class PtrPolicy> bool ptr_assert(const PtrPolicy<T>& p)
{
	return p.get() != NULL;
}

/// Retrun true if the parameter is not NULL
template<class PTR> bool ptr_assert(const PTR* p)
{
	return p != NULL;
}

/// Debug configuration assertion of the pointer is NULL
#define DEBUG_ASSERT_PTR(p) assert(ptr_assert((p)))

END_CORALMESSAGING_NAMESPACE

#endif /* POINTERS_H_ */
