/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_STREAM_DATASTRINGSTREAM_H_
#define CORALMESSAGING_STREAM_DATASTRINGSTREAM_H_

#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/ICoralIOHandler.h"

#include <sstream>

START_CORALMESSAGING_NAMESPACE

/** Stream implemented on string streams */
namespace Stream {

class DataStringStream : public ICoralIOHandler, public MonitorObject
{
public:
	/// Constructor
	explicit DataStringStream(const std::string& str = "");

	/// return the underlaying string stream
	const std::stringstream& str() const {return _str;}

	/// Clear the string buffer
	void reset();

	/// Output stream concept method. No functionality.
	bool flush(bool) {return true;}

	// From ICoralIOHandler
	virtual size_t read(char* buffer, size_t len);

	virtual bool readAll(char* buf, size_t len, const Timeout& to);

	virtual size_t write(const char* buf, size_t len);

	virtual bool writeAll(const char* buf, size_t len, const Timeout& to);

	virtual bool fail() const;

	virtual bool waitForData(const Timeout& to);

	virtual bool isClosed() const;

private:
	template<class OP, class BUF>
	bool _readwriteAll(BUF buf, size_t len, const Timeout& to, OP op);

	std::stringstream _str;
};

} // namespace Stream

END_CORALMESSAGING_NAMESPACE

#endif /*CORALMESSAGING_STREAM_DATASTRINGSTREAM_H_*/
