/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef TCPSOCKETFACTORY_H_
#define TCPSOCKETFACTORY_H_

#include <string>

#include "CoralMessaging/Pointers.h"
#include "TcpSocket.h"

START_CORALMESSAGING_NAMESPACE

namespace Tcp {

/** Create and open different TCP sockets.
 * 
 * The class provides 3 socket types: active, connecting and passive sockets. All the
 * sockets are wrapped into @see TcpSocket -s, and returned as shared pointers.
 */
class TcpSocketFactory 
{
public:
	/// Destructor
	virtual ~TcpSocketFactory() {};
	
	/// Create an active socket (for the server process) by accepting an incoming conenction request
	/// on a passive socket.
	static Pointer<TcpSocket>::Shared CreateActiveSocket(unsigned int fd /**< The passive (listening) socket */);
	
	/// Create a listening (passive) socket for a server process.
	static Pointer<TcpSocket>::Shared CreateListeningSocket(unsigned int port, /**< The port where the server listens */ 
			                                                const std::string& ip /**< The IP address of the server */);
	
	/// Create a connecting socket (for a client process).
	static Pointer<TcpSocket>::Shared CreateConnectingSocket(unsigned int port, /**< the port number of the server */
	                                                         const std::string& host /**< The host name of the server */);
private:
    static const unsigned int _BACKLOG = 10;
};

} // namespace Tcp

END_CORALMESSAGING_NAMESPACE

#endif /*TCPSOCKETFACTORY_H_*/
