/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef _ERROR_H_
#define _ERROR_H_

#include <string>
#include <exception>

#include "CoralMessaging/base.h"
#include "CoralMessaging/Utils/Logger.h"

#define EXCEPTION_THROW(aError) { \
	aError.log(__FILE__, __FUNCTION__, __LINE__); \
	throw aError; \
}

#define EXCEPTION_LOGERROR(aError) {aError.log(__FILE__, __FUNCTION__, __LINE__);}

START_CORALMESSAGING_NAMESPACE

class Err : public std::exception
{
public:
    typedef enum ErrorType {e_defaultReport,
                            e_detailedReport};
        
    void log(const char* aFile, const char* aFunc, const int aLineNo);
    virtual const char* what() const throw();

protected:
    virtual void _logSystemError() {};
    virtual std::string _description() const { return ""; };
};


template<bool IS_SYSTEM_ERROR, Err::ErrorType = Err::e_defaultReport>
class Error : virtual public Err
{
private:
    virtual const char* what() const throw() { 
        return Err::what(); 
    };

protected:
    virtual void _logSystemError() {};
};

// Errors
class Err_System : public Error<true>
{
public:
	Err_System(const std::string& userDesc = "System Error.") : _userDesc(userDesc) {};
	virtual ~Err_System() throw() {};
	
private:
    virtual std::string _description() const;
    
    std::string _userDesc;
};


class Err_Custom : public Error<false>
{
public:
    Err_Custom(const std::string& aDesc) : _desc(aDesc) {};
    virtual ~Err_Custom() throw() {};
    
private:
    virtual std::string _description() const; 

    std::string _desc;
};

END_CORALMESSAGING_NAMESPACE

#endif // _ERROR_H_
