/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_UTILS_LOGGER_H__
#define CORALMESSAGING_UTILS_LOGGER_H__

#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/Insulation/MessageStream.h"
#include "CoralMessaging/base.h"

START_CORALMESSAGING_NAMESPACE

class Logger : public coral::MonitorObject
{
	friend Logger& addErr(Logger&);

	friend Logger& commit(Logger&);
public:
	Logger();

	virtual ~Logger();

	Logger& setEnvironment(const std::string& appLabel);

	Logger& setLogOn();

	Logger& setLogOff();

	Logger& newLog(const coral::MsgLevel aLogLevel, const char* aFile,
                   const char* aFunc, const int aLineNo);

	template<class T> Logger& operator << (const T& aSrc);

	Logger& operator << (Logger& (*aFunc)(Logger &aLogger));

	static const std::string& separator();

private:
	void _commit();

private:
    bool                 _isLogOn;
    std::stringstream    _logLine;
    coral::MsgLevel      _actLogLevel;
    std::string          _appLabel;
};


// Modifiers
Logger& addErr(Logger& aLogger);
Logger& commit(Logger& aLogger);

template<class T>
Logger& Logger::operator << (const T& aSrc)
{
	if (_isLogOn)
		_logLine << aSrc;

	return *this;
}

inline Logger& theLogger()
{
  static Logger logger;
  return logger;
}

END_CORALMESSAGING_NAMESPACE

// Debug info logging
#define LOGGER_NEWLOG(aLevel)	\
	CORALMESSAGING_NAMESPACE_NAME::theLogger().newLog(aLevel, __FILE__, __FUNCTION__, __LINE__)

#endif /* #ifndef CORALMESSAGING_UTILS_LOGGER_H__ */
