/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_UTILS_RWLOCKABLE_H_
#define CORALMESSAGING_UTILS_RWLOCKABLE_H_

#include "CoralBricks/ThreadTraits.h"
#include "CoralMessaging/Pointers.h"

START_CORALMESSAGING_NAMESPACE

/** Provides scoped read/write lock. 
 * 
 * Actually, the lock is independent from the fact that they are used for read-write locking,
 * the names are given to express the development context of the locks. They are simple scoped
 * locks associated for a mutex pair.
 */
class RWLockable
{
public:
	/// Type of the system lock. It must conform the scoped lock concept.
	typedef Pointer<ThreadTraits::LOCK>::Shared LockType;
	
	/// Obtain a read lock. Busy wait until the lock can be obtained.
	LockType readLock() const {return LockType(new ThreadTraits::LOCK(_read_mutex));}
	
	/// Obtain a write lock. Busy wait until the lock can be obtained.
	LockType writeLock() const {return LockType(new ThreadTraits::LOCK(_write_mutex));}
		
private:
	mutable ThreadTraits::MUTEX _read_mutex; /**< Mutex for the read lock */
	mutable ThreadTraits::MUTEX _write_mutex; /**< Mutex for the write lock */
};

END_CORALMESSAGING_NAMESPACE

#endif /*CORALMESSAGING_UTILS_RWLOCKABLE_H_*/
