/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_STATISTICSCOLLECTOR_H_
#define CORALMESSAGING_STATISTICSCOLLECTOR_H_

#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/Insulation/TimeStamp.h"
#include "CoralMessaging/IMessage.h"

#include <map>

START_CORALMESSAGING_NAMESPACE

using coral::TimeStamp;

class StatisticsCollector : public coral::MonitorObject
{
public:
	StatisticsCollector();
	
	virtual ~StatisticsCollector();
	
	void incRecvPacketNumber()
	{
	MONITOR_START_CRITICAL
		++_recvPacketNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void incSentPacketNumber()
	{
	MONITOR_START_CRITICAL
		++_sentPacketNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void incDiscardedPacketNumber()
	{
	MONITOR_START_CRITICAL
		++_discPacketNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addRecvMessage(const IMessage& msg)
	{
	MONITOR_START_CRITICAL
		++_msgStat[msg.opcode()].recvMsgNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addSentMessage(const IMessage& msg)
	{
	MONITOR_START_CRITICAL
		++_msgStat[msg.opcode()].sentMsgNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addDiscardedMessage()
	{
	MONITOR_START_CRITICAL	
		++_discMsgNum;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addMessageEncodeTime(const IMessage& msg, TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_msgStat[msg.opcode()].encodeTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addMessageDecodeTime(const IMessage& msg, TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_msgStat[msg.opcode()].decodeTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addMessageHandlerTime(const IMessage& msg, TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_msgStat[msg.opcode()].handlerTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addMessageReadTime(const IMessage& msg, TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_msgStat[msg.opcode()].readTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addMessageWriteTime(const IMessage& msg, TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_msgStat[msg.opcode()].writeTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addStreamReadTime(TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_streamReadTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void addStreamWriteTime(TimeStamp::ValueType t)
	{
	MONITOR_START_CRITICAL	
		_streamWriteTime += t;
		++_synchNum;
	MONITOR_END_CRITICAL
	}
	
	void print() const;
	
	static StatisticsCollector& Instance()
	{
	MONITOR_START_STATIC_CRITICAL
		static StatisticsCollector s;
		return s;
	MONITOR_END_CRITICAL
	}
	
private:
	bool _lockOverhead(unsigned long& s) const
	{
	MONITOR_START_CRITICAL	
		return static_cast<bool>(s--);
	MONITOR_END_CRITICAL
	}
	
	struct _MsgStatData
	{
		_MsgStatData();
		
		void print() const;
		
		unsigned long sentMsgNum;
		unsigned long recvMsgNum;
		TimeStamp::ValueType encodeTime;
		TimeStamp::ValueType decodeTime;
		TimeStamp::ValueType readTime;
		TimeStamp::ValueType writeTime;
		TimeStamp::ValueType handlerTime;
	};
	
	typedef std::map<int, _MsgStatData> _MsgStatContainer;
	_MsgStatContainer _msgStat;
	
	unsigned long _recvPacketNum;
	unsigned long _sentPacketNum;
	unsigned long _discPacketNum;
	unsigned long _discMsgNum;
	TimeStamp::ValueType _streamReadTime;
	TimeStamp::ValueType _streamWriteTime;
	unsigned long _synchNum;
};

END_CORALMESSAGING_NAMESPACE

#endif /* CORALMESSAGING_STATISTICSCOLLECTOR_H_ */
