/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_UTILS_CTL_H_
#define CORALMESSAGING_UTILS_CTL_H_

#include "CoralMessaging/base.h"

#include <boost/type_traits/remove_pointer.hpp>
#include <boost/preprocessor/arithmetic/inc.hpp>
#include <boost/preprocessor/comparison/not_equal.hpp>
#include <boost/preprocessor/repetition/for.hpp>
#include <boost/preprocessor/tuple/elem.hpp>
#include <boost/foreach.hpp>

#include <iosfwd>
#include <iterator>
#include <vector>

template<class T> inline std::ostream& operator << (std::ostream& os, const std::vector<T>& x)
{
	std::copy(x.begin(), x.end(), std::ostream_iterator<const T>(os, "\n"));
	return os;
}

template<class T> inline std::istream& operator >> (std::istream& is, std::vector<T>& x)
{
	std::copy(std::istream_iterator<T>(is), std::istream_iterator<T>(), std::back_inserter(x));
	return is;
}

START_CORALMESSAGING_NAMESPACE

namespace ctl {

template <class Container>
struct _PtrContainerComparator : std::binary_function<typename Container::value_type, typename Container::value_type, bool>
{
	bool operator() (const typename Container::value_type lhs, const typename Container::value_type rhs) const
	{
		if (lhs == rhs) return true;
		return lhs && rhs ? *rhs == *lhs : false;
	}
};

} // namespace ctl

#define CORALMESSAGING_CTL_FOR_PRED(r, state) \
	BOOST_PP_NOT_EQUAL( \
		BOOST_PP_TUPLE_ELEM(2, 0, state), \
		BOOST_PP_INC(BOOST_PP_TUPLE_ELEM(2, 1, state)) \
	)

#define CORALMESSAGING_CTL_FOR_INC(r, state) \
( \
	BOOST_PP_INC(BOOST_PP_TUPLE_ELEM(2, 0, state)), \
	BOOST_PP_TUPLE_ELEM(2, 1, state) \
)

#define CORALMESSAGING_CTL_FOR(start, stop, MACRO) \
	BOOST_PP_FOR((start, stop), CORALMESSAGING_CTL_FOR_PRED, CORALMESSAGING_CTL_FOR_INC, MACRO);

END_CORALMESSAGING_NAMESPACE

#endif // CORALMESSAGING_UTILS_CTL_H_
