/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <iterator>

#include "CoralMessaging/Message/Array.h"

START_CORALMESSAGING_NAMESPACE

namespace Message { 

Array::Array(const std::string& data) 
	: _data(data) 
{
	// EMPTY
}	
Array& Array::operator = (const Array& x) 
{
	if (&x != this) _data = x._data;
	return *this;
}

bool Array::operator == (const Array& x) const
{
	return _data == x._data;
}

bool Array::operator != (const Array& x) const
{
	return !(*this == x);
}

const size_t Array::size() const 
{
	return _data.size();
}

Array& Array::append(const char* data, size_t size) 
{
	_data.append(data, size);
	return *this;
}

Array& Array::append(const std::string& data) 
{
	_data.append(data);
	return *this;
}

std::ostream& operator << (std::ostream& ost, const Array& a) 
{
	ost << "Size: " << a.size() << ", data: ";
	ost.width(2);
	ost.fill('0');
	ost << std::hex;
	std::copy(a.data().begin(), a.data().end(), std::ostream_iterator<unsigned>(ost, " "));
	return ost;
}

} // namespace Message

END_CORALMESSAGING_NAMESPACE
