/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralProtocol/PacketHeader.h"
#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/CoralSocket/CoralInputSocket.h"

START_CORALMESSAGING_NAMESPACE

CoralInputSocket::CoralInputSocket(Pointer<ICoralIOHandler>::Shared str, uint32_t clientID, uint32_t requestID) 
	: _stream(str), _clientID(clientID), _requestID(requestID) 
{
	// EMPTY
}

CoralInputSocket::CoralInputSocket(Pointer<ICoralIOHandler>::Shared str, const CoralProtocol::PacketHeader& phead) 
	: _stream(str), _clientID(phead.clientID()), _requestID(phead.requestID()) 
{
	// EMPTY
}

CoralInputSocket::CoralInputSocket(const CoralInputSocket& csock) 
	: _stream(csock._stream), _clientID(csock._clientID), _requestID(csock._requestID) 
{
	// EMPTY
}

Pointer<ICoralIOHandler>::Shared CoralInputSocket::stream() 
{
	return _stream;
}

bool CoralInputSocket::operator < (const CoralInputSocket& x) const 
{
	return _clientID < x._clientID || _requestID < x._requestID;
}

bool CoralInputSocket::operator == (const CoralInputSocket& x) const 
{
	return _clientID == x._clientID && _requestID == x._requestID;
}

bool CoralInputSocket::operator != (const CoralInputSocket& x) const 
{
	return !(*this == x);
}

END_CORALMESSAGING_NAMESPACE

