/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Utils/StatisticsCollector.h"
#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/IMessage.h"
#include "CoralMessaging/CoralSocket.h"

START_CORALMESSAGING_NAMESPACE

CoralSocket::CoralSocket(Pointer<ICoralIOHandler>::Shared str, const CoralProtocol::PacketHeader& phead, const IMessage& msg)
	: CoralInputSocket(str, phead), _opcode(msg.opcode()), _cacheable(msg.cacheable()), _fail(false)
{
	// EMPTY
}

CoralSocket::CoralSocket(Pointer<ICoralIOHandler>::Shared str, uint32_t clientID, uint32_t requestID, const IMessage& msg)
	: CoralInputSocket(str, clientID, requestID), _opcode(msg.opcode()), _cacheable(msg.cacheable()), _fail(false)
{
	// EMPTY
}

CoralSocket::CoralSocket(Pointer<ICoralIOHandler>::Shared str, uint32_t clientID, uint32_t requestID, uint8_t opcode, bool cacheable)
	: CoralInputSocket(str, clientID, requestID), _opcode(opcode), _cacheable(cacheable), _fail(false)
{
	// EMPTY
}

CoralSocket::CoralSocket(const CoralInputSocket& csock, const IMessage& msg)
	: CoralInputSocket(csock), _opcode(msg.opcode()), _cacheable(msg.cacheable()), _fail(false)
{
	// EMPTY
}

void CoralSocket::write(const char* data, size_t size)
{
	_data.append(data, size);
}

bool CoralSocket::flush(bool isFinal)
{
	if (_fail) return false;
	if (isFinal)
	{
		_seqNum.setFinal();
		LOGGER_NEWLOG(coral::Info) << "Message with opcode " << static_cast<int>(_opcode) << " sent. " << commit;
	}

	CoralProtocol::PacketHeader phead;
	phead.setCacheable(_cacheable);
	phead.setClientID(this->clientID());
	phead.setRequestID(this->requestID());
	phead.setOpcode(_opcode);
	phead.setPacketSize(_data.size());
	phead.setSequenceNum(_seqNum);
	// This will unlock the stream upon exiting the function
	DEBUG_ASSERT(this->stream().get())
	std::string str;
	str.append(phead.raw(), phead.size());
	str.append(_data.data(), _data.size());
	ICoralIOHandler& s = *this->stream();
	ICoralIOHandler::LockType lock = s.writeLock();
	InfiniteTimeout to;
	s.writeAll(str.data(), str.size(), to);
	_data.resize(0);
	if (!isFinal) ++_seqNum;

	DEBUG(StatisticsCollector::Instance().incSentPacketNumber());

	return !(_fail = s.fail());
}

END_CORALMESSAGING_NAMESPACE

