/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/Stream/DataStreamLogic.h"

START_CORALMESSAGING_NAMESPACE

namespace Stream {

DataStreamLogic::DataStreamLogic(std::string& buf, DataStreamLogic::DataStream& str) 
	: _buf(buf), _cursor(0), _str(str) 
{
	// EMPTY
}

size_t DataStreamLogic::size() const 
{ 
	return _cursor; 
}

DataStreamLogic::State DataStreamLogic::state() const 
{ 
	return _str->fail() ? (_state = Failed) : _state; 
}

DataStreamLogic::State DataStreamLogic::read(const Timeout& to)
{	
	// Do not wait for data if we do not expect data.
	if (_buf.size() == 0) return _state = Finished;
	
	DEBUG_ASSERT(_str.get());
	
	while (_str->waitForData(to)) {
		size_t toRead = _buf.size() - _cursor;
		size_t nbytes = _str->read(const_cast<char*>(_buf.data()) + _cursor, toRead);
		
		if (nbytes == toRead) {
			return _state = Finished; 
		} else 	if (_str->fail()) {
			return _state = Failed;
		} else {
			_cursor += nbytes;
		}
	}
	
	return _state = Receiving;
}

} // namespace Stream

END_CORALMESSAGING_NAMESPACE
