/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Stream/DataStringStream.h"

START_CORALMESSAGING_NAMESPACE

namespace Stream {

DataStringStream::DataStringStream(const std::string& str)
	: _str(str)
{
	// EMPTY
}

void DataStringStream::reset()
{
	_str.str("");
}

size_t DataStringStream::read(char* buffer, size_t len)
{
MONITOR_START_CRITICAL
	DEBUG_ASSERT(buffer);
	_str.read(buffer, len);
	return _str.gcount();
MONITOR_END_CRITICAL
}

bool DataStringStream::readAll(char* buf, size_t len, const Timeout& to)
{
	return _readwriteAll(buf, len, to, &DataStringStream::read);
}

size_t DataStringStream::write(const char* buf, size_t len)
{
MONITOR_START_CRITICAL
	_str.write(buf, len);
	_str.clear();
	return len;
MONITOR_END_CRITICAL
}

bool DataStringStream::writeAll(const char* buf, size_t len, const Timeout& to)
{
	return _readwriteAll(buf, len, to, &DataStringStream::write);
}

bool DataStringStream::fail() const
{
MONITOR_START_CRITICAL
	return _str.fail();
MONITOR_END_CRITICAL
}

bool DataStringStream::waitForData(const Timeout& to)
{
	// Busy wait!
	do
	{
	MONITOR_START_CRITICAL
		if (!_str.eof()) return true;
	MONITOR_END_CRITICAL
	} while(!to.occured());

	return false;
}

template<class OP, class BUF>
bool DataStringStream::_readwriteAll(BUF buf, size_t len, const Timeout& to, OP op)
{
	size_t nbytes = 0;

	while (nbytes != len)
	{
		nbytes += (this->*op)(buf + nbytes, len - nbytes);
		if (fail() || to.occured()) return false;
	}

	return true;
}

bool DataStringStream::isClosed() const
{
	return false;
}

} // namespace Stream

END_CORALMESSAGING_NAMESPACE
