/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <iostream>
#include <string.h>
#include <errno.h>

#include "CoralBricks/ThreadTraits.h"
#include "CoralMessaging/Insulation/MessageStream.h"
#include "CoralMessaging/Insulation/TimeStamp.h"
#include "CoralMessaging/Utils/Logger.h"

#include "MsgReporter.h"

START_CORALMESSAGING_NAMESPACE

Logger::Logger()
	: _isLogOn( true )
	, _actLogLevel( coral::Debug )
	, _appLabel( "CoralMessaging" )
{
  if ( !getenv("CORAL_DISABLE_CORALMESSAGING_MSGREPORTER" ) )
  {
    coral::MessageStream::installMsgReporter(new MsgReporter());
    coral::MessageStream::setMsgVerbosity(coral::Verbose);
  }
}

Logger::~Logger()
{
	// EMPTY
}

Logger& Logger::setLogOn()
{
MONITOR_START_CRITICAL
	_isLogOn = true;
MONITOR_END_CRITICAL
	return *this;
}

Logger& Logger::setLogOff()
{
MONITOR_START_CRITICAL
	_isLogOn = false;
MONITOR_END_CRITICAL
	return *this;
}

Logger& Logger::newLog(const coral::MsgLevel aLevel, const char* aFile, const char* aFunc, const int aLineNo)
{
	commit(*this);
	MONITOR_START_CRITICAL 
	_actLogLevel = aLevel;
    _logLine << ::coral::ThreadTraits::get_id() << separator();
	//if (_actLogLevel == coral::Debug) {
	_logLine << aFile << separator() << aFunc << separator() << std::dec << aLineNo << separator();
	//}
	MONITOR_END_CRITICAL
	return *this;
}

Logger& Logger::operator << (Logger& (*aFunc)(Logger &aLogger))
{
	return (aFunc)(*this);
}

Logger& Logger::setEnvironment(const std::string& appLabel)
{
	_appLabel = appLabel;
	return *this;
}

const std::string& Logger::separator()
{
	static std::string s(";");
	return s;
}

Logger& addErr(Logger& aLogger)
{
	aLogger << strerror(errno);
	return aLogger;
}

void Logger::_commit()
{
	MONITOR_START_CRITICAL 
	if (_isLogOn && !_logLine.str().empty()) { 
	    coral::MessageStream log(_appLabel);
	    log << _actLogLevel << separator();
	    log << TimeStamp::now().toString() << separator();
	    log << _logLine.str() << coral::MessageStream::endmsg;
	}
	
    _logLine.str("");
	MONITOR_END_CRITICAL 
}

Logger& commit(Logger& aLogger)
{
	aLogger._commit(); 
	return aLogger;
}

END_CORALMESSAGING_NAMESPACE
