/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/preprocessor/seq/for_each.hpp>
#include <boost/preprocessor/seq/seq.hpp>

#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/IMessage.h"
#include "CoralMessaging/Message/MessageFormatter.h"
#include "CoralMessaging/Utils/StatisticsCollector.h"
#include "CoralMessaging/CTL/CTLMessages.h"
#include "CoralMessaging/CTL/CTLMessageBuffer.h"
#include "CoralMessaging/Insulation/TimeStamp.h"

START_CORALMESSAGING_NAMESPACE

namespace Message {

Pointer<IMessage>::Shared MessageFormatter::format(CTL::MessageBuffer& mbuf)
{
	#define CAL_MESSAGE_FORMATTER(r, data, MSGNAME) \
		case coral::CoralProtocol::Opcode::BOOST_PP_CAT(E_, MSGNAME): {\
			Pointer<CAL::MSGNAME>::Shared msg(new CAL::MSGNAME); \
			TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds(); \
			msg->read(&mbuf); \
			StatisticsCollector::Instance().addMessageReadTime(*msg, TimeStamp::now().total_nanoseconds() - start); \
			\
			if (!msg->valid()) \
			{ \
				DEBUG(StatisticsCollector::Instance().addDiscardedMessage()); \
				return Pointer<CAL::MSGNAME>::Shared(); \
			} \
			return msg; }

	#define CTL_MESSAGE_FORMATTER(r, data, MSGNAME) \
		case CTL::ErrorCode::BOOST_PP_CAT(E_, MSGNAME): {\
			Pointer<CTL::MSGNAME>::Shared msg(new CTL::MSGNAME); \
			TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds(); \
			msg->read(&mbuf); \
			StatisticsCollector::Instance().addMessageDecodeTime(*msg, TimeStamp::now().total_nanoseconds() - start); \
			\
			if (!msg->valid()) \
			{ \
				DEBUG(StatisticsCollector::Instance().addDiscardedMessage()); \
				return Pointer<CTL::MSGNAME>::Shared(); \
			} \
			return msg; }

	switch (mbuf.opcode()) {
		BOOST_PP_SEQ_FOR_EACH(CAL_MESSAGE_FORMATTER, ~, BOOST_PP_SEQ_TAIL(OPCODE_LABELS));
	case coral::CoralProtocol::Opcode::E_CTL:
		switch (mbuf.firstByte()) {
			BOOST_PP_SEQ_FOR_EACH(CTL_MESSAGE_FORMATTER, ~, ERRCODE_LABELS);
		default:
			DEBUG(StatisticsCollector::Instance().addDiscardedMessage());
			LOGGER_NEWLOG(coral::Info) << "Unknown opcode: " << mbuf.opcode()
			                           << ", message dropped" << commit;
			return Pointer<IMessage>::Shared();
		}
	default:
		// should not get here..
		DEBUG_ASSERT(0);
	}
}

} // namespace Message

END_CORALMESSAGING_NAMESPACE

