/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include "CoralMessaging/Utils/StatisticsCollector.h"
#include "CoralMessaging/Message/MessageFormatter.h"
#include "CoralMessaging/Utils/Error.h"
#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/IMessage.h"
#include "CoralMessaging/MessageReceiver.h"

START_CORALMESSAGING_NAMESPACE

MessageReceiver::MessageReceiver(Pointer<ICoralIOHandler>::Shared& str)
	: _stream(str), _preader(str)
{
	// EMPTY
}

bool MessageReceiver::receive(const Timeout& to)
{
	do {
		// Receive packets
		Pointer<CTL::Packet>::Shared packet = _preader.pickPacket(to);

		if (packet.get()) {
			// Packet received, add to the corresponding incoming message
			Pointer<CoralInputSocket>::Shared csock(new CoralInputSocket(_stream, packet->header()));
			IncomingMessages::iterator i = _incoming.find(csock);

			if (i == _incoming.end()) i = _incoming.insert(csock, new CTL::MessageBuffer).first;

			CTL::MessageBuffer& mbuf = *(i->second);
			// Check if the message has been fully arrived, if so, return it
			if (mbuf.addPacket(packet, *csock)) {
				Pointer<IMessage>::Shared msg(Message::MessageFormatter::format(mbuf));
				_incoming.erase(i);

				if (!msg.get()) {
					LOGGER_NEWLOG(coral::Info) << "Message syntax error" << commit;
					CTL::SyntaxError errMsg;
					CoralSocket output(*csock, errMsg);
					errMsg.write(output);
				} else {
					DEBUG(StatisticsCollector::Instance().addRecvMessage(*msg));
					_lastFrom = csock;
					_lastMessage = msg;
					return true;
				}
			} else if (mbuf.fail()) {
				_incoming.erase(i);
			}
		} else if (_preader.terminated()) {
			EXCEPTION_THROW(Err_Custom("Broken line or connection dropped"));
		}

	} while (!to.occured());

	return false;
}

const Pointer<CoralInputSocket>::Shared MessageReceiver::from() const
{
	DEBUG_ASSERT(_lastMessage.get());
	return _lastFrom;
}

const Pointer<IMessage>::Shared MessageReceiver::message() const
{
	DEBUG_ASSERT(_lastMessage.get());
	return _lastMessage;
}

END_CORALMESSAGING_NAMESPACE

