/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include "CoralMessaging/Insulation/TimeStamp.h"
#include "CoralMessaging/Utils/Logger.h"
#include "CoralMessaging/Utils/StatisticsCollector.h"

START_CORALMESSAGING_NAMESPACE

StatisticsCollector::StatisticsCollector()
	: _recvPacketNum(0), _sentPacketNum(0), _discPacketNum(0), _discMsgNum(0), _streamReadTime(0), _synchNum(0)
{
	// EMPTY
}

StatisticsCollector::~StatisticsCollector()
{
	// EMPTY
}

void StatisticsCollector::print() const
{
	unsigned long s = 0;
	unsigned long sent = 0;
	unsigned long recv = 0;
	TimeStamp::ValueType readTime = 0;
	TimeStamp::ValueType writeTime = 0;
	TimeStamp::ValueType handlerTime = 0;
	
MONITOR_START_CRITICAL
	LOGGER_NEWLOG(coral::Info) << "\nMESSAGE_STATISTICS START\n" 
		<< "PACKETS START\n"
		<< "RecvPackets" << theLogger().separator() << _recvPacketNum << "\n"
		<< "SentPackets" << theLogger().separator() << _sentPacketNum << "\n"
		<< "DiscardedPackets" << theLogger().separator() << _discPacketNum << "\n"
		<< "PACKETS END\nMESSAGE STAT START\n";
	
	_MsgStatContainer::const_iterator end = _msgStat.end();

	for (_MsgStatContainer::const_iterator i = _msgStat.begin(); i != end; ++i) 
	{
		theLogger() << "MESSAGE: " << coral::CoralProtocol::Opcode::name(i->first) << "\n";
		i->second.print();
		theLogger() << "\n";
		sent += i->second.sentMsgNum;
		recv += i->second.recvMsgNum;
		readTime += i->second.readTime;
		writeTime += i->second.writeTime;
		handlerTime += i->second.handlerTime;
	}
	
	theLogger() << "SUM SENT" << theLogger().separator() << sent 
		        << "\nSUM RECV" << theLogger().separator() << recv << "\nEND\n" 
	            << "DiscardedMessages" << theLogger().separator() << _discMsgNum
	            << "\nMESSAGE STAT END";
	
	s = _synchNum;
MONITOR_END_CRITICAL

	// Calculate time effect of this object's synchronization activities	
	TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds();
	while(_lockOverhead(s)) continue;
	TimeStamp::ValueType t = TimeStamp::now().total_nanoseconds() - start;
	
MONITOR_START_CRITICAL	
	const int us = 1000;

	theLogger() << "\nStreamReadTime" << theLogger().separator() << _streamReadTime / us << " us"
	            << "\nStreamWriteTime" << theLogger().separator() << _streamWriteTime / us << " us"
	            << "\nMessageReadTime" << theLogger().separator() << readTime / us << " us"
	            << "\nMessageWriteTime" << theLogger().separator() << writeTime / us << " us"
	            << "\nMessageHandlingTime" << theLogger().separator() << handlerTime / us << " us"
			    << "\nCollectorOverhead" << theLogger().separator() << t / us << " us"
				<< "\n" << commit;
MONITOR_END_CRITICAL			   
}


StatisticsCollector::_MsgStatData::_MsgStatData()
	: sentMsgNum(0), recvMsgNum(0), encodeTime(0), decodeTime(0), readTime(0), writeTime(0), handlerTime(0)
{
	// EMPTY
}

void StatisticsCollector::_MsgStatData::print() const
{
	const int us = 1000;
	
	theLogger() << "\tSENT" << theLogger().separator() << sentMsgNum 
	            << "\n\tRECV" << theLogger().separator() << recvMsgNum
	            << "\n\tENCODE" << theLogger().separator() << encodeTime / us << " us"
	            << "\n\tDECODE" << theLogger().separator() << decodeTime / us << " us"
	            << "\n\tREAD" << theLogger().separator() << readTime / us << " us"
	            << "\n\tWRITE" << theLogger().separator() << writeTime / us << " us"
	            << "\n\tHANDLER" <<theLogger().separator() << handlerTime / us << " us";
}

END_CORALMESSAGING_NAMESPACE
