/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <sys/time.h>
#include <errno.h>

#include "CoralMessaging/Utils/Error.h"
#include "CoralMessaging/Tcp/TcpSocketSelector.h"
#include "CoralMessaging/Tcp/TcpSocket.h"

START_CORALMESSAGING_NAMESPACE

namespace Tcp {

TcpSocketSelector::TcpSocketSelector() : _servNum(0)
{
	FD_ZERO(&_registered);
	FD_ZERO(&_active);
}

TcpSocketSelector::~TcpSocketSelector()
{
	// EMPTY
}

void TcpSocketSelector::insert(Pointer<ICoralIOHandler>::Shared sh)
{
	insert(dynamic_cast<TcpSocket&>(*sh));
}

void TcpSocketSelector::remove(Pointer<ICoralIOHandler>::Shared sh)
{
	const TcpSocket& sock = dynamic_cast<TcpSocket&>(*sh);
MONITOR_START_CRITICAL
	FD_CLR(sock.getFd(), &_registered);
	--_servNum;
MONITOR_END_CRITICAL
}

bool TcpSocketSelector::eventOccurred(Pointer<ICoralIOHandler>::Shared sh)
{
	return eventOccurred(dynamic_cast<TcpSocket&>(*sh));
}

bool TcpSocketSelector::eventOccurred(TcpSocket& sock)
{
MONITOR_START_CRITICAL
	return FD_ISSET(sock.getFd(), &_active);
MONITOR_END_CRITICAL
}

void TcpSocketSelector::insert(TcpSocket& sock)
{
MONITOR_START_CRITICAL
	FD_SET(sock.getFd(), &_registered);
	++_servNum;
MONITOR_END_CRITICAL	
}

bool TcpSocketSelector::wait(const Timeout& td)
{
	_active = _registered;
	timeval* tv = td.tval();
    int res = select (FD_SETSIZE, &_active, NULL, NULL, tv);
    
	if (res < 0) 
    {
        if (errno != EINTR) EXCEPTION_THROW (Err_System("Select error."));
    }
    
    return res > 0;
}

bool TcpSocketSelector::allServiceClosed()
{
	return _servNum == 0;
}

} // namespace Tcp

END_CORALMESSAGING_NAMESPACE
