/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <boost/assign/std/vector.hpp>

#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/CoralSocket.h"
#include "CoralMessaging/CTL/CTLMessageBuffer.h"
#include "CoralMessaging/CTL/CTLPacketReader.h"

#include "CTLMessageBufferTest.h"

START_CORALMESSAGING_NAMESPACE

namespace CTL {

void MessageBufferTest::test()
{
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	CAL::MsgObj::Connection mo_conn(true, "oracle://lcg_coral_nightly/CORAL_WRITER");
	CAL::MsgObj::MonitoringInformation mi_conn;
	CAL::Connect msg1(mo_conn, mi_conn);
	Pointer<Stream::DataStringStream>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);
	Pointer<ICoralIOHandler>::Shared str2(new Stream::DataStringStream(str1->str().str()));
	PacketReader preader(str2);
	Pointer<Packet>::Shared packet;
	while (!packet.get()) {
		InfiniteTimeout to;
		packet = preader.pickPacket(to);
		CPPUNIT_ASSERT(!preader.terminated());
	}

	MessageBuffer mbuf;
	CPPUNIT_ASSERT(mbuf.addPacket(packet, csock1));
	// Now reconstruct the message...
	CAL::Connect msg2;
	msg2.read(&mbuf);
	CPPUNIT_ASSERT_EQUAL(msg1, msg2);
}

void MessageBufferTest::testListTablesReply()
{
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	std::vector<std::string> names;
	boost::assign::push_back(names)("T1")("T2")("T3");
	CAL::ListTablesReply msg1(names);
	Pointer<Stream::DataStringStream>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);
	Pointer<ICoralIOHandler>::Shared str2(new Stream::DataStringStream(str1->str().str()));
	PacketReader preader(str2);
	Pointer<Packet>::Shared packet;
	while (!packet.get()) {
		InfiniteTimeout to;
		packet = preader.pickPacket(to);
		CPPUNIT_ASSERT(!preader.terminated());
	}

	MessageBuffer mbuf;
	CPPUNIT_ASSERT(mbuf.addPacket(packet, csock1));
	// Now reconstruct the message...
	CAL::ListTablesReply msg2;
	msg2.read(&mbuf);
	CPPUNIT_ASSERT_EQUAL(msg1, msg2);
}

} // namespace CTL

END_CORALMESSAGING_NAMESPACE
