/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/CoralException.h"
#include "CoralMessaging/CoralSocket.h"
#include "CoralMessaging/MessageReceiver.h"
#include "CoralMessaging/Stream/DataStringStream.h"
#include "CoralExceptionTest.h"

START_CORALMESSAGING_NAMESPACE

void CoralExceptionTest::testExceptionTypes()
{
	// LookupServiceException-s
	_checkAnException(coral::LookupServiceException());
	_checkAnException(coral::InvalidReplicaIdentifierException());
	// MonitoringException-s
	_checkAnException(coral::MonitoringException());
	// RelationalServiceException-s
	_checkAnException(coral::RelationalServiceException());
	_checkAnException(coral::NonExistingDomainException());
	_checkAnException(coral::InvalidOperationException());
	// SchemaException-s
	_checkAnException(coral::SchemaException());
	_checkAnException(coral::TableAlreadyExistingException());
	_checkAnException(coral::TableNotExistingException());
	_checkAnException(coral::ViewAlreadyExistingException());
	_checkAnException(coral::QueryException());
	_checkAnException(coral::QueryExecutedException());
	_checkAnException(coral::InvalidUniqueConstraintIdentifierException());
	_checkAnException(coral::InvalidForeignKeyIdentifierException());
	_checkAnException(coral::InvalidIndexIdentifierException());
	_checkAnException(coral::InvalidColumnIndexException());
	_checkAnException(coral::InvalidColumnNameException());
	_checkAnException(coral::NoPrimaryKeyException());
	_checkAnException(coral::ExistingPrimaryKeyException());
	_checkAnException(coral::UniqueConstraintAlreadyExistingException());
	_checkAnException(coral::DataEditorException());
	_checkAnException(coral::DuplicateEntryInUniqueKeyException());
	_checkAnException(coral::ViewNotExistingException());
	// SessionException-s
	_checkAnException(coral::SessionException());
	_checkAnException(coral::MaximumNumberOfSessionsException());
	_checkAnException(coral::StartSessionException());
	_checkAnException(coral::InvalidOperationInReadOnlyModeException());
	_checkAnException(coral::InvalidSchemaNameException());
	_checkAnException(coral::NoAuthenticationServiceException());
	_checkAnException(coral::AuthenticationFailureException());
	_checkAnException(coral::MonitoringServiceNotFoundException());
	_checkAnException(coral::ServerException());
	_checkAnException(coral::DatabaseNotAccessibleException());
	_checkAnException(coral::ConnectionException());
	_checkAnException(coral::ConnectionNotActiveException());
	_checkAnException(coral::TransactionException());
	_checkAnException(coral::TransactionNotStartedException());
	_checkAnException(coral::TransactionNotCommittedException());
	_checkAnException(coral::TransactionNotActiveException());
	_checkAnException(coral::TypeConverterException());
	_checkAnException(coral::UnSupportedSqlTypeException());
	_checkAnException(coral::UnSupportedCppTypeException());
	// WebCacheControlException-s
	_checkAnException(coral::WebCacheControlException());
	// The general exception
	_checkAnException(coral::Exception());
}

void CoralExceptionTest::testExceptionContent()
{
	std::string message("message");
	std::string methodName("methodName");
	std::string moduleName("moduleName");

	coral::MonitoringException ex(message, methodName, moduleName);
	coral::Exception captured (_checkAnException(ex));
	CPPUNIT_ASSERT_EQUAL(message + " ( CORAL : \"" + methodName + "\" from \"" + moduleName + "\" )", std::string(captured.what()));
}

template<class EXCEPTION>
coral::Exception CoralExceptionTest::_checkAnException(const EXCEPTION& ex)
{
	const uint32_t clientID = 1;
	const uint32_t reqID = 1;
	Pointer<ICoralIOHandler>::Shared str1(new Stream::DataStringStream);

	try {
		throw ex;
	} catch (coral::Exception& catched) {
		uint32_t id = CORALMESSAGING_NAMESPACE_NAME::CoralException::getID(typeid(catched));
		CORALMESSAGING_NAMESPACE_NAME::CAL::Exception msg(id, std::string(catched.what()));
		CoralSocket csock(str1, clientID, reqID, msg);
		msg.write(csock);
	}

	try {
		MessageReceiver receiver(str1);
		InfiniteTimeout to;
		CPPUNIT_ASSERT(receiver.receive(to));
		CORALMESSAGING_NAMESPACE_NAME::CoralException::checkAndThrow(*(receiver.message()));
	} catch (EXCEPTION& catched) {
		CPPUNIT_ASSERT(1);
		return catched;
	}

	CPPUNIT_ASSERT(0);
	return coral::Exception();
}

END_CORALMESSAGING_NAMESPACE
