/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALMESSAGING_TESTS_CORALIOHANDLERTESTBASE_H_
#define CORALMESSAGING_TESTS_CORALIOHANDLERTESTBASE_H_

#include "CoralMessaging/Timeout.h"
#include "CoralMessaging/ICoralIOHandler.h"

#include "TestBase.h"

START_CORALMESSAGING_NAMESPACE

class CoralIOHandlerTestBase
{
protected:
	typedef void (*forceFailure_t)(ICoralIOHandler& _ioh);
	
	// -------------------- methods --------------------------------------------
	void _testReadWrite(ICoralIOHandler& sender, ICoralIOHandler& receiver);
	void _testReadWriteAll(ICoralIOHandler& sender, ICoralIOHandler& receiver);
	void _testReadFailure(ICoralIOHandler& sender, ICoralIOHandler& receiver, forceFailure_t);
	void _testWriteFailure(ICoralIOHandler& sender, ICoralIOHandler& receiver, forceFailure_t);
	void _testWaitForData(ICoralIOHandler& sender, ICoralIOHandler& receiver);
	
private:
	struct ThrCommand
	{
		ThrCommand(ICoralIOHandler* _ioh, const std::string& _s)
			: ioh(_ioh), s(_s) {}
		
		ICoralIOHandler* ioh;
		const std::string& s;
	};
	
	// -------------------- static data ----------------------------------------
	static Timeout& _GetTimeout()
	{
		static Timeout to(30, 0, 0);
		to.actualize();
		return to;
	}
	
	static Timeout& _GetShortWaitTimeout()
	{
		static Timeout to(1, 0, 0);
		to.actualize();
		return to;
	}
	
	static Timeout& _GetWaitTimeout()
	{
		static Timeout to(3, 0, 0);
		to.actualize();
		return to;
	}
	
	static Timeout& _GetLongerWaitTimeout()
	{
		static Timeout to(4, 0, 0);
		to.actualize();
		return to;
	}
	
	static const size_t _DataSize = 2000000; 
	static const size_t _DataSizeUntilFailure = _DataSize / 2;
	
	// -------------------- ststic methods -------------------------------------
	static void _ThrReadWrite(const ThrCommand& cmd);
	static void _ThrReadWriteAll(const ThrCommand& cmd);
	static void _ThrReadFailure(ThrCommand& cmd);
	static void _ThrWriteFailure(ThrCommand& cmd);
	static void _ThrWait(ThrCommand& cmd);
	
	static const std::string& _GenerateLongRandomData();
};

END_CORALMESSAGING_NAMESPACE

#endif /*CORALMESSAGING_TESTS_CORALIOHANDLERTESTBASE_H_*/
