/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/mpl/map.hpp>
#include <boost/mpl/for_each.hpp>

#include "CoralMessaging/CAL/CppType_ProtocolID_Map.h"
#include "CppType_ProtocolID_MapTest.h"

START_CORALMESSAGING_NAMESPACE

using CAL::CppType_ProtocolID_Map;

struct CppType_ProtocolID_MapTest_TestClass : public CppType_ProtocolID_Map
{
	enum {E_Int = 1, E_Double};
	// Restart the type list
	enum {E_Float = 3, E_Bool};
};

struct CppType_ProtocolID_MapTest_TestClass_2 : public CppType_ProtocolID_MapTest_TestClass
{
	CppType_ProtocolID_MapTest_TestClass_2()
	{
		_map[&typeid(int)] = static_cast<uint32_t>(E_Int);
		_map[&typeid(double)] = static_cast<uint32_t>(E_Double);
	}
};



void CppType_ProtocolID_MapTest::testMap()
{
	CppType_ProtocolID_MapTest_TestClass_2 m;
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Int), m(typeid(int)));
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Double), m(typeid(double)));
}

void CppType_ProtocolID_MapTest::testInserter()
{
	CppType_ProtocolID_MapTest_TestClass m;
	CppType_ProtocolID_MapTest_TestClass::Insert ins1(m);

	typedef boost::mpl::map <
		CAL::ElemGen<static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Int), int>::type,
		CAL::ElemGen<static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Double), double>::type
	> Creatormap_1;

	boost::mpl::for_each<Creatormap_1>(ins1);
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Int), m(typeid(int)));
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Double), m(typeid(double)));

	typedef boost::mpl::map <
		CAL::ElemGen<1, float>::type,
		CAL::ElemGen<2, bool>::type
	> Creatormap_2;

	CppType_ProtocolID_MapTest_TestClass::Insert ins2(m, 2);
	boost::mpl::for_each<Creatormap_2>(ins2);
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Float), m(typeid(float)));
	CPPUNIT_ASSERT_EQUAL(static_cast<uint32_t>(CppType_ProtocolID_MapTest_TestClass::E_Bool), m(typeid(bool)));
}

END_CORALMESSAGING_NAMESPACE
