/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Stream/DataStringStream.h"
#include "DataStringStreamTest.h"

START_CORALMESSAGING_NAMESPACE

void DataStringStreamTest::setUp (void)
{
	theLogger().setLogOff();
}

void DataStringStreamTest::tearDown (void) 
{
	theLogger().setLogOn();
}

void DataStringStreamTest::testReadWrite()
{
	Stream::DataStringStream str;
	_testReadWrite(str, str);
}

void DataStringStreamTest::testReadWriteAll()
{
	Stream::DataStringStream str;
	_testReadWriteAll(str, str);
}

void DataStringStreamTest::testReadFailure()
{
	Stream::DataStringStream str;
	_testReadFailure(str, str, DataStringStreamTest::_forceFailure);
}

void DataStringStreamTest::testWriteFailure()
{
	// This test is irrelevant here, because writing into a stream never fails
	// except extreme conditions that we do not test here.
}

void DataStringStreamTest::testWaitForData()
{
	Stream::DataStringStream str;
	_testWaitForData(str, str);
}

void DataStringStreamTest::_forceFailure(ICoralIOHandler& ioh)
{
	Stream::DataStringStream& dsock = dynamic_cast<Stream::DataStringStream&>(ioh);
	// Well, it circumvents that str() returns const reference. We need to do it here,
	// to force failure
	std::stringstream& s = const_cast<std::stringstream&>(dsock.str());
	s.setstate(std::ios::failbit);
}

END_CORALMESSAGING_NAMESPACE
