/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/foreach.hpp>
#include <boost/assign/std/vector.hpp>
#include <limits>

#include "CoralMessaging/CAL/MsgObj/Objects.h"
#include "CoralMessaging/CAL/MsgObj/AttributeTypeTools.h"
#include "FloatRepresentationTest.h"

START_CORALMESSAGING_NAMESPACE

using CAL::MsgObj::AttributeTypeTools;

void FloatRepresentationTest::test()
{
	_test<float, uint32_t>();
	_test<double, uint64_t>();
	_test<long double, CAL::MsgObj::LongDoubleRepresentation>();
}

template<class FloatType, class ReprType>
void FloatRepresentationTest::_test()
{
	std::vector<FloatType> f;
	boost::assign::push_back(f) (0)(1)(-1)(12535.56257)(-12535.56257)(-0.001)(0.00001)
								(std::numeric_limits<FloatType>::max())(std::numeric_limits<FloatType>::min())
								(std::numeric_limits<FloatType>::infinity())(std::numeric_limits<FloatType>::epsilon())
								(std::numeric_limits<FloatType>::denorm_min());

	BOOST_FOREACH(FloatType x, f) {
		// Convert to..
		ReprType converted(AttributeTypeTools::GetMsgObjConverter<FloatType>::type::convert(x));
		// Convert back...
		FloatType backConverted(AttributeTypeTools::GetMsgObjBackConverter<FloatType>::type::convert(converted));
		// Compare
		CPPUNIT_ASSERT_EQUAL(x, backConverted);
	}
}

END_CORALMESSAGING_NAMESPACE
