/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/CTL/CTLPacketReader.h"
#include "CoralMessaging/CTL/CTLMessageBuffer.h"
#include "CoralMessaging/Message/MessageFormatter.h"
#include "CoralMessaging/Stream/DataStringStream.h"
#include "MessageFormatterTest.h"

#include <boost/assign/std/vector.hpp>

START_CORALMESSAGING_NAMESPACE

void MessageFormatterTest::testMessage(void)
{
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	Pointer<Stream::DataStringStream>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	CAL::MsgObj::Connection mo_conn(true, "oracle://lcg_coral_nightly/CORAL_WRITER");
	CAL::MsgObj::MonitoringInformation mi_conn;
	CAL::Connect msg1(mo_conn, mi_conn);
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);
	Pointer<Stream::DataStringStream>::Shared str2(new Stream::DataStringStream(str1->str().str()));
	CTL::PacketReader preader(str2);
	Pointer<CTL::Packet>::Shared packet;
	InfiniteTimeout to;

	while (!packet.get()) {
		packet = preader.pickPacket(to);
		CPPUNIT_ASSERT(!preader.terminated());
	}

	CTL::MessageBuffer mbuf;
	CoralSocket csock2(str2, clientID, reqID, msg1);
	CPPUNIT_ASSERT(mbuf.addPacket(packet, csock2));
	// Now reconstruct the message...
	CPPUNIT_ASSERT(Message::MessageFormatter::format(mbuf).get());
}

void MessageFormatterTest::testListTablesReply(void)
{
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	Pointer<Stream::DataStringStream>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	std::vector<std::string> names;
	boost::assign::push_back(names)("T1")("T2")("T3");
	CAL::ListTablesReply msg1(names);
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);
	Pointer<Stream::DataStringStream>::Shared str2(new Stream::DataStringStream(str1->str().str()));
	CTL::PacketReader preader(str2);
	Pointer<CTL::Packet>::Shared packet;
	InfiniteTimeout to;

	while (!packet.get()) {
		packet = preader.pickPacket(to);
		CPPUNIT_ASSERT(!preader.terminated());
	}

	CTL::MessageBuffer mbuf;
	CoralSocket csock2(str2, clientID, reqID, msg1);
	CPPUNIT_ASSERT(mbuf.addPacket(packet, csock2));
	// Now reconstruct the message...
	Pointer<IMessage>::Shared msg2(Message::MessageFormatter::format(mbuf));
	CPPUNIT_ASSERT(msg2.get());
	CPPUNIT_ASSERT(names == dynamic_cast<CAL::ListTablesReply&>(*msg2).tableList());
}

void MessageFormatterTest::testCTLMessage(void)
{
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	Pointer<Stream::DataStringStream>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	CTL::VersionMismatch msg1(2, "desc");
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);
	Pointer<Stream::DataStringStream>::Shared str2(new Stream::DataStringStream(str1->str().str()));
	CTL::PacketReader preader(str2);
	Pointer<CTL::Packet>::Shared packet;
	InfiniteTimeout to;

	while (!packet.get()) {
		packet = preader.pickPacket(to);
		CPPUNIT_ASSERT(!preader.terminated());
	}

	CTL::MessageBuffer mbuf;
	CPPUNIT_ASSERT(mbuf.addPacket(packet, csock1));
	// Now reconstruct the message...
	CPPUNIT_ASSERT(Message::MessageFormatter::format(mbuf).get());
}

END_CORALMESSAGING_NAMESPACE
