/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include "CoralMessaging/CoralSocket.h"
#include "CoralMessaging/MessageReceiver.h"
#include "CoralMessaging/CTL/CTLMessages.h"
#include "CoralMessaging/CAL/CALMessages.h"
#include "CoralMessaging/Stream/DataStringStream.h"
#include "MessageReceiverTest.h"

START_CORALMESSAGING_NAMESPACE

void MessageReceiverTest::test(void)
{
	Pointer<ICoralIOHandler>::Shared str1(new Stream::DataStringStream);
	// Write a message into the stream
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	// Write a message into a coral socket, then read it back
	CAL::MsgObj::Connection mo_conn(true, "oracle://lcg_coral_nightly/CORAL_WRITER");
	CAL::MsgObj::MonitoringInformation mi_conn;
	CAL::Connect msg1(mo_conn, mi_conn);
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);

	// Now read it back with the MessageReceiver
	MessageReceiver receiver(str1);
	InfiniteTimeout to;
	CPPUNIT_ASSERT(receiver.receive(to));
	CPPUNIT_ASSERT_EQUAL(coral::CoralProtocol::Opcode_gen<coral::CoralProtocol::Opcode::E_Connect>(), receiver.message()->opcode());

	// Send another message to see if it is able to receive it
	msg1.write(csock1);
	CPPUNIT_ASSERT(receiver.receive(to));
	CPPUNIT_ASSERT_EQUAL(coral::CoralProtocol::Opcode_gen<coral::CoralProtocol::Opcode::E_Connect>(), receiver.message()->opcode());
}

void MessageReceiverTest::testCasts(void)
{
	// Write a message into the stream
	uint32_t clientID = 1;
	uint32_t reqID = 2;
	Pointer<ICoralIOHandler>::Shared str1(new Stream::DataStringStream);
	// Write a message into a coral socket, then read it back
	CTL::OK msg1;
	CoralSocket csock1(str1, clientID, reqID, msg1);
	msg1.write(csock1);

	// Now read it back with the MessageReceiver
	MessageReceiver receiver(str1);
	InfiniteTimeout to;
	CPPUNIT_ASSERT(receiver.receive(to));
	const IMessage& msg = *receiver.message();

	const CTL::OK* casted_ptr = message_cast<CTL::OK>(&msg);
	CPPUNIT_ASSERT(casted_ptr);
	CPPUNIT_ASSERT_EQUAL(static_cast<uint8_t>(CTL::ErrorCode::E_OK), casted_ptr->errorCode().value());

	// should not throw bad_cast
	const CTL::OK& casted_ref = message_cast<CTL::OK>(msg);
	CPPUNIT_ASSERT_EQUAL(static_cast<uint8_t>(CTL::ErrorCode::E_OK), casted_ref.errorCode().value());
}

END_CORALMESSAGING_NAMESPACE
