/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <boost/assign/std/vector.hpp>

#include "CoralMessaging/Stream/MessageStreamReader.h"
#include "CoralMessaging/Stream/MessageStreamWriter.h"
#include "CoralMessaging/Utils/ctl.h"

#include "MessageStreamReaderTest.h"

START_CORALMESSAGING_NAMESPACE

void MessageStreamReaderTest::testReadingSimpleValues()
{
	uint32_t v1 = 123456;
	CPPUNIT_ASSERT_EQUAL(v1, _testReadingValue(v1));

	std::string v2("A String");
	CPPUNIT_ASSERT_EQUAL(v2, _testReadingValue(v2));

	std::vector<std::string> v3;
	boost::assign::push_back(v3)("T1")("T2")("T3");
	CPPUNIT_ASSERT_EQUAL(v3, _testReadingValue(v3));

	std::vector<uint32_t*> v4;
	boost::assign::push_back(v4)(new uint32_t(1))(new uint32_t(2))(new uint32_t(3));
	CPPUNIT_ASSERT(std::equal(v4.begin(), v4.end(), _testReadingValue(v4).begin(), ctl::_PtrContainerComparator<std::vector<uint32_t*> >()));
}

void MessageStreamReaderTest::testWritingPointerVector()
{
	StrType str(new Stream::DataStringStream);
	Stream::MessageStreamWriter<StrType> msw(str);

	str->reset();
	std::vector<uint8_t*> v1;
	boost::assign::push_back(v1)(new uint8_t(1))(new uint8_t(2))(new uint8_t(3));
	std::string res1;
	uint32_t size1 = v1.size();
	res1.append(reinterpret_cast<const char*>(&size1), sizeof(uint32_t));
	boost::assign::push_back(res1)(1)(2)(3);
	msw.write(v1);
	CPPUNIT_ASSERT_EQUAL(res1, str->str().str());
}

void MessageStreamReaderTest::testReadingPointerVector()
{
	std::string val1;
	uint32_t size1 = 3;
	val1.append(reinterpret_cast<const char*>(&size1), sizeof(uint32_t));
	boost::assign::push_back(val1)(1)(2)(3);
	StrType str(new Stream::DataStringStream(val1));
	Stream::MessageStreamReader<StrType> msr(str);
	std::vector<uint8_t*> v1;
	msr.read(v1);
	std::vector<uint8_t*> v2;
	boost::assign::push_back(v2)(new uint8_t(1))(new uint8_t(2))(new uint8_t(3));
	CPPUNIT_ASSERT_EQUAL(v1.size(), v2.size());
	CPPUNIT_ASSERT(NULL != v1[0]);
	CPPUNIT_ASSERT(std::equal(v1.begin(), v1.end(), v2.begin(), ctl::_PtrContainerComparator<std::vector<uint8_t*> >()));
}

template<class T>
T MessageStreamReaderTest::_testReadingValue(const T& val)
{
	StrType str(new Stream::DataStringStream);
	Stream::MessageStreamWriter<StrType> msw(str);
	msw.write(val);
	Stream::MessageStreamReader<StrType> msr(str, CORALPROTOCOL_NAMESPACE_NAME::_system_endian);
	T val3;
	msr.read(val3);
	CPPUNIT_ASSERT(!msr.fail());
	return val3;
}

END_CORALMESSAGING_NAMESPACE
