/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Stream/MessageStreamReader.h"
#include "CoralMessaging/Stream/MessageStreamWriter.h"
#include "CoralMessaging/CAL/MsgObj/ParameterCodecs.h"
#include "ParameterCodecsTest.h"

START_CORALMESSAGING_NAMESPACE

using CAL::MsgObj::Parameter_1;

void ParameterCodecsTest::testEncode()
{
	Parameter_1<uint32_t> p1(5);
	IMessageObject::EncodedType enc(p1.encode());
	CPPUNIT_ASSERT_EQUAL(sizeof(uint32_t), enc->size());
	CPPUNIT_ASSERT_EQUAL(p1.p1(), *reinterpret_cast<const uint32_t*>(enc->data()));
}

void ParameterCodecsTest::testDecode()
{
	std::string str;
	uint32_t val = 5;
	str.append(reinterpret_cast<const char*>(&val), sizeof(uint32_t));
	Parameter_1<uint32_t> p1(7);
	CPPUNIT_ASSERT(p1.decode(str));
	CPPUNIT_ASSERT_EQUAL(p1.p1(), val);
}

void ParameterCodecsTest::testReadWrite()
{
	StrType str(new Stream::DataStringStream);
	Stream::MessageStreamWriter<StrType> msw(str);
	Parameter_1<uint32_t> p1(5);
	msw.write(p1);
	// 2 * sizeof... : first: string size, second: the uint32_t size. IMsgObject-s are
	// written as Array-s (size + data)
	CPPUNIT_ASSERT_EQUAL(2 * sizeof(uint32_t), str->str().str().size());
	// The + sizeof(): see the commemt above. The first uint32 is the size
	CPPUNIT_ASSERT_EQUAL(p1.p1(), *reinterpret_cast<const uint32_t*>(str->str().str().data() + sizeof(uint32_t)));
	Parameter_1<uint32_t> p2(3);
	Stream::MessageStreamReader<StrType> msr(str);
	msr.read(p2);
	CPPUNIT_ASSERT_EQUAL(p1, p2);
}

END_CORALMESSAGING_NAMESPACE
