/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */


#include "CoralMessaging/CAL/MsgObj/QueryRepresentation.h"
#include "CoralMessaging/CAL/MsgObj/AttributeListRepresentation.h"

#include "TestQueryDefinition.h"

START_CORALMESSAGING_NAMESPACE

using CORALMESSAGING_NAMESPACE_NAME::CAL::MsgObj::QueryRepresentationTypes;
using CORALMESSAGING_NAMESPACE_NAME::CAL::MsgObj::AttributeListRepresentation;

TestQueryDefinition::TestQueryDefinition(const std::string& schemaName)
{
	_eqr.setSchemaName(schemaName);
}

//replace
void TestQueryDefinition::setDistinct()
{
	typedef QueryRepresentationTypes::setDistinct repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type;
	_eqr.replaceMethodCall(repr);
}

//insert
void TestQueryDefinition::addToOutputList(const std::string& expression, std::string alias)
{
	typedef QueryRepresentationTypes::addToOutputList repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(expression, alias);
	_eqr.insertMethodCall(repr);
}

//insert
void TestQueryDefinition::addToTableList(const std::string& tableName, std::string alias)
{
	typedef QueryRepresentationTypes::addToTableList repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(tableName, alias);
	_eqr.insertMethodCall(repr);
}

//insert
IQueryDefinition& TestQueryDefinition::defineSubQuery(const std::string& alias)
{
	Pointer<TestQueryDefinition>::Shared subQuery(new TestQueryDefinition(_eqr.schemaName()));
	_subQueries.push_back(subQuery);
	typedef QueryRepresentationTypes::defineSubQuery repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(alias, subQuery->repr());
	_eqr.insertMethodCall(repr);
	return *subQuery;
}

// replace
void TestQueryDefinition::setCondition(const std::string& condition, const AttributeList& inputData)
{
	typedef QueryRepresentationTypes::setCondition repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(condition, AttributeListRepresentation(inputData));
	_eqr.replaceMethodCall(repr);
}

// replace
void TestQueryDefinition::groupBy(const std::string& expression)
{
	typedef QueryRepresentationTypes::groupBy repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(expression);
	_eqr.replaceMethodCall(repr);
}

// insert
void TestQueryDefinition::addToOrderList(const std::string& expression)
{
	typedef QueryRepresentationTypes::addToOrderList repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(expression);
	_eqr.insertMethodCall(repr);
}

// replace
void TestQueryDefinition::limitReturnedRows(int maxRows, int offset)
{
	typedef QueryRepresentationTypes::limitReturnedRows repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(maxRows, offset);
	_eqr.replaceMethodCall(repr);
}

// replace
IQueryDefinition& TestQueryDefinition::applySetOperation(SetOperation operationType)
{
	_setQuery.reset(new TestQueryDefinition(_eqr.schemaName()));
	typedef QueryRepresentationTypes::applySetOperation repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(static_cast<uint8_t>(operationType), _setQuery->repr());
	_eqr.replaceMethodCall(repr);
	return *_setQuery;
}

END_CORALMESSAGING_NAMESPACE
