/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <assert.h>
#include <errno.h>
#include <boost/bind.hpp>

#include "CoralBricks/ThreadTraits.h"
#include "CoralMessaging/Utils/Error.h"
#include "CoralMessaging/Tcp/TcpSocketFactory.h"
#include "CoralMessaging/Tcp/TcpSocketSelector.h"

#include "TestSocketServer.h"

START_CORALMESSAGING_NAMESPACE

TestSocketServer::TestSocketServer()
{
	Pointer<Tcp::TcpSocket>::Shared listening(Tcp::TcpSocketFactory::CreateListeningSocket(_Port, _Hostname().c_str()));
	ThreadTraits::THREAD thr(boost::bind(&TestSocketServer::_connect, this));
	// The thread above will connect. Accept the connection here.
	int fd = listening->getFd();
    struct sockaddr_in clientname;
    unsigned int size = sizeof (clientname);
    // Wait until the thread ready to connect (and does it)
    Tcp::TcpSocketSelector selector;
    selector.insert(listening);
    Timeout to(1, 0, 0);
    assert(selector.wait(to));
    // Accept the connection
    int newsock = accept (fd, (struct sockaddr *) &clientname, &size);  
    
    if (newsock < 0) EXCEPTION_THROW (Err_System("Error accepting connection."));

    _server = Tcp::TcpSocketFactory::CreateActiveSocket(newsock);
    thr.join();
}

void TestSocketServer::_connect()
{
	_client = Tcp::TcpSocketFactory::CreateConnectingSocket(_Port, _Hostname().c_str());
}
	
END_CORALMESSAGING_NAMESPACE

