/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALPROTOCOL_OPCODE_H_
#define CORALPROTOCOL_OPCODE_H_

#include <string>

#include <boost/static_assert.hpp>
#include <boost/preprocessor/seq/for_each_i.hpp>
#include <boost/preprocessor/seq/seq.hpp>
#include <boost/preprocessor/seq/reverse.hpp>

#include "base.h"
#include "arch.h"

START_CORALPROTOCOL_NAMESPACE

#define OPCODE_LABELS (CTL)(Connect)(ConnectReply)(CloseSession)(StartTransaction)\
(CommitTransaction)(ExecuteQuery)(ExecuteQueryReply)(OK)(Exception)(TableSchemaRead)\
(TableSchemaReadReply)(TableSchemaExists)(ListTables)(ListTablesReply)(Ping0)(Ping1)(Ping2)(Ping3)

#define LAST_OPCODE BOOST_PP_CAT(E_, BOOST_PP_SEQ_HEAD(BOOST_PP_SEQ_REVERSE(OPCODE_LABELS)))

class Opcode
{
public:
	#define ENUM_GEN(r, data, i, elem) BOOST_PP_CAT(E_, elem) = i,

	enum { BOOST_PP_SEQ_FOR_EACH_I(ENUM_GEN, ~, OPCODE_LABELS)
		   E_Invalid = 0xFF
	};

	#undef ENUM_GEN

	Opcode(const uint8_t opcode = E_Invalid) : _opcode(opcode) {}
	bool valid() const {return _opcode <= LAST_OPCODE;}
	operator uint8_t () const {return _opcode;}
	static const std::string& name(uint8_t);

private:
	uint8_t _opcode;
};

template <int OPCODE> inline Opcode Opcode_gen()
{
	BOOST_STATIC_ASSERT((OPCODE <= Opcode::LAST_OPCODE));
	return Opcode(OPCODE);
}

#undef LAST_OPCODE

END_CORALPROTOCOL_NAMESPACE

#endif /*CORALPROTOCOL_OPCODE_H_*/
