#ifndef CORALPROTOCOL_SEQUENCENUMBER_H_
#define CORALPROTOCOL_SEQUENCENUMBER_H_

#include <ostream>

#include "arch.h"
#include "error.h"

START_CORALPROTOCOL_NAMESPACE

class SequenceNumber
{
public:
	SequenceNumber() : _seqNum(_MORECOMING_BITMASK()) {}
	SequenceNumber(const uint24_t seqNum) : _seqNum(seqNum) {}
	operator uint24_t () const {return _seqNum;}
	
	SequenceNumber& operator = (const SequenceNumber& x) 
	{
		_seqNum = x._seqNum;
		return *this;
	}
	
	uint32_t value() const {return _seqNum;}
	bool setValue(const uint24_t seqNum) 
	{
		_seqNum = seqNum;
		return true;
	}
	
	bool operator == (const SequenceNumber& x) const {return _seqNum == x._seqNum;};
	bool operator != (const SequenceNumber& x) const {return !(x == *this);};
	
	SequenceNumber& operator ++ () 
	{	
		// Check overloading
		if (_seqNum & 0x00010000) CORALPROTOCOL_EXCEPTION_THROW("Sequence number overloading");
		++_seqNum;
		return *this;
	}
	
	bool isFirst() const {return !(_seqNum & 0x00FFFFFF);}
	bool isLast() const {return !(_MORECOMING_BITMASK() & _seqNum);}
	void setFinal() {_seqNum &= ~_MORECOMING_BITMASK();} 
	bool isNextOf(const SequenceNumber s) const {return (0x007FFFFF & _seqNum) + 1 == (0x007FFFFF & s._seqNum);}
	
private:
	static const uint32_t _MORECOMING_BITMASK() {return 0x00800000;}
	uint32_t _seqNum;
};

struct FirstSequenceNumber : public SequenceNumber
{
	FirstSequenceNumber() : SequenceNumber(0) {}
};

inline std::ostream& operator << (std::ostream& os, const SequenceNumber& s)
{
	os << std::hex << "0x" << s.value() << "(" << (s.isLast() ? "last sequence" : "more coming") << ")";
	return os;
}

END_CORALPROTOCOL_NAMESPACE

#endif /* CORALPROTOCOL_SEQUENCENUMBER_H_*/
