/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#ifndef CORALPROTOCOL_ARCH_H_
#define CORALPROTOCOL_ARCH_H_

#include <stdint.h>

#ifdef WIN32
//TODO
#else
#ifdef __APPLE__
#include <machine/endian.h>
#else
#include <endian.h>
#endif
#endif

#include "base.h"

START_CORALPROTOCOL_NAMESPACE

#ifdef WIN32
//See http://svn.mysql.com/svnpublic/connector-odbc3/branches/guffert/test/odbctap.h
#if !defined(__attribute__) && !defined(__GNUC__)
# define __attribute__(arg)
#endif
#endif

#define PACK_STRUCT __attribute__((packed)) 

/** Collection of supported endianness-es */
enum t_endianness {
    e_little, /**< for little endian systems */
    e_big, /**< for big endian systems */
    e_unknown /**< for unknown or unsupported endian systems */
}; 

#if (__BYTE_ORDER == __LITTLE_ENDIAN)
	/**< Global constant representing the system endianness where the software is compiled on */
    const t_endianness _system_endian = e_little;
#elif (__BYTE_ORDER == __BIG_ENDIAN)
    /**< Global constant representing the system endianness where the software is compiled on */
    const t_endianness _system_endian = e_big;
#else
    // Egzotic endiannes is not supported, only big and little.
#  error Unsupported endianness detected.
#endif

/** Encapsulates an integer represented on 3 bytes.
  *
  * In the CoralProtocol, we use 3-byte (24 bit) fields. This class implements
  * a 3-byte number. It is a simple wrapper arount the 4-byte machine word. Actually,
  * it represents the 3-byte number on a word, and performs simple runtime check
  * on the highes byte them must be zero. If it is not zero, it generates runtime
  * error. All the numeric operations are executed on words (casted to words), the
  * resumt must be casted back to uint24_t. Overflow will generate runtime error
  * at this point. 
  */
class uint24_t 
{
public:
	/// Constructor. The value's highest byte must be 0.
    uint24_t(const uint32_t aVal) 
        : _val(aVal)
    {
        CORALPROTOCOL_DEBUG_ASSERT(!(aVal & 0xFF000000));
    }

    /// Convert to a word.
    operator uint32_t () const {
        return _val;
    }
    
private:
    uint32_t _val; /**< The value stored in a word */   
};

END_CORALPROTOCOL_NAMESPACE

#endif // #ifndef CORALPROTOCOL_ARCH_H_
