#include "CoralProtocol/PacketHeader.h"

START_CORALPROTOCOL_NAMESPACE

PacketHeader::PacketHeader(const t_endianness e) {
	::memset(&_header, 0, sizeof(_header));
    _header.delim = (e == e_little ? LITTLE_ENDIAN_MAGIC_WORD() : BIG_ENDIAN_MAGIC_WORD());
    _header.psize = size();
    _setProtocolVersion(_PROTOCOL_VERSION);
}

PacketHeader* PacketHeader::create(const std::string& data)
{
	CORALPROTOCOL_DEBUG_ASSERT(data.size() >= size());
    const uint32_t endian = *reinterpret_cast<const uint32_t*>(data.c_str());
        
    if (endian == LITTLE_ENDIAN_MAGIC_WORD() || endian == BIG_ENDIAN_MAGIC_WORD()) {
        return new PacketHeader(data);
    } else {
        return NULL;
    }
}

std::ostream& operator << (PacketHeader& phead, std::ostream& os)
{
	os << "Delim     | Version | ClientID  | Packet Size | Opcode | RequestID | Flags | Sequence Num" << std::endl;
	os << "0x";
	os.width(8);
	os << std::hex << std::left << (uint32_t)phead._header.delim;
	os << "| 0x";
	os.width(2);
	os << std::hex << std::left << (uint32_t)phead.version();
	os << "    | 0x";
	os.width(8);
	os << std::hex <<  (uint32_t)phead.clientID();
	os << "| 0x";
	os.width(10);
	os << std::hex <<  (uint32_t)phead.packetSize();
	os << "| 0x";
	os.width(5);
	os << std::hex <<  (uint32_t)phead.opcode();
	os << "| 0x";
	os.width(8);
	os << std::hex <<  (uint32_t)phead.requestID();
	os << "| 0x";
	os.width(4);
	os << std::hex <<  (uint32_t)phead._flags();
	os << "| " << phead.sequenceNum() << std::endl;
    return os;
} 

END_CORALPROTOCOL_NAMESPACE

