/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "CoralMessaging/Tcp/TcpSocketFactory.h"
#include "CoralMessaging/Tcp/TcpSocketSelector.h"
#include "CoralMessaging/Utils/StatisticsCollector.h"

#include "Error.h"
#include "ThreadPool.h"
#include "Environment.h"
#include "Dispatcher.h"
#include "CoralDataAcceptor.h"
#include "DebugController.h"
#include "CoralServer/CoralServer.h"

START_CORALSERVER_NAMESPACE

void CoralServer::start()
{
	unsigned int port = theEnvironment().get<unsigned int>("port");
	const std::string& ip = theEnvironment().get<std::string>("ip");
	DebugController::Instance().start();
	typedef Dispatcher<CORALMESSAGING_NAMESPACE_NAME::Tcp::TcpSocketSelector> DispatcherType;
	DispatcherType portDisp("for incoming TCP connections");
	Pointer<ICoralIOHandler>::Shared dataChannel = CORALMESSAGING_NAMESPACE_NAME::Tcp::TcpSocketFactory::CreateListeningSocket(port, ip);
	Pointer<IEventHandler>::Shared cdacc(new CoralDataAcceptor());
	portDisp.registerService(dataChannel, cdacc);
	portDisp.start_p();
	ThreadPool::ThreadPool::instance().wait();
}

void CoralServer::stop()
{
    LOGGER_NEWLOG(coral::Info) << "Stopping...";
    DebugController::Instance().stop();
    //ThreadPool::ThreadPool::instance().stop();
    CORALMESSAGING_NAMESPACE_NAME::StatisticsCollector::Instance().print();
    theLogger() << commit;
}

END_CORALSERVER_NAMESPACE
