/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALSERVER_DEBUGCONTROLLER_H_
#define CORALSERVER_DEBUGCONTROLLER_H_

#include <fstream>
#include <string>
#include <map>

#include "CoralBricks/MonitorObject.h"

#include "ActiveObject.h"
#include "ThreadPool.h"

START_CORALSERVER_NAMESPACE

class DebugController : public ActiveObject<ThreadPool::ThreadPool>
{
public:
	DebugController();
	virtual ~DebugController();
	
    void start();
    void stop();
    
    static DebugController& Instance()
    {
    	static DebugController dc;
    	return dc;
    }
    
private:
    void _start_a();
    void _stop_a();
    bool _createFifo();
    bool _isValid();
    void _handleCommand();
    
	static Timeout& _TIMEOUT() {
		static InfiniteTimeout td;
		return td;
	}
	
	static const std::string _FifoPathPrefix()
	{
		static std::string s("/tmp/CoralServer.");
		return s;
	}
    
	static void _HandleStat();
	static void _HandleLogOn();
	static void _HandleLogOff();
	static void _HandleStop();
	
    std::fstream _fifo;
    std::string _fileName;
    
    typedef std::map<std::string, void (*)(void)> _CmdMap_type;
    _CmdMap_type _cmdMap;
};

END_CORALSERVER_NAMESPACE

#endif // #ifndef CORALSERVER_DEBUGCONTROLLER_H_
