/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALSERVER_DISPATCHER_H_
#define CORALSERVER_DISPATCHER_H_

#include <map>

#include "CoralBricks/MonitorObject.h"
#include "CoralMessaging/ICoralIOHandler.h"
#include "CoralMessaging/Timeout.h"

#include "ActiveObject.h"
#include "IEventHandler.h"
#include "Traced.h"
#include "ThreadPool.h"

START_CORALSERVER_NAMESPACE

template<class Demultiplexer>
class Dispatcher : public ActiveObject<ThreadPool::ThreadPool, Traced<Dispatcher<Demultiplexer> > >
{
public:
	using ActiveObject<ThreadPool::ThreadPool, Traced<Dispatcher<Demultiplexer> > >::_enqueue;
	
	Dispatcher(const std::string& desc = "") 
		: ActiveObject<ThreadPool::ThreadPool, Traced<Dispatcher<Demultiplexer> > >("Dispatcher", desc) {}
	virtual ~Dispatcher() {}
	
	void registerService(Pointer<ICoralIOHandler>::Shared servh, 
						 Pointer<IEventHandler>::Shared evh)
	{
		DEBUG_ASSERT(servh.get());
		DEBUG_ASSERT(evh.get());
		// TODO what if the service is already registered?
	MONITOR_START_CRITICAL
		_servHandlers[servh] = evh;
		_eventDemux.insert(servh);
	MONITOR_END_CRITICAL
	}
	
	void unregisterService(Pointer<ICoralIOHandler>::Shared sh)
	{
		DEBUG_ASSERT(sh.get());
		// TODO what if the service is not registered?
	MONITOR_START_CRITICAL
		_eventDemux.remove(sh);
		_servHandlers.erase(sh);
	MONITOR_END_CRITICAL
	}
	
	// Active and proxy methods
	void start_p()
	{
		boost::function0<void> op = boost::bind(&Dispatcher::_start_a, this);
		this->_enqueue(op);
	}
	
private:
	virtual void _gotUnreferenced() {delete this;}
	
	void _start_a()
	{
		while(1) {
			_TIMEOUT().actualize();
			bool isEventHappened = _eventDemux.wait(_TIMEOUT());
			_servHandlers_type triggered;
		MONITOR_START_CRITICAL
			if (_eventDemux.allServiceClosed()) return;
			if (!isEventHappened) continue;
			// Poll for each event to see which one was triggered, and collect them
			for (_servHandlers_type::iterator itr = _servHandlers.begin(); itr != _servHandlers.end(); ++itr) {
				DEBUG_ASSERT(itr->first.get());
				if (_eventDemux.eventOccurred(itr->first)) triggered.insert(*itr);
			}
		MONITOR_END_CRITICAL
			for (_servHandlers_type::iterator itr = triggered.begin(); itr != triggered.end(); ++itr) {
				DEBUG_ASSERT(itr->second.get());
				Pointer<ICoralIOHandler>::Shared p = itr->first;
				itr->second->handleEvents(p);
			}
		}
	}
	
	typedef std::map<Pointer<ICoralIOHandler>::Shared, 
	 			     Pointer<IEventHandler>::Shared> _servHandlers_type;
	_servHandlers_type _servHandlers;
	
	Demultiplexer _eventDemux;
	
	static Timeout& _TIMEOUT() {
		static InfiniteTimeout td;
		return td;
	}
};

END_CORALSERVER_NAMESPACE

#endif /*CORALSERVER_DISPATCHER_H_*/
