/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <unistd.h>
#include <boost/lexical_cast.hpp>
#include <boost/program_options.hpp>

#include "Error.h"
#include "Environment.h"

START_CORALSERVER_NAMESPACE

Environment::Environment()
{
    _vars["port"] = "8080";
    _vars["host"] = "localhost";
    _vars["ip"] = "0.0.0.0";
    _vars["coral_auth_path"] = "/afs/cern.ch/sw/lcg/app/pool/db";
    _vars["coral_dblookup_path"] = "/afs/cern.ch/sw/lcg/app/pool/db";
    _vars["thread_num"] = "10";
    _vars["disable_reporter"] = "0";
}

void Environment::setup(int argc, char** argv)
{
	namespace po = boost::program_options;
	po::options_description desc("Allowed options");

	desc.add_options()
	    ("help", "Display this help page")
	    ("port",  po::value<std::string>(), "Listening port")
	    ("thread_num", po::value<std::string>(), "Number of threads")
	    ("disable_reporter", po::value<std::string>(), "1: Disable the CORAL message reporter")
	;

	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);

	if (vm.count("help")) {
	    std::cout << desc << "\n";
	    exit(1);
	}

	if (vm.count("port"))
		_vars["port"] = vm["port"].as<std::string>();

	if (vm.count("thread_num"))
		_vars["thread_num"] = vm["thread_num"].as<std::string>();

	if (vm.count("disable_reporter"))
		_vars["disable_reporter"] = vm["disable_reporter"].as<std::string>();
}

const std::string& Environment::_get_str(const std::string& aVariable)
{
    _t_vars::iterator itr = _vars.find(aVariable);

    if (itr == _vars.end())
    	EXCEPTION_THROW(Err_Custom("Variable " + aVariable + " not defined."));
    // No worry, it will not be 0 pointer due to the exception
    return itr->second;
}

END_CORALSERVER_NAMESPACE
