/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include "MessageHandler.h"

START_CORALSERVER_NAMESPACE

namespace MessageHandlerSM {

using CORALMESSAGING_NAMESPACE_NAME::CoralSocket;
using CORALMESSAGING_NAMESPACE_NAME::message_cast;

void StateGeneral::sendNotConnected(const Pointer<CoralInputSocket>::Shared in, const std::string& desc)
{
	LOGGER_NEWLOG(coral::Info) << "Not connected" << commit;
	CORALMESSAGING_NAMESPACE_NAME::CTL::NotConnected msg(desc);
	CoralSocket csock(*in, msg);
	msg.write(csock);
}


sc::result StConnecting::react(const Event<CORALMESSAGING_NAMESPACE_NAME::CAL::Connect>& ev)
{	
	this->outermost_context().handler.reset(new CoralMessageHandler);
	Pointer<CORALMESSAGING_NAMESPACE_NAME::CAL::Connect>::Shared msg(message_cast<CORALMESSAGING_NAMESPACE_NAME::CAL::Connect>(ev.message));
	bool res = this->outermost_context().handler->handle(msg, ev.csock);
	
	return res ? this->transit<StConnected>() : this->transit<StUnconnected>();
}

sc::result StUnconnected::react(const Event<CORALMESSAGING_NAMESPACE_NAME::CAL::Connect>& ev)
{
	LOGGER_NEWLOG(coral::Info) << "Connect message arrived, connecting..." << commit;
	this->post_event(ev);
	return this->transit<StConnecting>();
}

sc::result StConnected::react(const Event<CORALMESSAGING_NAMESPACE_NAME::CAL::Connect>& ev)
{
	LOGGER_NEWLOG(coral::Info) << "Connect message arrived, connecting..." << commit;
	this->post_event(ev);
	return this->transit<StConnecting>();
}

sc::result StConnected::react(const Event<CORALMESSAGING_NAMESPACE_NAME::CAL::CloseSession>& ev)
{
	LOGGER_NEWLOG(coral::Info) << "CloseSession message arrived, unconnecting..." << commit;
	Pointer<CORALMESSAGING_NAMESPACE_NAME::CAL::CloseSession>::Shared msg(CORALMESSAGING_NAMESPACE_NAME::message_cast<CORALMESSAGING_NAMESPACE_NAME::CAL::CloseSession>(ev.message));
	this->outermost_context().handler->handle(msg, ev.csock);
	this->outermost_context().handler.reset(NULL);
	return this->transit<StUnconnected>();
}

} // namespace MessageHandlerSM


MessageHandler::MessageHandler() 
{
	_sm.initiate();
}
	
bool MessageHandler::handle(Pointer<IMessage>::Shared message, Pointer<CoralInputSocket>::Shared csock) 
{
	#define MESSAGE_HANDLER_GENERATOR(r, data, MSGNAME) \
		case coral::CoralProtocol::Opcode::BOOST_PP_CAT(E_, MSGNAME): \
			LOGGER_NEWLOG(coral::Info) << "Message " << message->name() << " is dispatched to handler" << commit; \
			_sm.process_event(MessageHandlerSM::Event<CORALMESSAGING_NAMESPACE_NAME::CAL::MSGNAME>(message, csock)); \
			break;
	
	DEBUG_ASSERT_PTR(message);
	
	switch (message->opcode()) {
		BOOST_PP_SEQ_FOR_EACH(MESSAGE_HANDLER_GENERATOR, ~, ALL_HANDLED_MESSAGES);
	default:
		// Ignore the message
		LOGGER_NEWLOG(coral::Info) << "Message " << message->name() << " is ignored" << commit;
	}
	
	#undef MESSAGE_HANDLER_GENERATOR
	return !terminated();
}
	
bool MessageHandler::terminated() const 
{
	return _sm.terminated();
}

END_CORALSERVER_NAMESPACE
