/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <boost/bind.hpp>

#include "Logger.h"
#include "Error.h"
#include "TcpSocketAcceptor.h"

START_CORALSERVER_NAMESPACE

Pointer<TcpSocket>::Shared TcpSocketAcceptor::Accept(const TcpSocket& passive)
{
	int fd = passive.getFd();
    struct sockaddr_in clientname;
    unsigned int size = sizeof (clientname);
    int newsock = accept (fd, (struct sockaddr *) &clientname, &size);   
    
    if (newsock < 0) {
        if (errno == EAGAIN) {
        	LOGGER_NEWLOG(coral::Error) << "accept error (EAGAIN): no connection waiting" << commit;
            return Pointer<TcpSocket>::Shared();
        }
        
        EXCEPTION_THROW (Err_System("Error accepting connection."));
    }
    
    LOGGER_NEWLOG(coral::Info) << "Connection from " << inet_ntoa(clientname.sin_addr) << commit;
    return Pointer<TcpSocket>::Shared(new TcpSocket(newsock));
}

END_CORALSERVER_NAMESPACE
