/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALSERVER_UTILS_H_
#define CORALSERVER_UTILS_H_

#include <iosfwd>
#include <vector>
#include <string>

#include "Pointers.h"

START_CORALSERVER_NAMESPACE

// Unary operator for an object pointed by a boost::shared_ptr
template <class OP>
class _UnaryForSharedPtr {
public:
	_UnaryForSharedPtr(const OP& op) : _op(op) {};
	
	typename OP::result_type operator () (const typename Pointer<typename OP::argument_type>::Shared a) {
		DEBUG_ASSERT_PTR(a);
		return _op(*a);
	}
	
private:
	OP _op;
};

template <class OP> _UnaryForSharedPtr<OP> _UnaryForSharedPtr_f (const OP& op) {
	return _UnaryForSharedPtr<OP>(op);
}

class NullType {};
struct EmptyType {};

std::ostream& operator << (std::ostream& os, const std::vector<std::string>& data);

END_CORALSERVER_NAMESPACE

#endif // #ifndef CORALSERVER_UTILS_H_
