/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <vector>

#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ISessionProperties.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IQuery.h"

#include "CoralMessaging/Stream/DataStringStream.h"
#include "CoralMessaging/CAL/CALMessages.h"

#include "UnitTests/gnuplot.h"
#include "../../src/CoralMessageHandler.h"
#include "QueryHandlerPerformance.h"

START_CORALSERVER_NAMESPACE

using namespace CORALMESSAGING_NAMESPACE_NAME;

void QueryHandlerPerformance::handleManyQueriesWithOracle()
{
	coral::ConnectionService connService;
	Pointer<coral::ISessionProxy>::Scoped s(connService.connect(OracleServiceName(), coral::ReadOnly));

	ISessionProxy& session = *s;
	session.transaction().start(true);

	TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds();

	for (int i = 0; i < _QueryNum(); ++i) {
		coral::ISchema& schema = session.nominalSchema();
		Pointer<coral::IQuery>::Scoped query(schema.newQuery());
		query->addToTableList(TableName());
		ICursor& cursor = query->execute();

		std::vector<CAL::MsgObj::AttributeListRepresentation> resultSet;

		while (cursor.next())
			resultSet.push_back(CAL::MsgObj::AttributeListRepresentation(cursor.currentRow()));

		if (!(i % 100))
			std::cout << i << "; " << std::flush;
	}

	std::cout << "\n\nExecution time of " << _QueryNum() << " queries with server: "
		<< (TimeStamp::now().total_nanoseconds() - start) / 1000 / 1000 / 1000 << " sec" << std::endl;

	session.transaction().commit();
}

void QueryHandlerPerformance::handleManyQueriesWithHandler()
{
	theLogger().setLogOff();
	CoralMessageHandler cmh;

	Pointer<Stream::DataStringStream>::Shared str(new Stream::DataStringStream);
	// Handle a Connect to set up the session
	CAL::MsgObj::Connection mo_conn(true, "oracle://lcg_coral_nightly/CORAL_WRITER");
	CAL::MsgObj::MonitoringInformation mi_conn;
	Pointer<CAL::Connect>::Shared msg_conn(new CAL::Connect(mo_conn, mi_conn));
	Pointer<CoralInputSocket>::Shared csock(new CoralInputSocket(str, 1, 1));
	// Handle a Connect to set up the session
	cmh.handle(msg_conn, csock);
	// Now the queries
	CAL::MsgObj::QueryRepresentation qr;
	typedef CAL::MsgObj::QueryRepresentationTypes::addToTableList repr_type;
	repr_type repr;
	repr.ptr = new repr_type::type(TableName(), "");
	qr.insertMethodCall(repr);

	Pointer<CAL::ExecuteQuery>::Shared query(new CAL::ExecuteQuery(qr));

	TimeStamp::ValueType start = TimeStamp::now().total_nanoseconds();

	for (int i = 0; i < _QueryNum(); ++i) {
		cmh.handle(query, csock);
		cmh.waitUntilRunning();

		if (!(i % 100))
			std::cout << i << "; " << std::flush;
	}

	std::cout << "\n\nExecution time of " << _QueryNum() << " queries: "
		<< (TimeStamp::now().total_nanoseconds() - start) / 1000 / 1000  / 1000 << " sec" << std::endl;
}

END_CORALSERVER_NAMESPACE
