/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <iostream>
#include <boost/program_options.hpp>

#include "TestTableCreator.h"
#include "QueryHandlerPerformance.h"

using namespace CORALSERVER_NAMESPACE_NAME;

int main(int argc, char ** argv)
{
	namespace po = boost::program_options;
	po::options_description desc("Allowed options");

	desc.add_options()
	    ("help", "Display this help page")
	    ("oracle", "Run the tests directly against the Oracle server")
	    ("ct", "Create the table")
	;

	po::variables_map vm;
	po::store(po::parse_command_line(argc, argv, desc), vm);
	po::notify(vm);

	if (vm.count("help")) {
	    std::cout << desc << "\n";
	    return 1;
	}

	if (vm.count("ct")) {
		std::cout << "Creating tables..." << std::endl;
		TestTableCreator::Create(QueryHandlerPerformance::OracleServiceName(), QueryHandlerPerformance::TableName());
	}

	QueryHandlerPerformance handler;

	if (vm.count("oracle")) {
		handler.handleManyQueriesWithOracle();
	} else {
		handler.handleManyQueriesWithHandler();
	}

	return 1;
}
