#include "SynchronizedQueueTest.h"

START_CORALSERVER_NAMESPACE

void SynchronizedQueueTest::elementOrder(void)
{
	_TestContext tc;
	CPPUNIT_ASSERT(tc.q().empty());
	CPPUNIT_ASSERT(!tc.q().full());
	CPPUNIT_ASSERT(tc.push(0));
	CPPUNIT_ASSERT(!tc.q().empty());
	CPPUNIT_ASSERT(!tc.q().full());
	CPPUNIT_ASSERT(tc.push(1));
	CPPUNIT_ASSERT(!tc.q().empty());
	CPPUNIT_ASSERT(tc.q().full());
	CPPUNIT_ASSERT_EQUAL(0, (int) tc.pop());
	CPPUNIT_ASSERT(!tc.q().empty());
	CPPUNIT_ASSERT(!tc.q().full());
	CPPUNIT_ASSERT_EQUAL(1, (int) tc.pop());
	CPPUNIT_ASSERT(tc.q().empty());
	CPPUNIT_ASSERT(!tc.q().full());
}

void SynchronizedQueueTest::blockOnFull(void)
{
	_TestContext tc;
	CPPUNIT_ASSERT(tc.push(0));
	CPPUNIT_ASSERT(tc.push(1));
	// Ok, now pop a value to release the blocked thread
	bool pushres = tc.push(2);
	// We must do it before the assert, because in case of assertion, the blocked
	// thread never joins, and the execution is blocked on the tc destructor.
	tc.q().pop();
	CPPUNIT_ASSERT(!pushres);
}

void SynchronizedQueueTest::blockOnEmpty(void)
{
	_TestContext tc;
	CPPUNIT_ASSERT(tc.push(0));
	CPPUNIT_ASSERT(tc.push(1));
	CPPUNIT_ASSERT_EQUAL(0, (int) tc.pop());
	CPPUNIT_ASSERT_EQUAL(1, (int) tc.pop());
	bool res = tc.isTimeout(tc.pop());
	// Ok, now push a value to release the blocked thread to avoid resource leakage
	// We must do it before the assert, because in case of assertion, the blocked
	// thread never joins, and the execution is blocked on the tc destructor.
	tc.q().push(_TestContext::queue_t::element_type(new _TestContext::queue_e_t(2)));
	CPPUNIT_ASSERT(res);
}

void SynchronizedQueueTest::notifyIfFullLostElement(void)
{
	_TestContext tc;
	CPPUNIT_ASSERT(tc.push(0));
	CPPUNIT_ASSERT(tc.push(1));
	// This thread should block
	CPPUNIT_ASSERT(!tc.push(2));
	// Now we pop an element, the previous thread should wake up and push 2 in the queue
	CPPUNIT_ASSERT_EQUAL(0, (int) tc.pop());
	CPPUNIT_ASSERT_EQUAL(1, (int) tc.pop());
	CPPUNIT_ASSERT_EQUAL(2, (int) tc.pop());
}

void SynchronizedQueueTest::notifyIfEmptyGotElement(void)
{
	_TestContext tc;
	bool res = tc.isTimeout(tc.pop());
	CPPUNIT_ASSERT(res);
	CPPUNIT_ASSERT(tc.push(0));
	CPPUNIT_ASSERT(tc.push(1));
	CPPUNIT_ASSERT(!tc.isTimeout(tc.pop()));
	// It must block again as the queue got empty
	//CPPUNIT_ASSERT(tc.isTimeout(tc.pop()));
}

END_CORALSERVER_NAMESPACE
