/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#ifndef CORALSERVER_TASKTEST_H_
#define CORALSERVER_TASKTEST_H_

#include "TestBase.h"

START_CORALSERVER_NAMESPACE

/** Unit test for the Task class */
class TaskTest : public CPPUNIT_NS :: TestFixture
{
    CPPUNIT_TEST_SUITE (TaskTest);
    /** Test if the task is able to execute its operation */
    CPPUNIT_TEST (testExecutingOperation);
    /** Test if the task is able to catch unhandled exceptions */
    CPPUNIT_TEST(testCatchingUnhandledException);
    CPPUNIT_TEST_SUITE_END ();

public:
    void setUp (void);
    void tearDown (void) {};

protected:
    void testExecutingOperation (void);
    void testCatchingUnhandledException (void);
    
private:
	/** The operation stores a value in an external (referenced) value */
	class TestOp {
	public:
		typedef int value_type;
		typedef enum enabled_value_type {INVALID_VALUE, VALUE_1, VALUE_2, VALUE_3};
		
		TestOp(value_type& ret, /* The external variable */
				int expected /* The value to be stored */) : _ret(ret), _exp(expected) {};
		
		void operator () () {
			_ret = _exp;
		};
		
		static value_type val(enabled_value_type e) {
			return static_cast<value_type>(e);
		}
		
	private:
		value_type& _ret;
		const value_type _exp;
	};
	
	/** The operation throws an unhandled exception */
	class TestOpThrows {
	public:	
		void operator () () {
			throw exception_type(1);
		}
		
		typedef int exception_type; 
	};
};

END_CORALSERVER_NAMESPACE

#endif /*CORALSERVER_TASKTEST_H_*/
