#ifndef THREADFUNCTION_H_
#define THREADFUNCTION_H_

#include "CoralBricks/ThreadTraits.h"

#include "TestBase.h"
#include "src/Pointers.h"

START_CORALSERVER_NAMESPACE

class ThreadFunction 
{
private:

	class ProgressCondition 
	{
	public:
		ProgressCondition() : _stopped(false) {};
		
		bool wait(const boost::xtime t) {
			ThreadTraits::LOCK lock(_mutex);
			bool ret = _stopped ? true : _cond.timed_wait(lock, t);
			return ret;
		};
		
		void finished() {
			ThreadTraits::LOCK lock(_mutex);
			_stopped = true;
			_cond.notify_all();
		}
		
	private:
		bool _stopped;
		mutable ThreadTraits::MUTEX _mutex;
		ThreadTraits::CONDITION _cond;
	};

public:
	ThreadFunction() : _pcond(new ProgressCondition) {};
	ThreadFunction(const ThreadFunction& tf) : _pcond(tf._pcond) {};
	virtual ~ThreadFunction() {};
	
	bool wait(const boost::xtime t) {
		return _pcond->wait(t);
	};
	
protected:
	void _finished() {
		_pcond->finished();
	}
	
	virtual void operator () () = 0;
	
private:
	Pointer<ProgressCondition>::Shared _pcond;
};

END_CORALSERVER_NAMESPACE

#endif /*THREADFUNCTION_H_*/
