/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */

#include <errno.h>
#include <fstream>
#include <sstream>

#ifdef WIN32
#else
#include <syslog.h>
#endif

#include "src/Logger.h"
#include "DaemonMsgReporter.h"

START_CORALSERVER_NAMESPACE

DaemonMsgReporter::DaemonMsgReporter()
	: coral::MsgReporter()
{
	setOutputLevel(coral::Verbose);
	openlog("CoralServer", LOG_USER, LOG_PID | LOG_PERROR);
}

DaemonMsgReporter::~DaemonMsgReporter()
{
	closelog();
}

void DaemonMsgReporter::report(const int level, const std::string& src, const std::string& msg)
{
	if (level < outputLevel()) return;
	const char* levelStr = "Unknown";
	switch (level)    
	{
#define SET(x)  case coral::x:  levelStr=#x;  break 
    	SET( Nil );
    	SET( Verbose );
    	SET( Debug );
    	SET( Info );
    	SET( Warning );
    	SET( Error );
    	SET( Fatal );
    case coral::Always:
    	levelStr="Success";
    	break;
    default:
    	levelStr="Unknown";
    	break;
#undef SET
	}

    syslog(_getSysLogLevel(level), (std::string(levelStr) + Logger::separator() + src 
                                    + Logger::separator() + msg + "\n").c_str());
}

int DaemonMsgReporter::_getSysLogLevel(const int level)
{
	switch (level) {
	case Verbose:
	case Debug:
		return LOG_DEBUG;
	case Warning:
		return LOG_WARNING;
	case Fatal:
		return LOG_ERR;
	default:
		return LOG_INFO;
	};
}

END_CORALSERVER_NAMESPACE
