#include <unistd.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/types.h>
#include <fcntl.h>
#include <syslog.h>

#include "src/Error.h"
#include "src/Logger.h"
#include "DaemonMsgReporter.h"
#include "daemonize.h"

START_CORALSERVER_NAMESPACE

// Handler of SIGCHLD
static void _handle_sigchld(int)
{
    int state;
    wait(&state);
}

// The usual Linux daemon start-up sequence,
void daemonize()
{
    struct sigaction reap_action;
    reap_action.sa_handler = _handle_sigchld;
    sigemptyset(&reap_action.sa_mask);
    reap_action.sa_flags = SA_RESTART; 

    if (sigaction(SIGCHLD, &reap_action, NULL) == -1) {
        EXCEPTION_THROW(Err_System());
    }
    
    int status = fork();   

    if (status < 0) {
        EXCEPTION_THROW(Err_System());
    }
    else if (status > 0) {
        exit(EXIT_SUCCESS);
    }

    close(STDIN_FILENO);
    close(STDOUT_FILENO);
    //close(STDERR_FILENO);

    if (setsid() < 0) {
        EXCEPTION_THROW(Err_System());
    }

    status = fork();

    if (status < 0) {
        EXCEPTION_THROW(Err_System());
    }
    else if (status > 0) {
        exit(EXIT_SUCCESS);
    }

    chdir("/");
    umask(0);
    int fileDesc = open("/dev/null", O_RDWR);/* stdin */
    (void) dup(fileDesc);  /* stdout */
    //(void) dup(fileDesc);  /* stderr */
    
    coral::MessageStream::installMsgReporter(new DaemonMsgReporter());
    LOGGER_NEWLOG(coral::Info) << "Daemon is initialized" << commit;
}

END_CORALSERVER_NAMESPACE
