/*
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 *
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include <signal.h>
#include <unistd.h>

#include "src/Error.h"
#include "src/Logger.h"
#include "src/Environment.h"
#include "CoralServer/CoralServer.h"
#include "daemonize.h"

using namespace CORALSERVER_NAMESPACE_NAME;

// Handler of SIGCHLD
void _handle_sigint(int)
{
    theServer().stop();
    kill(getpid(), 9);
    exit(EXIT_SUCCESS);
}

int main (int argc, char** argv)
{
	theLogger().setEnvironment("CoralServer");
	theEnvironment().setup(argc, argv);

    try {
        struct sigaction action;
        action.sa_handler = _handle_sigint;
        sigemptyset(&action.sa_mask);
        action.sa_flags = SA_RESTART;

        if (sigaction(SIGINT, &action, NULL) == -1) {
            EXCEPTION_THROW(Err_System());
        }

        //daemonize();
        LOGGER_NEWLOG(coral::Info) << "Starting server..." << commit;
        theServer().start();
    }
    catch(Err e)
    {
        std::string msg = "Unhandled exception, exiting...";
        LOGGER_NEWLOG(coral::Error) << msg << commit;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        std::string msg = "Unknown exception, exiting...";
        LOGGER_NEWLOG(coral::Error) << msg << commit;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

