#include "BulkOperationWithQuery.h"
#include "BulkOperation.h"
#include "QueryDefinition.h"

coral::FrontierAccess::BulkOperationWithQuery::BulkOperationWithQuery( const coral::FrontierAccess::ISessionProperties& properties,
                                                                     int cacheSize,
                                                                     const std::string& statement ):
  m_properties( properties ),
  m_rowsInCache( cacheSize ),
  m_sqlPrefix( statement ),
  m_queryDefinition( new coral::FrontierAccess::QueryDefinition( m_properties ) ),
  m_bulkOperation( 0 )
{}


coral::FrontierAccess::BulkOperationWithQuery::~BulkOperationWithQuery()
{
  if ( m_bulkOperation ) delete m_bulkOperation;
}


coral::IQueryDefinition&
coral::FrontierAccess::BulkOperationWithQuery::query()
{
  return *m_queryDefinition;
}


void
coral::FrontierAccess::BulkOperationWithQuery::processNextIteration()
{
  if ( ! m_bulkOperation ) {
    m_bulkOperation = new coral::FrontierAccess::BulkOperation( m_properties,
                                                              m_queryDefinition->bindData(),
                                                              m_rowsInCache,
                                                              m_sqlPrefix + " ( " + m_queryDefinition->sqlFragment() + " )" );
  }

  m_bulkOperation->processNextIteration();
}


void
coral::FrontierAccess::BulkOperationWithQuery::flush()
{
  if ( ! m_bulkOperation ) {
    m_bulkOperation = new coral::FrontierAccess::BulkOperation( m_properties,
                                                              m_queryDefinition->bindData(),
                                                              m_rowsInCache,
                                                              m_sqlPrefix + " ( " + m_queryDefinition->sqlFragment() + " )" );
  }
  m_bulkOperation->flush();
}
