#include "DataEditor.h"
#include "Statement.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "BulkOperation.h"
#include "OperationWithQuery.h"
#include "BulkOperationWithQuery.h"

#include "RelationalAccess/ITableDescription.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/SchemaException.h"
#include "RelationalAccess/RelationalServiceException.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"

#include "CoralKernel/Service.h"

#include <sstream>

coral::FrontierAccess::DataEditor::DataEditor( const coral::FrontierAccess::ISessionProperties& properties, const coral::ITableDescription& description )
  : m_properties( properties ), m_tableDescription( description )
{
}

void coral::FrontierAccess::DataEditor::rowBuffer( coral::AttributeList& buffer )
{
  int numberOfColumns = m_tableDescription.numberOfColumns();
  for ( int iColumn = 0; iColumn < numberOfColumns; ++iColumn )
  {
    const coral::IColumn& column = m_tableDescription.columnDescription( iColumn );
    buffer.extend( column.name(), column.type() );
  }
}

void coral::FrontierAccess::DataEditor::insertRow( const coral::AttributeList& /*dataBuffer*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::insertRow" );
}

coral::IOperationWithQuery* coral::FrontierAccess::DataEditor::insertWithQuery()
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::insertWithQuery" );
  coral::IOperationWithQuery* op = 0;
  return op;
}

coral::IBulkOperation* coral::FrontierAccess::DataEditor::bulkInsert( const coral::AttributeList& /*dataBuffer*/, int /*rowCacheSize*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::bulkInsert" );
  coral::IBulkOperation* op = 0;
  return op;
}

coral::IBulkOperationWithQuery* coral::FrontierAccess::DataEditor::bulkInsertWithQuery( int /*dataCacheSize*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::bulkInsertWithQuery" );
  coral::IBulkOperationWithQuery* op = 0;
  return op;
}

long coral::FrontierAccess::DataEditor::updateRows( const std::string& /*setClause*/, const std::string& /*condition*/, const coral::AttributeList& /*inputData*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::updateRows" );
  return 0;
}

coral::IBulkOperation* coral::FrontierAccess::DataEditor::bulkUpdateRows( const std::string& /*setClause */, const std::string& /*condition*/, const coral::AttributeList& /*inputData*/, int /*dataCacheSize*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::bulkUpdateRows" );
  coral::IBulkOperation* op = 0;
  return op;
}

long coral::FrontierAccess::DataEditor::deleteRows( const std::string& /*condition*/, const coral::AttributeList& /*conditionData*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::deleteRows" );
  return 0;
}

coral::IBulkOperation* coral::FrontierAccess::DataEditor::bulkDeleteRows( const std::string& /*condition*/, const coral::AttributeList& /*conditionData*/, int /*dataCacheSize*/ )
{
  throw coral::InvalidOperationException( m_properties.domainProperties().service()->name(), "Frontier does not support DML operations", "ITableDataEditor::bulkDeleteRows" );
  coral::IBulkOperation* op = 0;
  return op;
}
