// $Id: Table.cpp,v 1.3 2008/04/03 21:42:02 rado Exp $
#include "Table.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "TableDescriptionProxy.h"
#include "DataEditor.h"
#include "PrivilegeManager.h"
#include "Query.h"

#include "RelationalAccess/SessionException.h"

#include "CoralKernel/Service.h"

coral::FrontierAccess::Table::Table( const coral::FrontierAccess::ISessionProperties& sessionProperties, const std::string& tableName )
  : m_sessionProperties( sessionProperties )
  , m_descriptionProxy( new coral::FrontierAccess::TableDescriptionProxy( sessionProperties, tableName ) )
  , m_privilegeManager( new coral::FrontierAccess::PrivilegeManager( sessionProperties, sessionProperties.schemaName() + ".\"" + tableName + "\"" ) )
  , m_dataEditor( new coral::FrontierAccess::DataEditor( sessionProperties, *m_descriptionProxy ) )
{
}

coral::FrontierAccess::Table::~Table()
{
  delete m_descriptionProxy;
}

const coral::ITableDescription& coral::FrontierAccess::Table::description() const
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "coral::FrontierAccess::Table::description" );
  return static_cast< const coral::ITableDescription& >( *m_descriptionProxy );
}

coral::ITableSchemaEditor& coral::FrontierAccess::Table::schemaEditor()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "coral::FrontierAccess::Table::schemaEditor" );
  return static_cast< coral::ITableSchemaEditor& >( *m_descriptionProxy );
}

coral::ITableDataEditor& coral::FrontierAccess::Table::dataEditor()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "coral::FrontierAccess::Table::dataEditor" );
  return *m_dataEditor;
}

coral::ITablePrivilegeManager& coral::FrontierAccess::Table::privilegeManager()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "coral::FrontierAccess::Table::privilegeManager" );
  return *m_privilegeManager;
}

coral::IQuery* coral::FrontierAccess::Table::newQuery() const
{
  return new coral::FrontierAccess::Query( m_sessionProperties, m_descriptionProxy->name() );
}
