#include "TestBase.h"

#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IConnectionService.h"

#include "CoralKernel/Context.h"

#include <stdexcept>

TestBase::TestBase()
{
  coral::Context& ctx = coral::Context::instance();
  
  ctx.loadComponent( "CORAL/RelationalPlugins/frontier" );
  ctx.loadComponent( "CORAL/RelationalPlugins/oracle" );
  
  if( getenv( "CORAL_AUTH_PATH" ) )
    ctx.loadComponent( "CORAL/Services/XMLAuthenticationService" );
  else
    ctx.loadComponent( "CORAL/Services/EnvironmentAuthenticationService" );
  
  ctx.loadComponent( "CORAL/Services/ConnectionService" );
}

TestBase::~TestBase()
{
}

coral::IConnection* connection = 0;

coral::ISession* TestBase::connect( const std::string& connectionString, const std::string& userName, const std::string& password, coral::AccessMode mode  )
{
  coral::Context& ctx = coral::Context::instance();
  
  std::pair<std::string,std::string> cs;
  
  if( connectionString.find( "oracle" ) != std::string::npos )
  {
    coral::IHandle<coral::IRelationalDomain> iHandle = ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/oracle" );

    if ( ! iHandle.isValid() )
    {
      throw coral::NonExistingDomainException( "oracle" );
    }

    cs = iHandle->decodeUserConnectionString( connectionString );
    
    connection = iHandle->newConnection( cs.first );
  }
  else if( connectionString.find( "frontier" ) != std::string::npos )
  {
    coral::IHandle<coral::IRelationalDomain> iHandle = ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/frontier" );

    if ( ! iHandle.isValid() )
    {
      throw coral::NonExistingDomainException( "frontier" );
    }

    cs = iHandle->decodeUserConnectionString( connectionString );
    
    connection = iHandle->newConnection( cs.first );
  }

  connection->connect();
  
  coral::ISession* session = connection->newSession( cs.second, mode );
  
  if ( session )
  {
    session->startUserSession( userName, password );
  }

  return session;
}

void TestBase::disconnect()
{
  if( connection->isConnected() )
    connection->disconnect();

  delete connection;
  connection = 0;
}

coral::ITypeConverter& TestBase::typeConverter() const 
{
  return connection->typeConverter();
}

coral::IHandle<coral::IConnectionService> TestBase::connectionService()
{
  coral::Context& ctx = coral::Context::instance();
  
  coral::IHandle<coral::IConnectionService> svc = ctx.query<coral::IConnectionService>();

  if( ! svc.isValid() )
  {
    throw std::runtime_error( "Could not locate the connection service" );
  }

  return svc;
}
