#include "LFCDbLookupService.h"
#include <iostream>
#include <fstream>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <uuid/uuid.h>
#include <unistd.h>
#include <lfc_api.h>
#include <serrno.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>

extern char *getenv();
using namespace std;
static const std::string BASE_DIR="/database";

// Constructor

LFCDbLookupService::LFCDbLookupService::LFCDbLookupService()
{	
	guid="";
	m_data.clear();
}

std::string LFCDbLookupService::LFCDbLookupService::baseDir(){
  char* userBaseDir = ::getenv("CORAL_LFC_BASEDIR");
  if(userBaseDir){
    return std::string(userBaseDir);
  } else{
    return BASE_DIR;
  }
}


// Changes the directory to /database  

int LFCDbLookupService::LFCDbLookupService::chDir()
{
	if(lfc_chdir(baseDir().c_str()) < 0)
		throw LFCDbLookupException("Could not change the current directory to '"+baseDir()+"'", serrno);

	return 0;
}

/* Parse the logical connection string */
std::string LFCDbLookupService::LFCDbLookupService::parseLFN(std::string &logicalConnectionString)
{
	string dirPath("");
        if ( logicalConnectionString.substr(0,1) == "/")
        {
                if (logicalConnectionString.substr(0,baseDir().size()) != baseDir())
               	{
                       	dirPath = baseDir();
                       	dirPath.append(logicalConnectionString);
                       	logicalConnectionString = dirPath;
               	}
        }
        else
        {
               	dirPath=baseDir();
               	dirPath.append("/");
               	dirPath.append(logicalConnectionString);
               	logicalConnectionString=dirPath;
        }

	return logicalConnectionString;	
}

/* Checks the existence of logical connection string in the specfied path in LFC directory /database.  
If the logical connection string exists, the corresponding guid is stored in class variable and function returns true, 
otherwise false is returned. */

bool LFCDbLookupService::LFCDbLookupService::checkLFN( std::string &logicalConnectionString)
{
	// Returns the guid of the specified logical connection string 

	struct lfc_filestatg statg;
	string message;
	
	try
	{
		logicalConnectionString=parseLFN(logicalConnectionString);
		
		if (lfc_statg(logicalConnectionString.c_str(), NULL, &statg) < 0)
		{
			message.append("Logical connection string '");
			message.append(logicalConnectionString.substr(baseDir().size()+1));
			message.append("' does not exist.");
			throw LFCDbLookupException(message,serrno);
		}
		else
		{
			guid.assign(statg.guid);
			return true;
		}
  	}
  	catch (LFCDbLookupException echeckLFN)
  	{
		echeckLFN.showLFCErrorMessage();
		return false;
  	} 
}

/* If the logical connection string does not exist in the specified path in /database LFC directory, then Logical File Name is
created  with a unique guid which is stored in Cns_file_metadata LFC Table */  

int LFCDbLookupService::LFCDbLookupService::createLFN(std::string &logicalConnectionString)
{
	string message;
	struct lfc_filestatg statg;
	const char *lfnGuid;

	try
	{
		logicalConnectionString=parseLFN(logicalConnectionString);

		if (lfc_statg(logicalConnectionString.c_str(), NULL, &statg) < 0)
		{
			if(serrno==ENOENT)
			{
				Guid lfcGuid;
				Guid::create(lfcGuid);
				guid=lfcGuid.toString();
				lfnGuid=guid.c_str();

				if (lfc_creatg(logicalConnectionString.c_str(), lfnGuid, 755) < 0)
					throw LFCDbLookupException("Cannot add the logical connection string to LFC Database.",serrno);
				return 0;
			}
			else
			{
				message.append("GUID cannot be obtained for this Logical connection string: '");
				message.append(logicalConnectionString.substr(baseDir().size()+1));
				message.append("'");
				throw LFCDbLookupException(message,serrno);
			}
		}
		else
		{
			guid.assign(statg.guid);
			return 0;
		}
	   	
	}
	catch (LFCDbLookupException ecreateLFN)
	{
		ecreateLFN.showLFCErrorMessage();
		return -1;
	}
}

// Adds a new replica for the specified logical connection string.
	
int LFCDbLookupService::LFCDbLookupService::addReplica(const std::string &contactString,  std::string &logicalConnectionString, const std::string &physicalReplica, std::string accessMode="", std::string aMethod="")
{
	const char *tguid;
	std::string message("");
	const char *errmsg;

	if(chDir()!=0)
		return -1;

	// Starts a transaction 
	if (lfc_starttrans(::getenv("LFC_HOST"), const_cast<char*>("Starts transaction mode...")) < 0)
		throw LFCDbLookupException("Could not start the transaction.", serrno);

	if(createLFN(logicalConnectionString)!=0)
	{
		lfc_aborttrans();	
		return -1;
	}
	
	tguid=guid.c_str();
	
	/* Access mode is stored in poolname and authentication mechanism is stored in fs in Cns_file_replica table.
	By default access mode is set as Update and authentication mechanism as password.*/

	if (accessMode==""||accessMode.empty())
		accessMode="update";

	if (aMethod==""||aMethod.empty())
		aMethod="password";

	if (lfc_addreplica(tguid, NULL, contactString.c_str(), physicalReplica.c_str(), 'P','P' , accessMode.c_str(), aMethod.c_str()) < 0 )
	{
		if(serrno==17)
			errmsg="Physical Replica Already exists";
		else
			errmsg=sstrerror(serrno);

		message.append("Could not add the replica. ");
		message.append(errmsg);
		lfc_aborttrans();
		throw LFCDbLookupException(message,serrno);
		return -1;
	}
	else
		std::cout<<"Physical Replica Added Successfully"<<std::endl;

	lfc_endtrans();
	return 0;
}
	
// Removes an existing replica mapped to a logical connection string. Optional filter - physical replica 
	  		
int  LFCDbLookupService::LFCDbLookupService::deleteReplica( std::string &logicalConnectionString, const std::string &physicalReplica="")
{
	int flags=0;
        int result=0;
       	int numberOfReplicas = 0;
	std::string message("");
	const char *tguid, *errmsg;

	if(chDir()!=0)
		return -1;

	if(!checkLFN(logicalConnectionString))
	{
		lfc_endtrans();
		return -1;
	}

	// Starts a transaction 
	if (lfc_starttrans(::getenv("LFC_HOST"), const_cast<char*>("Starts transaction mode...")) < 0)
		throw LFCDbLookupException("Could not start the transaction.", serrno);

	tguid=guid.c_str();
	
	std::cout<<"Logical Connection String :"<<logicalConnectionString.substr(baseDir().size()+1)<<std::endl;

	struct lfc_filereplica *lfcFileReplica;
	lfc_list lfcList;
	flags=CNS_LIST_BEGIN;

	// Deletes all replica entries for a specified logical connection string 
	if(physicalReplica=="" || physicalReplica.empty())
	{
		// flag to check if any replica exists for the logical connection string 
		bool replicaflag=false; 

		while((lfcFileReplica = lfc_listreplica(logicalConnectionString.c_str(), NULL, flags, &lfcList)) != NULL)
		{
			replicaflag=true;
			std::cout<<"Deleting Replica  : "<<lfcFileReplica->sfn<<" "<<lfcFileReplica->poolname<<" "<<lfcFileReplica->fs<<std::endl;
			flags=CNS_LIST_CONTINUE;

			if (lfc_delreplica(tguid, NULL, lfcFileReplica->sfn) < 0 ){
				lfc_aborttrans();
				throw LFCDbLookupException("Could Not Delete the Replica.",serrno);
				return -1;
			}
		}
				
		// Final call to terminate list and free the resources. Refer lfc_listreplica man pages. 
		(void)   lfc_listreplica(logicalConnectionString.c_str(),  NULL,  CNS_LIST_END, &lfcList);

		if (replicaflag == false)
		{
			std::cout<<"No replicas exist for this logical connection string"<<std::endl;
			lfc_aborttrans();
			return -1;
		}
		else
		{
			std::cout<<"All replicas deleted, now deleting the logical connection string"<<std::endl;
			lfc_unlink(logicalConnectionString.c_str());
		} 
	}
	else
	// Deletes specified replica entry 
	{
		if (lfc_delreplica(tguid, NULL, physicalReplica.c_str()) < 0 )
		{
			if(serrno==2)
				errmsg="Physical Replica does not exist";
			else
   	 			errmsg=sstrerror(serrno);

			lfc_aborttrans();
			throw LFCDbLookupException("Could not delete the replica.",serrno);
			return -1;
		}
		else
		{
			std::cout<<"Replica deleted Successfully"<<std::endl;

        		struct lfc_filereplica *replicaEntries;

        		result=lfc_getreplica(logicalConnectionString.c_str(), NULL, NULL,  &numberOfReplicas, &replicaEntries);

			if (numberOfReplicas == 0)
			{
				std::cout<<"No more replicas associated, hence deleting the logical connection string"<<std::endl;
				lfc_unlink(logicalConnectionString.c_str());
			}
			free (replicaEntries);
		}
	}
	lfc_endtrans();
	return 0;
}

// Reads the /database LFC directory for listing available Replicas for a logical connection string. 
int LFCDbLookupService::LFCDbLookupService::readDirList( const char *currentDir, std::string &accessMode)
{
	std::string logicalConnectionString("");
        char *pathName=NULL;

        lfc_DIR *lfcDir;
        struct lfc_direnstatg *dent;

        if((lfcDir=lfc_opendir(currentDir))== NULL)
                std::cout<<"Could not open '"<<currentDir<<"' directory for listing of replicas"<<std::endl;

        serrno=0;

        while ((dent=lfc_readdirg(lfcDir)))
        {
                pathName=new char[strlen(currentDir)+strlen(dent->d_name)+2];
                sprintf(pathName, "%s/%s",currentDir, dent->d_name);

                if (S_ISDIR(dent->filemode))
                        readDirList(pathName, accessMode);
                else
                {
                        logicalConnectionString.assign(pathName);
                        createReplicaMap(logicalConnectionString);
                        showReplicaList(logicalConnectionString, accessMode);
                        delete pathName;
                }
        }
        if (serrno!=0)
        {
                if(serrno== SECOMERR ||serrno == ENSNACT)
                        throw LFCDbLookupException("Could not list the replicas.", serrno);
        }
        lfc_closedir(lfcDir);
	return 0;
}

int LFCDbLookupService::LFCDbLookupService::readDirXML( const char *currentDir, std::string &accessMode, fstream& dbCatalogue)
{
	std::string logicalConnectionString("");
        char *pathName=NULL;

        lfc_DIR *lfcDir;
        struct lfc_direnstatg *dent;

        if((lfcDir=lfc_opendir(currentDir))== NULL)
                std::cout<<"Could not open '"<<currentDir<<"' directory for listing of replicas"<<std::endl;

        serrno=0;

        while ((dent=lfc_readdirg(lfcDir)))
        {
                pathName=new char[strlen(currentDir)+strlen(dent->d_name)+2];
                sprintf(pathName, "%s/%s",currentDir, dent->d_name);

                if (S_ISDIR(dent->filemode))
                        readDirXML(pathName, accessMode,dbCatalogue);
                else
                {
                        logicalConnectionString.assign(pathName);
                        createReplicaMap(logicalConnectionString);
			writeXMLFile(logicalConnectionString, accessMode,dbCatalogue);
                        delete pathName;
                }
        }
        if (serrno!=0)
        {
                if(serrno== SECOMERR ||serrno == ENSNACT)
                        throw LFCDbLookupException("Could not list the replicas.", serrno);
        }
        lfc_closedir(lfcDir);
	return 0;
}

//Creates a map of Replica Entries and stores them in vector
int LFCDbLookupService::LFCDbLookupService::createReplicaMap( std::string &logicalConnectionString)
{
	int flags =0;
	string  logicalServiceName;

	struct lfc_filereplica *lfcFileReplica;
	lfc_list lfcList;
	flags=CNS_LIST_BEGIN;

	if(!checkLFN(logicalConnectionString))
	{
		lfc_endtrans();
		return -1;
	}
       	else
	{
		m_data[logicalConnectionString]= vector<LFCDbLookupService::LFCDbLookupService::DbEntry >();
 		vector<LFCDbLookupService::LFCDbLookupService::DbEntry >& vectorOfEntries = m_data[logicalConnectionString];
		while((lfcFileReplica = lfc_listreplica(logicalConnectionString.c_str(), NULL, flags, &lfcList)) != NULL)
		{
			flags=CNS_LIST_CONTINUE;
			vectorOfEntries.push_back(LFCDbLookupService::LFCDbLookupService::DbEntry(lfcFileReplica->sfn,lfcFileReplica->fs,lfcFileReplica->poolname));
		}

		// Final call to terminate list and free the resources. Refer lfc_listreplica man pages. 
		(void)   lfc_listreplica(logicalConnectionString.c_str(),  NULL,  CNS_LIST_END, &lfcList);
	}
	return 0;
}

// Displays available replicas from the map. Optional filters -  readonly, update, authentication method 

void LFCDbLookupService::LFCDbLookupService::showReplicaList(const std::string &logicalConnectionString, std::string& accessMode)
{
	map <string, vector<LFCDbLookupService::LFCDbLookupService::DbEntry > >::const_iterator iLogicalEntry;
	iLogicalEntry = m_data.find(logicalConnectionString);
	const vector<LFCDbLookupService::LFCDbLookupService::DbEntry >& EntryVector = iLogicalEntry->second;
	
	std::cout<<"\nReplica list for: "<<logicalConnectionString.substr(baseDir().size()+1)<<std::endl;

	if (EntryVector.size() == 0)
		std::cout<<"No replicas for the given logical connection string."<<std::endl;

	if (accessMode == ""||accessMode.empty())
	{
		for(vector<LFCDbLookupService::LFCDbLookupService::DbEntry>::const_iterator iEntry = EntryVector.begin(); iEntry != EntryVector.end(); ++iEntry)
		{
			const LFCDbLookupService::LFCDbLookupService::DbEntry& Entry = *iEntry;
			std::cout<<Entry.physicalReplica<<", "<<Entry.authenticationMechanism<<", "<<Entry.accessMode<<std::endl;
		}
	}
	else
	{
		for(vector<LFCDbLookupService::LFCDbLookupService::DbEntry>::const_iterator iEntry = EntryVector.begin(); iEntry != EntryVector.end(); ++iEntry)
		{
			const LFCDbLookupService::LFCDbLookupService::DbEntry& Entry = *iEntry;
			if ((Entry.accessMode=="readonly") || (accessMode==Entry.accessMode))
				std::cout<<Entry.physicalReplica<<", "<<Entry.authenticationMechanism<<", "<<Entry.accessMode<<std::endl;
		}
	}
}
		
// Exports available replicas from the map to an XML File. Optional filters -  readonly, update, authentication method 

void LFCDbLookupService::LFCDbLookupService::writeXMLFile(const std::string &logicalConnectionString, std::string& accessMode,fstream& dbCatalogue)
{
	map <string, vector<LFCDbLookupService::LFCDbLookupService::DbEntry > >::const_iterator iLogicalEntry;
	iLogicalEntry = m_data.find(logicalConnectionString);
	const vector<LFCDbLookupService::LFCDbLookupService::DbEntry >& EntryVector = iLogicalEntry->second;

	if (EntryVector.size() == 0)
		std::cout<<"No replicas for the logical connection string '"<<logicalConnectionString.substr(baseDir().size()+1)<<"'."<<std::endl;

	// To check the existence of replica according to access mode
	else
	{
  		dbCatalogue<<"<logicalservice name=\""<<logicalConnectionString.substr(baseDir().size()+1)<<"\">\n";
		if (accessMode == "" || accessMode.empty())
		{
			for(vector<LFCDbLookupService::LFCDbLookupService::DbEntry>::const_iterator iEntry = EntryVector.begin(); iEntry != EntryVector.end(); ++iEntry)
			{
				const LFCDbLookupService::LFCDbLookupService::DbEntry& Entry = *iEntry;
				dbCatalogue<<"<service name=\""<<Entry.physicalReplica<<"\" accessMode=\""<<Entry.accessMode<<"\" authentication=\""<<Entry.authenticationMechanism<<"\" />\n";
			}
		}
		else
		{
			for(vector<LFCDbLookupService::LFCDbLookupService::DbEntry>::const_iterator iEntry = EntryVector.begin(); iEntry != EntryVector.end(); ++iEntry)
			{
				const LFCDbLookupService::LFCDbLookupService::DbEntry& Entry = *iEntry;
				if (accessMode==Entry.accessMode)
					dbCatalogue<<"<service name=\""<<Entry.physicalReplica<<"\" accessMode=\""<<Entry.accessMode<<"\" authentication=\""<<Entry.authenticationMechanism<<"\" />\n";
			}
		}
        	dbCatalogue<<"</logicalservice>\n\n";
	}
}
 
// Lists Available Replica. Optional filters -  Logical Connection String, readonly, update.
 
int LFCDbLookupService::LFCDbLookupService::listReplica(std::string& logicalConnectionString, std::string& accessMode)
{
	if(chDir()!=0)
		return -1;

	// Starts a transaction 
	if (lfc_starttrans(::getenv("LFC_HOST"), const_cast<char*>("Starts transaction mode...")) < 0)
		throw LFCDbLookupException("Could not start the transaction.", serrno);

	// Lists all replica entries for all logical connection strings 
	if(logicalConnectionString == "" || logicalConnectionString.empty())
	{
		logicalConnectionString=baseDir();
		readDirList(const_cast<char*>(logicalConnectionString.c_str()),accessMode);
	}
	else
	// Lists replica entries for the specified logical connection string 
	{
		if(!checkLFN(logicalConnectionString))
		{
			lfc_endtrans();
			return -1;
		}
		else
		{
			createReplicaMap(logicalConnectionString);	
			showReplicaList(logicalConnectionString, accessMode);
		}
	}
	lfc_endtrans();
	return 0;
}
	
/* Export the dbcatalogue. Extracts subset of dbreplica catalogue creating  xml files in the format like dblookup.xml.  Optional filters - filename, logical connection string, readonly or update */

int LFCDbLookupService::LFCDbLookupService::exportDbCatalogue(std::string& fileName, std::string& logicalConnectionString, std::string& accessMode)
{
	if(chDir()!=0)
		return -1;
	
	if (fileName==""||fileName.empty())
		fileName="dblookup.xml";

	fstream dbCatalogue;
	dbCatalogue.open(fileName.c_str(),ios::out|ios::trunc);

	if(!dbCatalogue)
		throw LFCDbLookupException("Could not open the file for writing.",errno);

	// Starts a transaction 
	if (lfc_starttrans(::getenv("LFC_HOST"), const_cast<char*>("Starts transaction mode...")) < 0)
		throw LFCDbLookupException("Could not start the transaction.", serrno);

	// Exports replica entries for all logical connection strings to XML file 
	if(logicalConnectionString == "" || logicalConnectionString.empty())
	{

		dbCatalogue<<"<?xml version=\"1.0\" ?>\n";
		dbCatalogue<<"<servicelist>\n\n";

		logicalConnectionString=baseDir();
                readDirXML(const_cast<char*>(logicalConnectionString.c_str()),accessMode,dbCatalogue);
	}
	else
	// Exports replica entries for the specified logical connection string to XML file 
	{
		if(!checkLFN(logicalConnectionString))
		{
				lfc_endtrans();	
				return -1;
		}
        	else
		{
			if (createReplicaMap(logicalConnectionString) == 0)
			{	
				dbCatalogue<<"<?xml version=\"1.0\" ?>\n";
				dbCatalogue<<"<servicelist>\n\n";
				writeXMLFile(logicalConnectionString, accessMode,dbCatalogue);
			}
		}
	}

	dbCatalogue<<"</servicelist>\n";
	dbCatalogue.close();
	std::cout<<"DB Replica catalogue exported to : "<<fileName<<std::endl;
	lfc_endtrans();
	return 0;
}

// Destructor of LFCDbLookupService
LFCDbLookupService::LFCDbLookupService::~LFCDbLookupService()
{
	m_data.clear();
}

// Function definitions of Guid class.  These functions are taken from pool->LFCCatalogue for GUID generation 

static const char* fmt_Guid ="%08lX-%04hX-%04hX-%02hhX%02hhX-%02hhX%02hhX%02hhX%02hhX%02hhX%02hhX";

void LFCDbLookupService::Guid::create(Guid& guid)
{
	uuid_t me_;
	::uuid_generate_time(me_);
	unsigned int  *d1=(unsigned int*)me_;
	unsigned short *d2=(unsigned short*)(me_+4);
	unsigned short *d3=(unsigned short*)(me_+6);
	guid.Data1 = *d1;
	guid.Data2 = *d2;
	guid.Data3 = *d3;
	for (int i=0; i<8; i++){
		guid.Data4[i]=me_[i+8];
  	}
}

const std::string LFCDbLookupService::Guid::toString() const
{
	char text[128];
	::sprintf(text, fmt_Guid,
	Data1, Data2, Data3, 
	Data4[0], Data4[1], Data4[2], Data4[3], 
	Data4[4], Data4[5], Data4[6], Data4[7]);
	return text;
}

// Automatic conversion to string representation 
const LFCDbLookupService::Guid& LFCDbLookupService::Guid::fromString(const std::string& source)
{
	unsigned char d[8];
	::sscanf( source.c_str(), fmt_Guid, &Data1, &Data2, &Data3, 
            &Data4[0], &Data4[1], &Data4[2], &Data4[3], &d[0], &d[1], &d[2], &d[3]);
	*(int*)&Data4[4] = *(int*)d;
	return *this;
}

bool LFCDbLookupService::Guid::operator < (const Guid& g)  const  
{
	return ::memcmp(&g.Data1, &Data1, 16) < 0;
}

LFCDbLookupService::Guid::Guid& LFCDbLookupService::Guid::operator=(const Guid& g)
{
	if ( this != &g )
	{
		Data1 = g.Data1;
		Data2 = g.Data2;
		Data3 = g.Data3;
               unsigned int       *p = (unsigned int*)&Data4[0];
               const unsigned int *q = (const unsigned int*)&g.Data4[0];
               *(p+1) = *(q+1);
               *p     = *q;
        }
        return *this;
}
 bool LFCDbLookupService::Guid::operator==(const Guid& g)  const  
{
        if ( this != & g )  
	{
            if (Data1 != g.Data1 ) return false;
            if (Data2 != g.Data2 ) return false;
            if (Data3 != g.Data3 ) return false;
            const unsigned int *p = (const unsigned int*)&Data4[0],
            *q = (const unsigned int*)&g.Data4[0];
            return *p++ == *q++ && *p == *q;
         }
         return true;
}

bool LFCDbLookupService::Guid::operator!=(const Guid& g)  const  
{
	return !(this->operator==(g));
}

// Function definitions for LFCDbLookupException class 

// Constructor
LFCDbLookupService::LFCDbLookupException::LFCDbLookupException()
{
	lfcErrorMessage="";
	lfcErrorNo=0;
}
LFCDbLookupService::LFCDbLookupException::LFCDbLookupException(std::string errMessage, int& errNo)
{
	lfcErrorMessage=errMessage;
	lfcErrorNo=errNo;
}
void LFCDbLookupService::LFCDbLookupException::showLFCErrorMessage(void)
{
	std::cout<<"Error No: "<<lfcErrorNo;
	std::cout<<" : "<<lfcErrorMessage<<std::endl;
}
LFCDbLookupService::LFCDbLookupException::~LFCDbLookupException()
{
	lfcErrorMessage="";
	lfcErrorNo=0;
}
