#ifndef LFCDBLOOKUP_SERVICE_H
#define LFCDBLOOKUP_SERVICE_H 1

#include <string>
#include <map>
#include <vector>

using namespace std;

namespace LFCDbLookupService 
	{

	class LFCDbLookupService
	{

	//Private Variables

	private:
		// Stores the GUID for a LFC file entry (logical service name in this context)
		string guid;

    static std::string baseDir();
		
	public:
		
		// Constructor 
		LFCDbLookupService();

		// Changes the directory to /database and starts a transaction. On success returns 0, otherwise returns -1 
		int chDir();
			
		/* Checks the existence of the logical connection string in  LFC directory /database.
		If the service name exists, the corresponding guid is stored in class variable and function 
		returns true, otherwise false is returned. */
		bool checkLFN( std::string&);
		
		/* If the logical connection string does not exist in /database LFC directory, then LFN is
         	created  with a unique guid which is stored in Cns_file_metadata LFC Table */
		int createLFN( std::string&);
		
		/* Parse the logical connection string to absolute path */
		std::string parseLFN(std::string&);
		
		// Adds a new replica for the specified logical connection string
		int addReplica(const std::string&, std::string&, const std::string&, std::string , std::string);
		
       		// Removes an existing replica mapped to a logical connection string. 
		int deleteReplica( std::string&,const std::string&);
		
		// Lists the available Replicas.    
		int listReplica(std::string&, std::string&);
                                          
		// Creates a map of all available Replicas for a logical connection string.    
		int createReplicaMap( std::string&);

		// Lists all available replicas recursively.    
		int readDirList(const char *, std::string&);

		int readDirXML(const char *, std::string&, fstream&);

		// Lists all available Replicas for a logical connection string.    
		void showReplicaList(const std::string& , std::string&);

		// Exports the dbcatalogue. Extracts subset of dbreplica catalogue creating xml files in the format like dblookup.xml. 
		int exportDbCatalogue(std::string&, std::string& , std::string&);

		// Writes all available Replicas for a logical connection string to XML file.    
		void writeXMLFile(const std::string& , std::string&, fstream&);

		// Standard Destructor
		~LFCDbLookupService();
		
		// Class used to store replicas of a logical connection string in a vector.
		class DbEntry {
		public:
			DbEntry(string _physicalReplica, string _authenticationMechanism, string _accessMode=""):physicalReplica(_physicalReplica),authenticationMechanism(_authenticationMechanism), accessMode(_accessMode){}
			string physicalReplica;
		        string authenticationMechanism;
			string accessMode;
		};

		// A map container for storing logical connection strings and their corresponding replicas.
		map<string,vector< DbEntry > > m_data;
		
	};

        // Class for Guid Creation. This is taken from pool->LFCCatalogue for GUID generation. 
  	class Guid 		
	{	
  		public:
    			unsigned int  Data1;
    			unsigned short Data2;
    			unsigned short Data3;
    			unsigned char  Data4[8];

    			/// Standard constructor (No initialization of data for performance reasons)
    			Guid()             {                                 }
    			/// Standard constructor (With possible initialization)
    			explicit Guid(bool assign)  { if ( assign ) create(*this);    }
    			/// Constructor for Guid from char*
    			explicit Guid(const char* s)        { fromString(s);          }
    			/// Constructor for Guid from string
    			explicit Guid(const std::string& s) { fromString(s);          }
    			/// Copy constructor
    			Guid(const Guid& c)                 { *this = c;              }
    			/// Assignment operator
    			Guid& operator=(const Guid& g);
		
			/// Smaller operator
			bool operator<(const Guid& g)  const;
			/// Equality operator
			bool operator==(const Guid& g)  const ; 

			/// Non-equality operator
			bool operator!=(const Guid& g)  const;  

			/// Automatic conversion from string reprentation
			const std::string toString() const;
			/// Automatic conversion to string representation
			const Guid& fromString(const std::string& s);
			/// NULL-Guid: static class method
			static const Guid& null();
			/// Create a new Guid
			static void create(Guid& gUid);
	};
	
	// Class for exception handling
	class LFCDbLookupException
	{
		public:
		std::string lfcErrorMessage;
		int lfcErrorNo;
		
		// Constructor
		LFCDbLookupException(); 
		LFCDbLookupException(std::string , int&);
		void showLFCErrorMessage(void);
		// Destructor
		~LFCDbLookupException();
	}; 
}

#endif  // LFCDBLOOKUP_SERVICE_H
