#include "LFCLookupService.h"
#include "LFCDbLookupService.h"
#include <iostream>
#include <string>
#include "CoralCommon/DatabaseServiceSet.h"
#include "SealKernel/PropertyManager.h"
#include "SealKernel/MessageStream.h"
#include "SealBase/SearchPath.h"
#include "SealBase/Filename.h"
#include "lfc_api.h"
#include "serrno.h"
#include <stdio.h>
#include <cstdlib>

using namespace std;

DEFINE_SEAL_COMPONENT ( coral::LFCLookupService::LFCLookupService, "CORAL/Services/LFCLookupService")


coral::LFCLookupService::LFCLookupService::LFCLookupService( seal::Context* c ):
  seal::Service( c, classContextKey() )
{
}

coral::LFCLookupService::LFCLookupService::LFCLookupService( seal::Context* c,
                                                             const std::string& key ):
  seal::Service( c, key )
{
	LFCDbLookupService::LFCDbLookupService lfcObject;
	if (lfc_starttrans(::getenv("LFC_HOST"), const_cast<char*>("Starts transaction mode...")) < 0)
	{
		seal::MessageStream log( this, this->name() );
                log << seal::Msg::Error << "Could not start the transaction " << serrno <<  seal::flush;
	}
}

coral::LFCLookupService::LFCLookupService::~LFCLookupService()
{
		lfc_endtrans();
}

/* Definition of virtual function "lookup" defined in ILookupService */

coral::IDatabaseServiceSet*
coral::LFCLookupService::LFCLookupService::lookup( const std::string& logicalName,
                                                   coral::AccessMode accessMode,
                                                   std::string authenticationMechanism ) const
{
    	coral::AccessMode aMode;
	coral::DatabaseServiceSet* result = new coral::DatabaseServiceSet(this->name());
	LFCDbLookupService::LFCDbLookupService lfcObject;
		
	try 
	{
		/* Function of LFCDbLookupService to create the map of logical connection string and the vector containing
		   the replica entries */
		lfcObject.chDir();
		lfcObject.createReplicaMap(const_cast<string &>(logicalName));

		coral::DatabaseServiceSet* result = new coral::DatabaseServiceSet(this->name());
  		std::map< std::string, std::vector< LFCDbLookupService::LFCDbLookupService::DbEntry > >::const_iterator iLogicalEntry = lfcObject.m_data.find( logicalName );

  		if ( iLogicalEntry == lfcObject.m_data.end() )
			 return static_cast <coral::IDatabaseServiceSet*>( result );

 		const std::vector< LFCDbLookupService::LFCDbLookupService::DbEntry >& entryVector = iLogicalEntry->second;
  		for ( std::vector< LFCDbLookupService::LFCDbLookupService::DbEntry >::const_iterator iEntry = entryVector.begin(); iEntry != entryVector.end(); ++iEntry ) 
		{
    			const LFCDbLookupService::LFCDbLookupService::DbEntry& entry = *iEntry;
			aMode=(entry.accessMode=="readonly")?coral::ReadOnly:coral::Update;
		
			/* If the access mode is ReadOnly, then all the replicas (both ReadOnly and Update) are 
			   appended to the DatabaseServiceSet Object, otherwise adds only the replicas with 
			   access mode Update. */

    			if ((accessMode == coral::ReadOnly)|| (accessMode == aMode ) &&( authenticationMechanism.empty() ||	entry.authenticationMechanism.find( authenticationMechanism ) != std::string::npos ) )
		 	{
      				result->appendReplica( entry.physicalReplica, ( authenticationMechanism.empty() ) ? entry.authenticationMechanism : authenticationMechanism, aMode );
    		 	}

  		} 
 		return static_cast<coral::IDatabaseServiceSet*>( result );
	}
	catch(LFCDbLookupService::LFCDbLookupException &eLookup)
	{
		const_cast<coral::LFCLookupService::LFCLookupService*>(this)->logMessage(eLookup.lfcErrorMessage);
		return static_cast <coral::IDatabaseServiceSet*>( result );
	}
}

void
coral::LFCLookupService::LFCLookupService::logMessage( std::string& mesg )
{
seal::MessageStream log( this,this->name() );
log<<seal::Msg::Error<<mesg<<seal::flush;
}

