#include <iostream>
#include <sstream>
#include <stdexcept>
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/Context.h"
#include "SealKernel/Service.h"
#include "SealKernel/PropertyManager.h"
#include "SealKernel/IMessageService.h"
#include "RelationalAccess/ILookupService.h"
#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/IDatabaseServiceDescription.h"


void reportReplica( const coral::IDatabaseServiceDescription& replica,
                    const std::string& logicalName )
{
  std::cout << logicalName << " : "
            << replica.connectionString() << ", "
            << replica.authenticationMechanism() << ", "
            << ( ( replica.accessMode() == coral::ReadOnly ) ? std::string("Read-Only") : std::string("Update") )
            << std::endl;
}


int main( int, char** )
{

  try {

    int numberOfReplicas = 0;
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

// Code for Seal Message Service
   // Load the SEAL Message service and set the output threshold to Debug
   loader->load( "SEAL/Services/MessageService" );
   std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
   context->query( v_msgSvc );
   if ( ! v_msgSvc.empty() ) {
     seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
     msgSvc->setOutputLevel( seal::Msg::Debug);
   }
	
// Code for Seal Message Service
    loader->load( "CORAL/Services/LFCLookupService" );

    seal::IHandle<seal::Service> svc = context->query<seal::Service>( "CORAL/Services/LFCLookupService" );

    std::vector< seal::IHandle<coral::ILookupService> > v_lookupSvc;
    context->query( v_lookupSvc );
    if ( ! v_lookupSvc.empty() ) {
    	const coral::ILookupService& lookupSvc = *( v_lookupSvc.front() );

       coral::IDatabaseServiceSet* dbSet = lookupSvc.lookup( "myservice1");
       numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "myservice1" );
      delete dbSet;

      dbSet = lookupSvc.lookup( "F1/Service1");
      numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "F1/Service1" );
      delete dbSet;

      dbSet = lookupSvc.lookup( "/F1/F2/F3/F4/F5/F6/F7/F8/F9/Service1");
      numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "F1/F2/F3/F4/F5/F6/F7/F8/F9/Service1" );
      delete dbSet;

      dbSet = lookupSvc.lookup( "myservice2", coral::ReadOnly);
      numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "myservice2" );
      delete dbSet;

      dbSet = lookupSvc.lookup( "reader_test", coral::Update);
      numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "reader_test" );
      delete dbSet;

      dbSet = lookupSvc.lookup( "write_test", coral::Update);
      numberOfReplicas = dbSet->numberOfReplicas();

      for ( int i = 0; i < numberOfReplicas; ++i )
        reportReplica( dbSet->replica( i ), "write_test" );
      delete dbSet;

    }
    else {
      throw std::runtime_error("Could not retrieve an lookup service.");
    }
    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
