//To add new replica
/*
         * Usage :
         * coral-add-db-replica 	[-h contact_string] (can be skipped with enviornment variable) 
	 *				[-l logical_connection_string]
         *                       	[-p physical_replica ] 
	 *				[-r optional for readonly replica]
         *                       	[-a authentication_method] (optional, default is password)
*/

#include <stdio.h>
#include <string>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include "../src/LFCDbLookupService.h"
#include <iostream>


extern	char	*getenv();
extern	char	*optarg;
extern  int 	optind, optopt, opterr;

void showErrorMessage(char *);

using namespace std;

int main(int argc, char **argv)
{
	char *optionContactString=NULL;
	char *optionLogicalString=NULL;
	char *optionPhysicalReplica=NULL;
	const char *optionReplicaAccessMode=NULL;
	char *optionAuthenticationMethod=NULL;
	
	string optContactString("");
	string optLogicalString("");
	string optPhysicalReplica(""); 
	string optReplicaAccessMode("");
	string optAuthenticationMethod("");

	int argCount,lflag=0,pflag=0;
	int errorFlag=0;
	
	while( (argCount = getopt (argc, argv, ":h:l:p:ra:")) != EOF )
	{
		switch(argCount)	
		{
			case 'h': 
					optionContactString=optarg;
					break;
			case 'l': 
					lflag++;
					optionLogicalString=optarg;
				  	break;
			case 'p': 
					pflag++;
					optionPhysicalReplica=optarg;
				  	break;
			case 'r': 
				  	optionReplicaAccessMode="readonly";
				 	break;
			case 'a': 	
					optionAuthenticationMethod=optarg;
				  	break;
			case ':': 	/* -h, -l, -p , -a without argument */
				 	fprintf(stdout,"Option -%c requires an operand\n", optopt);
					errorFlag++;
					break;
			case '?': 	/*invalid switch*/
				 	fprintf(stdout,"Invalid Switch -%c \n", optopt);
					errorFlag++;
		}
	}			

	if (optind < argc) {
              fprintf (stderr,"Argument should be given with switch : ");
              while (optind < argc)
			fprintf (stderr,"%s ", argv[optind++]);
              printf ("\n");
	      errorFlag++;
           }
	if(!(lflag && pflag))
	{
		if(!lflag)
			fprintf(stdout,"option -l [logical_connection_string] is mandatory\n");
		
		if(!pflag)
			fprintf(stdout,"option -p [physical_replica] is mandatory\n");

		errorFlag++;
	}

	if (errorFlag)
	{
		showErrorMessage(argv[0]);
		exit(0);
	}

	if (optionContactString == NULL)
	{
		/* If environment variable is not set */
		if(!getenv("LFC_HOST"))
		{
			fprintf(stdout, "\n\tProbably the LFC_HOST environment variable is not set.\n\n");	
			return -1;
		}
		optionContactString=::getenv("LFC_HOST");		
	}
		

	if (optionContactString != NULL)
		optContactString.assign(optionContactString);
	if (optionLogicalString != NULL)
		optLogicalString.assign(optionLogicalString);
	if (optionPhysicalReplica != NULL)
		optPhysicalReplica.assign(optionPhysicalReplica);
	if (optionReplicaAccessMode != NULL) 	
		optReplicaAccessMode.assign(optionReplicaAccessMode);
	if (optionAuthenticationMethod != NULL)	
		optAuthenticationMethod.assign(optionAuthenticationMethod);
		
	LFCDbLookupService::LFCDbLookupService cAddReplica;

	try
	{
		cAddReplica.addReplica(optContactString, optLogicalString, optPhysicalReplica, optReplicaAccessMode, optAuthenticationMethod);
	}
	catch (LFCDbLookupService::LFCDbLookupException eAddReplicaException)
	{
		eAddReplicaException.showLFCErrorMessage();
		return -1;
	}

	return 0;
}

void showErrorMessage(char *cmdArgument)
{
 	fprintf(stdout, "\n\tUsage: %s [-h contact_string] \n\t [-l logical_connection_string] \n\t [-p physical_replica] \n\t [-r] < optional for read-only replica > \n\t [-a  authentication_method] <optional, default is password> \n\n", cmdArgument);

}

