/*
         * Usage :
         * coral-export-db-catalogue 	[-h contact_string] 
         *				[-f filename] (default dblookup.xml)
	 *				[-l logical_connection_string] (optional filter)
	 *				[-r optional filter for read-only ]
	 *				[-u optional filter for update ]
*/

#include <iostream>
#include <string>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include "../src/LFCDbLookupService.h"

extern	char	*getenv();
extern	char	*optarg;
extern  int 	optind, optopt, opterr;

void showErrorMessage(char *);

using namespace std;

int main(int argc, char **argv)
{
	char *optionContactString=NULL;
	char *optionLogicalString=NULL;
	const char *optionReplicaAccessMode=NULL;
	char *optionFileName=NULL;
	
	string optLogicalString("");
	string optFileName("");
        string aMode("");
	
	int argCount,uflag=0,rflag=0;
	int errorFlag=0;
	
	while( (argCount = getopt (argc, argv, ":h:f:l:ru")) != EOF ){
		switch(argCount){
			case 'h': 
					optionContactString=optarg;
				  	break;
			case 'f': 
					optionFileName=optarg;
				  	break;
			case 'l': 
					optionLogicalString=optarg;
				  	break;
			case 'r':
					if(uflag)
					{
					fprintf(stdout,"Either option -r or -u can be used, not both");	
					errorFlag++;
					}
					else
					{
				  	optionReplicaAccessMode="readonly";
					rflag++;
					}
				  	break;
			 case 'u': 
					if(rflag)
					{
					fprintf(stdout,"Either option -r or -u can be used, not both");	
					errorFlag++;
					}
					else
					{
				  	optionReplicaAccessMode="update";
					uflag++;
					}
				  	break; 
			case ':':	/* -h,-l,-r,-u */
				 	fprintf(stdout,"Option -%c requires an operand\n", optopt);
					errorFlag++;
					break;
			case '?':
					/*invalid switch*/
                                        fprintf(stdout,"Invalid Switch -%c \n", optopt);
					errorFlag++;
		}
	}			
	if (optind < argc) {
               fprintf (stderr,"Argument should be given with switch : ");
               while (optind < argc)
                   fprintf (stderr,"%s ", argv[optind++]);
               printf ("\n");
		errorFlag++;
           }

	if (errorFlag){
		showErrorMessage(argv[0]);
		exit(0);
	}

	if (optionContactString == NULL)
	{
		if(!getenv("LFC_HOST"))
		{
			fprintf(stdout, "\n\tProbably the LFC_HOST environment variable is not set.\n\n");	
			return -1;
		}
		optionContactString=getenv("LFC_HOST");		
	}
	
	if (optionLogicalString != NULL)
		optLogicalString.assign(optionLogicalString);

	if (optionFileName != NULL)
	 	optFileName.assign(optionFileName);

	if (optionReplicaAccessMode != NULL)
	 aMode.assign(optionReplicaAccessMode);

        LFCDbLookupService::LFCDbLookupService cDbCatalogue;

	try
	{
        	cDbCatalogue.exportDbCatalogue(optFileName,optLogicalString, aMode);
	}
 	catch (LFCDbLookupService::LFCDbLookupException exportReplicaException)
        {
                exportReplicaException.showLFCErrorMessage();
                return -1;
        }

        return 0;
	
}

void showErrorMessage(char *cmdArgument)
{
 	fprintf(stdout, "\n\tUsage: %s -h <contact_string> \n\t -f <file_name> [Optional,default is dblookup.xml]\n\t -l <Logical Connection String> \n\t -r <read-only>[Optional] \n\t -u <Update>[Optional] \n\n", cmdArgument);

}



