/*
         * Usage :
         * coral-remove-db-replica	[-h contact_string] 
	 *				[-l logical_connection_string]
         *                       	[-p physical_replica ] (if specified will remove the specified 
	 *				replica otherwise all the replica entries for given logical 
	 *				connection string will be deleted.) 
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <getopt.h>
#include "../src/LFCDbLookupService.h"

extern	char	*getenv();
extern	char	*optarg;
extern  int 	optind, optopt, opterr;

void showErrorMessage(char *);

//using namespace LFCDbLookupService;
using namespace std;

int main(int argc, char **argv)
{
	char *optionContactString=NULL;
	char *optionLogicalString=NULL;
	char *optionPhysicalReplica=NULL;
	int argCount,lflag=0;
	int errorFlag=0;

	string optLogicalString("");
	string optPhysicalReplica("");
	
	while( (argCount = getopt (argc, argv, ":h:l:p:")) != EOF )
	{
		switch(argCount){
			case 'h': 
					optionContactString=optarg;
					  break;
			case 'l': 
					lflag++;
					optionLogicalString=optarg;
				  	break;
			case 'p': 
					optionPhysicalReplica=optarg;
				  	break;
			case ':': 	/* -h, -l, -p */
				 	fprintf(stdout,"Option -%c requires an operand\n", optopt);
					errorFlag++;
					break;
			case '?': 	/*invalid switch*/
				 	fprintf(stdout,"Invalid Switch -%c \n", optopt);
					errorFlag++;
		}
	}			

	if (optind < argc) {
               fprintf (stderr,"Argument should be given with switch : " );
               while (optind < argc)
                   fprintf (stderr,"%s ", argv[optind++]);
               printf ("\n");
		errorFlag++;
           }
	if(!lflag)
	{
		fprintf(stdout,"option -l [logical_connection_string] is mandatory\n");

		errorFlag++;
	}

	if (errorFlag){
		showErrorMessage(argv[0]);
		exit(0);
	}


	if (optionContactString == NULL)
	{
		if(!getenv("LFC_HOST"))
		{
			fprintf(stdout,"\n\tProbably the LFC_HOST environment variable is not set.\n\n");	
			return -1;
		}
		optionContactString=getenv("LFC_HOST");		
	}
		

	if (optionLogicalString != NULL)
		optLogicalString.assign(optionLogicalString);
	
	if (optionPhysicalReplica != NULL)	
		optPhysicalReplica.assign(optionPhysicalReplica);

	LFCDbLookupService::LFCDbLookupService cDeleteReplica;

	try
	{
		cDeleteReplica.deleteReplica(optLogicalString,optPhysicalReplica);
	}
	catch (LFCDbLookupService::LFCDbLookupException deleteReplicaException)
        {
                deleteReplicaException.showLFCErrorMessage();
                return -1;
        }

	return 0;
}

void showErrorMessage(char *cmdArgument)
 {
	fprintf(stdout, "\n\tUsage: %s [-h contact_string] \n\t [-l logical_connection_string] \n\t [-p physical_replica] (optional)  \n\n", cmdArgument);

}

