#include "LFCDatabaseServiceSet.h"
#include "RelationalAccess/LookupServiceException.h"

std::string coral::LFCReplicaService::LFCDatabaseServiceDescription::serviceParameter(const std::string& parameterName) const {
  if(parameterName==coral::IDatabaseServiceDescription::serverNameParam()){
    return m_replica->serverName();
  } else if (parameterName==coral::IDatabaseServiceDescription::serverStatusParam()){
    std::string stat = coral::IDatabaseServiceDescription::serverStatusOnline();
    if(!m_replica->status()) stat = coral::IDatabaseServiceDescription::serverStatusOffline();
    return stat;
  } else {
    return "";
  }
}

/// Destructor
coral::LFCReplicaService::LFCDatabaseServiceSet::~LFCDatabaseServiceSet(){
  for(std::map<std::string,LFCDatabaseServiceDescription*>::const_iterator iD=m_replicaDescriptions.begin();
      iD!=m_replicaDescriptions.end();
      ++iD){
    delete iD->second;
  }
}

bool
coral::LFCReplicaService::LFCDatabaseServiceSet::appendReplica( const ReplicaDescription* replica ){
  bool inserted = false;
  if(replica){
    if(m_replicaDescriptions.find(replica->physicalConnectionString())==m_replicaDescriptions.end()){
      m_replicaDescriptions.insert(std::make_pair(replica->physicalConnectionString(),new LFCDatabaseServiceDescription(replica)));
      inserted = true;
      m_sorted = false;
    }
  }
  return inserted;
}

int coral::LFCReplicaService::LFCDatabaseServiceSet::numberOfReplicas() const {
  return static_cast<int>( m_replicaDescriptions.size());
}

const coral::IDatabaseServiceDescription& coral::LFCReplicaService::LFCDatabaseServiceSet::replica( int index ) const {
  if(index < 0 || index >= static_cast<int>( m_replicaDescriptions.size()) ){
    throw coral::InvalidReplicaIdentifierException( "LFCReplicaService" );
  }
  if(!m_sorted){
    m_replicas.clear();
    for(std::map<std::string,LFCDatabaseServiceDescription*>::const_iterator iR=m_replicaDescriptions.begin();
        iR!=m_replicaDescriptions.end();
        ++iR){
      m_replicas.push_back(iR->second);
    }    
    m_sorted = true;
  }
  return *(m_replicas[index]);
}


