#ifndef LFCREPLICASERVICE_LFCDATABASESERVICESET_H
#define LFCREPLICASERVICE_LFCDATABASESERVICESET_H

#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/IDatabaseServiceDescription.h"
#include "RelationalAccess/AccessMode.h"
#include <map>
#include <vector>
#include "ReplicaSet.h"

namespace coral {

  namespace LFCReplicaService {

    class LFCDatabaseServiceDescription : virtual public IDatabaseServiceDescription {
      public:

    /// Constructor
    explicit LFCDatabaseServiceDescription( const ReplicaDescription* replica );

    virtual ~LFCDatabaseServiceDescription();
    
    std::string connectionString() const;
  
    std::string authenticationMechanism() const;

    coral::AccessMode accessMode() const;
    
    std::string serviceParameter(const std::string& parameterName) const;

    private:

    const ReplicaDescription* m_replica;

  };

  class LFCDatabaseServiceSet : virtual public IDatabaseServiceSet {
  public:
    /// Constructor
    LFCDatabaseServiceSet();

    /// Destructor
    virtual ~LFCDatabaseServiceSet();

    bool appendReplica(const ReplicaDescription* replica);
    
  public:
    /**
     * Returns the number of actual database services corresponding to
     * the logical name.
     */
    int numberOfReplicas() const;
    
    /**
     * Returns a reference to the service description object corresponding
     * to the specified index.
     * If the index is out of range an InvalidReplicaIdentifierException is thrown.
     */
    const IDatabaseServiceDescription& replica( int index ) const;

    private:
    mutable bool m_sorted;
    mutable std::vector<LFCDatabaseServiceDescription*> m_replicas;
    std::map<std::string,LFCDatabaseServiceDescription*> m_replicaDescriptions;
  };
  }

}

inline
coral::LFCReplicaService::LFCDatabaseServiceSet::LFCDatabaseServiceSet():m_sorted(false),m_replicas(),m_replicaDescriptions(){
}

inline
coral::LFCReplicaService::LFCDatabaseServiceDescription::LFCDatabaseServiceDescription( const ReplicaDescription* replica ):
  m_replica(replica){}

inline
coral::LFCReplicaService::LFCDatabaseServiceDescription::~LFCDatabaseServiceDescription(){
  if(m_replica) delete m_replica;
}

inline
std::string coral::LFCReplicaService::LFCDatabaseServiceDescription::connectionString() const {
  return m_replica->physicalConnectionString();
}

inline
std::string coral::LFCReplicaService::LFCDatabaseServiceDescription::authenticationMechanism() const {
  return m_replica->authenticationMechanism();
}

inline
coral::AccessMode coral::LFCReplicaService::LFCDatabaseServiceDescription::accessMode() const {
  return m_replica->accessMode();
}
#endif
