#include "LFCDbReplicaMapping.h"
#include "LFCReplica.h"
#include "ReplicaSet.h"
#include "LFCPhysicalConnectionString.h"
#include "LFCCredentialString.h"

namespace coral {
  namespace LFCReplicaService {
    static const std::string ON("1");
    static const std::string OFF("0");
    static const std::string RO("R");
    static const std::string RW("W");
  }
}

coral::LFCReplicaService::LFCDbReplicaMapping::LFCDbReplicaMapping():m_lfcReplica(0),
                                                                     m_lfcReplicaOwned(new LFCReplica),
                                                                     m_owned(true){
}

std::string coral::LFCReplicaService::LFCDbReplicaMapping::statusCode(ReplicaStatus status){
  return ((status==coral::LFCReplicaService::On)?ON:OFF);
}

std::string coral::LFCReplicaService::LFCDbReplicaMapping::accessModeCode(coral::AccessMode accessMode){
  return ((accessMode==coral::ReadOnly)?RO:RW);
}

coral::LFCReplicaService::ReplicaStatus coral::LFCReplicaService::LFCDbReplicaMapping::statusFlag(const std::string& code){
  return ((code==ON)?On:Off);
}

coral::AccessMode coral::LFCReplicaService::LFCDbReplicaMapping::accessModeFlag(const std::string& code){
  return ((code==RO)?coral::ReadOnly:coral::Update);  
}

int coral::LFCReplicaService::LFCDbReplicaMapping::permissionMode(ReplicaPermission permission){
  int mode = 0;
  switch (permission){
    case NoPerm:
      break;
    case RPerm:
      mode = 4;
      break;
    case WPerm:
      mode = 2;
      break;
    case RWPerm:
      mode = 6;
      break;
  };
  return mode;
}

coral::LFCReplicaService::LFCDbReplicaMapping::LFCDbReplicaMapping(const ReplicaDescription& dbReplica):
  m_lfcReplica(0),
  m_lfcReplicaOwned(new LFCReplica),
  m_owned(true){
  std::string authMech = dbReplica.authenticationMechanism();
  if(authMech.empty()) authMech = "None";
  m_lfcReplicaOwned->pfn = LFCPhysicalConnectionString::pfn(dbReplica.physicalConnectionString(),dbReplica.userName());
  m_lfcReplicaOwned->status = statusCode(dbReplica.status());
  m_lfcReplicaOwned->serverParameter = dbReplica.serverName();
  m_lfcReplicaOwned->f_typeParameter = accessModeCode(dbReplica.accessMode());
  m_lfcReplicaOwned->poolnameParameter = authMech;
  m_lfcReplicaOwned->fsParameter = LFCCredentialString::credentials(dbReplica.password(),"");
}

coral::LFCReplicaService::LFCDbReplicaMapping::LFCDbReplicaMapping(const ReplicaDescription& dbReplica,
                                                                   const std::string& guid):
  m_lfcReplica(0),
  m_lfcReplicaOwned(new LFCReplica),
  m_owned(true){
  std::string authMech = dbReplica.authenticationMechanism();
  if(authMech.empty()) authMech = "None";
  m_lfcReplicaOwned->pfn = LFCPhysicalConnectionString::pfn(dbReplica.physicalConnectionString(),dbReplica.userName(),guid);
  m_lfcReplicaOwned->status = statusCode(dbReplica.status());
  m_lfcReplicaOwned->serverParameter = dbReplica.serverName();
  m_lfcReplicaOwned->f_typeParameter = accessModeCode(dbReplica.accessMode());
  m_lfcReplicaOwned->poolnameParameter = authMech;
  m_lfcReplicaOwned->fsParameter = LFCCredentialString::credentials(dbReplica.password(),guid);
}

coral::LFCReplicaService::LFCDbReplicaMapping::LFCDbReplicaMapping(const LFCReplica& lfcReplica):m_lfcReplica(&lfcReplica),
                                                                                                 m_lfcReplicaOwned(0),
                                                                                                 m_owned(false){
}

coral::LFCReplicaService::LFCDbReplicaMapping::~LFCDbReplicaMapping(){
  if(m_owned && m_lfcReplicaOwned) delete m_lfcReplicaOwned;
}

coral::LFCReplicaService::ReplicaDescription*
coral::LFCReplicaService::LFCDbReplicaMapping::dbReplica() const{
  if(lfcReplica().pfn.empty()) throw LFCDbReplicaMappingException("PFN retrieved from LFC database is empty.",
                                                                  "LFCDbReplicaMapping::dbReplica()");
  LFCPhysicalConnectionString lfcPfn(lfcReplica().pfn);
  if(!lfcPfn.isCoralPfn())
    throw LFCDbReplicaMappingException("PFN retrieved from LFC database is not a CORAL entry: \""+lfcReplica().pfn+"\".",
                                       "LFCDbReplicaMapping::dbReplica()");
  if(!lfcPfn.parse())
    throw LFCDbReplicaMappingException("PFN retrieved from LFC database could not be parsed successfully: \""+lfcReplica().pfn+"\".",
                                       "LFCDbReplicaMapping::dbReplica()");
  std::string password = LFCCredentialString::password(lfcReplica().fsParameter,lfcPfn.guid());
  std::string authMech = lfcReplica().poolnameParameter;
  if(authMech == "None") authMech.clear();
  ReplicaDescription* descr = new ReplicaDescription(lfcPfn.guid(),
                                                     lfcPfn.connectionString(),
                                                     authMech,
                                                     lfcPfn.userName(),
                                                     password,
                                                     lfcReplica().serverParameter,
                                                     statusFlag(lfcReplica().status),
                                                     accessModeFlag(lfcReplica().f_typeParameter));
  return descr;               
}

const coral::LFCReplicaService::LFCReplica& coral::LFCReplicaService::LFCDbReplicaMapping::lfcReplica() const {
  const LFCReplica* theLFCReplica  = 0;
  if(m_owned){
    theLFCReplica = m_lfcReplicaOwned;
  } else {
    theLFCReplica = m_lfcReplica;
  } 
  return *theLFCReplica;
}

