#ifndef LFCREPLICASERVICE_LFCDBREPLICAMAPPING_H
#define LFCREPLICASERVICE_LFCDBREPLICAMAPPING_H

#include <string>
#include "RelationalAccess/AccessMode.h"
#include "ReplicaStatus.h"
#include "ReplicaPermission.h"
#include "CoralBase/Exception.h"

namespace coral {

    namespace LFCReplicaService {

    class ReplicaDescription;
    class LFCReplica;
    
    class LFCDbReplicaMapping {

      public:

      LFCDbReplicaMapping();

      LFCDbReplicaMapping(const ReplicaDescription& dbReplica);

      LFCDbReplicaMapping(const ReplicaDescription& dbReplica,
                          const std::string& guid);

      LFCDbReplicaMapping(const LFCReplica& lfcReplica);

      virtual ~LFCDbReplicaMapping();
      
      ReplicaDescription* dbReplica() const;

      const LFCReplica& lfcReplica() const;

      static std::string statusCode(ReplicaStatus status);
      
      static std::string accessModeCode(coral::AccessMode accessMode);
      
      static ReplicaStatus statusFlag(const std::string& code);
      
      static coral::AccessMode accessModeFlag(const std::string& code);

      static int permissionMode(ReplicaPermission permission);
      
      private:

      const LFCReplica* m_lfcReplica;
      
      LFCReplica* m_lfcReplicaOwned;

      bool m_owned;
    };

    class LFCDbReplicaMappingException: public coral::Exception {
      public:

      /// Constructors
      LFCDbReplicaMappingException(const std::string& message,
                                   const std::string& methodName):
        Exception( message, methodName, "CORAL/Services/LFCReplicaService" ){}

      
      /// Destructor
      virtual ~LFCDbReplicaMappingException() throw() {}
      
    }; 
    
  }
}

#endif
    
      
