#include "LFCFile.h"
#include <sys/stat.h> //mode_t in lfc_api.h
#include <lfc_api.h>
#include <unistd.h>
#include "LFCException.h"
#include <serrno.h>

coral::LFCReplicaService::LFCFile::LFCFile():m_absolutePath(""),
                                             m_exists(false),
                                             m_isDir(false),
                                             m_readOk(false),
                                             m_writeOk(false),
                                             m_executeOk(false),
                                             m_content(),
                                             m_guid(""){}

coral::LFCReplicaService::LFCFile::LFCFile(const std::string& absolutePath):m_absolutePath(absolutePath),
                                                                            m_exists(false),
                                                                            m_isDir(false),
                                                                            m_readOk(false),
                                                                            m_writeOk(false),
                                                                            m_executeOk(false),
                                                                            m_content(),
                                                                            m_guid(""){}

coral::LFCReplicaService::LFCFile::LFCFile(const LFCFile& aFile):m_absolutePath(aFile.m_absolutePath),
                                                                 m_exists(aFile.m_exists),
                                                                 m_isDir(aFile.m_isDir),
                                                                 m_readOk(aFile.m_readOk),
                                                                 m_writeOk(aFile.m_writeOk),
                                                                 m_executeOk(aFile.m_executeOk),
                                                                 m_content(aFile.m_content),
                                                                 m_guid(aFile.m_guid){}

coral::LFCReplicaService::LFCFile::LFCFile(const std::string& absoluteFolderPath,
                                           const std::string& fileName ):m_absolutePath(absoluteFolderPath),
                                                                         m_exists(false),
                                                                         m_isDir(false),
                                                                         m_readOk(false),
                                                                         m_writeOk(false),
                                                                         m_executeOk(false),
                                                                         m_content(),
                                                                         m_guid(""){
  m_absolutePath.append("/").append(fileName);
}

coral::LFCReplicaService::LFCFile::~LFCFile(){}

coral::LFCReplicaService::LFCFile& coral::LFCReplicaService::LFCFile::operator=(const LFCFile& aFile){
  if(&aFile!=this){
    m_absolutePath = aFile.m_absolutePath;
    m_exists = aFile.m_exists;
    m_isDir = aFile.m_isDir;
    m_readOk = aFile.m_readOk;
    m_writeOk = aFile.m_writeOk;
    m_executeOk = aFile.m_executeOk;
    m_content = aFile.m_content;
    m_guid = aFile.m_guid;
  }
  return *this;
}

std::string coral::LFCReplicaService::LFCFile::fileName() const {
  std::string fileName("");
  std::string::size_type pos = m_absolutePath.rfind("/");
  if( (pos!=std::string::npos) && !(pos==m_absolutePath.size()-1) )
  {
    fileName =  m_absolutePath.substr( pos+1 );
  }
  return fileName;
}

const std::string& coral::LFCReplicaService::LFCFile::path() const {
  return m_absolutePath;
}

const std::string& coral::LFCReplicaService::LFCFile::guid() const {
  return m_guid;
}

bool coral::LFCReplicaService::LFCFile::lookup(){
  m_content.clear();
  if(m_absolutePath.empty()) return false;
	struct lfc_filestatg statg;
  m_exists = (lfc_statg(m_absolutePath.c_str(), NULL, &statg) == 0);
  if(m_exists){
    m_guid = std::string(statg.guid);
    m_isDir = S_ISDIR(statg.filemode);
    m_readOk = (lfc_access(m_absolutePath.c_str(),R_OK)==0);
    m_writeOk = (lfc_access(m_absolutePath.c_str(),W_OK)==0);
    m_executeOk = (lfc_access(m_absolutePath.c_str(),X_OK)==0);
    // search (dir) permission requires both read and execute permission!
    if(m_isDir && m_readOk && m_executeOk){
      lfc_DIR *lfcDir;
      struct lfc_direnstatg *dent;
      serrno = 0;
      if((lfcDir=lfc_opendirg(m_absolutePath.c_str(),NULL))== NULL)
      {  
        throw LFCException("Could not open '"+m_absolutePath+"' directory for listing of replicas.","lfc_opendir");
      }
      while ((dent=lfc_readdirg(lfcDir))){
        m_content.push_back(std::string(dent->d_name));
      }
      if(serrno){
        throw LFCException("Could not open '"+m_absolutePath+"' directory for listing of replicas.","lfc_readdirg");
      }
    }
  }
  return m_exists;
}

bool coral::LFCReplicaService::LFCFile::exists() const {
  return m_exists;  
}

bool coral::LFCReplicaService::LFCFile::isDirectory() const {
  return m_isDir;
}

bool coral::LFCReplicaService::LFCFile::isReadable() const {
  return m_readOk||m_writeOk;
}

bool coral::LFCReplicaService::LFCFile::isWritable() const {
  return m_writeOk;
}

bool coral::LFCReplicaService::LFCFile::isExecutable() const {
  return m_executeOk;
}

const std::vector<std::string>& coral::LFCReplicaService::LFCFile::listFiles() const {
  return m_content;
}



  
  
