#ifndef LFCREPLICASERVICE_REPLICASET_H
#define LFCREPLICASERVICE_REPLICASET_H

#include "RelationalAccess/AccessMode.h"
#include "ReplicaStatus.h"
#include <map>
#include <set>
#include <vector>
#include <string>

namespace coral {

  namespace LFCReplicaService {

   class ReplicaDescription{

      public:

      ReplicaDescription();

      ReplicaDescription(const std::string& physicalConnectionString,
                         const std::string& authenticationMechanism,
                         const std::string& userName,
                         const std::string& password,
                         const std::string& serverName,
                         ReplicaStatus status,
                         coral::AccessMode accessMode);

      ReplicaDescription(const std::string& guid,
                         const std::string& physicalConnectionString,
                         const std::string& authenticationMechanism,
                         const std::string& userName,
                         const std::string& password,
                         const std::string& serverName,
                         ReplicaStatus status,
                         coral::AccessMode accessMode);

      const std::string& guid() const;

      const std::string& physicalConnectionString() const;

      const std::string& authenticationMechanism() const;

      const std::string& userName() const;

      const std::string& password() const;

      const std::string& serverName() const;

      ReplicaStatus status() const;

      coral::AccessMode accessMode() const;

     private:

      std::string m_guid;

      std::string m_physicalConnectionString;

      std::string m_authenticationMechanism;

      std::string m_userName;

      std::string m_password;

      std::string m_serverName;

      ReplicaStatus m_status;

      coral::AccessMode m_accessMode;      
      
    };
    
    class ReplicaSet {

      public:
      
      ReplicaSet();

      virtual ~ReplicaSet();
      
      const std::map<std::string,std::map<std::string,std::multimap<std::string,ReplicaDescription*> > >& replicaMap() const;

      void appendReplica(const std::string& logicalName,
                         const std::string& role,
                         ReplicaDescription* replica );

      void prettyPrint(std::ostream& out) const;

      void XMLPrint(std::ostream& lookupOut, std::ostream& authenticationOut) const;

      private:
      
      std::map<std::string,std::map<std::string,std::multimap<std::string,ReplicaDescription*> > > m_replicaMap;

      std::set<ReplicaDescription*> m_descriptions;
    };    

    class ReplicaFilter {

      public:

      ReplicaFilter();

      ReplicaFilter(const ReplicaFilter& filter);

      void setPhysicalConnectionString(const std::string& physicalConnectionString);

      void setAuthenticationMechanism(const std::string& authenticationMechanism);

      void setUserName(const std::string& userName);

      void setServerName(const std::string& serverName);

      void setStatus(ReplicaStatus status);

      void setAccessMode(coral::AccessMode accessMode);
 
      bool select(const ReplicaDescription& replica) const;

      private:

      std::pair<std::string,bool>  m_physicalConnectionStringFilter;

      std::pair<std::string,bool>  m_authenticationMechanismFilter;

      std::pair<std::string,bool>  m_userNameFilter;

      std::pair<std::string,bool>  m_passwordFilter;

      std::pair<std::string,bool>  m_serverNameFilter;

      std::pair<ReplicaStatus,bool>  m_statusFilter;

      std::pair<coral::AccessMode,bool>  m_accessModeFilter;

      bool m_doFilter;
      
    };
    
  }
}

inline
coral::LFCReplicaService::ReplicaSet::ReplicaSet():m_replicaMap(),m_descriptions(){}

inline
const std::map<std::string,std::map<std::string,std::multimap<std::string,coral::LFCReplicaService::ReplicaDescription*> > >& coral::LFCReplicaService::ReplicaSet::replicaMap() const{
  return m_replicaMap;
}

inline
coral::LFCReplicaService::ReplicaDescription::ReplicaDescription():
  m_guid(""),
  m_physicalConnectionString(""),
  m_authenticationMechanism(""),
  m_userName(""),
  m_password(""),
  m_serverName(""),
  m_status(coral::LFCReplicaService::On),
  m_accessMode(coral::Update){
}

inline
coral::LFCReplicaService::ReplicaDescription::ReplicaDescription(const std::string& physicalConnectionString,
                                              const std::string& authenticationMechansim,
                                              const std::string& userName,
                                              const std::string& password,
                                              const std::string& serverName,
                                              coral::LFCReplicaService::ReplicaStatus status,
                                              coral::AccessMode accessMode ):
  m_guid(""),
  m_physicalConnectionString(physicalConnectionString),
  m_authenticationMechanism(authenticationMechansim),
  m_userName(userName),
  m_password(password),
  m_serverName(serverName),
  m_status(status),
  m_accessMode(accessMode){
}

inline
coral::LFCReplicaService::ReplicaDescription::ReplicaDescription(const std::string& guid,
                                              const std::string& physicalConnectionString,
                                              const std::string& authenticationMechansim,
                                              const std::string& userName,
                                              const std::string& password,
                                              const std::string& serverName,
                                              coral::LFCReplicaService::ReplicaStatus status,
                                              coral::AccessMode accessMode ):
  m_guid(guid),
  m_physicalConnectionString(physicalConnectionString),
  m_authenticationMechanism(authenticationMechansim),
  m_userName(userName),
  m_password(password),
  m_serverName(serverName),
  m_status(status),
  m_accessMode(accessMode){
}

inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::guid() const {
  return m_guid;
}

inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::physicalConnectionString() const {
  return m_physicalConnectionString;
}

inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::userName() const {
  return m_userName;
}


inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::password() const {
  return m_password;
}


inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::serverName() const {
  return m_serverName;
}

inline
const std::string&
coral::LFCReplicaService::ReplicaDescription::authenticationMechanism() const {
  return m_authenticationMechanism;
}

inline
coral::LFCReplicaService::ReplicaStatus
coral::LFCReplicaService::ReplicaDescription::status() const {
  return m_status;
}

inline
coral::AccessMode
coral::LFCReplicaService::ReplicaDescription::accessMode() const {
  return m_accessMode;
}

inline
coral::LFCReplicaService::ReplicaFilter::ReplicaFilter():
  m_physicalConnectionStringFilter(std::string(""),false),
  m_authenticationMechanismFilter(std::string(""),false),
  m_userNameFilter(std::string(""),false),
  m_passwordFilter(std::string(""),false),
  m_serverNameFilter(std::string(""),false),
  m_statusFilter(coral::LFCReplicaService::On,false),
  m_accessModeFilter(coral::Update,false),
  m_doFilter(false){
}

inline
coral::LFCReplicaService::ReplicaFilter::ReplicaFilter( const ReplicaFilter& rhs):
  m_physicalConnectionStringFilter(rhs.m_physicalConnectionStringFilter),
  m_authenticationMechanismFilter(rhs.m_authenticationMechanismFilter),
  m_userNameFilter(rhs.m_userNameFilter),
  m_passwordFilter(rhs.m_passwordFilter),
  m_serverNameFilter(rhs.m_serverNameFilter),
  m_statusFilter(rhs.m_statusFilter),
  m_accessModeFilter(rhs.m_accessModeFilter),
  m_doFilter(false){
}

inline
void coral::LFCReplicaService::ReplicaFilter::setPhysicalConnectionString(const std::string& physicalConnectionString){
  m_physicalConnectionStringFilter.first = physicalConnectionString;
  m_physicalConnectionStringFilter.second = true;
  m_doFilter = true;  
}

inline
void coral::LFCReplicaService::ReplicaFilter::setAuthenticationMechanism(const std::string& authenticationMechanism){
  m_authenticationMechanismFilter.first = authenticationMechanism;
  m_authenticationMechanismFilter.second = true;
  m_doFilter = true;  
}

inline
void coral::LFCReplicaService::ReplicaFilter::setUserName(const std::string& userName){
  m_userNameFilter.first = userName;
  m_userNameFilter.second = true;
  m_doFilter = true;  
}

inline
void coral::LFCReplicaService::ReplicaFilter::setServerName(const std::string& serverName){
  m_serverNameFilter.first = serverName;
  m_serverNameFilter.second = true;
  m_doFilter = true;  
}

inline
void coral::LFCReplicaService::ReplicaFilter::setStatus(coral::LFCReplicaService::ReplicaStatus status){
  m_statusFilter.first = status;
  m_statusFilter.second = true;
  m_doFilter = true;  
}

inline
void coral::LFCReplicaService::ReplicaFilter::setAccessMode(coral::AccessMode accessMode){
  m_accessModeFilter.first = accessMode;
  m_accessModeFilter.second = true;
  m_doFilter = true;  
}

#endif
