#include <iostream>
#include <sstream>
#include <stdexcept>
#include "CoralKernel/Context.h"
#include "CoralBase/Exception.h"
#include "RelationalAccess/ILookupService.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/IAuthenticationCredentials.h"
#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/IDatabaseServiceDescription.h"

void reportReplica( const coral::IDatabaseServiceDescription& replica,
                    const std::string& logicalName )
{
  std::cout << "#############################################"<<std::endl;
  std::cout << logicalName << " : "
            << replica.connectionString() << ", "
            << replica.authenticationMechanism() << ", "
            << ( ( replica.accessMode() == coral::ReadOnly ) ? std::string("Read-Only") : std::string("Update") )
            << std::endl;
}


int main( int, char** )
{

  try {

    int numberOfReplicas = 0; 
	
    coral::Context& ctx = coral::Context::instance();
    ctx.loadComponent( "CORAL/Services/LFCReplicaService" );
    coral::IHandle<coral::ILookupService> lookupSvc = ctx.query<coral::ILookupService>();
    coral::IHandle<coral::IAuthenticationService> authenticationSvc = ctx.query<coral::IAuthenticationService>();
    coral::IDatabaseServiceSet* dbSet = lookupSvc->lookup( "test1/ecal_calib0");
    numberOfReplicas = dbSet->numberOfReplicas();
    std::cout <<" Found ["<<numberOfReplicas<<"] replicas"<<std::endl;
    for ( int i = 0; i < numberOfReplicas; ++i ){
      reportReplica( dbSet->replica( i ), "test1/ecal_calib0" );
      if(dbSet->replica( i ).authenticationMechanism()!=""){
        std::cout <<" Default role:"<<std::endl;
        const coral::IAuthenticationCredentials& creds = authenticationSvc->credentials(dbSet->replica( i ).connectionString());
        std::cout <<" Found ["<<creds.numberOfItems()<<"] credential items"<<std::endl;
        for(int it=0;it<creds.numberOfItems();++it){
          std::cout <<"For item="<<creds.itemName(it)<<" value="<<creds.valueForItem(creds.itemName(it))<<std::endl;
        }
        std::cout <<" Role: calib_reader"<<std::endl;
        try{
          const coral::IAuthenticationCredentials& creds0 = authenticationSvc->credentials(dbSet->replica( i ).connectionString(),"calib_reader");
          std::cout <<" Found ["<<creds0.numberOfItems()<<"] credential items"<<std::endl;
          for(int it=0;it<creds0.numberOfItems();++it){
            std::cout <<"For item="<<creds0.itemName(it)<<" value="<<creds0.valueForItem(creds0.itemName(it))<<std::endl;
          }
        } catch (coral::Exception& e){
          std::cout << "Exception: "<<e.what()<<std::endl;
        }
        std::cout <<" Role: calib_writer"<<std::endl;
        try{
          const coral::IAuthenticationCredentials& creds1 = authenticationSvc->credentials(dbSet->replica( i ).connectionString(),"calib_writer");
          std::cout <<" Found ["<<creds1.numberOfItems()<<"] credential items"<<std::endl;
          for(int it=0;it<creds1.numberOfItems();++it){
            std::cout <<"For item="<<creds1.itemName(it)<<" value="<<creds1.valueForItem(creds1.itemName(it))<<std::endl;
          }
        } catch (coral::Exception& e){
          std::cout << "Exception: "<<e.what()<<std::endl;
        }
      }
    }
    delete dbSet;
    dbSet = 0;
    dbSet = lookupSvc->lookup( "test1/ecal_calib0", coral::ReadOnly);
    numberOfReplicas = dbSet->numberOfReplicas();
    std::cout <<" Found ["<<numberOfReplicas<<"] replicas"<<std::endl;
    for ( int i = 0; i < numberOfReplicas; ++i ){
      reportReplica( dbSet->replica( i ), "test1/ecal_calib0" );
      if(dbSet->replica( i ).authenticationMechanism()!=""){
        std::cout <<" Default role:"<<std::endl;
        const coral::IAuthenticationCredentials& creds = authenticationSvc->credentials(dbSet->replica( i ).connectionString());
        std::cout <<" Found ["<<creds.numberOfItems()<<"] credential items"<<std::endl;
        for(int it=0;it<creds.numberOfItems();++it){
          std::cout <<"For item="<<creds.itemName(it)<<" value="<<creds.valueForItem(creds.itemName(it))<<std::endl;
        }
        std::cout <<" Role: calib_reader"<<std::endl;
        try{
          const coral::IAuthenticationCredentials& creds0 = authenticationSvc->credentials(dbSet->replica( i ).connectionString(),"calib_reader");
          std::cout <<" Found ["<<creds0.numberOfItems()<<"] credential items"<<std::endl;
          for(int it=0;it<creds0.numberOfItems();++it){
            std::cout <<"For item="<<creds0.itemName(it)<<" value="<<creds0.valueForItem(creds0.itemName(it))<<std::endl;
          }
        } catch (coral::Exception& e){
          std::cout << "Exception: "<<e.what()<<std::endl;
        }
        std::cout <<" Role: calib_writer"<<std::endl;
        try{
          const coral::IAuthenticationCredentials& creds1 = authenticationSvc->credentials(dbSet->replica( i ).connectionString(),"calib_writer");
          std::cout <<" Found ["<<creds1.numberOfItems()<<"] credential items"<<std::endl;
          for(int it=0;it<creds1.numberOfItems();++it){
            std::cout <<"For item="<<creds1.itemName(it)<<" value="<<creds1.valueForItem(creds1.itemName(it))<<std::endl;
          }
        } catch (coral::Exception& e){
          std::cout << "Exception: "<<e.what()<<std::endl;
        }
      }
    }
    delete dbSet;
    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
