// $Id: View.cpp,v 1.2 2008/03/26 12:00:17 rado Exp $
#include "View.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "ErrorHandler.h"
#include "Statement.h"
#include "PrivilegeManager.h"

#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/SchemaException.h"
#include "RelationalAccess/ITypeConverter.h"

#include "CoralBase/Blob.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/MessageStream.h"

#include "CoralKernel/Service.h"

coral::MySQLAccess::View::View( const coral::MySQLAccess::ISessionProperties& sessionProperties,
                                 const std::string& viewName,
                                 OCIDescribe* descriptor ):
  m_sessionProperties( sessionProperties ),
  m_describeHandle( descriptor ),
  m_definitionString( "" ),
  m_description( new coral::TableDescription( m_sessionProperties.domainProperties().service()->name() ) ),
  m_privilegeManager( new coral::MySQLAccess::PrivilegeManager( m_sessionProperties, m_sessionProperties.schemaName() + ".\"" + viewName + "\"" ) )
{
  m_description->setName( viewName );
}


coral::ITablePrivilegeManager&
coral::MySQLAccess::View::privilegeManager()
{
  return *m_privilegeManager;
}

const coral::ITableDescription&
coral::MySQLAccess::View::description() const
{
  if ( m_description->numberOfColumns() == 0 )
    const_cast<coral::MySQLAccess::View*>( this )->refreshDescription();
  return *m_description;
}


coral::MySQLAccess::View::~View()
{
  delete m_privilegeManager;
  delete m_description;
// FIXME - clean up describe handle  OCIHandleFree( m_describeHandle, OCI_HTYPE_DESCRIBE );
}


std::string
coral::MySQLAccess::View::name() const
{
  return m_description->name();
}


std::string
coral::MySQLAccess::View::definition() const
{
  if ( m_definitionString.empty() )
    const_cast<coral::MySQLAccess::View*>( this )->refreshDefinition();
  return m_definitionString;
}


int
coral::MySQLAccess::View::numberOfColumns() const
{
  if ( m_description->numberOfColumns() == 0 )
    const_cast<coral::MySQLAccess::View*>( this )->refreshDescription();
  return m_description->numberOfColumns();
}


const coral::IColumn& coral::MySQLAccess::View::column( int index ) const
{
  if ( m_description->numberOfColumns() == 0 )
    const_cast<coral::MySQLAccess::View*>( this )->refreshDescription();
  
  return m_description->columnDescription( index );
}

void coral::MySQLAccess::View::refreshDescription()
{
//   // Retrieving the describe parameter handle
//
//   // Retrieving the column list
//   if ( status != SUCCESS ) {
//     coral::MySQLAccess::ErrorHandler errorHandler( m_sessionProperties.errorHandle() );
//     errorHandler.handleCase( status, "Retrieving the column list for a view" );
//     seal::MessageStream log( m_sessionProperties.domainProperties().service(), m_sessionProperties.domainProperties().service()->name() );
//     log << coral::Error << errorHandler.message() << seal::flush;    
//     throw coral::SchemaException( m_sessionProperties.domainProperties().service()->name(), "Could not retrieve the column list for a view", "View::refreshDescription" );
//   }

//   // Retrieving the number of columns
//   FIXME - For each column in the view retrieve its metadata, the same thing as done for table chema loader
//   FIXME - Can be done only for MySQL 5.0 & later  
}

void coral::MySQLAccess::View::refreshDefinition()
{
  std::ostringstream os;

  os << "SELECT TEXT FROM ALL_VIEWS WHERE VIEW_NAME='" << this->description().name() << "' AND OWNER='" << m_sessionProperties.schemaName() << "'";

  std::string sql = os.str();
  
  coral::MySQLAccess::Statement statement( m_sessionProperties, sql );
  
  if ( ! statement.execute( coral::AttributeList() ) )
    throw coral::SchemaException( m_sessionProperties.domainProperties().service()->name(), "Could not read the definition of a view", "View::refreshDefinition" );

  coral::AttributeList* output = new coral::AttributeList;
  output->extend( "TEXT", typeid(std::string) );
  output->begin()->data<std::string>() = "";
  statement.defineOutput( *output );
  statement.fetchNext();
  m_definitionString = output->begin()->data<std::string>();
  delete output;
  
  if ( m_definitionString.empty() )
  {
    throw coral::SchemaException( m_sessionProperties.domainProperties().service()->name(), "Could not read the definition of a view", "View::refreshDefinition" );
  }

  // Remove the parentheses
  while ( m_definitionString[0] == ' ' || m_definitionString[0] == '\t' || m_definitionString[0] == '\n' || m_definitionString[0] == '\r' )
  {
    m_definitionString = m_definitionString.substr( 1 );
  }

  std::string::size_type iPos = m_definitionString.size() - 1;
  while ( m_definitionString[iPos] == ' ' || m_definitionString[iPos] == '\t' || m_definitionString[iPos] == '\n' || m_definitionString[iPos] == '\r' )
  {
    m_definitionString = m_definitionString.substr( 0, iPos );
    --iPos;
  }

  if ( m_definitionString[0] == '(' && m_definitionString[iPos] == ')' )
  {
    m_definitionString = m_definitionString.substr( 1, iPos - 1 );

    while ( m_definitionString[0] == ' ' || m_definitionString[0] == '\t' || m_definitionString[0] == '\n' || m_definitionString[0] == '\r' )
    {
      m_definitionString = m_definitionString.substr( 1 );
    }

    iPos = m_definitionString.size() - 1;
    while ( m_definitionString[iPos] == ' ' || m_definitionString[iPos] == '\t' || m_definitionString[iPos] == '\n' || m_definitionString[iPos] == '\r' )
    {
      m_definitionString = m_definitionString.substr( 0, iPos );
      --iPos;
    }
  }
}
