// $Id: ViewFactory.cpp,v 1.2 2008/03/26 12:00:17 rado Exp $
#include "ViewFactory.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "Statement.h"

#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/SchemaException.h"

#include "CoralKernel/Service.h"

coral::MySQLAccess::ViewFactory::ViewFactory( const coral::MySQLAccess::ISessionProperties& properties )
  : coral::MySQLAccess::QueryDefinition( properties )
{
}

coral::MySQLAccess::ViewFactory::~ViewFactory()
{
}

coral::IView& coral::MySQLAccess::ViewFactory::create( const std::string& viewName )
{
  if ( this->sessionProperties().schema().existsView( viewName ) )
    throw coral::ViewAlreadyExistingException( this->sessionProperties().domainProperties().service()->name(), viewName );
  
  std::string sqlStatement = "CREATE VIEW " + this->sessionProperties().schemaName() + ".\"" + viewName + "\" AS ( " + this->sqlFragment() + " )";
  
  coral::MySQLAccess::Statement statement( this->sessionProperties(), sqlStatement );
  
  if ( ! statement.execute( this->bindData() ) )
    throw coral::SchemaException( this->sessionProperties().domainProperties().service()->name(), "Could not create view \"" + viewName + "\"", "IViewFactory::create" );

  return this->sessionProperties().schema().viewHandle( viewName );
}


coral::IView& coral::MySQLAccess::ViewFactory::createOrReplace( const std::string& viewName )
{
  std::string sqlStatement = "CREATE OR REPLACE VIEW " + this->sessionProperties().schemaName() + ".\"" + viewName + "\" AS ( " + this->sqlFragment() + " )";
  
  coral::MySQLAccess::Statement statement( this->sessionProperties(), sqlStatement );
  
  if ( ! statement.execute( this->bindData() ) )
    throw coral::SchemaException( this->sessionProperties().domainProperties().service()->name(), "Could not create view \"" + viewName + "\"", "IViewFactory::create" );
  
  return this->sessionProperties().schema().viewHandle( viewName );
}
