#include "TestBase.h"

#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/RelationalServiceException.h"

#include <iostream>

TestBase::TestBase()
{
  coral::Context& ctx = coral::Context::instance();
  
  ctx.loadComponent( "CORAL/RelationalPlugins/mysql" );
  
  const char* authvar = getenv( "CORAL_AUTH_PATH" );

  if( authvar ) 
  {    
    ctx.loadComponent( "CORAL/Services/XMLAuthenticationService" );
  }
}

TestBase::~TestBase()
{
}

coral::ISession* TestBase::connect( const std::string& connectionString, const std::string& userName, const std::string& password, coral::AccessMode mode )
{
  coral::Context& ctx = coral::Context::instance();
  
  coral::IHandle<coral::IRelationalDomain> domain = ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/mysql" );
  
  if ( ! domain.isValid() )
  {
    throw coral::NonExistingDomainException( "mysql" );
  }

  std::cout << "Retrieved relational domain \"" << domain->flavorName()
            << "\" implementation \"" << domain->implementationName()
            << "\" version " << domain->implementationVersion() << std::endl;

  std::pair<std::string,std::string> cs = domain->decodeUserConnectionString( connectionString );

  std::cout << "Connecting to: " << cs.first << " ing schema: " << cs.second << std::endl;
  
  coral::IConnection* connection = domain->newConnection( cs.first );

  // FIXME !!! call in try..catch..block
  coral::ISession* session = connection->newSession( cs.second /* the scheman name */, mode );

  if ( session ) {
    session->startUserSession( userName, password );
  }

  return session;
}
