#ifndef ORACLEACCESS_BUILK_OPERATION_H
#define ORACLEACCESS_BUILK_OPERATION_H

#include "RelationalAccess/IBulkOperation.h"
#include <string>
#include <vector>

struct OCIStmt;

namespace coral {

  class AttributeList;

  namespace OracleAccess {

    class ISessionProperties;
    class PolymorphicVector;
    class StatementStatistics;

    /**
     * Class BulkOperation
     *
     * Implementation of the IBulkOperation interface
     */

    class BulkOperation : virtual public coral::IBulkOperation
    {
    public:
      /// Constructor
      BulkOperation( const ISessionProperties& properties,
                     const coral::AttributeList& inputBuffer,
                     int cacheSize,
                     const std::string& statement );

      /// Destructor
      virtual ~BulkOperation();

      /**
       * Processes the next iteration
       */
      void processNextIteration();
    
      /**
       * Flushes the data on the client side to the server.
       */
      void flush();

    private:
      /// Resets the operation and closes the statement handle
      void reset();

    private:
      /// A reference to the sessopm properties
      const ISessionProperties&   m_properties;

      /// A reference to the input data buffer
      const coral::AttributeList& m_inputBuffer;

      /// The cache size
      int                         m_rowsInCache;

      /// The iterations counter
      int                         m_rowsUsed;

      /// The OCI statement handle
      OCIStmt*                    m_statementHandle;

      /// The data cache
      std::vector< PolymorphicVector* > m_dataCache;

      /// The statement statistics
      StatementStatistics*        m_statementStatistics;
    };

  }

}

#endif
