#include "BulkOperationWithQuery.h"
#include "BulkOperation.h"
#include "QueryDefinition.h"

coral::OracleAccess::BulkOperationWithQuery::BulkOperationWithQuery( const coral::OracleAccess::ISessionProperties& properties,
                                                                     int cacheSize,
                                                                     const std::string& statement ):
  m_properties( properties ),
  m_rowsInCache( ( ( cacheSize > 65534 ) ? 65534 : cacheSize ) ),
  m_sqlPrefix( statement ),
  m_queryDefinition( new coral::OracleAccess::QueryDefinition( m_properties ) ),
  m_bulkOperation( 0 )
{}


coral::OracleAccess::BulkOperationWithQuery::~BulkOperationWithQuery()
{
  if ( m_bulkOperation ) delete m_bulkOperation;
}


coral::IQueryDefinition&
coral::OracleAccess::BulkOperationWithQuery::query()
{
  return *m_queryDefinition;
}


void
coral::OracleAccess::BulkOperationWithQuery::processNextIteration()
{
  if ( ! m_bulkOperation ) {
    m_bulkOperation = new coral::OracleAccess::BulkOperation( m_properties,
                                                              m_queryDefinition->bindData(),
                                                              m_rowsInCache,
                                                              m_sqlPrefix + " ( " + m_queryDefinition->sqlFragment() + " )" );
  }

  m_bulkOperation->processNextIteration();
}


void
coral::OracleAccess::BulkOperationWithQuery::flush()
{
  if ( ! m_bulkOperation ) {
    m_bulkOperation = new coral::OracleAccess::BulkOperation( m_properties,
                                                              m_queryDefinition->bindData(),
                                                              m_rowsInCache,
                                                              m_sqlPrefix + " ( " + m_queryDefinition->sqlFragment() + " )" );
  }
  m_bulkOperation->flush();
}
