#include "ConnectionProperties.h"
#include "TypeConverter.h"
#include "oci.h"

coral::OracleAccess::ConnectionProperties::ConnectionProperties( const DomainProperties& domainProperties,
                                                                 const std::string& connectionString ):
  m_domainProperties( domainProperties ),
  m_connectionString( connectionString ),
  m_typeConverter( new coral::OracleAccess::TypeConverter( m_domainProperties ) ),
  m_environmentHandle( 0 ),
  m_errorHandle( 0 ),
  m_serverHandle( 0 ),
  m_serverVersion( 0 ),
  m_mutex()
{}



coral::OracleAccess::ConnectionProperties::~ConnectionProperties()
{
  delete m_typeConverter;
}


void
coral::OracleAccess::ConnectionProperties::setHandles( OCIEnv* environmentHandle,
                                                       OCIError* errorHandle,
                                                       OCIServer* serverHandle,
                                                       int majorServerVersion )
{
  boost::mutex::scoped_lock lock( m_mutex );
  m_environmentHandle = environmentHandle;
  m_errorHandle = errorHandle;
  m_serverHandle = serverHandle;
  m_typeConverter->reset( majorServerVersion );
  m_serverVersion = majorServerVersion;
}


coral::ITypeConverter&
coral::OracleAccess::ConnectionProperties::typeConverter()
{
  boost::mutex::scoped_lock lock( m_mutex );
  return *m_typeConverter;
}


const coral::ITypeConverter&
coral::OracleAccess::ConnectionProperties::typeConverter() const
{
  boost::mutex::scoped_lock lock( m_mutex );
  return *m_typeConverter;
}
