#ifndef ISESSION_PROPERTIES_H
#define ISESSION_PROPERTIES_H

#include <string>

struct OCIEnv;
struct OCIError;
struct OCISvcCtx;

namespace coral {

  class ITypeConverter;
  class ISchema;

  namespace monitor {
    class IMonitoringService;
  }

  namespace OracleAccess {

    class DomainProperties;
    class TypeConverter;

    /**
     * Class SessionProperties
     *
     * Interface for classes holding the domain properties, as well as the global
     * data and properties for the current session.
     */

    class ISessionProperties {
    public:
      virtual ~ISessionProperties(){}

      /// Returns the domain properties
      virtual const DomainProperties& domainProperties() const = 0;

      /// Returns the type converter
      virtual coral::ITypeConverter& typeConverter() = 0;
      virtual const coral::ITypeConverter& typeConverter() const = 0;

      /// Returns the OCI environment handle
      virtual OCIEnv* environmentHandle() const = 0;

      /// Returns the OCI error handle
      virtual OCIError* errorHandle() const = 0;

      /// Returns the OCI service context handle
      virtual OCISvcCtx* serviceContextHandle() const = 0;

      /// Returns the monitoring service
      virtual coral::monitor::IMonitoringService* monitoringService() const = 0;

      /// Returns the transaction state
      virtual bool isTransactionActive() const = 0;

      /// Returns the server version
      virtual int serverVersion() const = 0;

      /// Returns the corresponding schema
      virtual coral::ISchema& schema() const = 0;

      /// Returns the schema name
      virtual std::string schemaName() const = 0;

      /// Returns the connection string
      virtual std::string connectionString() const = 0;

      /// Returns the readOnly flag
      virtual bool isReadOnly() const = 0;
    };

  }

}


#endif
