#include "MonitorController.h"
#include "SessionProperties.h"
#include "DomainProperties.h"

#include "RelationalAccess/IMonitoringService.h"
#include "RelationalAccess/IMonitoringReporter.h"
#include "RelationalAccess/SessionException.h"

#include "CoralKernel/Service.h"
#include "CoralKernel/Context.h"

#include <iostream>


coral::OracleAccess::MonitorController::MonitorController( coral::OracleAccess::SessionProperties& properties ):
  m_properties( properties ),
  m_mutex()
{}


coral::OracleAccess::MonitorController::~MonitorController()
{}


void
coral::OracleAccess::MonitorController::start( coral::monitor::Level level )
{
  boost::mutex::scoped_lock lock( m_mutex );
  
  coral::monitor::IMonitoringService* monitoringService = m_properties.monitoringService();
  
  if ( monitoringService == NULL )
  {
    coral::IHandle<coral::monitor::IMonitoringService> monService = coral::Context::instance().query<coral::monitor::IMonitoringService>();
    
    if( ! monService.isValid() )
    {
      throw coral::MonitoringServiceNotFoundException( m_properties.domainProperties().service()->name() );
    }
  
    monitoringService = monService.get();
    m_properties.setMonitoringService( monitoringService );

  } 
  monitoringService->setLevel( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName(), level );
  monitoringService->enable( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName() );
}


void
coral::OracleAccess::MonitorController::stop()
{
  boost::mutex::scoped_lock lock( m_mutex );
  coral::monitor::IMonitoringService* monitoringService = m_properties.monitoringService();
  if ( monitoringService ) {
    monitoringService->disable( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName() );
  }
  m_properties.setMonitoringService( 0 );
}


void
coral::OracleAccess::MonitorController::reportToOutputStream( std::ostream& os ) const
{
  boost::mutex::scoped_lock lock( m_mutex );
  coral::monitor::IMonitoringService* monitoringService = m_properties.monitoringService();
  if ( ! monitoringService ) {
    throw coral::MonitoringServiceNotFoundException( m_properties.domainProperties().service()->name(),
                                                     "IMonitoring::reportToOutputStream" );
  }
  monitoringService->reporter().reportToOutputStream( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName(),
                                                      os );
}


void
coral::OracleAccess::MonitorController::report() const
{
  boost::mutex::scoped_lock lock( m_mutex );
  coral::monitor::IMonitoringService* monitoringService = m_properties.monitoringService();
  if ( ! monitoringService ) {
    throw coral::MonitoringServiceNotFoundException( m_properties.domainProperties().service()->name(),
                                                     "IMonitoring::report" );
  }
  monitoringService->reporter().report( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName() );
}


bool
coral::OracleAccess::MonitorController::isActive() const
{
  boost::mutex::scoped_lock lock( m_mutex );
  return ( m_properties.monitoringService() != 0 ||
           m_properties.monitoringService()->active( "oracle://" + m_properties.connectionString() + "/" + m_properties.schemaName() ) );
}
