#include "PrivilegeManager.h"
#include "OracleStatement.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"

#include "RelationalAccess/SchemaException.h"

#include "CoralBase/AttributeList.h"

#include "CoralKernel/Service.h"

#include <sstream>

coral::OracleAccess::PrivilegeManager::PrivilegeManager( const coral::OracleAccess::ISessionProperties& properties,
                                                         const std::string& tableName ):
  m_properties( properties ),
  m_tableName( tableName )
{}


void
coral::OracleAccess::PrivilegeManager::grantToUser( const std::string& userName,
                                                    coral::ITablePrivilegeManager::Privilege right )
{
  std::ostringstream os;
  os << "GRANT ";
  switch ( right ) {
  case coral::ITablePrivilegeManager::Select:
    os << "SELECT ";
    break;
  case coral::ITablePrivilegeManager::Update:
    os << "UPDATE ";
    break;
  case coral::ITablePrivilegeManager::Insert:
    os << "INSERT ";
    break;
  case coral::ITablePrivilegeManager::Delete:
    os << "DELETE ";
    break;
  default:
    os << "SELECT ";
  };

  os << "ON " << m_tableName << " TO " << userName;
  coral::OracleAccess::OracleStatement statement( m_properties,
                                                  os.str() );

  coral::AttributeList bindData;
  if ( ! statement.execute( bindData ) ) {
    throw coral::SchemaException( m_properties.domainProperties().service()->name(),
                                  "Could not grant access to a user",
                                  "ITablePrivilegeManager::grantToUser" );
  }
}


void
coral::OracleAccess::PrivilegeManager::revokeFromUser( const std::string& userName,
                                                       coral::ITablePrivilegeManager::Privilege right )
{
  std::ostringstream os;
  os << "REVOKE ";
  switch ( right ) {
  case coral::ITablePrivilegeManager::Select:
    os << "SELECT ";
    break;
  case coral::ITablePrivilegeManager::Update:
    os << "UPDATE ";
    break;
  case coral::ITablePrivilegeManager::Insert:
    os << "INSERT ";
    break;
  case coral::ITablePrivilegeManager::Delete:
    os << "DELETE ";
    break;
  default:
    os << "SELECT ";
  };

  os << "ON " << m_tableName << " FROM " << userName;
  coral::OracleAccess::OracleStatement statement( m_properties,
                                                  os.str() );

  coral::AttributeList bindData;
  if ( ! statement.execute( bindData ) ) {
    throw coral::SchemaException( m_properties.domainProperties().service()->name(),
                                  "Could not revoke access from a user",
                                  "ITablePrivilegeManager::revokeFromUser" );
  }
}


void
coral::OracleAccess::PrivilegeManager::grantToPublic( coral::ITablePrivilegeManager::Privilege right )
{
  this->grantToUser( "PUBLIC", right );
}


void
coral::OracleAccess::PrivilegeManager::revokeFromPublic( coral::ITablePrivilegeManager::Privilege right )
{
  this->revokeFromUser( "PUBLIC", right );
}
