#ifndef ORACLEACCESS_PRIVILEGEMANAGER_H
#define ORACLEACCESS_PRIVILEGEMANAGER_H

#include "RelationalAccess/ITablePrivilegeManager.h"

namespace coral {

  namespace OracleAccess {

    class ISessionProperties;

    /**
     * Class PrivilegeManager
     *
     * Implementation of the ITablePrivilegeManager interface for the OracleAccess module
     */

    class PrivilegeManager : virtual public coral::ITablePrivilegeManager
    {
    public:
      /// Constructor
      PrivilegeManager( const ISessionProperties& properties,
                        const std::string& tableName );

      /// Destructor
      virtual ~PrivilegeManager() {}

      /**
       * Grants an access right to a specific user.
       */
      void grantToUser( const std::string& userName,
                        coral::ITablePrivilegeManager::Privilege right );
    
      /**
       * Revokes a right from the specified user.
       */
      void revokeFromUser( const std::string& userName,
                           coral::ITablePrivilegeManager::Privilege right );
  
      /**
       * Grants the specified right to all users.
       */
      void grantToPublic( coral::ITablePrivilegeManager::Privilege right );
  
      /**
       * Revokes the specified right from all users.
       */
      void revokeFromPublic( coral::ITablePrivilegeManager::Privilege right );

    private:
      /// The session properties
      const ISessionProperties& m_properties;

      /// The fully qualified table name
      std::string              m_tableName;
    };

  }

}

#endif
