#ifndef SESSION_PROPERTIES_PROXY_H
#define SESSION_PROPERTIES_PROXY_H

#include "SessionProperties.h"

namespace coral {

  namespace OracleAccess {

    /**
     * Class SessionProperties
     *
     * An implementation of the ISessionProperties interface a proxy to the SessionProperties
     */

    class SessionPropertiesProxy : virtual public ISessionProperties
    {
    public:
      /// Constructor
      SessionPropertiesProxy( const SessionProperties& properties,
                              const std::string& schemaName );

      /// Destructor
      virtual ~SessionPropertiesProxy();

      /// Returns the domain properties
      const DomainProperties& domainProperties() const;

      /// Returns the type converter
      coral::ITypeConverter& typeConverter();
      const coral::ITypeConverter& typeConverter() const;

      /// Returns the OCI environment handle
      OCIEnv* environmentHandle() const;

      /// Returns the OCI error handle
      OCIError* errorHandle() const;

      /// Returns the OCI service context handle
      OCISvcCtx* serviceContextHandle() const;

      /// Returns the monitoring service
      coral::monitor::IMonitoringService* monitoringService() const;

      /// Returns the transaction state
      bool isTransactionActive() const;

      /// Returns the server version
      int serverVersion() const;

      /// Returns the corresponding schema
      coral::ISchema& schema() const;

      /// Returns the schema name
      std::string schemaName() const;

      /// Returns the connection string
      std::string connectionString() const;

      /// Returns the readOnly flag
      bool isReadOnly() const;

    private:
      /// The underlying properties object
      const SessionProperties& m_properties;

      /// The schema name
      const std::string        m_schemaName;
    };

  }

}






// Inline methods

inline
coral::OracleAccess::SessionPropertiesProxy::SessionPropertiesProxy( const coral::OracleAccess::SessionProperties& properties,
                                                                     const std::string& schemaName ):
  m_properties( properties ),
  m_schemaName( schemaName )
{}

inline
coral::OracleAccess::SessionPropertiesProxy::~SessionPropertiesProxy()
{}

inline const coral::OracleAccess::DomainProperties&
coral::OracleAccess::SessionPropertiesProxy::domainProperties() const
{
  return m_properties.domainProperties();
}

inline coral::ITypeConverter&
coral::OracleAccess::SessionPropertiesProxy::typeConverter()
{
  return const_cast< coral::OracleAccess::SessionProperties& >( m_properties ).typeConverter();
}

inline const coral::ITypeConverter&
coral::OracleAccess::SessionPropertiesProxy::typeConverter() const
{
  return m_properties.typeConverter();
}

inline OCIEnv*
coral::OracleAccess::SessionPropertiesProxy::environmentHandle() const
{
  return m_properties.environmentHandle();
}

inline OCIError*
coral::OracleAccess::SessionPropertiesProxy::errorHandle() const
{
  return m_properties.errorHandle();
}

inline OCISvcCtx*
coral::OracleAccess::SessionPropertiesProxy::serviceContextHandle() const
{
  return m_properties.serviceContextHandle();
}

inline
coral::monitor::IMonitoringService*
coral::OracleAccess::SessionPropertiesProxy::monitoringService() const
{
  return m_properties.monitoringService();
}

inline bool
coral::OracleAccess::SessionPropertiesProxy::isTransactionActive() const
{
  return m_properties.isTransactionActive();
}

inline int
coral::OracleAccess::SessionPropertiesProxy::serverVersion() const
{
  return m_properties.serverVersion();
}

inline std::string
coral::OracleAccess::SessionPropertiesProxy::schemaName() const
{
  return m_schemaName;
}


inline std::string
coral::OracleAccess::SessionPropertiesProxy::connectionString() const
{
  return m_properties.connectionString();
}


inline bool
coral::OracleAccess::SessionPropertiesProxy::isReadOnly() const
{
  return m_properties.isReadOnly();
}


#endif
