#ifndef ORACLEACCESS_TRANSACTION_H
#define ORACLEACCESS_TRANSACTION_H

#include "RelationalAccess/ITransaction.h"
#include "CoralBricks/ThreadTraits.h"

struct OCITrans;

namespace coral {

  namespace OracleAccess {

    class SessionProperties;
    class ITransactionObserver;

    /**
     * Class Transaction
     * Implementation of the ITransaction interface for the OracleAccess plugin
     */
    class Transaction : virtual public coral::ITransaction
    {
    public:
      /// Constructor
      Transaction( SessionProperties& properties,
                   ITransactionObserver& observer );

      /// Destructor
      virtual ~Transaction();

      /**
       * Starts a new transaction.
       * In case of failure a TransactionNotStartedException is thrown.
       * 
       */
      void start( bool readOnly = false );
  
      /**
       * Commits the transaction.
       * In case of failure a TransactionNotCommittedException is thrown.
       */
      void commit();
  
      /**
       * Aborts and rolls back a transaction.
       */
      void rollback();
    
      /**
       * Returns the status of the transaction (if it is active or not)
       */
      bool isActive() const;
  
      /**
       * Returns the mode of the transaction (if it is read-only or not)
       */
      bool isReadOnly() const;

    private:
      /// Allocates the necessary handles
      void allocateHandles();

      /// Releases the handles
      void releaseHandles();

    private:
      /// The session properties
      SessionProperties&    m_sessionProperties;

      /// The transaction observer
      ITransactionObserver& m_observer;

      /// The OCI transaction handle
      OCITrans*             m_transactionHandle;

      /// Flag indicating whether a transaction is read-only
      bool                  m_isReadOnly;

      /// The mutex lock
      mutable boost::mutex  m_mutex;
    };

  }

}

#endif
