#include "ViewFactory.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "OracleStatement.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/SchemaException.h"

#include "CoralKernel/Service.h"


coral::OracleAccess::ViewFactory::ViewFactory( const coral::OracleAccess::ISessionProperties& properties ):
  coral::OracleAccess::QueryDefinition( properties )
{}


coral::OracleAccess::ViewFactory::~ViewFactory()
{}


coral::IView&
coral::OracleAccess::ViewFactory::create( const std::string& viewName )
{
  if ( this->sessionProperties().schema().existsView( viewName ) )
    throw coral::ViewAlreadyExistingException( this->sessionProperties().domainProperties().service()->name(),
                                               viewName );
  std::string sqlStatement =
    "CREATE VIEW " + this->sessionProperties().schemaName() + ".\"" + viewName + "\" AS ( " + this->sqlFragment() + " )";
  coral::OracleAccess::OracleStatement statement( this->sessionProperties(),
                                                  sqlStatement );
  if ( ! statement.execute( this->bindData() ) )
    throw coral::SchemaException( this->sessionProperties().domainProperties().service()->name(),
                                  "Could not create view \"" + viewName + "\"",
                                  "IViewFactory::create" );
  return this->sessionProperties().schema().viewHandle( viewName );
}


coral::IView&
coral::OracleAccess::ViewFactory::createOrReplace( const std::string& viewName )
{
  std::string sqlStatement =
    "CREATE OR REPLACE VIEW " + this->sessionProperties().schemaName() + ".\"" + viewName + "\" AS ( " + this->sqlFragment() + " )";
  coral::OracleAccess::OracleStatement statement( this->sessionProperties(),
                                                  sqlStatement );
  if ( ! statement.execute( this->bindData() ) )
    throw coral::SchemaException( this->sessionProperties().domainProperties().service()->name(),
                                  "Could not create view \"" + viewName + "\"",
                                  "IViewFactory::create" );
  return this->sessionProperties().schema().viewHandle( viewName );
}
