#include "ConnectionApp.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"

#include <iostream>
#include <stdexcept>


ConnectionApp::ConnectionApp( const std::string& connectionString,
                              const std::string& userName,
                              const std::string& password ):
  TestBase(),
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password )
{}


ConnectionApp::~ConnectionApp()
{}


void
ConnectionApp::run()
{
  coral::ISession* session = this->connect( m_connectionString,
                                            m_userName,
                                            m_password );
  
  session->transaction().start();

  std::cout << "Tables in nominal schema (I) : " << std::endl;
  std::set<std::string> tables = session->nominalSchema().listTables();
  for ( std::set<std::string>::const_iterator iTable = tables.begin();
        iTable != tables.end(); ++iTable ) {
    std::cout << "   " << *iTable << std::endl;
  }

  session->transaction().commit();

  session->transaction().start( true );

  session->endUserSession();
  if ( session->isUserSessionActive() ) {
    throw std::runtime_error( "Session appears to be up and running !!!" );
  }

  session->startUserSession( m_userName, m_password );

  if ( ! session->isUserSessionActive() ) {
    throw std::runtime_error( "Connection lost..." );
  }

  session->transaction().start();

  delete session;
}
