#include "GroupApp.h"

#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Blob.h"

#include "CoralCommon/Sleep.h"

#include <iostream>
#include <stdexcept>


GroupApp::GroupApp( const std::string& connectionString,
                    const std::string& userName,
                    const std::string& password ):
  TestBase(),
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password )
{}


GroupApp::~GroupApp()
{}


void
GroupApp::run()
{
  this->fillData();
  coral::sys::sleep(1);
  this->readData();
}



void
GroupApp::fillData()
{
  coral::ISession* session = this->connect( m_connectionString,
                                            m_userName, m_password );
  session->transaction().start();

  coral::ISchema& schema = session->nominalSchema();

  std::cout << "About to drop previous table" << std::endl;
  schema.dropIfExistsTable( "ORA_UT_GB_T" );

  std::cout << "Describing new table" << std::endl;
  coral::TableDescription description;
  description.setName( "ORA_UT_GB_T" );
  description.insertColumn( "ID",
                            coral::AttributeSpecification::typeNameForId( typeid(int) ) );
  description.setPrimaryKey( "ID" );
  description.insertColumn( "x",
                            coral::AttributeSpecification::typeNameForId( typeid(float) ) );
  description.setNotNullConstraint( "x" );
  description.insertColumn( "GrB",
                            coral::AttributeSpecification::typeNameForId( typeid(int) ) );

  std::cout << "About to create the table" << std::endl;
  coral::ITable& table = schema.createTable( description );

  coral::AttributeList rowBuffer;
  rowBuffer.extend<int>( "ID" );
  rowBuffer.extend<float>( "x" );
  rowBuffer.extend<int>( "GrB" );

  for ( int i = 0; i < 100; ++i )
  {
    rowBuffer["ID"].data<int>() = i;
    rowBuffer["GrB"].data<int>() = i%10;
    rowBuffer["x"].data<float>() = ( i%14 ) + 0.1 * ( i%17 );

    table.dataEditor().insertRow( rowBuffer );
  }


  session->transaction().commit();
  delete session;
}



void
GroupApp::readData()
{
  coral::ISession* session = this->connect( m_connectionString,
                                            m_userName, m_password );
  session->transaction().start( true );

  coral::ISchema& schema = session->nominalSchema();

  coral::IQuery* query = schema.tableHandle("ORA_UT_GB_T").newQuery();

  coral::AttributeList rowBuffer;
  rowBuffer.extend<int>( "GrB" );
  rowBuffer.extend<float>( "RES" );

  query->addToOutputList( "GrB" );
  query->addToOrderList( "GrB" );
  query->addToOutputList( "AVG(x)", "RES" );
  query->groupBy( "GrB" );

  query->defineOutput( rowBuffer );

  coral::ICursor& cursor = query->execute();

  int i = 0;
  while ( cursor.next() ) {
    rowBuffer.toOutputStream( std::cout );
    std::cout << std::endl;
    if ( i != rowBuffer["GrB"].data<int>() )
      throw std::runtime_error( "Unexpected data" );
    ++i;
  }

  if ( i != 10 )
    throw std::runtime_error( "Unexpected number of rows returned" );

  delete query;

  session->transaction().commit();
  delete session;
}
