#include "TableCreateAndQueryThread.h"
#include "../Common/Reporter.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"

#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"

#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"

#include <stdexcept>
#include <sstream>
#include <memory>
#include <cmath>

void TableCreateAndQueryThread::operator()()
{
  try {
    std::ostringstream os1;
    os1 << "TableCreateAndQueryThread no " << m_tableNumber << " about to create and fill a table";
    m_reporter.reportToSTDOUT( os1.str() );

    // The table name
    std::ostringstream osTableName;
    osTableName << "ORA_UT_MT_T_" << m_tableNumber;
    const std::string tableName = osTableName.str();

    m_schema.dropIfExistsTable( tableName );

    coral::TableDescription description;
    description.setName( tableName );
    description.insertColumn( "I", "int" );
    description.insertColumn( "F", "float" );
    description.insertColumn( "D", "double" );
    description.setPrimaryKey( "I" );
    description.setNotNullConstraint( "F" );

    coral::ITable& table = m_schema.createTable( description );
    coral::AttributeList rowBuffer;
    rowBuffer.extend<int>( "I" );
    rowBuffer.extend<float>( "F" );
    rowBuffer.extend<double>( "D" );

    int& i = rowBuffer[0].data<int>();
    float& f = rowBuffer[1].data<float>();
    double& d = rowBuffer[2].data<double>();

    for ( int row01 = 0; row01 < 100; ++row01 ) {
      i = row01;
      f = row01 + 0.001 * m_tableNumber;
      d = row01 + 0.000001 * m_tableNumber;
      table.dataEditor().insertRow( rowBuffer );
    }

    std::ostringstream os2;
    os2 << "TableCreateAndQueryThread no " << m_tableNumber << " about to perform a query";
    m_reporter.reportToSTDOUT( os2.str() );

    std::auto_ptr<coral::IQuery> query( table.newQuery() );

    coral::AttributeList outputBuffer;
    outputBuffer.extend<double>( "RES" );

    query->addToOutputList( "F+D", "RES" );
    query->addToOrderList( "I" );
    query->defineOutput( outputBuffer );

    coral::ICursor& cursor = query->execute();
    int row02 = 0;
    double& res = outputBuffer[0].data<double>();
    while ( cursor.next() ) {
      if ( ::fabs( res - ( 2* row02 + 0.001001 * m_tableNumber ) ) > 0.00001 )
        throw std::runtime_error( "Unexpected data" );
      ++row02;
    }

    if ( row02 != 100 )
      throw std::runtime_error( "Unexpected number of rows" );

  }
  catch ( std::exception& e ) {
    std::ostringstream os;
    os << "[OVAL] : Standard C++ exception raised in TableCreateAndQueryThread no " << m_tableNumber << " : " << e.what();
    m_reporter.reportToSTDERR( os.str() );
  }
  catch (...) {
    std::ostringstream os;
    os << "[OVAL] : Unknonwn exception ... raised in TableCreateAndQueryThread no " << m_tableNumber;
    m_reporter.reportToSTDERR( os.str() );
  }

  std::ostringstream os;
  os << "TableCreateAndQueryThread no " << m_tableNumber << " ended gracefully";
  m_reporter.reportToSTDOUT( os.str() );
}
