#include "ThreadApp.h"
#include "TableCreateAndQueryThread.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "CoralBricks/ThreadTraits.h"
#include <vector>

ThreadApp::ThreadApp( const std::string& connectionString,
                      const std::string& userName,
                      const std::string& password,
                      Reporter& reporter ):
  TestBase(),
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password ),
  m_reporter( reporter )
{}


ThreadApp::~ThreadApp()
{}


void
ThreadApp::run()
{
  coral::ISession* session = this->connect( m_connectionString,
                                            m_userName, m_password );
  session->transaction().start();

  coral::ISchema& schema = session->nominalSchema();

  std::vector< TableCreateAndQueryThread* > threadBodies;

  for ( int i = 0; i < 10; ++i )
    threadBodies.push_back( new TableCreateAndQueryThread( schema, i, m_reporter ) );

  std::vector< boost::thread* > threads;
  for ( int i = 0; i < 10; ++i )
    threads.push_back( new boost::thread( *( threadBodies[i] ) ) );

  for ( int i = 0; i < 10; ++i )
    threads[i]->join();

  for ( int i = 0; i < 10; ++i ) {
    delete threads[i];
    delete threadBodies[i];
  }

  session->transaction().commit();
  delete session;
}
