#include "../Common/Reporter.h"

#include "MultipleSessionsApp.h"
#include "ThreadBody.h"

#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/RelationalServiceException.h"

#include "CoralKernel/Context.h"

#include "CoralBricks/ThreadTraits.h"

#include <vector>
#include <memory>


MultipleSessionsApp::MultipleSessionsApp( const std::string& connectionString,
                                          const std::string& userName,
                                          const std::string& password,
                                          Reporter& reporter ):
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password ),
  m_reporter( reporter )
{
}


MultipleSessionsApp::~MultipleSessionsApp()
{
}

void
MultipleSessionsApp::run()
{
  coral::Context& ctx = coral::Context::instance();
  
  ctx.loadComponent( "CORAL/RelationalPlugins/oracle" );
  
  coral::IHandle<coral::IRelationalDomain> iHandle = ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/oracle" );
  
  if ( ! iHandle.isValid() )
  {
    throw coral::NonExistingDomainException( "oracle" );
  }

  std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( m_connectionString );

  std::auto_ptr<coral::IConnection> connection( iHandle->newConnection( connectionAndSchema.first ) );
  connection->connect();

  // Launch the parallel threads here that share the same connection...
  std::vector< ThreadBody* > threadBodies;

  for ( int i = 0; i < 10; ++i )
    threadBodies.push_back( new ThreadBody( *connection, i, m_reporter,
                                            connectionAndSchema.second,
                                            m_userName, m_password ) );

  std::vector< boost::thread* > threads;
  for ( int i = 0; i < 10; ++i )
    threads.push_back( new boost::thread( *( threadBodies[i] ) ) );

  m_reporter.reportToSTDOUT( "Waiting for the threads..." );

  for ( int i = 0; i < 10; ++i )
    threads[i]->join();

  for ( int i = 0; i < 10; ++i ) {
    delete threads[i];
    delete threadBodies[i];
  }

}
