#include "ViewApp.h"

#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IView.h"
#include "RelationalAccess/IViewFactory.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/TableDescription.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/AttributeSpecification.h"

#include "CoralCommon/Sleep.h"

#include <iostream>
#include <stdexcept>


ViewApp::ViewApp( const std::string& connectionString,
                  const std::string& userName,
                  const std::string& password ):
  TestBase(),
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password )
{}


ViewApp::~ViewApp()
{}


void
ViewApp::run()
{
  coral::ISession* session = this->connect( m_connectionString,
                                            m_userName, m_password );
  session->transaction().start();

  coral::ISchema& schema = session->nominalSchema();

  std::cout << "About to drop previous tables and views" << std::endl;
  schema.dropIfExistsView( "Vt" );
  schema.dropIfExistsTable( "T" );

  std::cout << "Creating a new table" << std::endl;
  coral::TableDescription description;
  description.setName( "T" );
  description.insertColumn( "ID",
                            coral::AttributeSpecification::typeNameForId( typeid(int) ) );
  description.setPrimaryKey( "ID" );
  description.insertColumn( "x",
                            coral::AttributeSpecification::typeNameForId( typeid(float) ) );
  description.setNotNullConstraint( "x" );
  description.insertColumn( "Y",
                            coral::AttributeSpecification::typeNameForId( typeid(double) ) );
  schema.createTable( description );

  std::cout << "About to create a view" << std::endl;
  coral::IViewFactory* factory = schema.viewFactory();
  factory->addToOutputList( "tt.x", "x" );
  factory->addToOutputList( "tt.Y", "y" );
  factory->addToTableList( "T", "tt" );
  factory->setCondition( "ID > 2", coral::AttributeList() );
  factory->create( "Vt" );

  delete factory;

  session->transaction().commit();

  coral::sys::sleep(1);

  session->transaction().start( true );

  coral::IView& view = session->nominalSchema().viewHandle( "Vt" );
  std::cout << "View definition : \"" << view.definition() << "\"" << std::endl;

  int numberOfColumns = view.numberOfColumns();
  for ( int i = 0; i < numberOfColumns; ++i ) {
    const coral::IColumn& column = view.column( i );
    std::cout << "  " << column.name() << " (" << column.type() << ")";
    if ( column.isNotNull() ) std::cout << " NOT NULL";
    std::cout << std::endl;
  }

  delete session;
}
