#ifndef RELATIONALAUTHENTITACTIONSERVICE_RELATIONALAUTHENTITACTIONSERVICE_H
#define RELATIONALAUTHENTITACTIONSERVICE_RELATIONALAUTHENTITACTIONSERVICE_H

#include "RelationalAccess/IAuthenticationService.h"
#include "CoralKernel/Service.h"
#include "CoralKernel/Property.h"
#include "CoralBricks/ThreadTraits.h"

#include <map>
#include <string>

namespace coral {

  class AuthenticationCredentialSet;

  namespace RelationalAuthenticationService {

    /**
     * @class AuthenticationService AuthenticationService.h
     *
     * A simple implementation of the IAuthenticationService interface based on reading an XMl file
     */
    class RelationalAuthenticationService :
      public coral::Service,
      virtual public coral::IAuthenticationService {

    public:
      /// Standard Constructor
      explicit RelationalAuthenticationService( const std::string& componentName );
      
      /// Standard Destructor
      virtual ~RelationalAuthenticationService();

    public:
      /**
       * Returns a reference to the credentials object for a given connection string.
       * If the connection string is not known to the service an UnknownConnectionException is thrown.
       */
      const coral::IAuthenticationCredentials& credentials( const std::string& connectionString ) const;

      /**
       * Returns a reference to the credentials object for a given connection string.
       * If the connection string is not known to the service an UnknownConnectionException is thrown.
       * If the role is not known to the service an UnknownRoleException is thrown.
       */
      const coral::IAuthenticationCredentials& credentials( const std::string& connectionString,
                                                            const std::string& role ) const;

      public:
      
      /// Sets the initialization parameters
      void connectToDatabase( const std::string& connectionString, const std::string& encryptionKey );

      /// Sets the input connection string
      void setDataSource(  const std::string& connectionString );

      /// Sets the input connection string
      void setEncryptionKey(  const std::string& key );

      /// create the credential repository
      bool createCredentialDatabase();

      /// drop the credential repository
      bool dropCredentialDatabase();

      /// add a credential entry into the repository
      bool addCredentialEntry( const std::string& userName, const std::string& password,
                               const std::string& serviceConnectionString, std::string role="");

      /// remove a credential entry from the repository
      bool removeCredentialEntry( const std::string& serviceConnectionString, std::string role="");

      bool importCredentials( const std::string& xmlAuthenticationFile );

      bool exportCredentials( const std::string& xmlAuthenticationFile );

      bool listCredentials( std::ostream& out );

    private:
      /// Reset loaded data
      void reset();

      /// Loads data from the database
      bool loadCredentialData();

      /// bootstrap the credential data access when required (oracle, mysql)
      bool initCredentialAccess();

      private:
      
      /// Flag indicating whether the data has been loaded
      bool m_isLoaded;

      /// Status flag for the control flow
      bool m_isLoading;

      /// The database connection string
      std::string m_connectionString;

      /// The encryption key
      std::string m_key;

      /// The structure with the authentication data
      std::map< std::string, coral::AuthenticationCredentialSet* > m_data;

      /// the mutex lock
      //mutable boost::mutex m_mutexLock;

      /// The callback ids
      coral::Property::CallbackID m_callback0_ID;
      coral::Property::CallbackID m_callback1_ID;
    };

  }

}

#endif
