#include "../src/RelationalAuthenticationService.h"
#include "CoralBase/Exception.h"
#include "CoralBase/MessageStream.h"
#include "CoralCommon/CommandLine.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "CoralKernel/Context.h"

int main( int argc, char** argv ){

  std::vector<coral::Option> secondaryOptions;
  //
  coral::Option storPar("credentialDatabase");
  storPar.flag = "-db";
  storPar.helpEntry = "credential storage connection string";
  storPar.type = coral::Option::STRING;
  secondaryOptions.push_back(storPar);
  //
  coral::Option keyPar("key");
  keyPar.flag = "-k";
  keyPar.helpEntry = "the encoding key";
  keyPar.type = coral::Option::STRING;
  secondaryOptions.push_back(keyPar);
  //
  coral::Option servicePar("service");
  servicePar.flag = "-s";
  servicePar.helpEntry ="the service connection string";
  servicePar.type = coral::Option::STRING;
  secondaryOptions.push_back(servicePar);
  //
  coral::Option rolePar("role");
  rolePar.flag = "-r";
  rolePar.helpEntry ="the role associated to the credentials";
  rolePar.type = coral::Option::STRING;
  secondaryOptions.push_back(rolePar);
  //
  coral::Option userPar("username");
  userPar.flag = "-u";
  userPar.helpEntry = "the username of the database account";
  userPar.type = coral::Option::STRING;
  secondaryOptions.push_back(userPar);
  //
  coral::Option pwdPar("password");
  pwdPar.flag = "-p";
  pwdPar.helpEntry = "the password of the database account";
  pwdPar.type = coral::Option::STRING;
  secondaryOptions.push_back(pwdPar);
  //
  coral::Option debugOpt("debug");
  debugOpt.flag = "-debug";
  debugOpt.helpEntry = "set the debugging mode";
  debugOpt.type = coral::Option::BOOLEAN;
  secondaryOptions.push_back(debugOpt);
  //
  coral::Option filePar("file");
  filePar.flag = "-f";
  filePar.helpEntry = "the xml authentication file";
  filePar.type = coral::Option::STRING;
  secondaryOptions.push_back(filePar);
  //
  std::vector<coral::Command> mainSet;
  //
  coral::Command createComm("create");
  createComm.flag = "-create";
  createComm.helpEntry = "create a credential storage";
  createComm.addOption(storPar.name,true);
  createComm.addOption(debugOpt.name);
  mainSet.push_back(createComm);
  //
  coral::Command dropComm("drop");
  dropComm.flag = "-drop";
  dropComm.helpEntry = "drop a credential storage";
  dropComm.addOption(storPar.name,true);
  dropComm.addOption(debugOpt.name);
  mainSet.push_back(dropComm);
  //
  coral::Command addComm("add");
  addComm.flag = "-add";
  addComm.helpEntry = "add a credential entry in the storage";
  addComm.addOption(storPar.name,true);
  addComm.addOption(keyPar.name);
  addComm.addOption(servicePar.name,true);
  addComm.addOption(userPar.name,true);
  addComm.addOption(pwdPar.name,true);
  addComm.addOption(rolePar.name);
  addComm.addOption(debugOpt.name);
  mainSet.push_back(addComm);
  //
  coral::Command delComm("delete");
  delComm.flag = "-del";
  delComm.helpEntry = "remove a credential entry from the storage";
  delComm.addOption(storPar.name,true);
  delComm.addOption(keyPar.name);
  delComm.addOption(servicePar.name,true);
  delComm.addOption(rolePar.name);
  delComm.addOption(debugOpt.name);
  mainSet.push_back(delComm);
  //
  coral::Command importComm("import");
  importComm.flag = "-imp";
  importComm.helpEntry = "import a set of credential entries in the storage";
  importComm.addOption(storPar.name,true);
  importComm.addOption(keyPar.name);
  importComm.addOption(filePar.name);
  importComm.addOption(debugOpt.name);
  mainSet.push_back(importComm);
  //
  coral::Command exportComm("export");
  exportComm.flag = "-exp";
  exportComm.helpEntry = "export a set of credential entries from the storage";
  exportComm.addOption(storPar.name,true);
  exportComm.addOption(keyPar.name);
  exportComm.addOption(filePar.name);
  exportComm.addOption(debugOpt.name);
  mainSet.push_back(exportComm);
  //
  coral::Command listComm("list");
  listComm.flag = "-ls";
  listComm.helpEntry = "list the credential entries";
  listComm.addOption(storPar.name,true);
  listComm.addOption(keyPar.name);
  listComm.addOption(debugOpt.name);
  mainSet.push_back(listComm);

  std::string store("");
  std::string key("");
  std::string service("");
  std::string user("");
  std::string passwd("");
  std::string role("");
  std::string file("");
  bool create = false;
  bool drop = false;
  bool add = false;
  bool remove = false;
  bool debug=false;
  bool import = false;
  bool exports = false;
  bool list = false;

  try{
    coral::CommandLine cmd(secondaryOptions,mainSet);
    cmd.parse(argc,argv);
    
  if(cmd.userCommand()==coral::CommandLine::helpOption().name || cmd.userOptions().size()==0 ){
    cmd.help(std::cout);
    return 0;
  }
  if(cmd.find(coral::CommandLine::helpOption().name)){
    cmd.help(cmd.userCommand(),std::cout);
    return 0;
  }
  if(cmd.find(storPar.name)){
    store = cmd.optionValue( storPar.name);
    if(store.empty()) {
      std::cout << "ERROR: parameter \"credentialDatabase\" is empty string." << std::endl;
      return 1;
    }

  }
  if(cmd.find(keyPar.name)){
    key = cmd.optionValue( keyPar.name);
  }
  if(cmd.find(servicePar.name)){
    service = cmd.optionValue( servicePar.name);
  }
  if(cmd.find(rolePar.name)){
    role = cmd.optionValue( rolePar.name);
  }
  if(cmd.find(userPar.name)){
    user = cmd.optionValue( userPar.name);
  }
  if(cmd.find(pwdPar.name)){
    passwd = cmd.optionValue( pwdPar.name);
  }
  if(cmd.find(filePar.name)){
    file = cmd.optionValue( filePar.name);
  }
  if(cmd.find(debugOpt.name)){
    debug = true;
  }
  if(cmd.userCommand()==createComm.name){
    create = true;
  }
  if(cmd.userCommand()==dropComm.name){
    drop = true;
  }
  if(cmd.userCommand()==addComm.name){
    add = true;
  }
  if(cmd.userCommand()==delComm.name){
    remove = true;
  }
  if(cmd.userCommand()==importComm.name){
    import = true;
  }
  if(cmd.userCommand()==exportComm.name){
    exports = true;
  }
  if(cmd.userCommand()==listComm.name){
    list = true;
  }
  if(debug){
      std::cout << "credential store=\""<<store<<"\""<<std::endl;
      std::cout << "encoding key=\""<<key<<"\""<<std::endl;
      std::cout << "service connection string=\""<<service<<"\""<<std::endl;
      std::cout << "role=\""<<role<<"\""<<std::endl;
      std::cout << "user name=\""<<user<<"\""<<std::endl;
      std::cout << "password=\""<<passwd<<"\""<<std::endl;
      std::cout << "file=\""<<file<<"\""<<std::endl;
      std::cout << "create="<<(create?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "drop="<<(drop?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "add="<<(add?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "remove="<<(remove?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "import="<<(import?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "export="<<(exports?std::string("true"):std::string("false"))<<std::endl;
      std::cout << "list="<<(list?std::string("true"):std::string("false"))<<std::endl;
      coral::MessageStream::setMsgVerbosity(coral::Debug);
  }

  coral::Context::instance().loadComponent("CORAL/Services/ConnectionService");
  coral::IHandle<coral::IConnectionService> connServ = coral::Context::instance().query<coral::IConnectionService>();
  connServ->configuration().disablePoolAutomaticCleanUp();
  
  coral::RelationalAuthenticationService::RelationalAuthenticationService svc("CORAL/Services/RelationalAuthenticationService");
  svc.connectToDatabase( store, key );
  
  if( create ) {
    svc.createCredentialDatabase();
  }
  if( drop ) {
    svc.dropCredentialDatabase();
  }
  if( add ) {
    svc.addCredentialEntry(user,passwd,service,role);
  }
  if( remove ) {
    svc.removeCredentialEntry(service,role);
  }
  if( import ) {
    svc.importCredentials( file );
  }
  if( exports ) {
    svc.exportCredentials( file );
  }  
  if( list ) {
    svc.listCredentials( std::cout );
  }  
  return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return -1;
  }
}
