#ifndef SQLITEACCESS_SESSIONPROPERTIES_H
#define SQLITEACCESS_SESSIONPROPERTIES_H
#include "ISessionProperties.h"
#include <string>
//#include <iostream>
class sqlite3;
namespace coral {
  class DomainProperties;
  class ConnectionProperties;
  class ITypeConverter;
  class ISchema;
  class ISession;

  namespace monitor {
    class IMonitoringService;
  }
  
  namespace SQLiteAccess {
    
    class DomainProperties;
    class TypeConverter;

    /**
     * Class SessionProperties
     *
     * A simple class holding the connection properties, as well as the global
     * data and properties for the current session.
     */
    
    class SessionProperties : virtual public ISessionProperties{
    public:
      /// Constructor
      SessionProperties( ConnectionProperties& connectionProperties,
                         coral::ISession& session,
                         bool readOnly );     
      /// Destructor
      virtual ~SessionProperties();

      // Sets the service db handle
      void setdbHandle( sqlite3* dbHandle );

      /// Sets the monitoring service
      void setMonitoringService( coral::monitor::IMonitoringService* monitoringService );

      /// Returns the domain properties
      const DomainProperties& domainProperties() const;
      
      /// Returns the connection properties
      const ConnectionProperties& connectionProperties() const;
      
      /// Returns the connection string
      std::string connectionString() const;

      /// Returns the session object
      coral::ISession& session();

      /// Returns the type converter
      coral::ITypeConverter& typeConverter();
      const coral::ITypeConverter& typeConverter() const;

      /// Returns the monitoring service
      coral::monitor::IMonitoringService* monitoringService() const;

      /// Returns the transaction state
      bool isTransactionActive() const;

      /// Returns the server version
      //int serverVersion() const;

      /// Returns the corresponding schema
      coral::ISchema& schema() const;

      /// Returns the schema name
      std::string schemaName() const;

      /// Returns the readOnly flag
      bool isReadOnly() const;

      sqlite3* dbHandle();

      sqlite3* dbHandle() const;
    private:
       /// The connection properties
      ConnectionProperties& m_connectionProperties;

      /// The domain properties
      ///DomainProperties& m_domainProperties;

      /// The connection string
      std::string       m_connectionString;

      /// The type converter
      //coral::ITypeConverter& m_typeConverter;

      /// The session reference
      coral::ISession&  m_session;

      /// The sqlite db handle
      mutable sqlite3*          m_dbHandle;

      /// The monitoring service
      coral::monitor::IMonitoringService* m_monitoringService;

      /// The schema name
      //std::string       m_schemaName;

      /// The read-only flag
      bool              m_isReadOnly;
      
    };    
  }
}
#endif
