#include "Table.h"
#include "SessionProperties.h"
#include "DomainProperties.h"
#include "TableDescriptionProxy.h"
#include "DataEditor.h"
#include "PrivilegeManager.h"
#include "Query.h"

#include "RelationalAccess/SessionException.h"

#include "CoralKernel/Service.h"

//#include "sqlite3.h"
//#include <cstdlib>
//#include <iostream>
coral::SQLiteAccess::Table::Table(const SQLiteAccess::SessionProperties& sessionproperties, const std::string& tableName):
  m_sessionProperties(sessionproperties),
  m_descriptionProxy(new coral::SQLiteAccess::TableDescriptionProxy( sessionproperties, tableName)),
  m_privilegeManager( new coral::SQLiteAccess::PrivilegeManager( sessionproperties, sessionproperties.schemaName() + "." + tableName) ),
  m_dataEditor(new coral::SQLiteAccess::DataEditor( sessionproperties, *m_descriptionProxy))
{
}

coral::SQLiteAccess::Table::~Table()
{
  if ( m_dataEditor ) delete m_dataEditor;
  if ( m_privilegeManager ) delete m_privilegeManager;
  if ( m_descriptionProxy ) delete m_descriptionProxy;
}

const coral::ITableDescription&
coral::SQLiteAccess::Table::description() const
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(),"ITable::description" );
  return static_cast< const coral::ITableDescription& >( *m_descriptionProxy );
}

coral::ITableSchemaEditor&
coral::SQLiteAccess::Table::schemaEditor()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "ITable::schemaEditor" );
  if ( m_sessionProperties.isReadOnly() )
    throw coral::InvalidOperationInReadOnlyModeException( m_sessionProperties.domainProperties().service()->name(),
                                                          "ITable::schemaEditor" );
  return static_cast< coral::ITableSchemaEditor& >( *m_descriptionProxy );
}

coral::ITableDataEditor&
coral::SQLiteAccess::Table::dataEditor()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(), "ITable::dataEditor" );
  if ( m_sessionProperties.isReadOnly() )
    throw coral::InvalidOperationInReadOnlyModeException( m_sessionProperties.domainProperties().service()->name(),
                                                          "ITable::dataEditor" );
  return *m_dataEditor;
}

coral::ITablePrivilegeManager&
coral::SQLiteAccess::Table::privilegeManager()
{
  if ( ! m_sessionProperties.isTransactionActive() )
    throw coral::TransactionNotActiveException( m_sessionProperties.domainProperties().service()->name(),"ITable::privilegeManager" );
  if ( m_sessionProperties.isReadOnly() )
    throw coral::InvalidOperationInReadOnlyModeException( m_sessionProperties.domainProperties().service()->name(),
                                                          "ITable::privilegeManager" );
  return *m_privilegeManager;
}

coral::IQuery*
coral::SQLiteAccess::Table::newQuery() const
{
  return new coral::SQLiteAccess::Query( m_sessionProperties,
                                         m_descriptionProxy->name() );
}
