#include <iostream>
#include <stdexcept>
#include <memory>

#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/IBulkOperation.h"
#include "RelationalAccess/IOperationWithQuery.h"
#include "RelationalAccess/IBulkOperationWithQuery.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"

#include "CoralKernel/Context.h"

#include "src/DomainProperties.h"
#include "src/SessionProperties.h"
#include "src/Domain.h"
#include "src/Session.h"

#include <sstream>
#include <climits>
#include <float.h>

int main(){
  std::string sourcetab, desttab;
  std::string con="sqlite_file:zhen.db";
  try {
    coral::Context& ctx = coral::Context::instance();
    
    ctx.loadComponent( "CORAL/RelationalPlugins/sqlite" );
    
    coral::IHandle<coral::IRelationalDomain> iHandle=ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    
    if ( ! iHandle.isValid() )
    {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false) ;
    coral::ISchema& schema = session->nominalSchema();
    coral::TableDescription desc;
    sourcetab="bulksource";
    desc.setName( sourcetab );
    desc.insertColumn( "id", coral::AttributeSpecification::typeNameForId(typeid(int)) );
    desc.insertColumn( "myfloat", coral::AttributeSpecification::typeNameForId(typeid(float)) );
    schema.dropIfExistsTable( sourcetab );
    coral::ITable& table=schema.createTable(desc);
    desttab="bulkdest";
    desc.setName( desttab );
    schema.dropIfExistsTable( desttab );
    schema.createTable(desc);
    coral::AttributeList data;
    table.dataEditor().rowBuffer(data);
    std::cout << "Inserting new rows into the source table" << std::endl;
    coral::IBulkOperation* rowInserter = table.dataEditor().bulkInsert(data,10);
    for ( int i = 0; i < 50; ++i ) {
      data["id"].data<int>()=i ;
      data["myfloat"].data<float>()=i*3.1415;
      rowInserter->processNextIteration();
    }
    rowInserter->flush();
    session->transaction().commit();
    session->transaction().start(false); 
    coral::ITable& dest=session->nominalSchema().tableHandle( desttab );
    coral::IOperationWithQuery* op = dest.dataEditor().insertWithQuery();
    coral::IQueryDefinition& query = op->query();
    query.addToTableList( sourcetab );
    query.setCondition( "id<10", coral::AttributeList() );
    long numberOfRowsInserted = op->execute();
    long expectednumber=10;
    if(numberOfRowsInserted!=expectednumber){
      std::cerr<<"wrong number of inserted rows"<<std::endl;
      exit(1);
    }
    delete op;
    coral::IBulkOperationWithQuery* bulkOperation = dest.dataEditor().bulkInsertWithQuery( 3 );
    coral::IQueryDefinition& query2 = bulkOperation->query();
    query2.addToTableList( sourcetab );
    coral::AttributeList condition;
    condition.extend<long>( "idmin");
    condition.extend<long>( "idmax");
    query2.setCondition( "id>:idmin AND id<:idmax", condition );
    long& idmin = condition[0].data<long>();
    long& idmax = condition[1].data<long>();
    idmin = 10;
    idmax = 13;
    for ( int i = 0; i < 5; ++i ) {
      bulkOperation->processNextIteration();
      idmin += 4;
      idmax += 4;
    }
    bulkOperation->flush();
    delete bulkOperation;
    // Bulk delete some rows from source
    std::string deleteCondition = "id<:idmax AND id>:idmin ";
    coral::AttributeList deleteData;
    deleteData.extend<long>( "idmax" );
    deleteData.extend<long>( "idmin" );
    coral::ITable& source=session->nominalSchema().tableHandle( sourcetab );
    coral::IBulkOperation* bulkDeleter=source.dataEditor().bulkDeleteRows( deleteCondition,deleteData,3 );
    for ( int i = 0; i < 5; ++i ) {
      deleteData["idmin"].data<long>()=i;
      deleteData["idmax"].data<long>()=i+2;
      bulkDeleter->processNextIteration();
    }
    bulkDeleter->flush();
    delete bulkDeleter;
    session->transaction().commit();
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
  }
}
