#include <iostream>
#include <stdexcept>

#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/IPrimaryKey.h"
#include "RelationalAccess/IForeignKey.h"
#include "RelationalAccess/IIndex.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/RelationalServiceException.h"

#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Exception.h"

#include "CoralKernel/Context.h"

int main()
{
  std::string con="sqlite_file:zhen.db";
  
  try
  {
    coral::Context& ctx = coral::Context::instance();
    
    ctx.loadComponent( "CORAL/RelationalPlugins/sqlite" );
    
    coral::IHandle<coral::IRelationalDomain> iHandle=ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    
    if ( ! iHandle.isValid() )
    {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false);
    coral::ISchema& schema = session->nominalSchema();
    schema.dropIfExistsTable( "mytest" );
    coral::TableDescription description0;
    description0.setName( "mytest" );
    description0.insertColumn( "ID",coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    description0.insertColumn( "X",coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description0.insertColumn( "Y",coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description0.insertColumn( "ORDER",coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    std::vector<std::string> idx_cols;
    idx_cols.push_back("ORDER");
    description0.createIndex("IDX1",idx_cols,false);
    schema.createTable( description0 );
    std::set<std::string> result=schema.listTables();
    for(std::set<std::string>::iterator it=result.begin(); it!=result.end(); ++it){
      std::cout<<"table names: "<<*it<<std::endl;
      schema.dropTable(*it);
    }    
    //Committing the transaction
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    session->transaction().start();
    std::set<std::string> r=schema.listTables();
    for(std::set<std::string>::iterator it=r.begin(); it!=r.end(); ++it){
      std::cout<<"table names after drop: "<<*it<<std::endl;
    }    
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
  }
}
