#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/IPrimaryKey.h"
#include "RelationalAccess/IForeignKey.h"
#include "RelationalAccess/IIndex.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/SchemaException.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Exception.h"

#include "CoralKernel/Context.h"

#include <iostream>
#include <stdexcept>

int main( int, char** )
{
  std::string con = "sqlite_file:zhen.db";
  try
  {
    coral::Context& ctx = coral::Context::instance();
      
    ctx.loadComponent( "CORAL/RelationalPlugins/sqlite" );
    
    coral::IHandle<coral::IRelationalDomain> iHandle=ctx.query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    
    if ( ! iHandle.isValid() )
    {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false);
    //
    // Creating the table
    //
    std::cout<<"create table x1 (id UNSIGNEDLONG PRIMARY KEY, a STRING NOT NUUL UNIQUE, b STRING NOT NULL UNIQUE, c STRING NOT NULL UNIQUE)"<<std::endl;
    coral::TableDescription description1;
    description1.setName("x1");
    description1.insertColumn("id", coral::AttributeSpecification::typeNameForId( typeid(unsigned long) ) );
    description1.insertColumn("a", coral::AttributeSpecification::typeNameForId( typeid(std::string) ) );
    description1.insertColumn("b", coral::AttributeSpecification::typeNameForId( typeid(std::string) ) );
    description1.insertColumn("c", coral::AttributeSpecification::typeNameForId( typeid(std::string) ) );
    description1.setPrimaryKey( "id" );
    description1.setNotNullConstraint( "a" );
    description1.setNotNullConstraint( "b" );
    description1.setNotNullConstraint( "c" );
    description1.setUniqueConstraint( "a" );
    description1.setUniqueConstraint( "b" );
    //description1.setUniqueConstraint( "c" );
    session->nominalSchema().createTable(description1 );
    session->transaction().commit();
    std::cout<<"table x1 created"<<std::endl;
    //
    // insert new row
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start();

    coral::ITable& table =session->nominalSchema().tableHandle("x1");
    coral::ITableDataEditor& dataEditor = table.dataEditor();
    std::cout << "Adding 1 row into the table" << std::endl;
    coral::AttributeList newdata;
    table.dataEditor().rowBuffer( newdata );
    newdata["id"].data<unsigned long>()= 1 ;
    newdata["a"].data<std::string>()= "avalue" ;
    newdata["b"].data<std::string>()= "bvalue" ;
    newdata["c"].data<std::string>()= "cvalue";
    dataEditor.insertRow( newdata );
    
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();

    //
    // insert new duplicate row
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start();
    std::cout << "Adding duplicate row into the table" << std::endl;
    coral::AttributeList newdata2;
    table.dataEditor().rowBuffer( newdata2 );
    newdata2["id"].data<unsigned long>()= 1 ;
    newdata2["a"].data<std::string>()= "avalue" ;
    newdata2["b"].data<std::string>()= "bvalue" ;
    newdata2["c"].data<std::string>()= "cvalue";
    try{
      dataEditor.insertRow( newdata2 );    
      std::cout << "Committing..." << std::endl;
      session->transaction().commit();
    }catch(const coral::DuplicateEntryInUniqueKeyException& er){
      std::cout<<"OK caught the right exception"<<std::endl;
      session->transaction().rollback();
    }catch(...){
      std::cout<<"unexpected exception"<<std::endl;
      exit(1);
    }
    //
    // update row
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    coral::AttributeList emptybinddata;
    long nrow=dataEditor.updateRows( "a='anewvalue'", "a='avalue'",emptybinddata);
    if( nrow==0 ){
      throw std::runtime_error( "Could not update the row in the table." );
    }
    std::cout<<"numberOfRowsUpdated: "<<nrow<<std::endl;
    std::cout << "Committing..." << std::endl;
    session->transaction().commit() ;
    //
    // check updated a value
    //
    std::cout << "Checking updated values..." << std::endl;
    session->transaction().start(true);
    std::auto_ptr<coral::IQuery> query( table.newQuery() );
    query->addToOutputList( "a" );
    coral::AttributeList outBuffer;
    outBuffer.extend( "a",typeid(std::string)  );
    query->defineOutput( outBuffer );
    query->setForUpdate();
    coral::ICursor& cursor = query->execute();
    while( cursor.next() ){
      const coral::AttributeList& row = cursor.currentRow();
      std::cout<<"query result: "<<std::endl;
      row.toOutputStream( std::cout ) << std::endl;
      assert( row["a"].data<std::string>()==std::string("anewvalue" ));
    }
    cursor.close();
    session->transaction().commit() ;
    //
    // delete row
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    long drow=dataEditor.deleteRows( "",emptybinddata ); 
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    assert(drow==1);
    //
    // insert new row
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    std::cout << "Adding 2 rows into the table" << std::endl;
    newdata["id"].data<unsigned long>()=1;
    newdata["a"].data<std::string>()= "avalue1";
    newdata["b"].data<std::string>()= "bvalue1";
    newdata["c"].data<std::string>()= "cvalue1";
    dataEditor.insertRow( newdata ) ;
    newdata["id"].data<unsigned long>()= 2 ;
    newdata["a"].data<std::string>()= "avalue2";
    newdata["b"].data<std::string>()= "bvalue2";
    newdata["c"].data<std::string>()= "cvalue2";
    dataEditor.insertRow( newdata );
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    //
    // update row with binding
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    coral::AttributeList dataForUpdate;
    dataForUpdate.extend("newid", typeid(unsigned long) );
    dataForUpdate.extend( "idValue", typeid(unsigned long) );
    dataForUpdate["newid"].data<unsigned long>()=100;
    dataForUpdate["idValue"].data<unsigned long>()=1;
    long numberOfRowsUpdated = dataEditor.updateRows( "id = :newid",
					   "id >:idValue", dataForUpdate );
    if( nrow==0 ){
      throw std::runtime_error( "Could not update the row in the table." );
    }else{
      std::cout<<"numberOfRowsUpdated: "<<numberOfRowsUpdated<<std::endl;
    }
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    //
    // check updated id value
    //
    std::cout << "Checking updated values 2..." << std::endl;
    session->transaction().start(true);
    std::auto_ptr<coral::IQuery> query2( table.newQuery() );
    query2->addToOutputList( "id" );
    coral::AttributeList output;
    output.extend( "id",typeid(unsigned long)  );
    query2->defineOutput( output );
    query2->setForUpdate();
    coral::ICursor& cursor2 = query2->execute();
    while( cursor2.next() ){
      const coral::AttributeList& row = cursor2.currentRow();
      std::cout<<"query result: "<<std::endl;
      row.toOutputStream( std::cout ) << std::endl;
      if( row["id"].data<unsigned long>()>1 ){
	assert( row["id"].data<unsigned long>()==100 );
      }
    }
    cursor2.close();
    session->transaction().commit();
    //
    // check updated with not ordered binding
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    coral::AttributeList dataForUpdate2;
    dataForUpdate2.extend( "idValue", typeid(unsigned long) );
    dataForUpdate2.extend("newid", typeid(unsigned long) );
    dataForUpdate2["newid"].data<unsigned long>()=110;
    dataForUpdate2["idValue"].data<unsigned long>()=1;
    numberOfRowsUpdated = dataEditor.updateRows( "id = :newid",
					   "id >:idValue", dataForUpdate2 );
    if( nrow==0 ){
      throw std::runtime_error( "Could not update the row in the table." );
    }else{
      std::cout<<"numberOfRowsUpdated: "<<numberOfRowsUpdated<<std::endl;
    }
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    //
    // check updated id value
    //
    std::cout << "Checking updated values 3..." << std::endl;
    session->transaction().start(true);
    std::auto_ptr<coral::IQuery> query3( table.newQuery() );
    query3->addToOutputList( "id" );
    coral::AttributeList output3;
    output3.extend( "id",typeid(unsigned long)  );
    query3->defineOutput( output3 );
    query3->setForUpdate();
    coral::ICursor& cursor3 = query3->execute();
    while( cursor3.next() ){
      const coral::AttributeList& row = cursor3.currentRow();
      std::cout<<"query result: "<<std::endl;
      row.toOutputStream( std::cout ) << std::endl;
      if( row["id"].data<unsigned long>()>1 ){
	assert( row["id"].data<unsigned long>()==110 );
      }
    }
    cursor3.close();
    session->transaction().commit();

    //
    // update row without WHERE condition
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start(false);
    coral::AttributeList cForUpdate;
    cForUpdate.extend("c", typeid(std::string) );
    cForUpdate["c"].data<std::string>()="hello";
    numberOfRowsUpdated = dataEditor.updateRows( "c = :c","", cForUpdate );
    if( nrow==0 ){
      throw std::runtime_error( "Could not update the row in the table." );
    }else{
      std::cout<<"numberOfRowsUpdated: "<<numberOfRowsUpdated<<std::endl;
    }
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    //
    // check updated c value
    //
    std::cout << "Checking updated values c..." << std::endl;
    session->transaction().start(true);
    std::auto_ptr<coral::IQuery> queryc( table.newQuery() );
    queryc->addToOutputList( "c" );
    coral::AttributeList qoutput;
    qoutput.extend( "c",typeid(std::string)  );
    queryc->defineOutput( qoutput );
    queryc->setForUpdate();
    coral::ICursor& cursorc = queryc->execute();
    while( cursorc.next() ){
      const coral::AttributeList& row = cursorc.currentRow();
      std::cout<<"query result: "<<std::endl;
      row.toOutputStream( std::cout ) << std::endl;
      assert( row["c"].data<std::string>()=="hello" );
    }
    cursorc.close();
    session->transaction().commit() ;

    //
    // delete row with binding
    //
    std::cout << "Starting a new transaction" << std::endl;
    session->transaction().start();
    coral::AttributeList dataForDelete;
    dataForDelete.extend( "bValue", typeid(std::string) );
    dataForDelete["bValue"].data<std::string>()= "bvalue1";
    long numberOfRowsDeleted = dataEditor.deleteRows( "b =:bValue", dataForDelete );
    std::cout<<"numberOfRowsDeleted: "<<numberOfRowsDeleted<<std::endl;
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    assert(numberOfRowsDeleted==1);
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
    return 1;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  return 0;
}
