

#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/IQueryDefinition.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "RelationalAccess/ConnectionService.h"

#include <string>
#include <iostream>

static const int MAX_ID_LENGTH = 100;
static const int MAX_ALIAS_LENGTH = 100;
static const int MAX_CONVENTION_LENGTH = 100;
static const int MAX_SLOT_LENGTH = 100;
static const int MAX_FIELDNAME_LENGTH = 100;
static const int MAX_TAG_LENGTH = 100;
static const int MAX_OBJECTTYPE_LENGTH = 100;
static const std::string TABLENAME_PREFIX("Bug32314");
static const std::string SERVICE_ORACLE("oracle://lcg_coral_nightly/CORAL_WRITER");

static coral::ISessionProxy* m_session = NULL;

void createTransaction(const std::string& service)
{
	coral::ConnectionService connService; 
	if (m_session) delete m_session;
	m_session = connService.connect(service, coral::Update);
	m_session->transaction().start();
}

// Create master list table
void createMasterListTable ()
{
	coral::TableDescription masterDescr;
	masterDescr.setName(TABLENAME_PREFIX+"MASTERLIST");
	masterDescr.insertColumn("TAG",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_TAG_LENGTH, false);
	masterDescr.insertColumn("ID",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_TAG_LENGTH, false);
	masterDescr.insertColumn("IDTAG",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_ID_LENGTH, false);
	masterDescr.setPrimaryKey("IDTAG");
	// explicit NOT NULL constraints are needed in addition to the primary key, at least for SQLite
	masterDescr.setNotNullConstraint("TAG");
	masterDescr.setNotNullConstraint("ID");
	masterDescr.setNotNullConstraint("IDTAG");
	m_session->nominalSchema().dropIfExistsTable(TABLENAME_PREFIX+"MASTERLIST");
	m_session->nominalSchema().createTable( masterDescr );
	m_session->nominalSchema().tableHandle( masterDescr.name() ).privilegeManager().grantToPublic(coral::ITablePrivilegeManager::Select);
}

std::vector<std::string> makeStringVector(const std::string& s1, const std::string& s2, 
		const std::string& s3, const std::string& s4)
{
	std::vector<std::string> ret;
	ret.push_back(s1);
	ret.push_back(s2);
	ret.push_back(s3);
	ret.push_back(s4);
	return ret;
}

// Create connection table
void createConnectionTable ()
{
	coral::TableDescription connDescr;
	connDescr.setName(TABLENAME_PREFIX+"CONNECTIONS");
	connDescr.insertColumn("ID",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_ID_LENGTH, false);
	connDescr.insertColumn("IDTAG",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_TAG_LENGTH, false);
	connDescr.insertColumn("TAG",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_ID_LENGTH, false);
	connDescr.insertColumn("SLOT",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_SLOT_LENGTH, false);
	connDescr.insertColumn("TOID",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_ID_LENGTH, false);
	connDescr.insertColumn("TOSLOT",coral::AttributeSpecification::typeNameForType<std::string>(), MAX_SLOT_LENGTH, false);
	connDescr.setPrimaryKey(makeStringVector("ID", "IDTAG", "TAG", "SLOT"));
	// TOID and TOSLOT must also be unique. Can impose constraint in the current scheme without revision sharing.
	connDescr.setUniqueConstraint(makeStringVector("TOID", "IDTAG", "TAG", "TOSLOT"), TABLENAME_PREFIX+"CONNECTIONS_UC1");
	connDescr.setNotNullConstraint("TOID");
	connDescr.setNotNullConstraint("TOSLOT");
	// explicit NOT NULL constraints are needed in addition to the primary key, at least for SQLite
	connDescr.setNotNullConstraint("ID");
	connDescr.setNotNullConstraint("IDTAG");
	connDescr.setNotNullConstraint("TAG");
	connDescr.setNotNullConstraint("SLOT");
	m_session->nominalSchema().dropIfExistsTable(TABLENAME_PREFIX+"CONNECTIONS");
	m_session->nominalSchema().createTable( connDescr );
	m_session->nominalSchema().tableHandle( connDescr.name() ).privilegeManager().grantToPublic(coral::ITablePrivilegeManager::Select);
}

void test_bug () 
{
	createTransaction(SERVICE_ORACLE);
	createMasterListTable();
	createConnectionTable();
	coral::AttributeList data;
	data.extend<std::string>("TAG");
	std::string a("atyala");
	data["TAG"].data<std::string>() = a;
	coral::IQuery *qqq = m_session->nominalSchema().newQuery();
	qqq->addToTableList(TABLENAME_PREFIX+"CONNECTIONS", "CN");
	qqq->addToOutputList("ID");
	qqq->setCondition("TAG = :TAG", data);

	coral::IQueryDefinition& mlquery = qqq->applySetOperation(coral::IQueryDefinition::Union);
	mlquery.addToTableList(TABLENAME_PREFIX+"MASTERLIST", "ML");
	mlquery.addToOutputList("ID");
	mlquery.setCondition("TAG = :TAG", data);
	qqq->execute();
	m_session->transaction().commit();
}


void test_OK ()  
{
	createTransaction(SERVICE_ORACLE);
	createMasterListTable();
	createConnectionTable();
	coral::AttributeList data;
	data.extend<std::string>("TAG");
	std::string a("atyala");
	data["TAG"].data<std::string>() = a;
	coral::IQuery *qqq = m_session->nominalSchema().newQuery();
	qqq->addToTableList(TABLENAME_PREFIX+"CONNECTIONS", "CN");
	qqq->addToOutputList("ID");
	qqq->setCondition("TAG = :TAG", data);

	coral::AttributeList data2;  
	data2.extend<std::string>("TAG");
	std::string a2("atyala");
	data2["TAG"].data<std::string>() = a2;
	coral::IQueryDefinition& mlquery = qqq->applySetOperation(coral::IQueryDefinition::Union);
	mlquery.addToTableList(TABLENAME_PREFIX+"MASTERLIST", "ML");
	mlquery.addToOutputList("ID");
	mlquery.setCondition("TAG = :TAG", data2);
	qqq->execute();
	m_session->transaction().commit();
}

void test_thePoint()
{
	coral::AttributeList data1;
	data1.extend<std::string>("TAG1");

	coral::AttributeList data2;  
	data2.extend<std::string>("TAG2");
	
	coral::AttributeList data3;  
	data3.extend<std::string>("TAG3");
	
	data1.merge(data2);
	data3.merge(data2);
	data1.merge(data3);
}

int main() 
{
	if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
	    ::putenv( "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
	} 
	if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
	    ::putenv( "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
	}
	  
	//test_thePoint();
	test_bug();
	return 1;
}
