#include <iostream>
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/ICursor.h" // Needed on WIN32
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/TableDescription.h"

int main( int argc, char** )
{

	if ( !::getenv( "CORAL_AUTH_PATH" ) ) {
		::putenv( "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
	}
	if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
		::putenv( "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
	}

	try
	{

		coral::ConnectionService* connSvc = new coral::ConnectionService;
		std::string connectString = "oracle://lcg_coral_nightly/CORAL_WRITER"; // OK
		//std::string connectString = "mysql://pclhcb55.cern.ch/AVALASSI"; // OK
		//std::string connectString = "sqlite_file:test-sqlite.db"; // OK
		coral::AccessMode accessMode = coral::Update;
		std::auto_ptr<coral::ISessionProxy>
		session( connSvc->connect( connectString, accessMode ) );

		// Recreate the database only if an argument is provided
		std::string tableName = "TEST_BUG_33578";
		if ( argc> 1 )
		{
			std::cout << "Recreate test table" << std::endl;
			// Create table
			session->transaction().start( false );
			{
				session->nominalSchema().dropIfExistsTable( tableName );
				coral::TableDescription description( "Test table" );
				description.setName( tableName );
				description.insertColumn
				( "id",
				coral::AttributeSpecification::typeNameForId
				( typeid(long) ) );
				if ( !getenv( "CORAL_TESTBUG33578_NOCLOB" ) )
				description.insertColumn
				( "clob64k",
				coral::AttributeSpecification::typeNameForId
				( typeid(std::string) ), 65535, false );
				description.setPrimaryKey( "id" );
				session->nominalSchema().createTable( description );
			}
			{
				coral::ITable& table =
				session->nominalSchema().tableHandle( tableName );
				coral::ITableDataEditor& editor = table.dataEditor();
				coral::AttributeList rowBuffer;
				rowBuffer.extend<long>( "id" );
				long& id = rowBuffer[ "id" ].data<long>();
				for ( int i = 0; i < 10; ++i )
				{
					id = i + 1;
					editor.insertRow( rowBuffer );
				}
			}
			session->transaction().commit();
		}
		else
		{
			std::cout << "Use existing test table" << std::endl;
		}

		// Sleep
		sleep(1);

		// Query table
		session->transaction().start( true );
		if ( !session->nominalSchema().existsTable( tableName ) ) {
			std::cerr << "Table does not exist!" << std::endl;
			exit(-1);
		}
		bool leak=true;
		if ( getenv( "CORAL_TESTBUG33578_NOLEAK" ) ) leak=false;
		std::cout << "START QUERY LOOP" << std::endl;
		{
			//for ( unsigned i=0; i<1000; i++ ) // for valgrind
			while ( true )
			{
				std::auto_ptr<coral::IQuery>
				query( session->nominalSchema().newQuery() );
				query->addToTableList( tableName );
				if ( leak ) query->execute();
			}
		}
		session->transaction().commit();

	}

	catch ( std::exception& e )
	{
		std::cerr << "Standard C++ exception caught: " << e.what() << std::endl;
		return 1;
	}

	catch ( ... )
	{
		std::cerr << "Unknown exception caught" << std::endl;
		return 1;
	}

	return 0;

}


