#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"

#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ISessionProperties.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IView.h"
#include "RelationalAccess/ITablePrivilegeManager.h"

#include "CoralBase/Exception.h"

#include <stdexcept>
#include <iostream>

using namespace std;


int main(int argc, const char *argv[])
{
  cout << " --->>> Before new coral::ConnectionService() <<<---- " << endl;
  // Get connection service.
  auto_ptr<coral::ConnectionService> connectionService( new coral::ConnectionService() );

  coral::ISessionProxy              *m_session;

  // Retrieve a connection handle.
  try {
     cout << " --->>> Before ConnectionService()->connect() <<<---- " << endl;
     m_session = connectionService->connect( "oracle://lcg_coral_nightly/coral_writer", coral::ReadOnly);
     cout << " --->>> After ConnectionService()->connect() <<<---- " << endl;

     // Start a transaction.
     coral::ITransaction& transaction = m_session->transaction();
     if ( !transaction.isActive() ) {
        transaction.start( true );
     }


   coral::ISchema& nominalSchema = m_session->nominalSchema();



   const std::string linksTableName = "POOL_COLLECTION_LINKS";

  // Get new query object for links table.
   coral::IQuery* query = nominalSchema.tableHandle( linksTableName ).newQuery();

  // Select all columns from links table and order by link ID.
   coral::AttributeList outputDataBuffer;

   std::string linkIdColumnName = "LINK_ID";
   query->addToOutputList( linkIdColumnName );
   query->addToOrderList( linkIdColumnName );
   outputDataBuffer.extend< unsigned >( linkIdColumnName );

   std::string databaseIdColumnName = "DATABASE_ID";
   query->addToOutputList( databaseIdColumnName );
   outputDataBuffer.extend< std::string >( databaseIdColumnName );

   std::string containerIdColumnName = "CONTAINER_ID";
   query->addToOutputList( containerIdColumnName );
   outputDataBuffer.extend< std::string >( containerIdColumnName );

   std::string classIdColumnName = "CLASS_ID";
   query->addToOutputList( classIdColumnName );
   outputDataBuffer.extend< std::string >( classIdColumnName );

   std::string technologyIdColumnName = "TECHNOLOGY_ID";
   query->addToOutputList( technologyIdColumnName );
   outputDataBuffer.extend< unsigned >( technologyIdColumnName );

   // Execute query.
   query->defineOutput( outputDataBuffer );
   query->setRowCacheSize( 1000 );
   coral::ICursor& cursor = query->execute();

   while ( cursor.next() )   {
      unsigned linkId = outputDataBuffer[0].data< unsigned >();
      std::string databaseId = outputDataBuffer[1].data< std::string >();
      std::string containerId = outputDataBuffer[2].data< std::string >();
      std::string classId = outputDataBuffer[3].data< std::string >();
      unsigned technologyId = outputDataBuffer[4].data< unsigned >();

    cout << "-- Reading from "<< linksTableName << endl;
    cout << "-- LinkId=" << linkId << ", DB=" << databaseId << ", container=" <<  containerId
         << ", class=" << classId << ", tech=" << technologyId << endl;
  }
  delete query;  
 

     
  } catch( coral::Exception &e ) {
     cout << "  ***** CORAL Exception " << e.what() << endl;
     exit(1);
  }


  cout << " --->>> Finished CORAL read test <<<---- " << endl;  
  return 1;
}

