#include <boost/scoped_ptr.hpp>

#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"

#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Blob.h"
#include "CoralBase/Date.h"
#include "CoralBase/TimeStamp.h"

using namespace coral;

int main()
{
	const std::string db = "/tests/db_ora/admin";
	const std::string tableName = "testBugXXXX";
	coral::ConnectionService connService;
	boost::scoped_ptr<coral::ISessionProxy> session(connService.connect(db, coral::Update));
	session->transaction().start();
	coral::ISchema& schema = session->nominalSchema();
	schema.dropIfExistsTable(tableName);
	coral::TableDescription description;
	description.setName(tableName);
	description.insertColumn("Col", AttributeSpecification::typeNameForId(typeid(unsigned int)));
	coral::ITable& table = schema.createTable(description);
	coral::AttributeList rowBuffer;
	table.dataEditor().rowBuffer(rowBuffer);
	rowBuffer["Col"].data<int>() = -1;
	table.dataEditor().insertRow(rowBuffer);
	session->transaction().commit();

	sleep(1);

	session.reset(connService.connect(db, coral::ReadOnly));
	coral::ISchema& schema2 =session->nominalSchema();
	session->transaction().start(true);
	boost::scoped_ptr<coral::IQuery> query(schema2.tableHandle(tableName).newQuery());
	AttributeList attList;
	attList.extend<int>("Col");
	query->defineOutput(attList);
	ICursor& cursor = query->execute();

	while (cursor.next())
		std::cout << "Value: " << cursor.currentRow()["Col"] << std::endl;

	session->transaction().commit();
	std::cout << "Test is OK" << std::endl;
	return 1;
}
