#include "DmlOperations.h"
#include "TestEnv/TestingConfig.h"

DmlOperations::DmlOperations(TestEnv * env) : Testing(env) {
}

void
DmlOperations::atomicOperations()
{    
    session->transaction().start();

    coral::ITableDataEditor& editor1 = session->nominalSchema().tableHandle(T1).dataEditor();
    coral::AttributeList rowBuffer1;
    rowBuffer1.extend<long>( "id" );
    rowBuffer1.extend<long long>( "t" );
    rowBuffer1.extend<float>( "X" );
    rowBuffer1.extend<double>( "Y" );
    rowBuffer1.extend<double>( "Z" );
    rowBuffer1.extend<std::string>( "Comment" );
    rowBuffer1.extend<coral::Blob>( "Data" );

    long& id = rowBuffer1[ "id" ].data<long>();
    long long& t = rowBuffer1[ "t" ].data<long long>();
    float& x = rowBuffer1[ "X" ].data<float>();
    double& y = rowBuffer1[ "Y" ].data<double>();
    double& z = rowBuffer1[ "Z" ].data<double>();
    std::string& comment = rowBuffer1[ "Comment" ].data<std::string>();
    coral::Blob& blob = rowBuffer1[ "Data" ].data<coral::Blob>();

    for ( int i = 0; i < 10; ++i ) {
      id = i + 1;
      t = 1;
      t <<= 4*i;
      x = 1 + (i+1)*0.001;
      y = 2 + (i+1)*0.0001;
      z = i*1.1;
      std::ostringstream os;
      os << "Row " << i + 1;
      comment = os.str();

      int blobSize = 1000 * ( i + 1 );
      blob.resize( blobSize );
      unsigned char* p = static_cast<unsigned char*>( blob.startingAddress() );
      for ( int j = 0; j < blobSize; ++j, ++p ) *p = ( i + j )%256;

      if ( i%4 == 2 ) {
        rowBuffer1[ "Z" ].setNull( true );
      }
      else {
        rowBuffer1[ "Z" ].setNull( false );
      }

      // Insert the row
      editor1.insertRow( rowBuffer1 );
    }

    // Delete some rows.
    std::string deleteCondition = "Z > :z";
    coral::AttributeList deleteData;
    deleteData.extend<float>( "z" );
    deleteData[0].data<float>() = 7;
    long rowsDeleted = editor1.deleteRows( deleteCondition, deleteData );
    std::cout << "Deleted " << rowsDeleted << " rows." << std::endl;
    if ( rowsDeleted != 3 )
      throw std::runtime_error( "Unexpected number of rows deleted" );

    // Update some rows.
    std::string updateAction = "t = t + :offset";
    std::string updateCondition = "X < :x";
    coral::AttributeList updateData;
    updateData.extend<int>("offset");
    updateData.extend<float>("x");
    updateData[0].data<int>() = 111;
    updateData[1].data<float>() = 1.003;
    long rowsUpdated = editor1.updateRows( updateAction, updateCondition, updateData );
    std::cout << "Updated " << rowsUpdated << " rows." << std::endl;
    if ( rowsUpdated != 2 ) {
      throw std::runtime_error( "Unexpected number of rows updated" );
    }
    session->transaction().commit();
}


void
DmlOperations::bulkOperations()
{
    session->transaction().start();

    // Fill in table T2
    coral::ITableDataEditor& editor2 = session->nominalSchema().tableHandle(T2).dataEditor();
    coral::AttributeList rowBuffer2;
    rowBuffer2.extend<long>( "id" );
    rowBuffer2.extend<float>( "tx" );
    rowBuffer2.extend<double>( "ty" );

    long& id = rowBuffer2[0].data<long>();
    float& tx = rowBuffer2[1].data<float>();
    double& ty = rowBuffer2[2].data<double>();
    coral::IBulkOperation* bulkInserter = editor2.bulkInsert( rowBuffer2, 10 );
    for ( int i = 0; i < 35; ++i ) {
      id = i;
      tx = i + 100.1;
      ty = i + 200.234;
      bulkInserter->processNextIteration();
    }
    bulkInserter->flush();
    delete bulkInserter;

    // Bulk delete some rows
    std::string deleteCondition = "tx < :tx AND id > :id ";
    coral::AttributeList deleteData;
    deleteData.extend<float>( "tx" );
    deleteData.extend<long>( "id" );
    coral::IBulkOperation* bulkDeleter = editor2.bulkDeleteRows( deleteCondition,
                                                                 deleteData,
                                                                 3 );
    for ( int i = 0; i < 5; ++i ) {
      deleteData[0].data<float>() = 104 + 4*i;
      deleteData[1].data<long>() = 1 + 4*i;
      bulkDeleter->processNextIteration();
    }
    bulkDeleter->flush();
    delete bulkDeleter;

    // Bulk update some rows
    std::string updateAction = "tx = :offset";
    std::string updateCondition = "ty > :minValue AND ty < :maxMalue";
    coral::AttributeList updateData;
    updateData.extend<float>( "offset" );
    updateData.extend<float>( "minValue" );
    updateData.extend<float>( "maxMalue" );
    float& offset = updateData[0].data<float>();
    float& minValue = updateData[1].data<float>();
    float& maxValue = updateData[2].data<float>();

    coral::IBulkOperation* bulkUpdater = editor2.bulkUpdateRows( updateAction,
                                                                 updateCondition,
                                                                 updateData,
                                                                 3 );
    for ( int i = 0; i < 5; ++i ) {
      offset = i;
      minValue = 200 + i*3;
      maxValue = 202 + i*3;
      bulkUpdater->processNextIteration();
    }
    bulkUpdater->flush();
    delete bulkUpdater;

    // Fill-in table T3
    coral::ITableDataEditor& editor3 = session->nominalSchema().tableHandle(T3).dataEditor();
    coral::AttributeList rowBuffer3;
    rowBuffer3.extend<long>( "id" );
    rowBuffer3.extend<long>( "fk1" );
    rowBuffer3.extend<long>( "fk2" );
    rowBuffer3.extend<float>( "Qn" );

    long& id3 = rowBuffer3[0].data<long>();
    long& fk1 = rowBuffer3[1].data<long>();
    long& fk2 = rowBuffer3[2].data<long>();
    float& q = rowBuffer3[3].data<float>();
    bulkInserter = editor3.bulkInsert( rowBuffer3, 3 );
    for ( int i = 0; i < 6; ++i ) {
      id3 = i+1;
      fk1 = i+1;
      fk2 = i*4 + i%2;
      q = ( i + 1 ) * 0.3;
      bulkInserter->processNextIteration();
    }
    bulkInserter->flush();
    delete bulkInserter;



    session->transaction().commit();
}
