#include "InsertSelect.h"
#include "TestEnv/TestingConfig.h"

InsertSelect::InsertSelect(TestEnv * env) : Testing(env)
{
  env->getTableString(T4, "T4");
}


InsertSelect::~InsertSelect()
{
}


void
InsertSelect::run()
{

    session->transaction().start();

    coral::ISchema& workingSchema = session->nominalSchema();
    workingSchema.dropIfExistsTable(T4);
    coral::TableDescription description( workingSchema.tableHandle(T2).description() );
    description.setName(T4);
    coral::ITable& table = workingSchema.createTable( description );
    table.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

    coral::IOperationWithQuery* operation = table.dataEditor().insertWithQuery();
    coral::IQueryDefinition& query = operation->query();
    query.addToTableList(T2);
    query.setCondition( "id<10", coral::AttributeList() );
    long numberOfRowsInserted = operation->execute();
    delete operation;
    if ( numberOfRowsInserted != 6 )
      throw std::runtime_error( "Unexpected number of rows inserted." );

    coral::IBulkOperationWithQuery* bulkOperation = table.dataEditor().bulkInsertWithQuery( 3 );
    coral::IQueryDefinition& queryb = bulkOperation->query();
    queryb.addToTableList(T2);
    coral::AttributeList condition;
    condition.extend<long>( "idmin");
    condition.extend<long>( "idmax");
    queryb.setCondition( "id>:idmin AND id<:idmax", condition );
    long& idmin = condition[0].data<long>();
    long& idmax = condition[1].data<long>();

    idmin = 10;
    idmax = 13;
    for ( int i = 0; i < 5; ++i ) {
      bulkOperation->processNextIteration();
      idmin += 4;
      idmax += 4;
    }
    bulkOperation->flush();
    delete bulkOperation;

    // add three more rows (for the next tests to make better sense)
    coral::AttributeList rowBuffer;
    table.dataEditor().rowBuffer( rowBuffer );
    
    if ( rowBuffer[0].specification().type() == typeid(long long)){
      rowBuffer[0].data<long long>() = 100;
      rowBuffer[1].data<float>() = 200;
      rowBuffer[2].data<double>() = 300;
      for ( int i = 0; i < 3; ++i ) {
        table.dataEditor().insertRow( rowBuffer );
        rowBuffer[0].data<long long>() += 1;
        rowBuffer[1].data<float>() += 2;
        rowBuffer[2].data<double>() += 3;
      }      
    }else{
      rowBuffer[0].data<long>() = 100;
      rowBuffer[1].data<float>() = 200;
      rowBuffer[2].data<double>() = 300;
      for ( int i = 0; i < 3; ++i ) {
        table.dataEditor().insertRow( rowBuffer );
        rowBuffer[0].data<long>() += 1;
        rowBuffer[1].data<float>() += 2;
        rowBuffer[2].data<double>() += 3;
      }  
    }
    

    session->transaction().commit();

    coral::sleepSeconds( 1 );

}
