#include "SchemaDefinition.h"
#include "TestEnv/TestingConfig.h"

SchemaDefinition::SchemaDefinition(TestEnv * env) : Testing(env) {
}

void SchemaDefinition::prepareSchema(){
  session->transaction().start();

  coral::ISchema& schema = session->nominalSchema();
  
  // Get rid of the previous tables and views
  schema.dropIfExistsView(V0);
  schema.dropIfExistsTable(T3);
  schema.dropIfExistsTable(T2);
  schema.dropIfExistsTable(T1);

  // Create the first table
  coral::TableDescription description1( "SchemaDefinition_Test" );
  description1.setName(T1);
  
  description1.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    // Define primary key
    description1.setPrimaryKey( "id" );
    description1.insertColumn( "t",
                               coral::AttributeSpecification::typeNameForId( typeid(short) ) );
    description1.insertColumn( "Xx",
                              coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    // Set column not null
    description1.setNotNullConstraint( "Xx" );
    description1.insertColumn( "Y",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    // Add a unique constraint specifying a name
    description1.setUniqueConstraint( "Y", "U_" + T1 + "_Y" );
    description1.insertColumn( "Z",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    // Add a variable size string
    description1.insertColumn( "Comment",
                               coral::AttributeSpecification::typeNameForId( typeid(std::string) ),
                               100,
                               false );
    // Add a blob
    description1.insertColumn( "Data",
                               coral::AttributeSpecification::typeNameForId( typeid(coral::Blob) ) );

    // Create the actual table
  coral::ITable& table = schema.createTable( description1 );
  
    // Change the column name
    table.schemaEditor().renameColumn( "Xx", "X" );

    // Change the column type
    table.schemaEditor().changeColumnType( "t",
                                           coral::AttributeSpecification::typeNameForId( typeid(long long) ) );
    // Add a not null constaint
    table.schemaEditor().setNotNullConstraint( "t" );

    // Add a unique constraint made out of two columns
    std::vector< std::string > constraintColumns( 2 );
    constraintColumns[0] = "X";
    constraintColumns[1] = "Z";
    
    ///follwing was commented out of fialing for mysql plugin
    ///related to bug #36513
    
 //   table.schemaEditor().setUniqueConstraint( constraintColumns );

    /// Add a unique index
    table.schemaEditor().createIndex(T1 + "_IDX_t", "t", true );

    // Give privileges
    table.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

    // Create the second table
    coral::TableDescription description2( "SchemaDefinition_Test" );
    description2.setName(T2);
    description2.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description2.setUniqueConstraint( "id" );
    description2.setNotNullConstraint( "id" );
    description2.insertColumn( "tx",
                               coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description2.setNotNullConstraint( "tx" );
    description2.insertColumn( "ty",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    description2.setNotNullConstraint( "ty" );
    session->nominalSchema().createTable( description2 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
    
    // Create the third table
    coral::TableDescription description3( "SchemaDefinition_Test" );
    description3.setName(T3);
    description3.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.setPrimaryKey( "id" );
    description3.insertColumn( "fk1",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.createForeignKey( T3 + "_FK1", "fk1", T1, "id" );
    description3.insertColumn( "fk2",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.createForeignKey( T3 + "_FK2", "fk2", T2, "id" );
    description3.insertColumn( "Qn",
                               coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    session->nominalSchema().createTable( description3 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
     
     coral::sleepSeconds( 1 );
     
     ///following part was commented out of failing for mysql plugin
     ///related to bug #36512
     
   // Create a view
/*    std::cout << "ende10" << std::endl;
    coral::IViewFactory* factory = schema.viewFactory();
//    std::cout << "ende11" << std::endl;
    factory->addToOutputList( T3 + ".id", "id" );
    factory->addToOutputList( T1 + ".X", "x" );
    factory->addToOutputList( T1 + ".Y", "y" );
    factory->addToOutputList( T1 + ".Z", "z" );
    factory->addToOutputList( T2 + ".tx", "TX" );
    factory->addToOutputList( T2 + ".ty", "TY" );
    factory->addToTableList(T3);
    factory->addToTableList(T1);
    factory->addToTableList(T2);
//    std::cout << "ende12" << std::endl;
    factory->setCondition( T1 + ".id = " + T3 + ".fk1 AND " + T2 + ".id = " + T3 + ".fk2 AND " + T3 + ".Qn > 0.5",
                           coral::AttributeList() );

//    factory->setCondition( "T1.id = T3.fk1 AND T2.id = T3.fk2 ",
//                          coral::AttributeList() );

//    std::cout << "ende13" << std::endl;
    factory->create(V0).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
//    std::cout << "ende1" << std::endl;
    delete factory;
 */   
/*
     // Create a view
//    std::cout << "ende10" << std::endl;
    coral::IViewFactory* factory = session->nominalSchema().viewFactory();
//    std::cout << "ende11" << std::endl;
    factory->addToOutputList( "T3.id", "id" );
    factory->addToOutputList( "T1.X", "x" );
    factory->addToOutputList( "T1.Y", "y" );
    factory->addToOutputList( "T1.Z", "z" );
    factory->addToOutputList( "T2.tx", "TX" );
    factory->addToOutputList( "T2.ty", "TY" );
    factory->addToTableList( "T3" );
    factory->addToTableList( "T1" );
    factory->addToTableList( "T2" );
//    std::cout << "ende12" << std::endl;
    factory->setCondition( "T1.id = T3.fk1 AND T2.id = T3.fk2 AND T3.Qn > 0.5",
                           coral::AttributeList() );
//    std::cout << "ende13" << std::endl;
    factory->create( "V0" ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
//    std::cout << "ende1" << std::endl;
    delete factory;
    
  */          
  session->transaction().commit();
  
  //oracle feature
     coral::sleepSeconds( 1 );
 
}

void
SchemaDefinition::printViewInfo( const coral::IView& view ) const
{
  int numberOfColumns = view.numberOfColumns();
  std::cout << "View " << view.name();
  std::cout << " has" << std::endl
            << "  " << numberOfColumns << " columns :" << std::endl;
  for ( int i = 0; i < numberOfColumns; ++i ) {
    const coral::IColumn& column = view.column( i );
    std::cout << "    " << column.name() << " (" << column.type() << ")";
    if ( column.isUnique() ) std::cout << " UNIQUE";
    if ( column.isNotNull() ) std::cout << " NOT NULL";
    std::cout << std::endl;
  }
  std::cout << "  definition string : " << view.definition() << std::endl;
}

void
SchemaDefinition::readSchema()
{  
    session->transaction().start();

    // Examine the tables
    this->printTableInfo( session->nominalSchema().tableHandle(T1).description() );
    this->printTableInfo( session->nominalSchema().tableHandle(T2).description() );
    this->printTableInfo( session->nominalSchema().tableHandle(T3).description() );
    
    std::cout << "read view" << std::endl;
   
    this->printViewInfo( session->nominalSchema().viewHandle(V0) );

    session->transaction().commit();

    std::cout << "read done" << std::endl;

  
}

void
SchemaDefinition::printTableInfo( const coral::ITableDescription& description ) const
{
  int numberOfColumns = description.numberOfColumns();
  std::cout << "Table " << description.name();
  std::string tableSpaceName = description.tableSpaceName();
  if ( ! tableSpaceName.empty() )
    std::cout << " (created in tablespace " << tableSpaceName << ")";
  std::cout << " has" << std::endl
            << "  " << numberOfColumns << " columns :" << std::endl;
  for ( int i = 0; i < numberOfColumns; ++i ) {
    const coral::IColumn& column = description.columnDescription( i );
    std::cout << "    " << column.name() << " (" << column.type() << ")";
    if ( column.isUnique() ) std::cout << " UNIQUE";
    if ( column.isNotNull() ) std::cout << " NOT NULL";
    std::cout << std::endl;
  }

  if ( description.hasPrimaryKey() ) {
    const std::vector<std::string>& columnNames = description.primaryKey().columnNames();
    std::cout << "  Primary key defined for column";
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for ( std::vector<std::string>::const_iterator iColumn = columnNames.begin();
          iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = description.primaryKey().tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfUniqueConstraints = description.numberOfUniqueConstraints();
  std::cout << "  " << numberOfUniqueConstraints;
  if ( numberOfUniqueConstraints == 1 ) std::cout << " unique constraint:" << std::endl;
  else if ( numberOfUniqueConstraints == 0 ) std::cout << " unique constraints" << std::endl;
  else std::cout << " unique constraints:" << std::endl;
  for ( int i = 0; i < numberOfUniqueConstraints; ++i ) {
    const coral::IUniqueConstraint& uniqueConstraint = description.uniqueConstraint( i );
    std::cout << "    " << uniqueConstraint.name() << " defined for column";
    const std::vector<std::string>& columnNames = uniqueConstraint.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = uniqueConstraint.tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfIndices = description.numberOfIndices();
  std::cout << "  " << numberOfIndices;
  if ( numberOfIndices == 1 ) std::cout << " index:" << std::endl;
  else if ( numberOfIndices == 0 ) std::cout << " indices" << std::endl;
  else std::cout << " indices:" << std::endl;
  for ( int i = 0; i < numberOfIndices; ++i ) {
    const coral::IIndex& index = description.index( i );
    std::cout << "    " << index.name();
    if ( index.isUnique() ) std::cout << " (UNIQUE)";
    std::cout << " defined for column";
    const std::vector<std::string>& columnNames = index.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = index.tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfForeignKeys = description.numberOfForeignKeys();
  std::cout << "  " << numberOfForeignKeys;
  if ( numberOfForeignKeys == 1 ) std::cout << " foreign key:" << std::endl;
  else if ( numberOfForeignKeys == 0 ) std::cout << " foreign keys" << std::endl;
  else std::cout << " foreign keys:" << std::endl;
  for ( int i = 0; i < numberOfForeignKeys; ++i ) {
    const coral::IForeignKey& foreignKey = description.foreignKey( i );
    std::cout << "    " << foreignKey.name() << " defined for column";
    const std::vector<std::string>& columnNames = foreignKey.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::cout << " -> " << foreignKey.referencedTableName() << "( "; 
    const std::vector<std::string>& columnNamesR = foreignKey.referencedColumnNames();
    for (  std::vector<std::string>::const_iterator iColumn = columnNamesR.begin();
           iColumn != columnNamesR.end(); ++iColumn ) {
      if ( iColumn != columnNamesR.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::cout << " )" << std::endl;
  }
}

