
/* 
 * Project: POOL
 * Sub-project: CORAL
 * Web site: http://pool.cern.ch/coral
 * 
 * Author: Zsolt.Molnar@cern.ch (http://www.zsoltmolnar.hu)
 */
#include "RelationalAccess/SessionException.h"

#include "Transaction.h"
#include "TestEnv/TestingConfig.h"

namespace coral {

namespace Tests {

Transaction::Transaction(TestEnv * env) 
	: Testing(env)
{
	// EMPTY
}

void Transaction::run()
{
	_testInvalidOpeningMode();
}

void Transaction::_testInvalidOpeningMode()
{
	// Start a readonly session and check the transaction modes
	createSession(0, coral::ReadOnly);
	// Test readonly transaction - should pass
	session->transaction().start(true);
	session->transaction().commit();
	// Test update transaction - should throw coral::InvalidOperationInReadOnlyModeException
	try {
		session->transaction().start(false);
	} catch (coral::InvalidOperationInReadOnlyModeException ex) {
		// cool... let's continue
	}
	// start an update session and check the transaction modes
	createSession(0, coral::Update);
	// Test readonly transaction - should pass
	session->transaction().start(true);
	session->transaction().commit();
	// Test update transaction - should pass
	session->transaction().start(false);
	session->transaction().commit();
}

} // namespace coral

} // namespace Tests
