
#include "SchemaDefinition.h"
#include "DmlOperations.h"
#include "Queries.h"
#include "InsertSelect.h"
#include "SetOperations.h"
#include "Transaction.h"

#include "CoralBase/Exception.h"

#include "TestEnv/TestEnv.h" 

#include <stdio.h>
#include <string>
#include <iostream>

int main(int argc, char *argv[]){

  TestEnv TC01("BASIS");
  
  TestEnvWriters& wr01 = TC01.addWriters().addDefault("oracle").add("mysql").add("sqlite");
  wr01.readers("oracle").addDefault("oracle").add("coral");
  wr01.readers("mysql").addDefault("mysql");  
  wr01.readers("sqlite").addDefault("sqlite").add("coral");  
  
  if(TC01.check(argc, argv)){
  
    //add the default connection strings to the test application
    TC01.addServiceName(0, TEST_CORE_SCHEME_ADMIN, TEST_CORE_SCHEME_ADMIN);
    
    try{    
      std::cout << "Test 1: defining and reading back tables and views for dbtype " << TC01.getDBType(0) << std::endl;
        
      SchemaDefinition SchD(&TC01);
      //setup session for service string 0
      SchD.createSession(0);
      //create schema and view    
      SchD.createSchemaDefault();
      SchD.createViewDefault();
      //delete old session and create new one for reading
      SchD.deleteSession();
      std::cout << "ende schema" << std::endl;
    
      std::cout << "Test 2: DML operations" << std::endl;     
    
      DmlOperations DmlO(&TC01);
      //setup session for writing
      DmlO.createSession(0);
      DmlO.atomicOperations();
      DmlO.bulkOperations();   
      DmlO.deleteSession();
 
      std::cout << "Test 3: Read Only Part, SchemaDefinition, Queries" << std::endl;

      SchD.createSession(0, TESTENV_READER);
      //read from schema    
      SchD.readSchemaDefault();
    
      SchD.deleteSession();
          
      Queries Quer(&TC01);
    
      Quer.createSession(0, TESTENV_READER); 
      Quer.run();
      Quer.deleteSession(); 
         
      std::cout << "Test 4: Insert/Select statements" << std::endl;

      InsertSelect InsS(&TC01);
    
      InsS.createSession(0);
      InsS.run();
      InsS.deleteSession();
    
      std::cout << "Test 5: Queries involving set operations" << std::endl;
    
      SetOperations SetO(&TC01);
    
      SetO.createSession(0);
      SetO.run(0);
      SetO.deleteSession();
      
      std::cout << "Test 6: Test transactions" << std::endl;
      coral::Tests::Transaction tr(&TC01);
      tr.run();    
    }
    catch ( coral::Exception& e ) {
      std::cerr << "CORAL Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e ) {
      std::cerr << "C++ Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( ... ) {
      std::cerr << "Unhandled exception " << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
  
    return 0;
  }
  return 1;
}
