#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ISchema.h"
#include <string>
#include "Connection.h"
#include "SchemaTest.h"
#include "MysqlSchema.h"
#include <iostream>
#include <stdexcept>
#include "CoralBase/Exception.h"
#include "RelationalAccess/ITypeConverter.h"
#include <exception>
#include "RelationalAccess/ITransaction.h"

#include "TestEnv/Testing.h"

int main( int argc, char *argv[] )
{   
  Connection connService("CSERVICE");
  
  connService.addWriters().addDefault("mysql");
  connService.addWriters().addDefault("sqlite").add("mysql").add("oracle");
  connService.addWriters().addDefault("oracle").add("mysql").add("sqlite").readers("oracle").addDefault("oracle").add("coral");  
    
  if(connService.check(argc, argv)){
  
    try
    {  
      //define service name 0 from dbtype 0
      connService.addServiceName(0, TEST_CORE_SCHEME_ADMIN);
      //define service name 1 from dbtype 1
      connService.addServiceName(1, TEST_CORE_SCHEME_ADMIN);
      //define service name 2, 3 from dbtype 2
      connService.addServiceName(2, TEST_CORE_SCHEME_ADMIN);
      connService.addServiceName(2, TEST_CORE_SCHEME_READER, TEST_CORE_SCHEME_READER);
      
      /** Initialize Connection Service */
      connService.Init();
      /** open session proxy using MySQL technology providing logical service name & access mode */
      coral::ISessionProxy* proxy1 = connService.getSessionProxy(0, coral::ReadOnly);
    
      std::cout << "Test with mysql."<<std::endl;
      connService.listTables(proxy1);
	 
      /** connection exists and will be recycled */
      coral::ISessionProxy* proxy_update = connService.getSessionProxy(0, coral::Update);        
 //     connService.listTables("CORAL_WRITER");
      connService.listTables(proxy_update);
      delete proxy1;
      delete proxy_update;
      
      /** open session proxy using SQLite technology */
      proxy1 = connService.getSessionProxy(1, coral::Update);
      std::cout << "Supported C++ types: " << std::endl;
      std::set<std::string> cppTypes = proxy1->typeConverter().supportedCppTypes();
      for ( std::set<std::string>::const_iterator iType = cppTypes.begin();
          iType != cppTypes.end(); ++iType ) {
        std::cout << *iType << std::endl;
        std::cout << "                      -> "
                << proxy1->typeConverter().sqlTypeForCppType( *iType ) << std::endl;
      }
      delete proxy1;
       
      
      //we dont need schematest any more -> using testing
      
      Testing tests(&connService);
       
      std::cout << "Schema Test - writer." << std::endl;
      /** get proxy to session providing logical service name and access mode */
      coral::ISessionProxy* proxy = connService.getSessionProxy( 2, coral::Update);
      /** create and fill schema using same authentication parameters */
      tests.setSession(proxy);
      std::string schemaName = tests.getSchemaName();
      tests.createSchemaDefault();
      delete proxy;
       
      std::cout << "Schema Test - reader."<<std::endl;
      /** read created schema using new authentication on previously opened
        and released session - authentication parameters will be switched */ 
      coral::ISessionProxy* proxy2 = connService.getSessionProxy( 3, coral::ReadOnly );
      tests.setSession(proxy2);
      tests.readSchemaDefault(schemaName);
      delete proxy2;
    }
    catch ( coral::Exception& e )
    {
      std::cerr << "CORAL exception " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e )
    {
      std::cerr << "Standard C++ exception " << e.what() << std::endl;
      return 1;
    }
    catch( ... )
    {
      std::cerr << "Unknown exception ..." << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Test completed successfully" << std::endl;
    return 0;
  }
  return 1;
}
