#include "TestApp.h"
#include "TestEnv/Testing.h"
#include "CoralBase/Exception.h"
#include <iostream>
#include <exception>
#include <cstdlib>


int main( int argc, char *argv[] )
{
  //initialize the app 
  TestApp app("CS_CLEANUPPOOL");
  //setup default and supported db backends
  app.addWriters().addDefault("oracle").add("mysql").add("sqlite");
  //check environment parameters to set db backend or display
  if(app.check(argc, argv)){
    try {
      app.addServiceName(0, TEST_CORE_SCHEME_ADMIN);

      Testing tests(&app);
      tests.createSession(0);
      tests.createSimpleTable();
      tests.deleteSession();

          
      app.run();
    }  
    catch ( coral::Exception& e )
    {
      std::cerr << "CORAL exception " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e )
    {
      std::cerr << "Standard C++ exception " << e.what() << std::endl;
      return 1;
    }
    catch( ... )
    {
      std::cerr << "Unknown exception ..." << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
    return 0;
  }
  return 1;
}
