
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ISchema.h"
#include <string>
#include "Connection.h"
#include "SchemaTest.h"
#include "MysqlSchema.h"
#include <iostream>
#include <stdexcept>
#include <stdio.h>	
#include "CoralBase/Exception.h"
#include "RelationalAccess/ITypeConverter.h"
#include <exception>
#include "RelationalAccess/ITransaction.h"
using std::cout;
using std::endl;


int main( int, char** )
{
   
try {  
       /** Initialize Connection Service */
       Connection  connService; 
    
       /** open session proxy using MySQL technology providing logical service name & access
           mode */
       coral::ISessionProxy* proxy1 = connService.getSessionProxy("my_test",coral::ReadOnly);
    
       std::cout << "Test with mysql."<<std::endl;
	     MysqlSchema mysqlTest(proxy1);
	     mysqlTest.listTables("test");
	 
	     /** connection exists and will be recycled */
	     coral::ISessionProxy* proxy_update = connService.getSessionProxy("my_test",coral::Update);   
        MysqlSchema mysqlTest_update(proxy_update);
	     mysqlTest_update.listTables("RADO_POOL");
	     cout<<" Is connection shared: "<<proxy_update->isConnectionShared()<<endl;
	     delete proxy1;
	     delete proxy_update;
      
       /** open session proxy using SQLite technology */
       proxy1 = connService.getSessionProxy("sql_test",coral::Update);
       cout << "Supported C++ types: " << std::endl;
       std::set<std::string> cppTypes = proxy1->typeConverter().supportedCppTypes();
       for ( std::set<std::string>::const_iterator iType = cppTypes.begin();
        iType != cppTypes.end(); ++iType ) {
        std::cout << *iType << std::endl;
        std::cout << "                      -> "
              << proxy1->typeConverter().sqlTypeForCppType( *iType ) << std::endl;
       }
       delete proxy1;
       
       
       std::cout << "Schema Test - writer."<<std::endl;
       /** get proxy to session providing logical service name and access mode */
       coral::ISessionProxy* proxy = connService.getSessionProxy("oracle://devdb10/ral_writer",coral::Update);
       /** create and fill schema using same authentication parameters */
       SchemaTest *test1 = new (SchemaTest)(proxy);
       test1->createSchema();
       delete test1;
       delete proxy;
       
       std::cout << "Schema Test - reader."<<std::endl;
       /** read created schema using new authentication on previously opened
           and released session - authentication parameters will be switched */ 
       coral::ISessionProxy* proxy2 = connService.getSessionProxy("reader_test",coral::ReadOnly);
       coral::ISchema& schema = proxy2->nominalSchema();
       SchemaTest *test2 = new (SchemaTest)(proxy2);
       test2->readSchema("ral_writer");
       delete test2;
       delete proxy2;
       std::cout << "Test completed."<<std::endl;
    }
   catch ( coral::Exception& e )
  {
    std::cerr << "CORAL exception " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e )
  {
    std::cerr << "Standard C++ exception " << e.what() << std::endl;
    return 1;
  }
  catch( ... )
  {
    std::cerr << "Unknown exception ..." << std::endl;
    return 1;
  }
}
