#include "TestApp.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include "PluginManager/PluginManager.h"
#include <stdexcept>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/ConnectionServiceException.h"
#include "../../../ConnectionService/src/ConnectionService.h"
#include "../../../ConnectionService/src/SessionProxy.h"
#include "../../../ConnectionService/src/ConnectionPool.h"

const std::string TestApp::OVALTAG = "[OVAL]";

TestApp::TestApp():
  m_context( new seal::Context ){
  this->loadServices();
}


TestApp::~TestApp()
{
}


void
TestApp::loadServices()
{
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );

  // Load the SEAL Message service and set the output threshold to Info
  loader->load( "SEAL/Services/MessageService" );
  std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
  m_context->query( v_msgSvc );
  if ( ! v_msgSvc.empty() ) {
    seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
    msgSvc->setOutputLevel( seal::Msg::Debug);
  }

  // Load the Connection Service
  std::cout << "Loading Connection Service"<<std::endl;
  loader->load( "CORAL/Services/ConnectionService" );
  loader->load( "CORAL/Services/LFCLookupService" );
}

seal::IHandle<coral::IConnectionService>
TestApp::connectionService()
{
  std::vector< seal::IHandle<coral::IConnectionService> > v_svc;
  m_context->query( v_svc );
  if ( v_svc.empty() ) {
    throw std::runtime_error( "Could not locate the connection service" );
  }
  return v_svc.front();
}

//
void TestApp::setupDataSource( const std::string& connectionString){
  seal::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  coral::ISessionProxy* session = theConnectionService->connect( connectionString );
  session->transaction().start();
  // creates a dummy table to be looked up
  session->nominalSchema().dropIfExistsTable("T_CONNECTION_SERVICE_TEST");
  coral::TableDescription descr;
  descr.setName("T_CONNECTION_SERVICE_TEST");
  descr.insertColumn("N_X",coral::AttributeSpecification::typeNameForType<int>());
  descr.insertColumn("N_S",coral::AttributeSpecification::typeNameForType<std::string>());
  session->nominalSchema().createTable( descr );
  session->transaction().commit();
  delete session;
}

void
TestApp::run(){
  
  seal::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  if(! theConnectionService ){
    std::cout << "Connection Service not loaded."<<std::endl;
    return;
  }
  theConnectionService->configuration().setConnectionTimeOut(0);

  
  //setupDataSource("oracle://devdb10/ral_writer");
  setupDataSource("mysql://pcitdb59.cern.ch/RADO_POOL");

  ::sleep(1);
  theConnectionService->purgeConnectionPool();
  
  try {
	/*
    try {
      std::string connectionString = "service_1";  
      coral::ISessionProxy* session = theConnectionService->connect( connectionString, coral::ReadOnly  );
      session->transaction().start( true);
      session->transaction().commit();  
      delete session;
    } catch (const coral::ConnectionServiceException& ex){
      std::cout << OVALTAG <<"Exception:"<<ex.what()<<std::endl;
    }
	*/
    try {
      std::string connectionString = "F1/F2/F3/my_test";  
      coral::ISessionProxy* session = theConnectionService->connect( connectionString, coral::ReadOnly  );
      session->transaction().start( true);
      session->transaction().commit();  
      delete session;
    } catch (const coral::ConnectionServiceException& ex){
      std::cout << OVALTAG <<"Exception:"<<ex.what()<<std::endl;
    }
    theConnectionService->configuration().disableReadOnlySessionOnUpdateConnections();
    try {
      std::string connectionString = "write_test";  
      coral::ISessionProxy* session = theConnectionService->connect( connectionString, coral::ReadOnly  );
      session->transaction().start( true);
      std::set<std::string> tables = session->nominalSchema().listTables();
      session->transaction().commit();  
      delete session;
    } catch (const coral::ConnectionServiceException& ex){
      std::cout << OVALTAG <<"Exception:"<<ex.what()<<std::endl;
    }
    coral::ISessionProxy* session = 0;
    std::string connectionString = "service1";  
    session = theConnectionService->connect( connectionString, coral::ReadOnly  );
    session->transaction().start( true );
    std::set<std::string> tables = session->nominalSchema().listTables();
    session->transaction().commit();  
    delete session;
	/*
    session = 0;
    connectionString = "service_3";  
    session = theConnectionService->connect( connectionString, coral::Update  );
    session->transaction().start();
    tables = session->nominalSchema().listTables();
    if(tables.find("T_CONNECTION_SERVICE_TEST")==tables.end()){
      std::cout << OVALTAG <<"TEST ERROR with connection "<<connectionString<<" table T_CONNECTION_SERVICE_TEST is not found."<<std::endl;
    } else {
      std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    }
    session->transaction().commit();  
    delete session;
    session = 0;
    connectionString = "service_4";  
    try {
      session = theConnectionService->connect( connectionString, coral::Update  );
      session->transaction().start();
      tables = session->nominalSchema().listTables();
      session->transaction().commit();
      delete session;
    } catch (const coral::ConnectionServiceException& ex){
      std::cout << OVALTAG <<"Exception:"<<ex.what()<<std::endl;
    }  
	*/
  }catch ( const coral::Exception& exc) {
    std::cout << "TEST ERROR: "<<exc.what()<<std::endl;
  }
  
   
}
