#include "TestApp.h"
#include <stdexcept>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "../../../ConnectionService/src/ConnectionService.h"
#include "../../../ConnectionService/src/SessionProxy.h"
#include "../../../ConnectionService/src/ConnectionPool.h"
#include <iostream>

const std::string TestApp::OVALTAG = "[OVAL]";

TestApp::TestApp(){
}


TestApp::~TestApp()
{
}

//
void TestApp::setupDataSource( const std::string& connectionString){
  coral::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  coral::ISessionProxy* session = theConnectionService->connect( connectionString );
  session->transaction().start();
  // creates a dummy table to be looked up
  session->nominalSchema().dropIfExistsTable("T_CONNECTION_SERVICE_TEST");
  coral::TableDescription descr;
  descr.setName("T_CONNECTION_SERVICE_TEST");
  descr.insertColumn("N_X",coral::AttributeSpecification::typeNameForType<int>());
  descr.insertColumn("N_S",coral::AttributeSpecification::typeNameForType<std::string>());
  session->nominalSchema().createTable( descr );
  session->transaction().commit();
  delete session;
}

seal::IHandle<coral::IConnectionService>
TestApp::connectionService()
{
  coral::Context& ctx = coral::Context::instance();
  coral::IHandle<coral::IConnectionService> conSvc = ctx.query<coral::IConnectionService>();
  if ( ! conSvc.isValid() ) {
    std::string pluginName("CORAL/Services/ConnectionService");
    ctx.loadComponent( pluginName );
    conSvc = ctx.query<coral::IConnectionService>();
  }
  return conSvc;
}

void
TestApp::run(){
  coral::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  if(! theConnectionService.isValid() ){
    std::cout << "Connection Service not loaded."<<std::endl;
    return;
  }
  theConnectionService->configuration().setConnectionTimeOut(0);
  setupDataSource("oracle://lcg_coral_nightly/coral_writer");
  setupDataSource("sqlite_file:ral.db");
  ::sleep(1);
  theConnectionService->purgeConnectionPool();

  try {
    std::string connectionString("/SimpleFailover/dbdev/service_1");  
    coral::ISessionProxy* session = theConnectionService->connect( connectionString, coral::ReadOnly );
    session->transaction().start( true );
    std::set<std::string> tables = session->nominalSchema().listTables();
    session->transaction().commit();
    if(tables.find("T_CONNECTION_SERVICE_TEST")==tables.end()){
      std::cout << OVALTAG <<"TEST ERROR with connection "<<connectionString<<" table T_CONNECTION_SERVICE_TEST is not found."<<std::endl;
    } else {
      std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    }
    std::cout << "Trying to failover on the same db..."<<std::endl;
    dynamic_cast<coral::ConnectionService::SessionProxy*>(session)->session()->endUserSession();
    session->transaction().start( true );
    tables = session->nominalSchema().listTables();
    session->transaction().commit();  
    if(tables.find("T_CONNECTION_SERVICE_TEST")==tables.end()){
      std::cout << OVALTAG <<"TEST ERROR with connection "<<connectionString<<" table T_CONNECTION_SERVICE_TEST is not found."<<std::endl;
    } else {
      std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    }
    delete session;
    ::sleep(1);
    theConnectionService->purgeConnectionPool();
    connectionString = "/SimpleFailover/dbdev/service_2";  
    session = theConnectionService->connect( connectionString, coral::ReadOnly  );
    session->transaction().start( true );
    tables = session->nominalSchema().listTables();
    session->transaction().commit();  
    if(tables.find("T_CONNECTION_SERVICE_TEST")==tables.end()){
      std::cout << OVALTAG <<"TEST ERROR with connection "<<connectionString<<" table T_CONNECTION_SERVICE_TEST is not found."<<std::endl;
    } else {
      std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    }
    delete session;
 
  }catch ( const coral::Exception& exc) {
    std::cout << "TEST ERROR: "<<exc.what()<<std::endl;
  }
  
   
}
