
#include <stdio.h>
#include "CoralBase/Exception.h"
#include <iostream>
#include "TestDriver.h"
#include "TestEnv/TestEnv.h"
#include <exception>

int main(int argc, char *argv[]){

  TestEnv TC01("INOUT");
  
  TestEnvWriters& wr01 = TC01.addWriters().addDefault("oracle").add("mysql").add("sqlite");
  wr01.readers("oracle").addDefault("oracle").add("coral");
  wr01.readers("mysql").addDefault("mysql");  

  if(TC01.check(argc, argv)){

    TC01.addServiceName(0, TEST_CORE_SCHEME_ADMIN, TEST_CORE_SCHEME_ADMIN);
  
    try{
    
      //add the default connection strings to the test application
    
      InputOutput InOut(&TC01);
      InOut.createSession(0);

      std::cout << "    preparing the schema" << std::endl;
      InOut.prepareSchema();
  
      InOut.deleteSession();
  
      InOut.createSession(0);
  
      std::cout << "    simple row inserts ..." << std::endl;
      InOut.writeSimple();
  
      InOut.deleteSession();
      
      std::cout << "[OVAL] Tests for database service " << TC01.getServiceName(0) << " passed." << std::endl;
        
      InOut.createSession(0);
  
      std::cout << "    bulk row inserts in ..." << std::endl;

      InOut.writeBulk();
  
      InOut.deleteSession();
      
      std::cout << "[OVAL] Tests for database service " << TC01.getServiceName(0) << " passed." << std::endl;
        
      InOut.createSession(0, TESTENV_READER);
  
      std::cout << "    reading back the data from ..." << std::endl;
  
      InOut.read();
  
      InOut.deleteSession();

      std::cout << "[OVAL] Tests for database service " << TC01.getServiceName(0, TESTENV_READER) << " passed." << std::endl;

    }
    catch ( coral::Exception& e ) {
      std::cerr << "CORAL Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e ) {
      std::cerr << "C++ Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( ... ) {
      std::cerr << "Unhandled exception " << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success..." << std::endl;
 
    return 0;
  }
  return 1;
}
