#include "App.h"
#include "TestEnv/TestingConfig.h"

#ifdef CORAL_SEALED

#include "SealKernel/MessageStream.h"

#endif

App::App(TestEnv * env)
  : Testing(env)
{
}

void
App::setup01()
{
  coral::TableDescription desc01;
  desc01.insertColumn( "ID", "string", 255, false );
  desc01.insertColumn( "Name1", "string", 255, false );      
  desc01.setPrimaryKey( "ID" );
  desc01.setName(T1);
  
  session->transaction().start(false);
  
  session->nominalSchema().dropIfExistsTable( desc01.name() );
  coral::ITable& table = session->nominalSchema().createTable( desc01 );
  coral::AttributeList rowBuffer;
  table.dataEditor().rowBuffer( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "1";
  rowBuffer["Name1"].data<std::string>() = "row#1";
  table.dataEditor().insertRow( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "2";
  rowBuffer["Name1"].data<std::string>() = "row#2";
  table.dataEditor().insertRow( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "3";
  rowBuffer["Name1"].data<std::string>() = "row#3";
  table.dataEditor().insertRow( rowBuffer );
//  table.privilegeManager().grantToUser( "PUBLIC", coral::ITablePrivilegeManager::Select );

  session->transaction().commit();
  
  coral::sleepSeconds( 1 ); // Avoid ORA-01466
}

void
App::setup02()
{
  coral::TableDescription desc02;
  desc02.insertColumn( "ID", "string", 255, false );
  desc02.insertColumn( "Name2", "string", 255, false );      
  desc02.setPrimaryKey( "ID" );
  desc02.setName(T2);
  
  session->transaction().start( false ); // read-write  
        
  session->nominalSchema().dropIfExistsTable( desc02.name() );
  coral::ITable& table = session->nominalSchema().createTable( desc02 );
  coral::AttributeList rowBuffer;
  table.dataEditor().rowBuffer( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "1";
  rowBuffer["Name2"].data<std::string>() = "row#1";
  table.dataEditor().insertRow( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "2";
  rowBuffer["Name2"].data<std::string>() = "row#2";
  table.dataEditor().insertRow( rowBuffer );
  rowBuffer["ID"].data<std::string>() = "3";
  rowBuffer["Name2"].data<std::string>() = "row#3";
  table.dataEditor().insertRow( rowBuffer );
//  table.privilegeManager().grantToUser( "PUBLIC", coral::ITablePrivilegeManager::Select );


  session->transaction().commit();

  coral::sleepSeconds( 1 ); // Avoid ORA-01466
}

void
App::test()
{
  session->transaction().start(true);
  
  std::auto_ptr<coral::IQuery> mainquery( session->nominalSchema().newQuery() );
  
  coral::IQueryDefinition* query = mainquery.get();
  unsigned int nExpRows = 3;
  
  query->addToOutputList( "t.Name1" );
  query->addToOutputList( "T.Name2" );
  query->addToOutputList( "Name1" );
  query->addToOutputList( "Name2" );
  query->addToTableList( T1, "t" );
  query->addToTableList( T2, "T" );
  
  std::string whereClause = "t.ID = T.ID";
  coral::AttributeList whereData;
  query->setCondition( whereClause, whereData );
  
  coral::ICursor& cursor = mainquery->execute();
  
  unsigned int nRows = 0;
  std::cout << std::endl << "Query result:" << std::endl;
  while ( cursor.next() ) 
        {
          cursor.currentRow().toOutputStream( std::cout ) << std::endl;
          ++nRows;
        }
  
  if(nExpRows != nRows) throw std::runtime_error( "Unexpected number of rows fetched" );
   
  session->transaction().commit();
}
