#include "App.h"
#include "CoralBase/Exception.h"
#include <iostream>

#include "TestEnv/TestEnv.h"

int main(int argc, char *argv[]){

  TestEnv TC01("QUOTES");
  TestEnvWriters& wr01 = TC01.addWriters().addDefault("oracle").add("mysql");
  
  wr01.readers("oracle").addDefault("oracle").add("coral");
  wr01.readers("mysql").addDefault("mysql");

  if(TC01.check(argc, argv)){
    //add the default connection strings to the test application
    TC01.addServiceName(0, TEST_CORE_SCHEME_ADMIN, TEST_CORE_SCHEME_ADMIN);

    try {
      App app(&TC01);
      //the write part of the test
      app.createSession(0);
      app.setup01();
      app.setup02();
      app.deleteSession();
      //the read only part of the test
      app.createSession(0, TESTENV_READER);
      app.test();
      app.deleteSession();
    } 
    catch ( coral::Exception& e )
    {
      std::cerr << "CORAL Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e ) {
      std::cerr << "C++ Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( ... )
    {
      std::cerr << "Unhandled exception " << std::endl;
      return 1;
    }
    
    std::cout << "[OVAL] Success" << std::endl;
    return 0;
  }
  return 1;
} 
