#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "RandomQuery.h"

#include "CoralBase/Exception.h"


#include "TestEnv/TestEnv.h"

static void * testthread(void * param)
{
  TestEnv * TC01 = static_cast<TestEnv*>(param);
  RandomQuery rq(TC01);
  
  rq.createSession(0, TESTENV_READER, false);
  
  for(size_t i = 0; i < 10; i++){
    sleep((rand() % 2));
    rq.execute();
  }
  
  rq.deleteSession();
  
  return NULL;
}

int main(int argc, char *argv[]){

  TestEnv TC01("RANDOM");
  
  TestEnvWriters& wr01 = TC01.addWriters().addDefault("oracle").add("mysql").add("sqlite");
  wr01.readers("oracle").addDefault("oracle").add("coral");  
  wr01.readers("mysql").addDefault("mysql");
  wr01.readers("sqlite").addDefault("sqlite");
  
  if(TC01.check(argc, argv)){
  
    //add the default connection strings to the test application
    TC01.addServiceName(0, TEST_CORE_SCHEME_ADMIN, TEST_CORE_SCHEME_ADMIN);
 
    try {
      RandomQuery rq(&TC01);
      rq.createSession(0);
      rq.setup();
      rq.deleteSession();
      
      size_t size = 4;
  
      pthread_t id[size];
    
      for( size_t i = 1; i < size; i++ ){
        pthread_create(&id[i], NULL, testthread, (void*)&TC01);
      }
  
      testthread((void*)&TC01);
      
      //wait for all threads
      sleep(4);
    }
    catch ( coral::Exception& e ) {
      std::cerr << "CORAL Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e ) {
      std::cerr << "C++ Exception : " << e.what() << std::endl;
      return 1;
    }
    catch ( ... ) {
      std::cerr << "Unhandled exception " << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
      
    return 0;
  }
  return 1;
}
