#include "app.h"
#include <stdexcept>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITransaction.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include <iostream>

#include "TestingConfig.h"

#include "RelationalAccess/ConnectionService.h"

#include "CoralCommon/Utilities.h"

TestApp::TestApp(TestEnv * env) : Testing(env)
{
}


TestApp::~TestApp()
{
}
#if 0
static const RecordSpecification&
TestApp::referenceRecordSpec()
{
  static RecordSpecification spec;
  if ( spec.size() == 0 ) {
    // Payload metadata - unique ID and unique description
    spec.push_back( FieldSpecification( "ID", cool::StorageType::Int32 ) );
    spec.push_back( FieldSpecification( "Desc", cool::StorageType::String255 ) );
    // Payload data
    spec.push_back( FieldSpecification( "A_BOOL", cool::StorageType::Bool ) );
    spec.push_back( FieldSpecification( "A_UCHAR", cool::StorageType::UChar ) );
    spec.push_back( FieldSpecification( "A_INT16", cool::StorageType::Int16 ) );
    spec.push_back( FieldSpecification( "A_UINT16", cool::StorageType::UInt16 ) );
    spec.push_back( FieldSpecification( "A_INT32", cool::StorageType::Int32 ) );
    spec.push_back( FieldSpecification( "A_UINT32", cool::StorageType::UInt32 ) );
    spec.push_back( FieldSpecification( "A_UINT63", cool::StorageType::UInt63 ) );
    spec.push_back( FieldSpecification( "A_INT64", cool::StorageType::Int64 ) );
    spec.push_back( FieldSpecification( "A_FLOAT", cool::StorageType::Float ) );
    spec.push_back( FieldSpecification( "A_DOUBLE", cool::StorageType::Double ) );
    spec.push_back( FieldSpecification( "A_STRING255", cool::StorageType::String255 ) );
    spec.push_back( FieldSpecification( "A_STRING4K", cool::StorageType::String4k ) );
    spec.push_back( FieldSpecification( "A_STRING64K", cool::StorageType::String64k ) );
    spec.push_back( FieldSpecification( "A_STRING16M", cool::StorageType::String16M ) );
    spec.push_back( FieldSpecification( "A_BLOB64K", cool::StorageType::Blob64k ) );
    spec.push_back( FieldSpecification( "A_BLOB16M", cool::StorageType::Blob16M ) );
  }
  return spec;
}    

static const coral::AttributeList&
TestApp::referenceData( int index )
{
  static std::map<int,coral::AttributeList> dataMap;
  if ( dataMap.find( index ) == dataMap.end() ) {
        // ******************************************************
        // *** Index = -3       *** null value                ***
        // *** Index = -2       *** min value                 ***
        // *** Index = -1       *** max value                 ***
        // *** Index in [0,255] *** test all characters/bytes ***
        // ******************************************************
     if ( ( index>=-3 && index<=255) ){
       dataMap[index] = coral::AttributeList();
       coral::AttributeList& data = dataMap[index];
       const RecordSpecification& spec = referenceRecordSpec();
       for ( RecordSpecification::const_iterator 
                  it = spec.begin(); it != spec.end(); it++ ) 
       {
         data.extend( it->first, 
                         StorageType::storageType(it->second).cppType() );
       }
          data["ID"].setNull( false );
          data["ID"].setValue( index );
          data["Desc"].setNull( false );
          data["A_BOOL"].setNull( false );
          data["A_UCHAR"].setNull( false );
          data["A_INT16"].setNull( false );
          data["A_UINT16"].setNull( false );
          data["A_INT32"].setNull( false );
          data["A_UINT32"].setNull( false );
          data["A_UINT63"].setNull( false );
          data["A_INT64"].setNull( false );
          data["A_FLOAT"].setNull( false );
          data["A_DOUBLE"].setNull( false );
          data["A_STRING255"].setNull( false );
          data["A_STRING4K"].setNull( false );
          data["A_STRING64K"].setNull( false );
          data["A_STRING16M"].setNull( false );
          data["A_BLOB64K"].setNull( false );
          data["A_BLOB16M"].setNull( false );
          // ******************************************************
          // *** Index = -3       *** null value                ***
          // ******************************************************
          if ( index == -3 ) 
          {
            data["Desc"].setValue( std::string("NULL values") );
            data["A_BOOL"].setNull( true );
            data["A_UCHAR"].setNull( true );
            data["A_INT16"].setNull( true );
            data["A_UINT16"].setNull( true );
            data["A_INT32"].setNull( true );
            data["A_UINT32"].setNull( true );
            data["A_UINT63"].setNull( true );
            data["A_INT64"].setNull( true );
            data["A_FLOAT"].setNull( true );
            data["A_DOUBLE"].setNull( true );
            data["A_STRING255"].setNull( true );
            data["A_STRING4K"].setNull( true );
            data["A_STRING64K"].setNull( true );
            data["A_STRING16M"].setNull( true );
            data["A_BLOB64K"].setNull( true );
            data["A_BLOB16M"].setNull( true );
          }
          // ******************************************************
          // *** Index = -2       *** min value                 ***
          // ******************************************************
          else if ( index == -2 ) 
          {
            data["Desc"].setValue( std::string("min values") );
            data["A_BOOL"].setValue( false );
            data["A_UCHAR"].setValue( cool::UCharMin );
            data["A_INT16"].setValue( cool::Int16Min );
            data["A_UINT16"].setValue( cool::UInt16Min );
            data["A_INT32"].setValue( cool::Int32Min );
            data["A_UINT32"].setValue( cool::UInt32Min );
            data["A_UINT63"].setValue( cool::UInt63Min );
            data["A_INT64"].setValue( cool::Int64Min );
            data["A_FLOAT"].setValue( (cool::Float)(0.123456789012345678901234567890) );
            data["A_DOUBLE"].setValue( (cool::Double)(0.123456789012345678901234567890) );
            data["A_STRING255"].setValue( std::string("") );
            data["A_STRING4K"].setValue( std::string("") );
            data["A_STRING64K"].setValue( std::string("") );
            data["A_STRING16M"].setValue( std::string("") );
            data["A_BLOB64K"].data<coral::Blob>().resize(0);
            data["A_BLOB16M"].data<coral::Blob>().resize(0);
          }
          // ******************************************************
          // *** Index = -1       *** max value                 ***
          // ******************************************************
          else if ( index == -1 ) 
          {
            data["Desc"].setValue( std::string("MAX VALUES") );
            data["A_BOOL"].setValue( true );
            data["A_UCHAR"].setValue( cool::UCharMax );
            data["A_INT16"].setValue( cool::Int16Max );
            data["A_UINT16"].setValue( cool::UInt16Max );
            data["A_INT32"].setValue( cool::Int32Max );
            data["A_UINT32"].setValue( cool::UInt32Max );
            data["A_UINT63"].setValue( cool::UInt63Max );
            data["A_INT64"].setValue( cool::Int64Max );
            data["A_FLOAT"].setValue( (cool::Float)(0.987654321098765432109876543210) );
            data["A_DOUBLE"].setValue( (cool::Double)(0.987654321098765432109876543210) );
            std::string high = "HIGH";
            data["A_STRING255"].setValue( high );
            data["A_STRING4K"].setValue( high );
            data["A_STRING64K"].setValue( high );
            data["A_STRING16M"].setValue( high );
            data["A_BLOB64K"].data<coral::Blob>().resize(0);
            data["A_BLOB16M"].data<coral::Blob>().resize(0);
          }
          // ******************************************************
          // *** Index in [0,255] *** test all characters/bytes ***
          // ******************************************************
          else
          {
            std::stringstream sIndex;
            sIndex << index;
            data["Desc"].setValue( "Value #"+sIndex.str() );
            data["A_BOOL"].setValue( true );
            unsigned char uc = index;
            data["A_UCHAR"].setValue( uc );
            data["A_INT16"].setValue( (cool::Int16)(-index) );
            data["A_UINT16"].setValue( (cool::UInt16)(index) );
            data["A_INT32"].setValue( (cool::Int32)(-index) );
            data["A_UINT32"].setValue( (cool::UInt32)(index) );
            data["A_UINT63"].setValue( (cool::UInt63)(index) );
            data["A_INT64"].setValue( (cool::Int64)(-index) );
            data["A_FLOAT"].setValue( (cool::Float)(-index) );
            data["A_DOUBLE"].setValue( (cool::Double)(index) );
            std::string sUc = std::string( 1, (char)uc );
            data["A_STRING255"].setValue( sUc );
            data["A_STRING4K"].setValue( sUc );
            data["A_STRING64K"].setValue( sUc );
            data["A_STRING16M"].setValue( sUc );
            data["A_BLOB64K"].data<coral::Blob>().resize( sizeof(unsigned char) );
            unsigned char* pUc;
            pUc = static_cast<unsigned char*>( data["A_BLOB64K"].data<coral::Blob>().startingAddress() );
            *pUc = uc;
            data["A_BLOB16M"].data<coral::Blob>().resize( sizeof(unsigned char) );
            pUc = static_cast<unsigned char*>( data["A_BLOB16M"].data<coral::Blob>().startingAddress() );
            *pUc = uc;
          }
        }
        // Unknown index: throw an exception
        else
        {
          std::stringstream msg;
          msg << "Unknown index value for reference data: " << index;
          throw std::runtime_error( msg.str() );
        }
      }
      return dataMap[index];
    }

#endif

void
TestApp::fillReferencedData()
{
  session->transaction().start( false );
#if 0  
//  const coral::TableDescription& desc = referenceTableDescription();
  session->nominalSchema().dropIfExistsTable( T1 );
  coral::ITable& table = session->nominalSchema().createTable( T1 );
  for ( int index =-3; index<=255; index++ ){
    table.dataEditor().insertRow( referenceData( index ) );
  }
#endif
  session->transaction().commit();
}

void
TestApp::readReferencedData()
{

}

void
TestApp::testsAttrLists()
{
  session->transaction().start( false);

  coral::ISchema& schema = session->nominalSchema();

  std::set<std::string> list01 = schema.listTables();
  
  std::set<std::string>::const_iterator i01;
  for( i01 = list01.begin( ); i01 != list01.end( ); i01++  ) std::cout << "[ TEST ] Table : " << *i01 << std::endl;

  std::set<std::string> list02 = schema.listViews();
  
  std::set<std::string>::const_iterator i02;
  for( i02 = list02.begin( ); i02 != list02.end( ); i02++  ) std::cout << "[ TEST ] View : " << *i02 << std::endl;
  
  if(!schema.existsView( "myView" )) std::cout << "[ TEST ] View (" << "myView" << ") not exists " << std::endl;
  
  if(schema.existsTable( T1 )) std::cout << "[ TEST ] Table (" << T1 << ") exists " << std::endl;

  if(!schema.existsTable( T1 + "324" )) std::cout << "[ TEST ] Table (" << T1 + "324" << ") not exists " << std::endl;
    
  session->transaction().commit();
}

void
TestApp::testsQueries()
{
  session->transaction().start( false);
  
/*  
  m_rowBuffer = new coral::AttributeList;
  
  coral::ISchema& schema = session->nominalSchema();
  coral::ITable& table = schema.tableHandle(T1);
  
  coral::IQuery * query01 = table.newQuery(); 
  
  query01->addToOutputList("count(t)", "x" );
  query01->setCondition( " id > 5 ", *m_rowBuffer );  
  query01->groupBy("x");
  query01->setRowCacheSize( 0 );
  
  coral::ICursor& cursor01 = query01->execute();  
  while ( cursor01.next() ) {
    const coral::AttributeList& row01 = cursor01.currentRow();
//    std::cout << row[0].data<long long>() << "\t" << row[1].data<long long>() << std::endl;
    std::cout << row01[0].data<double>() << std::endl;
  }  
  
  coral::IQuery * query02 = table.newQuery(); 
  
  query02->addToOrderList("t");
  query02->setRowCacheSize( 0 );
  
  coral::ICursor& cursor02 = query02->execute();  
  while ( cursor02.next() ) {
    const coral::AttributeList& row02 = cursor02.currentRow();
//    std::cout << row[0].data<long long>() << "\t" << row[1].data<long long>() << std::endl;
    std::cout << row02[0].data<long long>() << std::endl;
  }  
  
  */
  coral::ISchema& schema = session->nominalSchema();
  
  coral::IQuery* query04 = schema.newQuery();
  query04->addToOutputList(T1 + ".X", "x" );
  query04->addToOutputList(T1 + ".t", "t" );
  query04->addToOutputList(T3 + ".Qn", "Qn" );
  query04->addToTableList(T1);
  query04->addToTableList(T3);

  std::string condition = T1 + ".id = " + T3 + ".fk1 AND " + T1 + ".Z < 3.0";
  coral::AttributeList conditionData;
  query04->setCondition( condition, conditionData );
  /*  
  std::string condition = T1 + ".id = " + T3 + ".fk1 AND " + T1 + ".Z<:zmax";
  conditionData.extend<float>( "zmax" );
//  query04->setMemoryCacheSize( 5 );
  conditionData[0].data<float>() = 3;
  */
  
  coral::ICursor& cursor04 = query04->execute();

  while ( cursor04.next() ) {
    cursor04.currentRow().toOutputStream( std::cout ) << std::endl;
  }
  delete query04;

      coral::IQuery* query5 = schema.newQuery();
    query5->addToOutputList( "A.t", "t" );
    query5->addToOutputList( "B.tx", "tx" );

    coral::IQueryDefinition& subQuery1 = query5->defineSubQuery( "A" );
    subQuery1.addToOutputList( "t" );
    subQuery1.addToOutputList( "id" );
    subQuery1.addToTableList(T1);
    
    coral::AttributeList cond1;
    cond1.extend<float>( "xmin" );
    subQuery1.setCondition( "X < :xmin", cond1 );
    
 //   subQuery1.setCondition( "X < 1.0049", cond1 );
    
    query5->addToTableList( "A" );

    coral::IQueryDefinition& subQuery2 = query5->defineSubQuery( "B" );
    subQuery2.addToOutputList( "id" );
    subQuery2.addToOutputList( "TX", "tx" );
    subQuery2.addToTableList(V0);
    coral::AttributeList cond2;
    
    cond2.extend<float>( "tymax" );
    subQuery2.setCondition( "TX < :tymax", cond2 );
    
 //   subQuery2.setCondition( "TX < 210", cond2 );
    query5->addToTableList( "B" );

    query5->setCondition( "A.id = B.id", coral::AttributeList() );

    
    //for mysql exact float values doesn't work propper
    //it could fluctuate between +0.1 and -0.1
    //cond1[0].data<float>() = 1.005;
    
    cond1[0].data<float>() = 1.0049;
    cond2[0].data<float>() = 210;
//    nRows = 0;
    coral::ICursor& cursor5 = query5->execute();
    while ( cursor5.next() ) {
      cursor5.currentRow().toOutputStream( std::cout ) << std::endl;
//      ++nRows;
    }
    delete query5;
    
    if(schema.existsTable( T1 )) std::cout << "Table " << T1 << "exists" << std::endl;;

  
 /* 
  delete query01;
  delete query02;
  delete m_rowBuffer;
  m_rowBuffer = NULL;
 */   
  session->transaction().commit();
}

