#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <sstream>
#include <stdexcept>
#include <sys/time.h>
#include "PluginManager/PluginManager.h"
#include "SealBase/Time.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/Context.h"
#include "SealKernel/Service.h"
#include "SealKernel/PropertyManager.h"
#include "SealKernel/IMessageService.h"
#include "RelationalAccess/ILookupService.h"
#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/IDatabaseServiceDescription.h"

// Wallclock time in seconds
time_t startWallClockTimeLFC, endWallClockTimeLFC, startWallClockTimeXML, endWallClockTimeXML;

void reportReplica( const coral::IDatabaseServiceDescription& replica,
                    const std::string& logicalName )
{
  std::cout << logicalName << " : "
            << replica.connectionString() << ", "
            << replica.authenticationMechanism() << ", "
            << ( ( replica.accessMode() == coral::ReadOnly ) ? std::string("Read-Only") : std::string("Update") )
            << std::endl;
}


int XMLLookupThread(int lookupCount)
{
 try {

    int numberOfReplicas=0;	
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

    loader->load( "CORAL/Services/XMLLookupService" );

    seal::IHandle<seal::Service> svc = context->query<seal::Service>( "CORAL/Services/XMLLookupService" );

    seal::Handle<seal::PropertyManager>& pmgr = svc->propertyManager();
    seal::PropertyBase* property = pmgr->property( "LookupFile" );
    property->set( std::string( "dblookup.xml" ) );

    std::vector< seal::IHandle<coral::ILookupService> > v_lookupSvc;
    context->query( v_lookupSvc );

    if ( ! v_lookupSvc.empty() )
    {
      const coral::ILookupService& lookupSvc = *( v_lookupSvc.front() );

	//std::cout<<"Performs the XMLLookupService test"<<std::endl;

	startWallClockTimeXML=time(NULL);

	//seal::Time start = seal::Time::current();

	for (int count=0; count < lookupCount; count++)
	{
      coral::IDatabaseServiceSet* dbSet = lookupSvc.lookup( "myservice1" );
      numberOfReplicas = dbSet->numberOfReplicas();
       }

	//seal::Time end = seal::Time::current();

	endWallClockTimeXML=time(NULL);

	printf("Time taken :: %ld seconds\n",(long)(endWallClockTimeXML - startWallClockTimeXML));   

    }
    else {
      throw std::runtime_error("Could not retrieve an lookup service.");
    }
 }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}

int LFCLookupThread(int lookupCount)
{
  try {
	

    int numberOfReplicas = 0;
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

   // Load the SEAL Message service and set the output threshold to Debug
   loader->load( "SEAL/Services/MessageService" );
   std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
   context->query( v_msgSvc );
   if ( ! v_msgSvc.empty() ) {
     seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
     msgSvc->setOutputLevel( seal::Msg::Debug);
   }
	
	
// Code for Seal Message Service
    loader->load( "CORAL/Services/LFCLookupService" );

    seal::IHandle<seal::Service> svc = context->query<seal::Service>( "CORAL/Services/LFCLookupService" );

    std::vector< seal::IHandle<coral::ILookupService> > v_lookupSvc;
    context->query( v_lookupSvc );
    if ( ! v_lookupSvc.empty() ) {
    	const coral::ILookupService& lookupSvc = *( v_lookupSvc.front() );

	//std::cout<<"Performs the LFCLookupService test"<<std::endl;

	startWallClockTimeLFC=time(NULL);
	
	//seal::Time start = seal::Time::current();

       coral::IDatabaseServiceSet* dbSet;

	std::cout<<" For = "<<lookupCount<<" lookups "<<std::endl;
	for (int count=0; count < lookupCount; count++)
	{

       dbSet = lookupSvc.lookup( "write_test");
       //dbSet = lookupSvc.lookup( "F1/Service1");
       numberOfReplicas = dbSet->numberOfReplicas();
	}
	
	//seal::Time end = seal::Time::current();

	endWallClockTimeLFC=time(NULL);

	printf("Time taken :: %ld seconds\n",(long)(endWallClockTimeLFC - startWallClockTimeLFC));   
    }
    else {
      throw std::runtime_error("Could not retrieve an lookup service.");
    }
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return -1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return -1;
  }
  return 0;
}

int main(int argc, char **argv)
{
	argc=1;

	//XMLLookupThread(atoi(argv[1]));

	LFCLookupThread(atoi(argv[1]));
}
