#include "DataFiller.h"

#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "RelationalAccess/IBulkOperation.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"

#include <memory>

coral::Testing::DataFiller::DataFiller():
  coral::Testing::ApplicationBase()
{}

coral::Testing::DataFiller::~DataFiller()
{}


void
coral::Testing::DataFiller::read1()
{
  std::auto_ptr< coral::ISessionProxy > session( this->connectionService().connect( "/connectivity/tests/writer1" ) );
  this->fillData( *session );
}

void
coral::Testing::DataFiller::read2()
{
  std::auto_ptr< coral::ISessionProxy > session( this->connectionService().connect( "/connectivity/tests/writer2" ) );
  this->fillData( *session );
}


void
coral::Testing::DataFiller::fillData( coral::ISessionProxy& session )
{
  coral::ITransaction& transaction = session.transaction();
  transaction.start();

  coral::ISchema& schema = session.nominalSchema();

  // Get rid of old tables first
  schema.dropIfExistsTable( "CT2" );
  schema.dropIfExistsTable( "CT1" );

  // Create the new tables
  coral::TableDescription description1;
  description1.setName( "CT1" );
  coral::AttributeList rowData1;
  rowData1.extend<long>( "id" );
  description1.insertColumn( "id",
                             coral::AttributeSpecification::typeNameForId( typeid(long) ) );
  description1.setPrimaryKey( "id" );
  rowData1.extend<short>( "t" );
  description1.insertColumn( "t",
                             coral::AttributeSpecification::typeNameForId( typeid(short) ) );
  rowData1.extend<float>( "X" );
  description1.insertColumn( "X",
                              coral::AttributeSpecification::typeNameForId( typeid(float) ) );
  description1.setNotNullConstraint( "X" );
  rowData1.extend<double>( "Y" );
  description1.insertColumn( "Y",
                             coral::AttributeSpecification::typeNameForId( typeid(double) ) );
  description1.setNotNullConstraint( "Y" );
  coral::ITable& table1 = schema.createTable( description1 );
  table1.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );


  coral::TableDescription description2;
  description2.setName( "CT2" );
  coral::AttributeList rowData2;
  rowData2.extend<long>( "fid" );
  description2.insertColumn( "fid",
                             coral::AttributeSpecification::typeNameForId( typeid(long) ) );
  description2.setNotNullConstraint( "fid" );
  description2.createForeignKey( "CT2_FK", "fid", "CT1", "id" );
  rowData2.extend<float>( "tx" );
  description2.insertColumn( "tx",
                             coral::AttributeSpecification::typeNameForId( typeid(float) ) );
  description2.setNotNullConstraint( "tx" );
  description2.insertColumn( "ty",
                             coral::AttributeSpecification::typeNameForId( typeid(double) ) );
  rowData2.extend<double>( "ty" );
  description2.setNotNullConstraint( "ty" );
  coral::ITable& table2 = schema.createTable( description2 );
  table2.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

  // Fill the data

  coral::IBulkOperation* inserter1 = table1.dataEditor().bulkInsert( rowData1, 100 );
  for ( int iRow = 0; iRow < 1000; ++iRow ) {
    rowData1["id"].data<long>() = iRow + 1;
    rowData1["t"].data<short>() = iRow % 33;
    rowData1["X"].data<float>() = 1.924 * (iRow - 134);
    rowData1["Y"].data<double>() = 40.12 * iRow - 2.1 * iRow * iRow;
    inserter1->processNextIteration();
  }
  inserter1->flush();
  delete inserter1;


  coral::IBulkOperation* inserter2 = table2.dataEditor().bulkInsert( rowData2, 100 );
  for ( int iRow1 = 0; iRow1 < 1000; ++iRow1 ) {
    rowData2["fid"].data<long>() = iRow1 + 1;
    for ( int iRow2 = 0; iRow2 < 10 + iRow1%10; ++iRow2 ) {
      rowData2["tx"].data<float>() = 1.1 * ( ( iRow2 + iRow1)% 21 );
      rowData2["ty"].data<double>() = 2.2 * ( ( iRow2 + iRow1)% 12 );
      inserter2->processNextIteration();
    }
  }
  inserter2->flush();
  delete inserter2;

  transaction.commit();
}
