#include "Reader.h"

#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"

#include <memory>
#include <iostream>
#include <sstream>

coral::Testing::Reader::Reader( const std::string& args ):
  coral::Testing::ApplicationBase()
{
  // Configure here the Connection Service
  coral::IConnectionServiceConfiguration& configuration = this->connectionService().configuration();

  int retrialPeriod = -1;
  int retrialTimeOut = -1;
  int failover = -1;
  std::istringstream is( args.c_str() );
  is >> retrialPeriod >> retrialTimeOut >> failover;

  if ( retrialPeriod >= 0 ) configuration.setConnectionRetrialPeriod( retrialPeriod );
  else retrialPeriod = configuration.connectionRetrialPeriod();
  if ( retrialTimeOut >= 0 ) configuration.setConnectionRetrialTimeOut( retrialTimeOut );
  else retrialTimeOut = configuration.connectionRetrialTimeOut();
  if ( failover >= 0 ) {
    if ( failover != 0 ) configuration.enableReplicaFailOver();
    else configuration.disableReplicaFailOver();
  }
  failover = ( configuration.isReplicaFailoverEnabled() ) ? 1 : 0;
  std::cout << "Configured with (retial period, retrial time out, fail over) = ("
            << retrialPeriod << "," << retrialTimeOut << "," << failover << ")" << std::endl;
}


coral::Testing::Reader::~Reader()
{}


void
coral::Testing::Reader::read1()
{
  std::auto_ptr< coral::ISessionProxy > session( this->connectionService().connect( "/connectivity/tests/reader",
                                                                                    coral::ReadOnly ) );
  coral::ITransaction& transaction = session->transaction();
  coral::ISchema& schema = session->nominalSchema();
  transaction.start( true );
  coral::IQuery* query = schema.newQuery();
  coral::AttributeList output;
  coral::AttributeList conditionData;

  output.extend< float >( "tx" );
  query->addToOutputList( "CT2.tx", "tx" );

  output.extend< double >( "ty" );
  query->addToOutputList( "CT2.ty", "ty" );

  query->addToTableList( "CT1" );
  query->addToTableList( "CT2" );

  query->setCondition( "CT1.id = CT2.fid AND CT1.t*20 < CT1.id",
                       conditionData );

  query->setRowCacheSize( 100 );
  query->defineOutput( output );
  coral::ICursor& cursor = query->execute();

  long numberOfRows = 0;
  while ( cursor.next() ) {
    ++numberOfRows;
  }
  std::cout << "Returned rows = " << numberOfRows << std::endl;

  delete query;
  transaction.commit();
}

void
coral::Testing::Reader::read2()
{
  std::auto_ptr< coral::ISessionProxy > session( this->connectionService().connect( "/connectivity/tests/reader",
                                                                                    coral::ReadOnly ) );
  coral::ITransaction& transaction = session->transaction();
  coral::ISchema& schema = session->nominalSchema();
  transaction.start( true );
  coral::IQuery* query = schema.newQuery();
  coral::AttributeList output;
  coral::AttributeList conditionData;

  output.extend< float >( "tx" );
  query->addToOutputList( "CT2.tx", "tx" );

  output.extend< double >( "ty" );
  query->addToOutputList( "CT2.ty", "ty" );

  query->addToTableList( "CT1" );
  query->addToTableList( "CT2" );

  query->setCondition( "CT1.id = CT2.fid AND CT1.X - CT1.Y < 0",
                       conditionData );

  query->setRowCacheSize( 100 );
  query->defineOutput( output );
  coral::ICursor& cursor = query->execute();

  long numberOfRows = 0;
  while ( cursor.next() ) {
    ++numberOfRows;
  }
  std::cout << "Returned rows = " << numberOfRows << std::endl;

  delete query;
  transaction.commit();
}
