#include "TestCore.h"

#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/IMonitoringService.h"
#include "RelationalAccess/IMonitoringReporter.h"


#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/IMonitoringService.h"

#include "CoralCommon/Utilities.h"

#include "CoralBase/Exception.h"

//constructor
TestCore::TestCore(const char * testName) : TestEnv(testName)
{
 // coral::Context& context = coral::Context::instance();
  //init the connection service
  connService = new coral::ConnectionService();
}

//destructor
TestCore::~TestCore()
{
  delete connService;
}

coral::ISessionProxy* 
TestCore::createSession(int index)
{
  return createSession(index, coral::Update);
}

coral::ISessionProxy* 
TestCore::createSession(int index, coral::AccessMode mode)
{
  coral::ISessionProxy * session;
  serviceName = getServiceName(index);
  session = connService->connect( serviceName, mode );
  if ( ! session )
    throw std::runtime_error( "Could not connect to " + serviceName );
  return session;
}

coral::ISessionProxy* 
TestCore::createSession(int index, coral::IHandle<coral::IConnectionService> handle, coral::AccessMode mode)
{
  coral::ISessionProxy * session;
  serviceName = getServiceName(index);
  session = handle->connect( serviceName , mode );
  if ( ! session )
    throw std::runtime_error( "Could not connect to " + serviceName );
  return session;
}

void 
TestCore::monitoringLoad()
{  
  //start the monitoring service
  coral::Context& ctx = coral::Context::instance();
    
  ctx.loadComponent( "CORAL/Services/MonitoringService" );   
}

void 
TestCore::monitoringStart(coral::monitor::Level level)
{  
  coral::IHandle<coral::monitor::IMonitoringService> svc = coral::Context::instance().query<coral::monitor::IMonitoringService>();
  
  if ( ! svc.isValid() )
  {
    throw std::runtime_error( "Could not locate the monitoring service" );
  }
  
  svc->setLevel( serviceName , level );
  svc->enable( serviceName );

}

void 
TestCore::monitoringStop()
{
  coral::IHandle<coral::monitor::IMonitoringService> svc = coral::Context::instance().query<coral::monitor::IMonitoringService>();
  
  if ( ! svc.isValid() )
  {
    throw std::runtime_error( "Could not locate the monitoring service" );
  }
  
  svc->disable( serviceName );

}

const coral::IMonitoringReporter& 
TestCore::getMonitoringReporter()
{
  coral::IHandle<coral::monitor::IMonitoringService> svc = coral::Context::instance().query<coral::monitor::IMonitoringService>();
  
  if ( ! svc.isValid() )
  {
    throw std::runtime_error( "Could not locate the monitoring service" );
  }
  
  return svc->reporter();
}

coral::IConnectionServiceConfiguration& 
TestCore::getConnectionConfig()
{
  return connService->configuration();
}

coral::Context&
TestCore::getContext()
{
  return coral::Context::instance();
}


