#include "TestEnv.h"

#include <map>

TestEnvDBType::TestEnvDBType() : buffer()
{
  currentType = 0;
}

TestEnvDBType::~TestEnvDBType()
{
  int size, i;
  size = dbTypes.size();
  for( i = 0; i < size; i++ ){
    delete dbTypes[i];
  }
  dbTypes.clear();
}

TestEnvDBType&
TestEnvDBType::add(const std::string& dbType)
{
  std::string * ns = new std::string(dbType);
  dbTypes.push_back(ns);
  return *this;
}

void
TestEnvDBType::setType(const char* dbType)
{
  int size, i;
  size = dbTypes.size();
  for( i = 0; i < size; i++ ){
    if(dbTypes[i]->compare(dbType) == 0){
      currentType = i;
      return;
    }
  }
}

std::string&
TestEnvDBType::getType()
{
  return *dbTypes[currentType];
}

std::string&
TestEnvDBType::toString()
{
  buffer.clear();
  int size, i;
  size = dbTypes.size();
  for( i = 0; i < size; i++ ){  
    buffer = buffer + " | " + *dbTypes[i];
  }
  return buffer;
}

std::string&
TestEnvDBType::toXML()
{
  buffer.clear();
  int size, i;
  size = dbTypes.size();
  for( i = 0; i < size; i++ ){
    buffer = buffer + "<type>" + *dbTypes[i] + "</type>\n\r";
  }
  return buffer;  
}


//constructor
TestEnv::TestEnv(const char * testName) : empty("")
{
  //set the name of the test core
  testCoreName = testName;
  //setup environment variables  
  if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
    ::putenv( const_cast<char*>("CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db") );
  } 
  if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
    ::putenv( const_cast<char*>("CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db") );
  }
  
  std::map<std::string, std::string> platformMatrix;
  typedef std::pair<std::string, std::string> TStrStrPair;
  //amd 64bit
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc34", "A0"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc34_dbg", "A1"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc41", "A2"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc41_dbg", "A3"));
  //intel 32bit
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc34", "B0"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc34_dbg", "B1"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc41", "B2"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc41_dbg", "B3"));
  //macosx for intel
  platformMatrix.insert(TStrStrPair("osx104_ia32_gcc401", "C0"));
  platformMatrix.insert(TStrStrPair("osx104_ia32_gcc401_dbg", "C1"));
  
  platformMatrix.insert(TStrStrPair("osx105_ia32_gcc401", "C2"));
  platformMatrix.insert(TStrStrPair("osx105_ia32_gcc401_dbg", "C3"));
  
  platformMatrix.insert(TStrStrPair("win32_vc71", "D0"));
  platformMatrix.insert(TStrStrPair("win32_vc71_dbg", "D1"));
  
  std::map<std::string, std::string> slotMatrix;
  
  slotMatrix.insert(TStrStrPair("dev", "A"));
  slotMatrix.insert(TStrStrPair("dev2", "B"));
  slotMatrix.insert(TStrStrPair("dev3", "C"));
  slotMatrix.insert(TStrStrPair("test", "T"));
  slotMatrix.insert(TStrStrPair("geant4", "G"));
  
  std::string slotPart1;
  //check the slot environment variable
  if ( ::getenv( "LCG_NGT_SLT_NAME" ) ) {
    slotPart1 = slotMatrix[::getenv( "LCG_NGT_SLT_NAME" )];
  }
  //if no evironment set take the test one
  if(slotPart1 == "") slotPart1 = "T";
  //check the slot environment variable
  
  std::string slotPart2;
  if ( ! ::getenv( "LCG_NGT_SLT_NUM" ) ) {
    slotPart2 = "0";
  } else {
    slotPart2 = ::getenv( "LCG_NGT_SLT_NUM" );
    if(!((slotPart2[0] >= '0') && (slotPart2[0] <= '9'))){
      slotPart2 = "0";
    }
  }
    
  slotPrefix = slotPart1 + slotPart2.substr(0,2);
    
  if ( ::getenv( "CMTCONFIG" ) ) {
    platformPrefix = platformMatrix[::getenv( "CMTCONFIG" )];
  }
  if(platformPrefix == "") platformPrefix = "NO";
  
  //debug output
 // std::cout << "LCG_NGT_SLT_NAME : (" << ::getenv( "LCG_NGT_SLT_NAME" ) << ")" << std::endl;
 // std::cout << "LCG_NGT_SLT_NUM : (" << ::getenv( "LCG_NGT_SLT_NUM" ) << ")" << std::endl;
  
}

//destructor
TestEnv::~TestEnv()
{
  int i, size;
  size = addArgs.size();
  for( i = 0; i < size; i++ ){
    delete addArgs[i];
  }
  
  size = DBTypes.size();
  for( i = 0; i < size; i++ ){
    delete DBTypes[i];
  }
  DBTypes.clear();
}

TestEnvDBType&
TestEnv::addDBType(const std::string& dbType)
{
  TestEnvDBType* ndbt = new TestEnvDBType();
  DBTypes.push_back(ndbt);
  ndbt->add(dbType);
  return *ndbt;
}

bool
TestEnv::check(int argc, char * argv[])
{
  if(argc > 1){
    if(strcmp(argv[1], "help") == 0){
      std::cout << "help screen for desperate users ;)" << std::endl << std::endl;
      std::cout << "test [parameter]" << std::endl << std::endl;
      std::cout << "view\tdisplays all supportet database backends" << std::endl;          
      std::cout << "viewxml\tdisplays all supportet database backends as xml output" << std::endl;          
      std::cout << "help\tthis screen" << std::endl;          
      return false;
    }else if(strcmp(argv[1], "view") == 0){
      int i, size;
      size = DBTypes.size();
      for( i = 0; i < size ; i++ ){
        std::cout << "Types for Argument(" << i << ") : " << DBTypes[i]->toString() << std::endl;
      }
      return false;      
    }else if(strcmp(argv[1], "viewxml") == 0){
      int i, size;
      size = DBTypes.size();
      std::cout << "<test>" << std::endl;
      for( i = 0; i < size ; i++ ){
	std::cout << "<argument id=\"" << i << "\">" << std::endl;
        std::cout << DBTypes[i]->toXML();
        std::cout << "</argument>" << std::endl;
      }
      std::cout << "</test>" << std::endl;
      return false;      
    }else{
      int size, i;
      size = DBTypes.size();
      for( i = 0; i < size ; i++ ){
        if(i < (argc - 1) ){
          DBTypes[i]->setType(argv[(i + 1)]);
        }
      }
      return true;     
    }  
  }
  return true;
}

std::string& 
TestEnv::getDBType(int pos)
{
  int size = DBTypes.size();
  if((pos > -1) && (pos < size)) return DBTypes[pos]->getType();
  return empty;
}

std::string& 
TestEnv::getTableString(std::string& source, const std::string& tableName)
{
  source = slotPrefix + platformPrefix + "_" + testCoreName + "_" + tableName;
  return source;
}

std::string&
TestEnv::getServiceName(int index)
{
  return *getServiceNamePtr(index);
}

/*
void 
TestEnv::setServiceName(int index, const std::string& type, const char * dbType, int argc, char * argv[], int position)
{
  std::string* serviceName = getServiceNamePtr(index);
  std::string temp;
  if(position > 0 && argc > position){
    temp = argv[position];
  }else{
    temp = dbType;
  }
  *serviceName = "/tests/db_" + temp.substr(0,3) + "/" + type;
}

void 
TestEnv::setServiceName(int index, const std::string& type)
{
  std::string* serviceName = getServiceNamePtr(index);
  std::string newName = serviceName->substr(0, serviceName->find_last_of('/') + 1) + type;
  *serviceName = newName;
}
*/

void
TestEnv::addServiceName(int pos, const std::string& type)
{  
  std::string* source = new std::string("/tests/db_" + getDBType(pos).substr(0,3) + "/" + type);
  addArgs.push_back(source);
}

std::string*
TestEnv::getServiceNamePtr(int index)
{
  return addArgs[index];
}


