#include "TestEnv.h"
#include "Testing.h"
#include "TestingConfig.h"

Testing::Testing(TestEnv * env) : m_rowBuffer( 0 )
{
  this->env = env;
  //set default table names
  env->getTableString(T1, "T1");
  env->getTableString(T2, "T2");  
  env->getTableString(T3, "T3");  
  env->getTableString(V0, "V0");
  //session
  session = NULL;
}
    
Testing::~Testing()
{
}

  
void 
Testing::createSession(int index)
{
  deleteSession();
  createSession(index, coral::Update);
}

void 
Testing::createSession(int index, coral::AccessMode mode)
{
  deleteSession();
  coral::ConnectionService * connService = new coral::ConnectionService();
  std::string serviceName = env->getServiceName(index);

  session = connService->connect( serviceName, mode );
  if ( ! session )
    throw std::runtime_error( "Could not connect to " + serviceName );
  delete connService;
}
 
/*   
void 
Testing::createSession(int index, coral::IHandle<coral::IConnectionService> handle, coral::AccessMode mode)
{
//  deleteSession();
//  session = testCore->createSession(index, handle, mode);
}
 */  
void 
Testing::deleteSession()
{
  if(session != NULL){
    delete session;
    session = NULL;
  }
}


void
Testing::setSession(coral::ISessionProxy * session)
{
  this->session = session;
}

std::string
Testing::getSchemaName()
{
  if(session != NULL) return session->nominalSchema().schemaName();
  return "";
}

/*
bool 
Testing::isDBType(const std::string& type)
{
//  return (testCore->getDBType(0).compare(type) == 0);
}
*/
void
Testing::fillData( int i )
{
  coral::AttributeList& row = *m_rowBuffer;

  row[0].data<int>() = i;

  if ( i%3 == 0 ) {
    row[1].setNull( true );
  }
  else {
    row[1].setNull( false );
    row[1].data<bool>() = ( i%2 == 0 ) ? true : false;
  }

  row[2].data<unsigned char>() = ( i*2 ) % 256;

  row[3].data<char>() = ( i*2 ) % 128;

  row[4].data<unsigned short>() = ( i*3 ) % 2048;

  row[5].data<short>() = ( i*3 ) % 2048 - 1024;

  row[6].data<unsigned int>() = 2147483648U - i;//( i*4 ) % 1000;

  row[7].data<int>() = ( i*4 ) % 1000 - 500;

  row[8].data<unsigned long>() = ( i*4 ) % 1001;

  row[9].data<long>() = ( i*4 ) % 1001 - 500;

  row[10].data<unsigned long long>() =  2147483648U - i;

  row[11].data<long long>() = ( (i%2 == 0 ) ? ( 2147483648ll - 2*i ) : ( i - 2147483648ll ) );

  if ( i%4 == 0 ) {
    row[12].setNull( true );
  }
  else {
    row[12].setNull( false );
    row[12].data<float>() = i + 0.123; 
  }

  row[13].data<double>() = 0.123456789 - 2.3*i; 

  row[14].data<double>() = 0.123456789 + 2.3*i; 

  coral::Blob& blob = row[15].data<coral::Blob>();
  int blobSize = 1000 * ( i%100 + 1 );
  blob.resize( blobSize );
  unsigned char* p = static_cast<unsigned char*>( blob.startingAddress() );
  for ( int j = 0; j < blobSize; ++j, ++p ) *p = ( i + j )%256;

  row[16].data<coral::Date>() = coral::Date( 2006, 1, i%31 + 1 );

  row[17].data<coral::TimeStamp>() = coral::TimeStamp( 2006, 1, 12, 15, 47, i%60, 0 );

  std::ostringstream os1;
  os1 << "A general String : " << i % 5;
  std::string s1 = os1.str();
  row[18].data<std::string>() = s1;

  std::ostringstream os2;
  os2 << "...." << i % 10;
  std::string s2 = os2.str();
  row[19].data<std::string>() = s2;

  std::ostringstream os3;
  os3 << "Value : " << i;
  std::string s3 = os3.str();
  row[20].data<std::string>() = s3;
}

void
Testing::checkData( int i ) const
{
  const coral::AttributeList& row = *m_rowBuffer;

  if ( row[0].data<int>() != i )
    throw std::runtime_error( "Unexpected value for variable " + row[0].specification().name() );

  if ( i%3 == 0 ) {
    if ( ! row[1].isNull() )
      throw std::runtime_error( "Unexpected NOT NULL data for variable " + row[1].specification().name() );
  }
  else {
    if ( row[1].isNull() )
      throw std::runtime_error( "Unexpected NULL data for variable " + row[1].specification().name() );
    if ( row[1].data<bool>() != ( ( i%2 == 0 ) ? true : false ) )
      throw std::runtime_error( "Unexpected value for variable " + row[1].specification().name() );
  }

  if ( row[2].data<unsigned char>() != ( i*2 ) % 256 )
    throw std::runtime_error( "Unexpected value for variable " + row[2].specification().name() );

  if ( row[3].data<char>() != ( i*2 ) % 128 )
    throw std::runtime_error( "Unexpected value for variable " + row[3].specification().name() );

  if ( row[4].data<unsigned short>() != ( i*3 ) % 2048 )
    throw std::runtime_error( "Unexpected value for variable " + row[4].specification().name() );

  if ( row[5].data<short>() != ( i*3 ) % 2048 - 1024 )
    throw std::runtime_error( "Unexpected value for variable " + row[5].specification().name() );

  if ( row[6].data<unsigned int>() != static_cast<unsigned int>( 2147483648U - i ) ) // ( i*4 ) % 1000 ) )
    throw std::runtime_error( "Unexpected value for variable " + row[6].specification().name() );

  if ( row[7].data<int>() != ( i*4 ) % 1000 - 500 )
    throw std::runtime_error( "Unexpected value for variable " + row[7].specification().name() );

  if ( row[8].data<unsigned long>() != static_cast<unsigned long>( ( i*4 ) % 1001 ) )
    throw std::runtime_error( "Unexpected value for variable " + row[8].specification().name() );

  if ( row[9].data<long>() != ( i*4 ) % 1001 - 500 )
    throw std::runtime_error( "Unexpected value for variable " + row[9].specification().name() );

  if ( row[10].data<unsigned long long>() != static_cast<unsigned long long>( 2147483648U - i ) )
    throw std::runtime_error( "Unexpected value for variable " + row[10].specification().name() );

  if ( row[11].data<long long>() != ( (i%2 == 0 ) ? ( 2147483648ll - 2*i ) : ( i - 2147483648ll ) ) )
    throw std::runtime_error( "Unexpected value for variable " + row[11].specification().name() );

  if ( i%4 == 0 ) {
    if ( ! row[12].isNull() )
      throw std::runtime_error( "Unexpected NOT NULL data for variable " + row[12].specification().name() );
  }
  else {
    if ( row[12].isNull() )
      throw std::runtime_error( "Unexpected NULL data for variable " + row[12].specification().name() );
    if ( std::abs( row[12].data<float>() / ( i + 0.123 ) - 1 ) > std::numeric_limits<float>::epsilon() )
      throw std::runtime_error( "Unexpected value for variable " + row[12].specification().name() );
  }

  if ( std::abs( row[13].data<double>() / ( 0.123456789 - 2.3*i ) - 1 ) > std::numeric_limits<double>::epsilon() )
    throw std::runtime_error( "Unexpected value for variable " + row[13].specification().name() );

  if ( std::abs( row[14].data<double>() / ( 0.123456789 + 2.3*i ) - 1 ) > std::numeric_limits<double>::epsilon() )
    throw std::runtime_error( "Unexpected value for variable " + row[14].specification().name() );

  const coral::Blob& blob = row[15].data<coral::Blob>();
  int blobSize = 1000 * ( i%100 + 1 );
  if ( blob.size() != blobSize )
    throw std::runtime_error( "Unexpected blob size for variable " + row[15].specification().name() );
  const unsigned char* p = static_cast<const unsigned char*>( blob.startingAddress() );
  for ( int j = 0; j < blobSize; ++j, ++p )
    if ( *p != ( i + j )%256 )
      throw std::runtime_error( "Unexpected value for variable " + row[15].specification().name() );

  if ( row[16].data<coral::Date>() != coral::Date( 2006, 1, i%31 + 1 ) )
    throw std::runtime_error( "Unexpected value for variable " + row[16].specification().name() );
  
  if ( row[17].data<coral::TimeStamp>() != coral::TimeStamp( 2006, 1, 12, 15, 47, i%60, 0 ) )
    throw std::runtime_error( "Unexpected value for variable " + row[17].specification().name() );

  std::ostringstream os1;
  os1 << "A general String : " << i % 5;
  std::string s1 = os1.str();
  if ( row[18].data<std::string>() != s1 )
    throw std::runtime_error( "Unexpected value for variable " + row[16].specification().name() );

  std::ostringstream os2;
  os2 << "...." << i % 10;
  std::string s2 = os2.str();
  if ( row[19].data<std::string>() != s2 )
    throw std::runtime_error( "Unexpected value for variable " + row[17].specification().name() );

  std::ostringstream os3;
  os3 << "Value : " << i;
  std::string s3 = os3.str();
  if ( row[20].data<std::string>() != s3 )
    throw std::runtime_error( "Unexpected value for variable " + row[18].specification().name() );
}

void 
Testing::createSchemaDefault()
{  
    session->transaction().start();

    // Get rid of the previous tables and views
    session->nominalSchema().dropIfExistsTable(T3);
    session->nominalSchema().dropIfExistsTable(T2);
    session->nominalSchema().dropIfExistsTable(T1);

    // Create the first table
    coral::TableDescription description1( "SchemaTest_Test" );
    description1.setName(T1);
    
    
    // mysql type problem, id should be long instead of int otherwise
    // foreign key constraint will fail of type missmatch
//    description1.insertColumn( "id",
//                               coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    
    description1.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    // Define primary key
    description1.setPrimaryKey( "id" );
    description1.insertColumn( "t",
                               coral::AttributeSpecification::typeNameForId( typeid(short) ) );
    description1.insertColumn( "Xx",
                              coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    // Set column not null
    description1.setNotNullConstraint( "Xx" );
    description1.insertColumn( "Y",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    // Add a unique constraint specifying a name
    description1.setUniqueConstraint( "Y", "U_" + T1 + "_Y" );
    description1.insertColumn( "Z",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    // Add a variable size string
    description1.insertColumn( "Comment",
                               coral::AttributeSpecification::typeNameForId( typeid(std::string) ),
                               100,
                               false );
    // Add a blob
    description1.insertColumn( "Data",
                               coral::AttributeSpecification::typeNameForId( typeid(coral::Blob) ) );

    // Create the actual table
    coral::ITable& table = session->nominalSchema().createTable( description1 );


    // Change the column name
    ///not workin in mysql !!!!
    table.schemaEditor().renameColumn( "Xx", "X" );

    // Change the column type
    table.schemaEditor().changeColumnType( "t", coral::AttributeSpecification::typeNameForId( typeid(long long) ) );
    // Add a not null constaint
    table.schemaEditor().setNotNullConstraint( "t" );

    //bug#1 small hack to solve it
    table.schemaEditor().changeColumnType( "t", coral::AttributeSpecification::typeNameForId( typeid(long long) ) );
    // Add a unique constraint made out of two columns
    
    //this is not working for mysql !!!!!!
    std::vector< std::string > constraintColumns( 2 );
    constraintColumns[0] = "X";
    constraintColumns[1] = "Z";
///    table.schemaEditor().setUniqueConstraint( constraintColumns );

    /// Add a unique index
    table.schemaEditor().createIndex( T1 + "_IDX_t", "t", true );

    // Give privileges
    table.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

    // Create the second table
    coral::TableDescription description2( "SchemaDefinition_Test" );
    description2.setName(T2);
    
    description2.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    
    
    description2.setUniqueConstraint( "id" );
    description2.setNotNullConstraint( "id" );
    description2.insertColumn( "tx",
                               coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description2.setNotNullConstraint( "tx" );
    description2.insertColumn( "ty",
                               coral::AttributeSpecification::typeNameForId( typeid(double) ) );
    description2.setNotNullConstraint( "ty" );
    session->nominalSchema().createTable( description2 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );



    // Create the third table
    coral::TableDescription description3( "SchemaTest_Test" );
    description3.setName(T3);
    description3.insertColumn( "id",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.setPrimaryKey( "id" );
    description3.insertColumn( "fk1",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.createForeignKey( T3 + "_FK1", "fk1", T1, "id" );
    description3.insertColumn( "fk2",
                               coral::AttributeSpecification::typeNameForId( typeid(long) ) );
    description3.createForeignKey( T3 + "_FK2", "fk2", T2, "id" );
    description3.insertColumn( "Qn",
                               coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    session->nominalSchema().createTable( description3 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

    session->transaction().commit();
    coral::sleepSeconds(2);
}
void 
Testing::readSchemaDefault()
{
  readSchemaDefault(getSchemaName());
}

void
Testing::readSchemaDefault(std::string SchemaName)
{
    session->transaction().start( true );

    // Examine the tables
    this->printTableInfo( session->schema(SchemaName).tableHandle(T1).description() );
    this->printTableInfo( session->schema(SchemaName).tableHandle(T2).description() );
    this->printTableInfo( session->schema(SchemaName).tableHandle(T3).description() );
 
    session->transaction().commit();
    
}

void 
Testing::printTableInfo( const coral::ITableDescription& description ) const
{
  int numberOfColumns = description.numberOfColumns();
  std::cout << "Table " << description.name();
  std::string tableSpaceName = description.tableSpaceName();
  if ( ! tableSpaceName.empty() )
    std::cout << " (created in tablespace " << tableSpaceName << ")";
  std::cout << " has" << std::endl
            << "  " << numberOfColumns << " columns :" << std::endl;
  for ( int i = 0; i < numberOfColumns; ++i ) {
    const coral::IColumn& column = description.columnDescription( i );
    std::cout << "    " << column.name() << " (" << column.type() << ")";
    if ( column.isUnique() ) std::cout << " UNIQUE";
    if ( column.isNotNull() ) std::cout << " NOT NULL";
    std::cout << std::endl;
  }

  if ( description.hasPrimaryKey() ) {
    const std::vector<std::string>& columnNames = description.primaryKey().columnNames();
    std::cout << "  Primary key defined for column";
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for ( std::vector<std::string>::const_iterator iColumn = columnNames.begin();
          iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = description.primaryKey().tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfUniqueConstraints = description.numberOfUniqueConstraints();
  std::cout << "  " << numberOfUniqueConstraints;
  if ( numberOfUniqueConstraints == 1 ) std::cout << " unique constraint:" << std::endl;
  else if ( numberOfUniqueConstraints == 0 ) std::cout << " unique constraints" << std::endl;
  else std::cout << " unique constraints:" << std::endl;
  for ( int i = 0; i < numberOfUniqueConstraints; ++i ) {
    const coral::IUniqueConstraint& uniqueConstraint = description.uniqueConstraint( i );
    std::cout << "    " << uniqueConstraint.name() << " defined for column";
    const std::vector<std::string>& columnNames = uniqueConstraint.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = uniqueConstraint.tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfIndices = description.numberOfIndices();
  std::cout << "  " << numberOfIndices;
  if ( numberOfIndices == 1 ) std::cout << " index:" << std::endl;
  else if ( numberOfIndices == 0 ) std::cout << " indices" << std::endl;
  else std::cout << " indices:" << std::endl;
  for ( int i = 0; i < numberOfIndices; ++i ) {
    const coral::IIndex& index = description.index( i );
    std::cout << "    " << index.name();
    if ( index.isUnique() ) std::cout << " (UNIQUE)";
    std::cout << " defined for column";
    const std::vector<std::string>& columnNames = index.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::string tableSpace = index.tableSpaceName();
    if ( ! tableSpace.empty() )
      std::cout << " in tablespace " << tableSpace;
    std::cout << std::endl;
  }

  int numberOfForeignKeys = description.numberOfForeignKeys();
  std::cout << "  " << numberOfForeignKeys;
  if ( numberOfForeignKeys == 1 ) std::cout << " foreign key:" << std::endl;
  else if ( numberOfForeignKeys == 0 ) std::cout << " foreign keys" << std::endl;
  else std::cout << " foreign keys:" << std::endl;
  for ( int i = 0; i < numberOfForeignKeys; ++i ) {
    const coral::IForeignKey& foreignKey = description.foreignKey( i );
    std::cout << "    " << foreignKey.name() << " defined for column";
    const std::vector<std::string>& columnNames = foreignKey.columnNames();
    if ( columnNames.size() > 1 ) std::cout << "s";
    std::cout << " ";
    for (  std::vector<std::string>::const_iterator iColumn = columnNames.begin();
           iColumn != columnNames.end(); ++iColumn ) {
      if ( iColumn != columnNames.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::cout << " -> " << foreignKey.referencedTableName() << "( "; 
    const std::vector<std::string>& columnNamesR = foreignKey.referencedColumnNames();
    for (  std::vector<std::string>::const_iterator iColumn = columnNamesR.begin();
           iColumn != columnNamesR.end(); ++iColumn ) {
      if ( iColumn != columnNamesR.begin() ) std::cout << ", ";
      std::cout << *iColumn;
    }
    std::cout << " )" << std::endl;
  }
}

void 
Testing::createViewDefault()
{
    session->transaction().start();
    
    coral::ISchema& schema = session->nominalSchema();
    //delete previous view creations
    schema.dropIfExistsView(V0);
    
   // Create a view
    
    coral::IViewFactory* factory = schema.viewFactory();
    
    factory->addToOutputList( T3 + ".id", "id" );
    factory->addToOutputList( T1 + ".X", "x" );
    factory->addToOutputList( T1 + ".Y", "y" );
    factory->addToOutputList( T1 + ".Z", "z" );
    factory->addToOutputList( T2 + ".tx", "TX" );
    factory->addToOutputList( T2 + ".ty", "TY" );
    factory->addToTableList(T3);
    factory->addToTableList(T1);
    factory->addToTableList(T2);

    factory->setCondition( T1 + ".id = " + T3 + ".fk1 AND " + T2 + ".id = " + T3 + ".fk2 AND " + T3 + ".Qn > 0.5", coral::AttributeList() );

//    factory->setCondition( "T1.id = T3.fk1 AND T2.id = T3.fk2 ",
//                          coral::AttributeList() );

 //   if(isDBType("mysql")){
 //     factory->create(V0);
 //   }else{
      factory->create(V0).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
 //   }  
    //here happens the segmentation fault somewhere
    //.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
    delete factory;
   
    session->transaction().commit();
}

void
Testing::fillSchemaDefault()
{
  int i;
  
  session->transaction().start();

  coral::ISchema& schema = session->nominalSchema();
  coral::ITable& table = schema.tableHandle(T1);
  coral::ITableDataEditor& dataEditor = table.dataEditor();
  
  if ( m_rowBuffer ) delete m_rowBuffer;
  m_rowBuffer = new coral::AttributeList;
  coral::AttributeList& row = *m_rowBuffer;

  const coral::ITableDescription& description = table.description();
  int numColumns = description.numberOfColumns();
  for ( i = 0; i < numColumns; i++ ) {
    const coral::IColumn& column = description.columnDescription( i );
    row.extend( column.name(), column.type() );
  }

  for ( i = 1; i < 11; i++ ) {
    row[0].data<long long>() = i;
    row[1].data<long long>() = 100 - i;
    row[2].data<float>() = 0.123456;
    row[3].data<double>() = 0.1 * i;
    row[4].data<double>() = 0.2;
    row[5].data<std::string>() = "hello world";
    
    coral::Blob& blob = row[6].data<coral::Blob>();
    int blobSize = 1000 * ( i%100 + 1 );
    blob.resize( blobSize );
    unsigned char* p = static_cast<unsigned char*>( blob.startingAddress() );
    for ( int j = 0; j < blobSize; ++j, ++p ) *p = ( i + j )%256;
    
    dataEditor.insertRow( row );
  }

  session->transaction().commit();

}

void
Testing::printSchemaDefault()
{
  int i;
  
  session->transaction().start();
  
  coral::ISchema& schema = session->nominalSchema();
  coral::ITable& table = schema.tableHandle(T1);
  coral::IQuery * query = table.newQuery();
  
//  if ( m_rowBuffer ) delete m_rowBuffer;
//  m_rowBuffer = new coral::AttributeList;

  
//  int numColumns = description.numberOfColumns();
//  for ( i = 0; i < numColumns; i++ ) {
//    const coral::IColumn& column = description.columnDescription( i );
//    row.extend( column.name(), column.type() );
//  }
    
//  query->addToOrderList( "id" );
//  std::cout << row.size() << std::endl;
//  query->defineOutput( *buffer );
  query->setRowCacheSize( 0 );

  coral::ICursor& cursor = query->execute();
  
  for ( i = 1; i < 11; i++ ) {
    if ( ! cursor.next() ) break;
    const coral::AttributeList& row = cursor.currentRow();
    std::cout << row[0].data<long long>() << "\t" << row[1].data<long long>() << "\t" << row[3].data<double>() << std::endl;

  }
  
  delete query;
  
  session->transaction().commit();
}

void
Testing::createSimpleTable()
{
  session->transaction().start();
  
  session->nominalSchema().dropIfExistsTable(T1);
  
  coral::TableDescription description1( "SchemaTest_Test" );
  description1.setName(T1);

  description1.insertColumn( "id", coral::AttributeSpecification::typeNameForId( typeid(long) ) );
  description1.setPrimaryKey( "id" );
  
  session->nominalSchema().createTable( description1 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

  session->transaction().commit();
}
