#include <cstdlib>
#include <cstring>

#include "TestEnv/TestEnv.h"

#include "CoralBase/Exception.h"
#include "PHash.hpp"
#include "CommandOptions.h"

#include <iostream>

TestEnv::TestEnv(std::string testName)
  : m_testName( testName )
{
  if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
    ::putenv( const_cast<char*>("CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db") );
  } 
  if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
    ::putenv( const_cast<char*>("CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db") );
  }
  
  std::map<std::string, std::string> platformMatrix;
  typedef std::pair<std::string, std::string> TStrStrPair;
  //amd 64bit
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc34", "A0"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc34_dbg", "A1"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc41", "A2"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc41_dbg", "A3"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc43", "A4"));
  platformMatrix.insert(TStrStrPair("slc4_amd64_gcc43_dbg", "A5"));
  //intel 32bit
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc34", "B0"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc34_dbg", "B1"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc41", "B2"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc41_dbg", "B3"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc43", "B4"));
  platformMatrix.insert(TStrStrPair("slc4_ia32_gcc43_dbg", "B5"));
  //macosx for intel
  platformMatrix.insert(TStrStrPair("osx104_ia32_gcc401", "C0"));
  platformMatrix.insert(TStrStrPair("osx104_ia32_gcc401_dbg", "C1"));
  
  platformMatrix.insert(TStrStrPair("osx105_ia32_gcc401", "C2"));
  platformMatrix.insert(TStrStrPair("osx105_ia32_gcc401_dbg", "C3"));
  
  platformMatrix.insert(TStrStrPair("win32_vc71", "D0"));
  platformMatrix.insert(TStrStrPair("win32_vc71_dbg", "D1"));
  platformMatrix.insert(TStrStrPair("i686-winxp-vc9-dbg ", "D2"));
  //new platforms
  platformMatrix.insert(TStrStrPair("i686-slc5-gcc34-opt", "E0"));
  platformMatrix.insert(TStrStrPair("i686-slc5-gcc34-dbg", "E1"));
  platformMatrix.insert(TStrStrPair("i686-slc5-gcc43-opt", "E2"));
  platformMatrix.insert(TStrStrPair("i686-slc5-gcc43-dbg", "E3"));
  
  platformMatrix.insert(TStrStrPair("x86_64-slc5-gcc34-opt", "F0"));
  platformMatrix.insert(TStrStrPair("x86_64-slc5-gcc34-dbg", "F1"));
  platformMatrix.insert(TStrStrPair("x86_64-slc5-gcc43-opt", "F2"));
  platformMatrix.insert(TStrStrPair("x86_64-slc5-gcc43-dbg", "F3"));
    
  std::map<std::string, std::string> slotMatrix;
  
  slotMatrix.insert(TStrStrPair("dev", "A"));
  slotMatrix.insert(TStrStrPair("dev2", "B"));
  slotMatrix.insert(TStrStrPair("dev3", "C"));
  slotMatrix.insert(TStrStrPair("test", "T"));
  slotMatrix.insert(TStrStrPair("geant4", "G"));
  
  std::string slotPart1;
  //check the slot environment variable
  if ( ::getenv( "LCG_NGT_SLT_NAME" ) ) {
    slotPart1 = slotMatrix[::getenv( "LCG_NGT_SLT_NAME" )];
  }
  //if no evironment set take the test one
  if(slotPart1 == "") slotPart1 = "T";
  //check the slot environment variable
  
  std::string slotPart2;
  if ( ! ::getenv( "LCG_NGT_SLT_NUM" ) ) {
    slotPart2 = "0";
  } else {
    slotPart2 = ::getenv( "LCG_NGT_SLT_NUM" );
    if(!((slotPart2[0] >= '0') && (slotPart2[0] <= '9'))){
      slotPart2 = "0";
    }
  }
    
  m_slotPrefix = slotPart1 + slotPart2.substr(0,2);
    
  if ( ::getenv( "CMTCONFIG" ) ) {
    m_platformPrefix = platformMatrix[::getenv( "CMTCONFIG" )];
  }
  if(m_platformPrefix == "") m_platformPrefix = "NO";

}

TestEnv::~TestEnv()
{
  size_t size = m_writers.size();
  for( size_t i = 0; i < size; i++ ) delete m_writers[i];
  m_writers.clear();
}

TestEnvWriters&
TestEnv::addWriters()
{
  TestEnvWriters * new_writers = new TestEnvWriters();
  m_writers.push_back(new_writers);
  return *new_writers;
}

void
TestEnv::setupUserInput(int argc, char * argv[])
{
  int size_writers = m_writers.size();
  int i;
  for( i = 1; i < argc; i++ ){
    if( i <= size_writers ){
      //if only writer was set
      std::string arg = argv[i];
      std::string::size_type next = arg.find(':');
      if(next != std::string::npos){
        std::string writer = arg.substr(0, next);
	std::string reader = arg.substr(next + 1);
        m_writers[i - 1]->setCurrentName(writer);
        m_writers[i - 1]->readers(m_writers[i - 1]->getCurrentName()).setCurrentName(reader);
      }else{
        m_writers[i - 1]->setCurrentName(arg);    
      }
    }
  }
}


bool
TestEnv::check(int argc, char * argv[])
{
  size_t i, size, j, size2, k, size3;
  std::vector<std::string> writers;
  if(argc > 1){
    std::string cmdLineOption = argv[1]; // First command line argument
    const CommandOption * option = Perfect_Hash::IsValidCommandLineOption(cmdLineOption.c_str(), cmdLineOption.length());
    if(option == NULL){
      setupUserInput(argc, argv);
      return true;
    }
    switch (option->OptionCode){
      case CommandOptionCode::HELP :
        std::cout << "TestEnv framework for the Coral Project, Version 0.2.0" << std::endl;
        std::cout << "Usage: " << argv[0] << " [OPTION]... [WRITER]:[READER]..." << std::endl << std::endl;
        std::cout << "  -v, --view\t\tdisplay all permutations" << std::endl;
        std::cout << "  -x, --xml\t\tdisplay all permutations in xml format" << std::endl;
        std::cout << "  -h, --help\t\tdisplay this help and exit" << std::endl << std::endl;
        std::cout << "Writers: oracle, mysql, sqlite" << std::endl;
        std::cout << "Readers: oracle, mysql, sqlite, coral, frontier" << std::endl;
      return false;
      case CommandOptionCode::VIEW :
        size = m_writers.size();
        for( i = 0; i < size; i++ ){
          std::cout << "+-- Writer " << (i + 1) << " --+-- Readers ---------" << std::endl;   
          m_writers[i]->entries(writers);
          size2 = writers.size();
          for( j = 0; j < size2; j++ ){
            std::vector<std::string>& readers = m_writers[i]->readers(writers[j]).entries();
            size3 = readers.size();
            std::cout << "| " << writers[j];
            for( k = 0; k < (12 - writers[j].size()); k++ ) std::cout << ' ';
            std::cout << " | ";
            if(size3 > 0)  std::cout << readers[0];
            for( k = 1; k < size3; k++ ) std::cout << ", " << readers[k];
            std::cout << std::endl;
          }
        }
        std::cout << "+--------------+--------------------" << std::endl;   
      return false;
      case CommandOptionCode::XML : 
	std::cout << "<test>" << std::endl;
        size = m_writers.size();
        for( i = 0; i < size; i++ ){
          std::cout << "  <option id=\"" << i << "\">" << std::endl;
          m_writers[i]->entries(writers);
          size2 = writers.size();
          for( j = 0; j < size2; j++ ){
            std::cout << "    <writer>" << writers[j] << "</writer>" << std::endl;
            std::vector<std::string>& readers = m_writers[i]->readers(writers[j]).entries();
            size3 = readers.size();
            for( k = 0; k < size3; k++ ) std::cout << "    <reader writer=\"" << writers[j] << "\">" << readers[k] << "</reader>" << std::endl;
          }
          std::cout << "  </option>" << std::endl;
        }      
        std::cout << "</test>" << std::endl;
      return false;
      default:
        setupUserInput(argc, argv);
      return true;
    }
  }
  return true;
}

size_t
TestEnv::addServiceName(size_t pos, const std::string writerexp, const std::string readerexp)
{
  if( pos >= m_writers.size() ) throw coral::Exception( "TestEnv", "Index out of range", "TestEnv::addServiceName" );
  TestEnvWriters * writer = m_writers[pos];
  std::string& writer_name = writer->getCurrentName();
  std::string& reader_name = writer->readers(writer_name).getCurrentName();
  //insert the calculated strings to the services
  //if the reader name is empty no reader is available
  if( reader_name.empty() || readerexp.empty() ){
    std::string expression = "/tests/db_" + writer_name.substr(0,3) + '/' + writerexp;
    m_services.push_back(std::pair<std::string, std::string>(expression, expression));
  }else{
    std::string expression01 = "/tests/db_" + writer_name.substr(0,3) + '/' + writerexp;
    std::string expression02 = "/tests/db_" + reader_name.substr(0,3) + '/' + readerexp;
    m_services.push_back(std::pair<std::string, std::string>(expression01, expression02));  
  }
  return m_services.size() - 1;
}

const std::string
TestEnv::getServiceName(size_t index, bool reader) const
{
  if( index >= m_services.size() ) throw coral::Exception( "TestEnv", "Index out of range", "TestEnv::getServiceName" );
  if( reader == TESTENV_READER ){
    return m_services[index].second;
  }else{
    return m_services[index].first;
  }
}

const std::string&
TestEnv::getTableString(std::string& source, const std::string& tableName) const
{
  source = m_slotPrefix + m_platformPrefix + "_" + m_testName + "_" + tableName;
  return source;
}

const std::string
TestEnv::getDBType(size_t pos, bool reader) const
{
  if( reader == TESTENV_READER ){
    return m_writers[pos]->readers(m_writers[pos]->getCurrentName()).getCurrentName();
  }else{
    return m_writers[pos]->getCurrentName();
  }
}


