#! /usr/bin/env python
# $Id: config.py,v 1.2 2007/12/10 16:33:47 rbasset Exp $

from xml.dom import minidom

class Config:

	# reads the content of the elements of a given xmlNode with a given tagName
	def getListFromXmlNode(self, xmlNode, tagName):
		liste = []
		for elem in xmlNode.getElementsByTagName(tagName):
			liste.append(str(elem.firstChild.data))
		return liste

	def __init__(self, confFilePath="config.xml"):
		self.config = {}

		sock = open(confFilePath)
		xmldoc = minidom.parse(sock).documentElement
		sock.close()
		
		for cfgElement in xmldoc.childNodes:
			if cfgElement.nodeType != cfgElement.ELEMENT_NODE:
				continue

			if cfgElement.tagName == 'machines':
				self.config['testNodes'] = self.getListFromXmlNode(cfgElement, "client")
				self.config['monServers'] = self.getListFromXmlNode(cfgElement, "server")
				self.config['numRACNodes'] = len(self.config['monServers'])

			if cfgElement.tagName == 'statistic':
				self.config['cpu'] = int(cfgElement.attributes["cpu"].value)
				self.config['waitEvents'] = int(cfgElement.attributes["waitEvents"].value)

			if cfgElement.tagName == 'execution':
				self.config['numBoxes'] = int(cfgElement.attributes["numCliNode"].value)
				self.config['numMaxClientsPerBox'] = int(cfgElement.attributes["numSessPerCliNode"].value)
				self.config['numClientsPerBoxInit'] = int(cfgElement.attributes["numSessIni"].value)
				self.config['numClientsPerStep'] = int(cfgElement.attributes["numSessInc"].value)
				self.config['stepPeriod'] = int(cfgElement.attributes["stepDuration"].value)
				self.config['testMaxDuration'] = int(cfgElement.attributes["maxTestDuration"].value)

			if cfgElement.tagName == 'locations':
				self.config['baseDir'] = str(cfgElement.attributes["baseDir"].value)
				self.config['clientLogDir'] = str(cfgElement.attributes["clientLogDir"].value)

			if cfgElement.tagName == 'metrics':
				self.config['dynamicMetrics'] = self.getListFromXmlNode(cfgElement, "dynamic")
				self.config['staticMetrics'] = self.getListFromXmlNode(cfgElement, "static")
				self.config['numMetrics'] = len(self.config['dynamicMetrics']) + len(self.config[ 'staticMetrics' ])

			if cfgElement.tagName == 'clientConf':
				self.config['db_username'] = str(cfgElement.attributes["dbUser"].value)
				self.config['testClientCmd'] = str(cfgElement.attributes["clientCmd"].value)
				self.config['testClientExe'] = str(cfgElement.attributes["clientExe"].value)
				self.config['reportPeriod'] = int(cfgElement.attributes["reportPeriod"].value)
				self.config['report_db_connection'] = str(cfgElement.attributes["reportDbConnect"].value)

			if cfgElement.tagName == 'fileToSave':
				self.config['fileToSave'] = {}
				for elem in cfgElement.getElementsByTagName("file"):
					self.config['fileToSave'][str(elem.attributes["name"].value)] = str(elem.firstChild.data)

