-- ----------------------------------------------------------------------------
-- Create an Oracle account for the monitoring feature.
-- ----------------------------------------------------------------------------

-- create account
CREATE USER &&Monitoring_user IDENTIFIED BY &Password;

-- grant required privileges
GRANT cern_app_role TO &&Monitoring_user;
GRANT SELECT ON sys.gv_$session TO &&Monitoring_user;
GRANT SELECT ON sys.v_$event_name TO &&Monitoring_user;
GRANT SELECT ON sys.gv_$waitclassmetric TO &&Monitoring_user;
GRANT SELECT ON sys.gv_$sysmetric TO &&Monitoring_user;
GRANT CREATE VIEW TO &&Monitoring_user;

-- create views
CREATE OR REPLACE VIEW &&Monitoring_user..st_session AS
SELECT inst_id, username, program
FROM sys.gv_$session;

CREATE OR REPLACE VIEW &&Monitoring_user..st_avg_active_sessions AS
SELECT class, sum(time_waited) / avg(intsize_csec) value
  FROM
 (
 SELECT wc.inst_id, en.wait_class class,wc.time_waited, wc.intsize_csec
   FROM gv$waitclassmetric wc, 
	(SELECT UNIQUE wait_class, wait_class# 
	 FROM v$event_name 
	 WHERE wait_class != 'Idle') en
  WHERE wc.wait_class# = en.wait_class#
 UNION ALL
 SELECT inst_id, 'CPU Used', value, 100
   FROM gv$sysmetric
  WHERE metric_name='CPU Usage Per Sec'
 )
GROUP BY class
ORDER BY class;

