#! /usr/bin/env python
# $Id: reportTest.py,v 1.17 2007/12/10 16:33:47 rbasset Exp $
# Analysis the log files of test clients

import csv, sys, getopt, os, string, time
from datetime import datetime, timedelta
from lemonData import LemonData

_isFinished = False
_verbose = False
_testStart = 0

class MetricAggregator(object):
   """ Class that computes the metric values to plot them."""

   def compute(self, values):
      """ Compute the metric values."""
      pass

   def close(self):
      """ Write to file the values needed for the graphs."""
      pass

class AllReportsPlot(MetricAggregator):

   def __init__(self, dynMetrics, staMetrics):
      self.nDynMetrics = len(dynMetrics) # number of metrics used for rate
      self.nStaMetrics = len(staMetrics) # number of metrics used as it is, witout calculation
      self.nMetrics = self.nDynMetrics + self.nStaMetrics # numbers of metrics (all of them)
      self.dynHeaders = dynMetrics
      self.staHeaders = staMetrics
      self.allTimestamps = []
      self.allRates = [[] for i in range(self.nDynMetrics)] # from dynamic metrics
      self.allValues = [[] for i in range(self.nStaMetrics)] # from static metrics
     
   def compute(self, timestamp, interval, values):
      self.allTimestamps.append(timestamp)
      for i in range(self.nDynMetrics):
         metric =  values[i]
	 rate = metric / interval
	 self.allRates[i].append(rate)
      for i in range(self.nStaMetrics):
	 self.allValues[i].append(values[i+self.nDynMetrics])

   def close(self, logDir):
      for i in range(self.nDynMetrics):
         metricdata = open(logDir + "/jpgraphdata/metricthroughput" + str(i) + ".php", 'wb')
         metricdata.write("<? $graphname = 'All client reports : " + self.dynHeaders[i] + " / sec'; ")
         metricdata.write("$metricname = '" + self.dynHeaders[i] + " / sec'; ")
         metricdata.write("$metricdata = array(" + str(self.allRates[i])[1:-1] + "); ?>")
         metricdata.close()
      for i in range(self.nStaMetrics):
         metricdata = open(logDir + "/jpgraphdata/metricthroughput" + str(i+self.nDynMetrics) + ".php", 'wb')
         metricdata.write("<? $graphname = 'All client reports : " + self.staHeaders[i] + "'; ")
         metricdata.write("$metricname = '" + self.staHeaders[i] + "'; ")
         metricdata.write("$metricdata = array(" + str(self.allValues[i])[1:-1] + "); ?>")
         metricdata.close()
      
      xdata = open(logDir + "/jpgraphdata/xdatathroughput.php", 'wb')
      xdata.write("<? $xdata = array(" + str(self.allTimestamps)[1:-1] + "); ?>")
      xdata.close()

class InstantRatePlot(MetricAggregator):

   def __init__(self, headers, plotInterval, testStart):
      self.nMetrics = len(headers)
      self.headers = headers
      self.plotInterval = plotInterval
      self.testStart = testStart
      self.before = []
      self.xPlot = testStart + plotInterval
      self.after = []
      self.xData = [self.xPlot]
      self.yData = [[] for i in range(self.nMetrics)]
     
   def compute(self, timestamp, values):
      while (timestamp > self.xPlot + self.plotInterval):
	 if (self.before == []) & (self.after == []):
	    rates = [ 0 for a in values ]
	    map(list.append, self.yData, rates)
	 elif (self.before == []):
	    rates = [a / (self.plotInterval*2) for a in self.after]
	    map(list.append, self.yData, rates)
	 elif (self.after == []):
	    rates = [a / (self.plotInterval*2) for a in self.before]
	    map(list.append, self.yData, rates)
	 else:
	    metrics = map(float.__add__, self.before, self.after)
	    rates = [a / (self.plotInterval*2) for a in metrics]
	    map(list.append, self.yData, rates)
	 
	 self.before = self.after
	 self.after = []
         self.xPlot += self.plotInterval
	 self.xData.append(self.xPlot)

      if timestamp < self.xPlot:
         if (self.before == []):
	    self.before = values
	 else:
	    self.before = map(float.__add__, self.before, values)
      elif timestamp < self.xPlot + self.plotInterval:
         if (self.after == []):
	    self.after = values
	 else:
	    self.after = map(float.__add__, self.after, values)

   def close(self, logDir):
      self.xData = self.xData[:-1]
      # flush the last plot
#       if (self.before == []):
# 	rates = [a / (self.plotInterval*2) for a in self.after]
# 	map(list.append, self.yData, rates)
#       elif (self.after == []):
#          pass
#       else:
# 	metrics = map(float.__add__, self.before, self.after)
# 	rates = [a / (self.plotInterval*2) for a in metrics]
# 	map(list.append, self.yData, rates)
     
      for i in range(self.nMetrics):
         metricdata = open(logDir + "/jpgraphdata/metricdata" + str(i) + ".php", 'wb')
         metricdata.write("<? $graphname = 'Aggregated rate : " + self.headers[i] + " / sec'; ")
         metricdata.write("$metricname = '" + self.headers[i] + "'; ")
         metricdata.write("$metricdata = array(" + str(self.yData[i])[1:-1] + "); ?>")
         metricdata.close()
      
      xdata = open(logDir + "/jpgraphdata/xdatametric.php", 'wb')
      xdata.write("<? $xdatametric = array(" + str(self.xData)[1:-1] + "); ?>")
      xdata.close()


###############################################################
def getTestDate ():
   # get the start date and the end date
   dateFile = open( cfg[ 'clientLogDir' ] + '/executionDates.log', 'r' )
   startTimestamp = float(dateFile.readline().strip())
   global _testStart
   _testStart = startTimestamp
   endTimestamp = dateFile.readline()
   if endTimestamp:
      endTimestamp = float(endTimestamp.strip())
      global _isFinished
      _isFinished = True
   else:
      endTimestamp = time.time()

   if ( endTimestamp - startTimestamp < 600 ):
      startTimestamp = endTimestamp - 600
   startDatetime = datetime.fromtimestamp(startTimestamp)
   endDatetime = datetime.fromtimestamp(endTimestamp)
   return (startDatetime, endDatetime)
   
###############################################################
# prints the test date in the report AND builds lemon base url
# html. report html file descriptor
def printTestDate( html ):

   dateTuple = getTestDate()

   # test start
   sDate = dateTuple[0].strftime('%d/%b/%Y at %H:%M')
   # test end
   eDate = dateTuple[1].strftime('%d/%b/%Y at %H:%M')

   html.write( '<hr><center>===============>>   Test run on from <b> ' + sDate + ' to ' + eDate + ' </b>   <<===============</center>' )

###############################################################
# LEMON plots links
# generates links and downloads plots from the lemon server to afs
# html. report html file descriptor
def printLemonPlots(html):

   dateTuple = getTestDate()
   startDatetime = dateTuple[0]-timedelta(0,300)
   endDatetime = dateTuple[1]+timedelta(0,300)

#### test start
   sYear = startDatetime.strftime('%Y')
   sMonth = startDatetime.strftime('%m')
   sDay = startDatetime.strftime('%d')
   sHour = startDatetime.strftime('%H')
   sMin = startDatetime.strftime('%M')

#### test end
   eYear = endDatetime.strftime('%Y')
   eMonth = endDatetime.strftime('%m')
   eDay = endDatetime.strftime('%d')
   eHour = endDatetime.strftime('%H')
   eMin = endDatetime.strftime('%M')

#### url model
   baseLemonUrl = 'http://lemonweb.cern.ch/lemon-web/metric_graph.php?type=host'
   baseLemonUrl += '&sdate='+sDay+'-'+sMonth+'-'+sYear+'&sdate_time='+sHour+':'+sMin+':00'
   baseLemonUrl += '&edate='+eDay+'-'+eMonth+'-'+eYear+'&edate_time='+eHour+':'+eMin+':00'

   html.write( '<HR><H2>MONITORING</H2>' )

   lemonData = LemonData(cfg['clientLogDir'], sDay+'-'+sMonth+'-'+sYear, sHour+':'+sMin+':00', eDay+'-'+eMonth+'-'+eYear, eHour+':'+eMin+':00')
   
   for serverName in cfg['monServers'] + cfg['testNodes'][:cfg[ 'numBoxes' ]]:
      lemonUrl = baseLemonUrl+'&entity='+serverName
      html.write('<CENTER><H2><A NAME="' + serverName + '">' + serverName + '</A></h2></CENTER>')
      net_metric = '&metric=networkInterfaceI0'
      net_in = '<a href="'+lemonUrl+net_metric+'&field=NUMKBREADAVG">IN</a>'
      net_out = '<a href="'+lemonUrl+net_metric+'&field=NUMKBWRITEAVG">OUT</a>'
      html.write('<H4>Network utilization : '+net_in+' - '+net_out+'</H4>')
      cpu_idle = '<a href="'+lemonUrl+'&metric=CPUutil&field=PercIdle">Idle %</a>'
      cpu_load = '<a href="'+lemonUrl+'&metric=LoadAvg&field=LOADAVG">Load Average</a>'
      html.write('<H4>CPU : '+cpu_idle+' - '+cpu_load+'</H4>')

      if _isFinished:
         print "Downloading lemon plots to disk for " + serverName
         lemonData.fetchHostImg(serverName)
         html.write('<IMG SRC="' + serverName + '_networkInterfaceI0_NUMKBREADAVG.png" ALT="Network input">')
         html.write('<IMG SRC="' + serverName + '_networkInterfaceI0_NUMKBWRITEAVG.png" ALT="Network output">')
         html.write('<img src="' + serverName + '_CPUutil_PercIdle.png" ALT="CPU Idle %">')
         html.write('<img src="' + serverName + '_LoadAvg_LOADAVG.png" ALT="Load average">')
#       else:
#          html.write('<img src="' + baseLemonUrl + '&metric=networkInterfaceI0.NUMKBREADAVG&host=' + serverName + '" ALT="Network input">')
#          html.write('<img src="' + baseLemonUrl + '&metric=networkInterfaceI0.NUMKBWRITEAVG&host=' + serverName + '" ALT="Network output">')
#          html.write('<H4>CPU (Idle Time % - Load Average)</H4>')
#          html.write('<img src="' + baseLemonUrl + '&metric=CPUutil.PERCIDLE&host=' + serverName + '" ALT="CPU Idle %">')
#          html.write('<img src="' + baseLemonUrl + '&metric=LoadAvg.LoadAvg&host=' + serverName + '" ALT="Load average">')

      html.write('<br/><br/>More: <a href="http://lemonweb.cern.ch/lemon-web/info.php?entity='+serverName+'&type=host&detailed=yes">'+serverName+' Lemon page</a>')
      html.write('<br/>Top : <a href="#top">top of the page</a>')

################################################
######          GENERATE TP PLOT         #######
################################################

def graphStats(html):
   """\
   
   """
#### First time : create folder for jpgraph data and copy php files
   if not os.path.exists(cfg['clientLogDir'] + "/jpgraphdata"):
      os.mkdir(cfg['clientLogDir'] + "/jpgraphdata")
      os.system("cp " + cfg["baseDir"] + "/etc/*.php " + cfg["clientLogDir"])
      
   statFile    = open(cfg["clientLogDir"] + "/clientCount.csv")
   csv_reader  = csv.reader(statFile, delimiter=';')
   headers     = csv_reader.next()
   global _testStart
   lastReport  = _testStart
   timestamps  = []
   clients     = []
   globalRate  = [[] for i in range(cfg['numMetrics'])]
   globalRatePerClient = [[] for i in range(cfg['numMetrics'])]
   statValues  = [[] for i in headers[cfg['numMetrics'] + 2:]]
   numDynaMetrics = len(cfg['dynamicMetrics'])
   numStatMetrics = len(cfg['staticMetrics'])

   for row in csv_reader:
      values = [float(cell) for cell in row]
      timestamps.append(values[0])
      clients.append(values[1])
      map(list.append, statValues, values[cfg['numMetrics'] + 2:])
      #sinceStartTest = values[0] - _testStart
      #for i in range(cfg['numMetrics']):
      #   metric =  values[2 + i]
      #   globalRateTmp = metric / sinceStartTest
      #   globalRate[i].append(globalRateTmp)
      #   globalRatePerClient[i].append(globalRateTmp / values[1])

   xdata = open(cfg["clientLogDir"] + "/jpgraphdata/xdata.php", 'wb')
   xdata.write("<? $xdata = array(" + str(timestamps)[1:-1] + ");")
   xdata.write(" $clientdata = array(" + str(clients)[1:-1] + "); ?>")
   xdata.close()

   html.write("<HR><H2>MONITORING</H2>")
   html.write("<H3>Metrics</H3>")
   for i in range(len(cfg["dynamicMetrics"])):
      html.write("<H4>" + headers[i+2] + "</H4>")
   #### Aggregated
      #metricdata = open(cfg["clientLogDir"] + "/jpgraphdata/metricdata" + str(i) + ".php", 'wb')
      #metricdata.write("<? $graphname = 'Aggregated rate : " + headers[i+2] + " / sec and number of clients'; ")
      #metricdata.write("$metricname = '" + headers[i+2] + "'; ")
      #metricdata.write("$metricdata = array(" + str(globalRate[i])[1:-1] + "); ?>")
      #metricdata.close()
      html.write('<IMG SRC="metricplot.php?metric=' + str(i) + '" ALT="Global ' + headers[i+2] + '"/><BR/>')

   if cfg['cpu']:
      cpudata = open(cfg["clientLogDir"] + "/jpgraphdata/cpudata.php", 'wb')
      cpudata.write("<? $mincpudata = array(" + str(statValues[0])[1:-1] + ");")
      cpudata.write(" $avgcpudata = array(" + str(statValues[1])[1:-1] + ");")
      cpudata.write(" $maxcpudata = array(" + str(statValues[2])[1:-1] + "); ?>")
      cpudata.close()
      html.write("<H3>Server CPU Utilization</H3>")
      html.write('<IMG SRC="cpuplot.php" ALT="Server CPU Utilization"/><BR/>')
      statValues = statValues[3:]
      headers = headers[cfg["numMetrics"]+5:]
   else:
      headers = headers[cfg["numMetrics"]+2:]

   if cfg['waitEvents']:
      waitclasses = ["CPU Used", "Scheduler", "User I/O", "System I/O", "Concurrency", "Application", "Commit", "Configuration", "Administrative", "Network", "Cluster", "Other"]
      waitdata = open(cfg["clientLogDir"] + "/jpgraphdata/waitdata.php", 'wb')
      waitdata.write("<? $waitdata = array(\n")
      separator = ""
      for waitclass in waitclasses:
         index = headers.index(waitclass)
         waitdata.write(separator + "array(" + str(statValues[index])[1:-1] + ")")
         separator = ",\n"
      waitdata.write("); \n$waitheaders = array(" + str(waitclasses)[1:-1] + "); ?>")
      waitdata.close()
      html.write("<H3>Average Active Sessions</H3>")
      html.write('<IMG SRC="waitplot.php" ALT="Average Active Sessions"/><BR/>')

   
   statFile    = os.popen("cat " + cfg["clientLogDir"] + "/log*.csv|sort -n")
   csv_reader  = csv.reader(statFile, delimiter=';')
   headers     = csv_reader.next()
   fewTimestamps  = []
   globalRate     = [[] for i in range(cfg['numMetrics'])]
   globalRateCounter = [] # where metrics are computed

   allPlots = AllReportsPlot(cfg['dynamicMetrics'],cfg['staticMetrics'])
   regularPlots = InstantRatePlot(headers[2:2+cfg['numMetrics']], cfg['reportPeriod'], _testStart)
   
   for row in csv_reader:
     if float(row[1]) > 0:
      values = [float(cell) for cell in row]
      allPlots.compute(values[0], values[1], values[2:cfg['numMetrics']+2])
      regularPlots.compute(values[0], values[2:numDynaMetrics+2])
#       for i in range(cfg['numMetrics']):
#          metric =  values[2 + i]
#          rate = metric / values[1]
#          globalRate[i].append(rate)
   allPlots.close(cfg["clientLogDir"])
   regularPlots.close(cfg["clientLogDir"])

   html.write("<HR><H2>Throughput</H2>")
   html.write("<H3>Metrics</H3>")
   for i in range(cfg["numMetrics"]):
      html.write("<H4>" + headers[i+2] + "</H4>")
   #### Aggregated
      html.write('<IMG SRC="metricplotthroughput.php?metric=' + str(i) + '" ALT="Global ' + headers[i+2] + '"/><BR/>')



###################################################################
###################################################################
# MAIN - reads client output and produces a report in a HTML file #
###################################################################
###################################################################
if __name__ == '__main__':
   _verbose = True
   if len( sys.argv ) < 2:
      print "Usage: reportClients.py <logDirName>"
      sys.exit()

   # loads config parameters
   from config import Config
   cfgTemp = Config().config
   cfg = Config(cfgTemp[ 'clientLogDir' ] + sys.argv[1] + "/config.xml").config # uses local configuration file
   cfg['clientLogDir'] += sys.argv[1]

   os.system('vi desc.txt');
   os.system("cp " + cfg["baseDir"] + "bin/desc.txt " + cfg["clientLogDir"])

   if _verbose:
      print "Reading ", cfg[ 'clientLogDir' ]
   
   ###############################
   # main report - SETUP section #
   ###############################
   html = open( cfg['clientLogDir'] + '/index.php', 'w' )
   html.write( '<HTML><HEAD><title>Test Results</title></HEAD><BODY>' )
   html.write( '<H2><A NAME="#top">Description</A></H2><p><pre><? include("desc.txt"); ?></pre></p>' )
   html.write( '<H2>SETUP</H2>' )
   html.write( '<b>Test directory:</b> ' + cfg[ 'clientLogDir' ] + '<br>' )
   html.write( '<br><b>Client test nodes:</b> ')
   for node in cfg['testNodes'][:cfg[ 'numBoxes' ]]:
      html.write('<A HREF="#' + node + '">' + node + '</A> ' )
   html.write( '<br/><br/><b>Servers Under Test:</b> ')
   for node in cfg['monServers']:
      html.write('<A HREF="#' + node + '">' + node + '</A> ' )
   html.write('<br/><br/><b>Number of test clients:</b> ' + str( cfg[ 'numBoxes' ] * cfg[ 'numMaxClientsPerBox' ] ) + ' -> ' + str( cfg[ 'numBoxes' ] ) + ' node(s) running ' + str( cfg[ 'numMaxClientsPerBox' ] ) + ' client(s each)<br>' )
   for file in cfg['fileToSave']:
      html.write('<br><b>' + file + ' :</b> <a href="' + cfg['fileToSave'][file] + '">' + cfg['fileToSave'][file]  + '</a><br>')
   printTestDate( html )
  
   #################################
   #   instant feedback section    #
   #################################
   graphStats(html)
   
   ####################################
   # main report - MONITORING section #
   ####################################
   printLemonPlots(html)

   ####################################
   # main report - JOB STATUS section #
   ####################################
   html.write( '</BODY></HTML>\n' )

   print "Done!"
