#! /usr/bin/env python
# $Id: coolClient.py,v 1.2 2007/11/27 10:29:32 rbasset Exp $

from PyCool import cool
from time import time
import os, sys
import random
from client import Client
from coolSetup import CoolSetup

class CoolClient(Client):

	def folderName(self, iFolder):
	    """ Return a folder name provided a folder number. """
	    fName = "/f" + repr(iFolder).zfill(3)
	    return fName

	def headerPayload(self, iObj, iChan, fName, tail, spec):
	    """ Return a different payload for each object. """
	    head = "Object %4d in channel %4d in folder %5s - " % (iObj,iChan,fName)
	    payload = cool.Record(spec)
	    payload["S"] = head + tail
	    return payload

	def insertIov(self, first, last ):
	    """ Insert the IOVs in a specified range of folders. """
	    app = cool.Application()
	    dbSvc = app.databaseService()
	    readOnly = False
	    db = dbSvc.openDatabase( self.connectString, readOnly )

	    ### report
	    lastReport = time()
	    self.logReport([lastReport, 0, "Num IOV", "Total IOV"])
	    iov = 0
	    totalIov = 0
	    ### end report

	    t = lastReport
	    for iIOV in range(0, CoolSetup.nIOV ) :
	    	for iFolder in range( first, last+1 ) :
			fName = self.folderName( iFolder )
			f = db.getFolder(fName)
			#f.setupStorageBuffer() # still memory leak problem with pyCool ?
			for iChan in range( 1, CoolSetup.nChan+1 ) :
				iov += 1
	                	keyS = iIOV*CoolSetup.iovValidity + CoolSetup.firstKey 
				keyU = keyS + CoolSetup.iovValidity
				payload = self.headerPayload( iIOV, iChan, fName, CoolSetup.payloadString, CoolSetup.spec )
				f.storeObject( keyS, keyU, payload, iChan )

			#f.flushStorageBuffer()

			### report
			now = time()
			if now-lastReport > self._reportEveryXSecs/2.0:
		       		totalIov += iov
	       			self.logReport([now, now-lastReport, iov,totalIov])
				iov=0
				lastReport = now
			### end report


	def run(self):
		### Determine which range of folders will be handled
		### by this client (depends on the client ID)
		numClients = self.cfg["numMaxClientsPerBox"] * self.cfg["numBoxes"]
		numFolders = CoolSetup.nFolders
		first = (self._id - 1) * (numFolders / numClients) + 1
		last = self._id * (numFolders / numClients)
		remain = numFolders % numClients
		if id <= remain:
			first += id - 1
			last += id
		else:
			first += remain
			last += remain

		self.insertIov(first, last)

	def __init__(self, logFile="client", reportEveryXSecs=10, id=0, cfg={}, **kw):
		Client.__init__(self, logFile, reportEveryXSecs, **kw)
		self.cfg = cfg

		### Determine which node will be used
		self._id = id
		node = id % cfg['numRACNodes'] + 1
		self.connectString = "oracle://"+CoolSetup.oracleService+str(node)\
			+";schema="+CoolSetup.clientUsername\
			+";dbname="+CoolSetup.coolDBName


if __name__ == '__main__':
		id = idOffset + numClientsStarted
		CoolClient(logFile, reportEveryXSecs, id, cfg).start()

