#! /usr/bin/env python
# $Id: startTest.py,v 1.20 2007/12/10 16:33:47 rbasset Exp $
"""\
 Start testing jobs
 Start the test nodes referenced in the configuration file (config.py)
"""

import csv
import os
import sys
import time


def statHeaders(schema):
   """ Function that returns the statistic headers """
   headers = []

   try:
      if cfg["cpu"]:
         headers.append("Minimum Host CPU Utilization (%)")
         headers.append("Average Host CPU Utilization (%)")
         headers.append("Maximum Host CPU Utilization (%)")

      if cfg["waitEvents"]:
         query = schema.newQuery()
         query.addToOutputList("CLASS")
         query.addToTableList("ST_AVG_ACTIVE_SESSIONS")

         for currentRow in query.execute():
            headers.append(currentRow["CLASS"].data())

   except Exception, e:
      print("manager error in statHeaders :\t" + str(e))

   return headers

def statValues(schema1, schema2):
   """ Function that queries GV$SYSMETRIC to get some statistics about the database server """
   stats = []

   try:
      if cfg["cpu"]:
         query = schema1.newQuery()
         query.addToOutputList("MIN(VALUE)")
         query.addToOutputList("AVG(VALUE)")
         query.addToOutputList("MAX(VALUE)")
         query.addToTableList("GV_$SYSMETRIC")
         query.setCondition("METRIC_ID = 2057 and GROUP_ID = 3", coral.AttributeList())

         for currentRow in query.execute():
            for currentValue in currentRow:
               metricValue = currentValue.data()
               stats.append(metricValue)

      if cfg["waitEvents"]:
         query = schema2.newQuery()
         query.addToOutputList("VALUE")
         query.addToTableList("ST_AVG_ACTIVE_SESSIONS")

         for currentRow in query.execute():
            metricValue = currentRow["VALUE"].data()
            stats.append(metricValue)


   except Exception, e:
      print("manager error in statValues :\t" + str(e))

   return stats

def reportStats(message):
   """ Function that reports statistics about the test and the server nodes, and logs them in a file """
   print message
   sys.stdout.flush()

   clientCountFile = open(cfg['clientLogDir'] + '/clientCount.csv', 'a')
   csv_writer = csv.writer(clientCountFile, delimiter=';')
   csv_writer.writerow(message)
   clientCountFile.close()

def countClients(schema):
   """ Function that queries ST_REPORT.ST_SESSION to get the number of clients """
   numClients = 0

   try:
      query = schema.newQuery()
      query.addToOutputList("count(*)")
      query.addToTableList("ST_SESSION")
      query.setCondition("USERNAME = '" + cfg['db_username'] + "' AND LOWER(PROGRAM) LIKE LOWER('%" + cfg['testClientExe'] + "%')", coral.AttributeList())
      for currentRow in query.execute():
         for cell in currentRow:
            numClients = int(cell.data())
   except Exception, e:
      print("manager error :\t" + str(e))

   return numClients

#######################
# python startTest.py #
#######################
if __name__ == '__main__':

#### Verify parameters
   if len(sys.argv) != 2:
      print "Usage: startClients.py <logDirName>"
      sys.exit()

#### Load config parameters
   from config import Config
   cfg = Config().config

#### Create the log directory
   cfg['clientLogDir'] = cfg['clientLogDir'] + sys.argv[1]
   if os.path.exists(cfg['clientLogDir']) :
      print "Log directory specified already exists! Try another name."
      sys.exit()
   os.mkdir(cfg['clientLogDir'])

#### Copy the configuration files
   for file in cfg['fileToSave'].values():
      os.system('cp ' + file + ' ' +  cfg['clientLogDir'])
   
   #TO DO : keep ? sys.stderr = open(cfg['clientLogDir']+"/log_manager_err.log",'w')

#### Start the clients on the working nodes
   idOffset = 0
   cfg['testNodes'] = cfg['testNodes'][0:cfg['numBoxes']]
   sshCmdArg = ['ssh']
   sshCmdArg += ['-2nqx']
   sshCmdArg += ['node']
   sshCmdArg += ['command']

   for n in cfg['testNodes']:
      sshCmdArg[2] = n
      # Client node command : go to baseDir/bin and execute startClients.py
      sshCmdArg[3] = "cd '" + cfg['baseDir'] + "bin/'; ./startClients.py " + n + " " + str(idOffset) + " " + cfg['clientLogDir']
      os.spawnvp(os.P_NOWAIT,"ssh",sshCmdArg)

      idOffset += cfg['numMaxClientsPerBox']

#### Create the connection to the database
   import coral
   context = coral.Context()
   context.setVerbosityLevel('ERROR')
   #context.setVerbosityLevel('VERBOSE')
   svc = coral.ConnectionService(context)
   session = svc.connect(cfg['report_db_connection'], accessMode = coral.access_ReadOnly)
   transaction = session.transaction()
   transaction.start(True)
   schema = session.nominalSchema()
   sysSchema = session.schema("SYS")

#### Print the headers
   # The log files are named as follow
   logFileTemplate = cfg['clientLogDir'] + "/log_*.csv"
   headersArray = []
   while (headersArray == []):
      time.sleep(0.1)
      headers = os.popen("head -qn1 " + logFileTemplate + " 2> /dev/null")
      csv_reader = csv.reader(headers, delimiter=';')
      try:
         headersArray = csv_reader.next()
      except:
         headersArray = []

   headers = ["TimeStamp","Clients"]
   headersArray = headersArray[1 :]
   numMetrics = cfg['numMetrics']
   # Get metric names
   for name in headersArray[1 : numMetrics + 1]:
      headers.append(name)
   # Get statistic names
   statNames = statHeaders(schema)
   reportStats(headers + statNames)

#### Write the start date to executionDates.log
   testStart = time.time()
   dateFile = open(cfg['clientLogDir'] + '/executionDates.log', 'w')
   dateFile.write(str(testStart) + '\n') 
   dateFile.close()

#### Aggregate the clients' reports with the same frequency the clients do
   testEnd = testStart + cfg['testMaxDuration']
   lastReport = testStart

   while ((cfg['testMaxDuration']==0) or (time.time() < testEnd)):
      sleepTime = cfg['reportPeriod'] - (time.time() - lastReport)
      if sleepTime > 0:
         time.sleep(sleepTime)
      lastReport = time.time()

      results = []
      tailLogs = os.popen("tail -qn1 " + logFileTemplate + " 2> /dev/null")
      csv_reader = csv.reader(tailLogs, delimiter=';')

      for row in csv_reader:
         elapsedtime = float(row[1])
         if elapsedtime != 0:
            resultTotal = [float(i) for i in row[numMetrics + 2:]]
            if float(row[0]) > lastReport:
               lastReport = float(row[0])
            if results == []:
               results = resultTotal
            else:
               results = map(float.__add__, results, resultTotal)

      if results != []:
         numClients = countClients(schema)
         if numClients ==0:
           break
         report = [lastReport, str(numClients)]
         report += results
         report += statValues(sysSchema, schema)
         reportStats(report)

#### End of the test
   dateFile = open(cfg['clientLogDir']+'/executionDates.log', 'a')
   dateFile.write(str(time.time()) + '\n')
   dateFile.close()
   sys.exit(0)

