#! /usr/bin/env python
# $Id: statClients.py,v 1.10 2007/12/07 16:34:18 rbasset Exp $

"""\
 Print the number of clients for each node
 (connected on the Oracle database with the username specified in the configuration)
"""

import coral, os, sys

def stat(verbose=False):
   # loads config parameters
   from config import Config
   cfg = Config().config

   context = coral.Context()
   context.setVerbosityLevel('ERROR')
   svc = coral.ConnectionService(context)
   try:
      session = svc.connect(cfg['report_db_connection'], accessMode = coral.access_ReadOnly)
      transaction = session.transaction() 
      transaction.start(True)
      schema = session.nominalSchema()

      query = schema.newQuery()
      query.addToOutputList("INST_ID")
      query.addToOutputList("count(*)","numclients")
      query.addToTableList("ST_SESSION")
      query.setCondition("USERNAME = upper('" + cfg['db_username'] + "')", coral.AttributeList())
      query.groupBy("INST_ID")
      if verbose:
         query.addToOutputList("PROGRAM")
         query.addToOrderList("PROGRAM")
         query.groupBy("INST_ID, PROGRAM")

      print "Sessions for user '" + cfg['db_username'] + "'"
      print 

      if verbose:
         print "RAC node \tClient program @ Client node \tNumber sessions"
         for currentRow in query.execute():
            print str(int(currentRow['INST_ID'].data())) + '\t\t' + currentRow['PROGRAM'].data() + ' \t' + str(int(currentRow['numclients'].data()))
      else:
         print "RAC node \tNumClients"
         for currentRow in query.execute():
            print str(int(currentRow['INST_ID'].data())) + '\t\t' + str(int(currentRow['numclients'].data()))


   except Exception, e:
      print("error :\t" + str(e))

if __name__ == '__main__':
   if len(sys.argv) == 1:
      stat()
   elif (len(sys.argv) == 2) & (sys.argv[1]=='-v'):
      stat(True)
   else:
      print """Usage :
         statClients.py [-v]
          -v \tVerbose (prints out the client program names)
         """


