#!/usr/bin/python

#   @author Alexander Kalkhof
#   @date   2008-04-16

import sys, re
if re.search("windows", sys.platform) <> None:
	platform = 1
else:
	platform = 0	

import os, popen2, select, time

import xml.sax

def vprint(s):
        if "v" in sys.argv:
                print s

## getCommandOutput
## starts a program and returns exitcode, stdout, stderr
## return trippel : exitcode, stdout, stderr

def getCommandOutput(command):
	#split the command before, avoiding running in a shell
	#for some reasons the return tuppel is mixed as described in the documentation
	out, w, err = popen2.popen3(command)
	
	stdout = out.read()
	stderr = err.read()

	out.close()
	err.close()
	return 0, stdout, stderr

def getPWD():
	return os.path.realpath(os.curdir) + os.sep

def getTestPath():
	if 'CMTCONFIG' in os.environ:
		return getPWD() + os.pardir + os.sep + os.pardir + os.sep + os.pardir + os.sep + os.getenv('CMTCONFIG') + os.sep + 'tests' + os.sep + 'bin' + os.sep
	else:
		return "error"

class FindPlatform(xml.sax.ContentHandler):
	def __init__(self, platform):
		self.search_platform = platform
		self.found = False
		self.tests = []

	def startElement(self, name, attrs):
		if name == 'test' and self.found:
			self.tests.append([attrs.get('name', None), attrs.get('exclude', None)])

        	elif name == 'platform':
	        	if (attrs.get('name', None) == self.search_platform):
        	    		self.found = True
			else:
				self.found = False

	def isTestFound(self, name):
		for i in self.tests:
			if name in i:
				return i
			else:
				#check also windows prefixes
				parts = name.split('.')
				vprint(parts)
				if len(parts) == 2:
					if parts[1] == 'exe' and parts[0] in i:
						return i
		return None

class FindParams(xml.sax.ContentHandler):
	def __init__(self):
		self.arguments = []
		self.argument = 0
		self.writer = False
		self.reader = False
		self.reader_writer = ""
		self.content = ""

	def startElement(self, name, attrs):
		if name == 'option':
			self.argument = int(attrs.get('id', 0))
		elif name == 'writer':
			self.writer = True
			self.content = ""
		elif name == 'reader':
			self.reader = True
			self.content = ""
			self.reader_writer = attrs.get('writer', "")

	def characters(self, ch):
        	self.content = self.content + ch

	def endElement(self, name):
        	if name == 'writer':
	        	self.writer = False
			self.arguments.append((self.argument, self.content, None));
        	elif name == 'reader':
	        	self.reader = False
			self.arguments.append((self.argument, self.content, self.reader_writer));

	def getResult(self):
		return self.arguments

def transformParams(name, exclusion, result):
	#prepare exclusions
	exclusions = []
	if exclusion != None:
		for i in re.split(',', exclusion):
			exclusions.append(i.strip())

	#prepare parameter
	realparams = []
	for i in result:
		while i[0] > (len(realparams) - 1):
			realparams.append({})

	#write data in	
	for i in result:
		level, writer, reader = i[0], i[1], i[2]
		if( reader == None):
			if not writer in exclusions:
				realparams[level][writer] = []
		else:
			if not reader in exclusions:
				realparams[level][reader].append(writer)
				
	return (name, realparams)

def getParams():
	#read all test from the file
	params = []

	parser01 = xml.sax.make_parser()
	dh01 = FindPlatform(os.getenv('CMTCONFIG'))
	parser01.setContentHandler(dh01)
	parser01.parse('testlist.xml')

	testPath = getTestPath()
	vprint(testPath)
	for i in os.listdir(testPath):
		#find the test_Integration, simple solution
		result = dh01.isTestFound(i)
		if result != None:
			#execute the test and get the stdout back
			exitcode, stdout, stderr = getCommandOutput(testPath + os.sep + i + ' -x')
			vprint(stdout)
			vprint(stderr)
			#initialize the xml sax parser
			parser02 = xml.sax.make_parser()
			#initialize the parser handler classes
			dh02 = FindParams()
			#set the content hanlder in the sax parser
			parser02.setContentHandler(dh02)
			#start parsing process
			parser02.feed(stdout)
			name, realparams = transformParams(result[0], result[1], dh02.getResult())
			if len(realparams) > 0 and len(realparams[0]) > 0:
				print('[GETP] (IN) ' + result[0])
				params.append((name, realparams))
			else:
				print('[GETP] (--) ' + result[0])

	return params

def matrixR(level, params):
	lenp = len(params)
	if(level == (lenp - 1)):
		result = []
		for i in params[level]:
			result.append([i])
		vprint(result)
		return result
	else:
		result2 = []
		result = matrixR(level + 1, params)
		vprint('[ADDM]')
		vprint(params[level])
		vprint('[WITH]')
		vprint(result)
		for i in params[level]:
			for j in result:
				result3 = [i]
				result3.extend(j)
				result2.append(result3)
		vprint('[RSLT]')
		vprint(result2)
		return result2
		

def getTestPermutations(testvalues):
	params = testvalues[1]
	return matrixR(0, params)
	
def writeParams(params):
	#remove all old files
	try:
		os.mkdir('integration.qms')
	except:
		pass
	for i in os.listdir('integration.qms'):
		os.unlink('integration.qms/' + i)

	for i in params:
		result = getTestPermutations(i)
		for j in result:
			testname = i[0].split('_')

			refpath = ''
			refpath += testname[1]
			for h in range(2, len(testname)):
				refpath += '_' + testname[h]
			refpath = '../../Tests/%s/' % refpath

			parameters01 = ''
			for h in j:
				parameters01 += ' ' + h
			
			parameters02 = ''
			for h in j:
				parameters02 += '_' + h

			reffile = i[0] + parameters02 + '.ref'
			if not os.path.exists(refpath + reffile):
				reffile = i[0] + '.ref'
			

			vprint('[PMUT] for test %s: %s' % (i[0], j))
			#write the xml output into a variable
			#sorry but not realy good design
			xmloutput = ''
			xmloutput += '<?xml version="1.0" ?>'
			xmloutput += '<!DOCTYPE extension  PUBLIC "-//QM/2.3/Extension//EN"  "http://www.codesourcery.com/qm/dtds/2.3/-//qm/2.3/extension//en.dtd">\n'
			xmloutput += '<extension class="LCG_QMTestExtensions.ShellCommandTest" kind="test">\n'
#			xmloutput += '<extension class="command.ExecTest" kind="test">\n'
			xmloutput += '<argument name="excluded_lines">\n'
			xmloutput += '<text />\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="target_group">\n'
			xmloutput += '<text>.*</text>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stderr">\n'
			xmloutput += '<text>*</text>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdout">\n'
			xmloutput += '<text />\n'
			xmloutput += '</argument>'
			xmloutput += '<argument name="prerequisites">\n'
			xmloutput += '<set />\n'
			xmloutput += '</argument>'
			xmloutput += '<argument name="stdout_ref">\n'
			xmloutput += '<text>%s</text>\n' % reffile
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdout_tol">\n'
			xmloutput += '<integer>0</integer>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="exit_code">\n'
			xmloutput += '<integer>0</integer>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdout_tag">\n'
			xmloutput += '<text>[OVAL]</text>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="environment">\n'
			xmloutput += '<set />\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdout_ref_path">\n'
			xmloutput += '<text>%s</text>\n' % refpath
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="timeout">\n'
			xmloutput += '<integer>40</integer>\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="description_test">\n'
			xmloutput += '<text />\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="command">\n'
#			xmloutput += '<text>python runTest.py %s</text>\n' % i[0]

			xmloutput += '<text>%s %s</text>\n' % (i[0], parameters01)
			xmloutput += '</argument>\n'

			xmloutput += '<argument name="resources">\n'
			xmloutput += '<set />\n'
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdin">\n'
			xmloutput += '<text />\n'
			xmloutput += '</argument>\n'
			xmloutput += '</extension>\n'

			vprint(xmloutput)

			filename = 'integration.qms/'
			filename += testname[2]
			for h in range(3, len(testname)):
				filename += '_' + testname[h]

			filename += parameters02 + '.qmt'
			filename = filename.lower()
			vprint('[WOUT] %s' % filename)
			fd = open(filename, 'w')
			fd.write(xmloutput)
			fd.close()

writeParams(getParams())
